#!/usr/bin/perl

use strict;
use warnings;

#check required modules
#use LWP::UserAgent;
my $module_status_lwp = 1;
eval "use LWP::UserAgent; 1" or $module_status_lwp = 0;
if ( !$module_status_lwp ) {
  print "can\'t load module \"LWP::UserAgent\"!\n";
  print "http://www.stor2rrd.com/install.htm\n";
  exit 1;
}

#use HTTP::Request;
my $module_status_http = 1;
eval "use HTTP::Request; 1" or $module_status_http = 0;
if ( !$module_status_http ) {
  print "can\'t load module \"HTTP::Request\"!\n";
  print "http://www.stor2rrd.com/install.htm\n";
  exit 1;
}

#use Xorux_lib;
my $module_status_xorux_lib = 1;
eval "use Xorux_lib; 1" or $module_status_xorux_lib = 0;
if ( !$module_status_xorux_lib ) {
  print "can\'t load module \"Xorux_lib\"!\n";
  print "http://www.stor2rrd.com/install.htm\n";
  exit 1;
}

#use JSON qw(encode_json decode_json);
my $module_status_json = 1;
eval "use JSON qw(encode_json decode_json); 1" or $module_status_json = 0;
if ( !$module_status_json ) {
  print "can\'t load module \"JSON qw(encode_json decode_json)\"!\n";
  print "http://www.stor2rrd.com/install.htm\n";
  exit 1;
}

my ( $host ) = @ARGV;
#my $host     = $ENV{'BNA_IP'}   ||= "";
my $port     = $ENV{'BNA_PORT'} ||= "";
my $user     = $ENV{'BNA_USER'} ||= "";
my $password = $ENV{'BNA_PASS'} ||= "";

$password = Xorux_lib::unobscure_password($password);

if ( ! defined $host || $host eq '' ) {
  error( "BNA IP must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( ! defined $port || $port eq '' ) {
  error( "BNA PORT must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( ! defined $user || $user eq '' ) {
  error( "BNA USER must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( ! defined $password || $password eq '' ) {
  error( "BNA PASSWORD must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}

# try this test 3 times
for my $test_idx (0..2) {
  my $status = test_api();
  if ( defined $status && $status eq "1" ) {
    exit 1;
  }
  else {
    if ( $test_idx < 2) {
      sleep 10;
    }
    else {
      exit 0;
    }
  }
}

exit 0;

sub test_api {
  # LOGIN to API
  #
  my $protocol="https";
  my $url;
  my $ua;
  my $req;
  my $res;

  $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  $url = $protocol . "://".$host. ":$port" ."/rest/login";
  $req = HTTP::Request->new( POST => $url );

  $req->header( 'Content-Type' => 'application/vnd.brocade.networkadvisor+json;version=v1' );
  $req->header('Accept'=>'application/vnd.brocade.networkadvisor+json;version=v1');
  $req->header('WSUsername'=> $user );
  $req->header('WSPassword'=> $password);
  $res = $ua->request($req);

  if (!$res->is_success ){
    my $st_line = $res->status_line;
    error("Request error: $st_line");
    error("POST url     : $url");
    error("               LOGIN failed!");

    return 0;
  }
  else {
    print "  LOGIN             : ok\n";
  }


  # API request
  #
  my $header = $res->headers;
  my $token  = $header->{"wstoken"};

  $url = $protocol . "://".$host. ":$port" ."/rest/resourcegroups/All/fcswitches/";
  $req = HTTP::Request->new( GET => $url );
  $req->header('Content-Type' => 'application/vnd.brocade.networkadvisor+json;version=v1' );
  $req->header('Accept'=>'application/vnd.brocade.networkadvisor+json;version=v1');
  $req->header('WStoken'=> $token);
  $res = $ua->request($req);
  if (!$res->is_success ) {
    my $st_line = $res->status_line;
    error("Request error: $st_line");
    error("GET url      : $url");

    return 0;
  }
  else {
    print "  API request       : ok\n";
  }


  # Switches
  #
  my $result     = decode_json( $res->content );
  my $switch_idx = 0;
  my @data;
  if ( defined $result->{'fcSwitches'} && $result->{'fcSwitches'} ne '' ) {
    foreach ( @{ $result->{'fcSwitches'} } ) {
      $switch_idx++;

      my $switch_name = "";
      my $oper_status = "";
      my $state       = "";
      my $status_reas = "";

      if ( defined $_->{'name'} && $_->{'name'} ne "" ) {
        $switch_name = $_->{'name'};
      }
      if ( defined $_->{'operationalStatus'} && $_->{'operationalStatus'} ne "" ) {
        $oper_status = $_->{'operationalStatus'};
      }
      if ( defined $_->{'state'} && $_->{'state'} ne "" ) {
        $state = $_->{'state'};
      }
      if ( defined $_->{'statusReason'} && $_->{'statusReason'} ne "" ) {
        $status_reas = $_->{'statusReason'};
      }

      push(@data, "  SAN switch        : $switch_name");
      push(@data, "  operationalStatus : $oper_status");
      push(@data, "  state             : $state");
      push(@data, "  statusReason      : $status_reas");
      push(@data, "");
    }
  }


  # LOGOUT
  #
  $url = $protocol . "://".$host. ":$port" ."/logout";
  $req->header( 'Content-Type' => 'application/vnd.brocade.networkadvisor+json;version=v1' );
  $req->header('Accept'=>'application/vnd.brocade.networkadvisor+json;version=v1');
  $req->header('WStoken'=> $token);
  $res = $ua->request($req);
  if (!$res->is_success ) {
    my $st_line = $res->status_line;
    error("Request error: $st_line");
    error("GET url      : $url");
    error("               LOGOUT failed!");

    return 0;
  }
  else {
    print "  LOGOUT            : ok\n";
  }


  print "  Switches found    : $switch_idx\n\n";
  print join("\n", @data);
  print "\n";

  return 1;
}

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  #print STDERR "$act_time: $text : $!\n";
  print STDERR "$act_time: $text\n";


  return 1;
} ## end sub error
