# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use LoadDataModule;

# get cmd line params
my $version = "$ENV{version}";
my $storage = $ENV{STORAGE_NAME};
my $st_type = $ENV{STORAGE_TYPE};
my $webdir  = $ENV{WEBDIR};
my $bindir  = $ENV{BINDIR};
my $basedir = $ENV{INPUTDIR};
my $DEBUG   = $ENV{DEBUG};
$DEBUG = 0;    # suppres any debug messages here as they come into output* logs what causes filling them up
my $new_change = "$basedir/tmp/$version-run";
my $wrkdir     = "$basedir/data";
my $act_time   = localtime();
my $STEP       = $ENV{SAMPLE_RATE};

if ( defined $ENV{STOR2RRD_STEP} ) {
  $STEP = $ENV{STOR2RRD_STEP};    # it is in $STEP variable, viz load.sh, so far implemented only for SVC/Storwize switch to 60 secs
}
my $no_time  = 1380;              # says the time interval when RRDTOOL consideres a gap in input data, usually 3 * 5 + 2 = 17mins
                                  # to avoid small gaps every midnight
                                  # --> use 23 minutes as default
                                  # XIV is further adjusted to 33 minutes (1980) as it uses 15 mins step
                                  # When you change it here then update also bin/data_load.pl !!!!
my $rrdtool  = $ENV{RRDTOOL};
my $upgrade  = $ENV{UPGRADE};
my $ds8k_cap = 0;
if ( defined( $ENV{DS8K_CAP} ) ) {
  $ds8k_cap = $ENV{DS8K_CAP};
  $DEBUG    = 1;                  # allow debug for custom capacity graphs
}

if ( -f "$bindir/premium.pl" ) {
  require "$bindir/premium.pl";
}
else {
  require "$bindir/standard.pl";
}

if ( !-d "$webdir" ) {
  die "$act_time: Pls set correct path to Web server pages, it does not exist here: $webdir\n";
}

# start RRD via a pipe
use RRDp;
RRDp::start "$rrdtool";

load_storage();

# close RRD pipe
RRDp::end;

exit(0);

sub load_storage {
  my $act_time = localtime();

  if ( $storage eq '' ) {
    error("storage not found : could not parse storage name ");
    exit(1);
  }
  if ( $st_type eq '' ) {
    error("storage type not found : could not parse storage name ");
    exit(1);
  }
  print "Storage        : $storage\n" if $DEBUG;
  print "Storage type   : $st_type\n" if $DEBUG;

  if ( !-d "$wrkdir/$storage" ) {
    print "mkdir          : $wrkdir/$storage\n" if $DEBUG;
    mkdir( "$wrkdir/$storage", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage: $!";
  }
  if ( !-d "$webdir/$storage" ) {
    print "mkdir          : $webdir/$storage\n" if $DEBUG;
    mkdir( "$webdir/$storage", 0755 ) || die "$act_time: Cannot mkdir $webdir/$storage: $!";
  }
  if ( !-d "$wrkdir/$storage/VOLUME" ) {
    print "mkdir          : $wrkdir/$storage/VOLUME\n" if $DEBUG;
    mkdir( "$wrkdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME: $!";
  }
  if ( !-d "$wrkdir/$storage/POOL" ) {
    print "mkdir          : $wrkdir/$storage/POOL\n" if $DEBUG;
    mkdir( "$wrkdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/POOL: $!";
  }
  if ( !-d "$wrkdir/$storage/HOST" ) {
    print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
    mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
  }

  if ( $st_type =~ m/^HUS$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
    }
    if ( !-d "$wrkdir/$storage/RANK" ) {
      print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
    }
    if ( !-d "$wrkdir/$storage/RANK" ) {
      print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$storage/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE-CACHE: $!";
    }
    if ( !-l "$wrkdir/$storage/RG" ) {
      print "ln -s          : $wrkdir/$storage/RANK $wrkdir/$storage/RG\n" if $DEBUG;
      symlink( "$wrkdir/$storage/RANK", "$wrkdir/$storage/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$storage/RG: $!";
    }
  } ## end if ( $st_type =~ m/^HUS$/)

  if ( $st_type =~ m/^NETAPP$/ ) {
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/POOL" ) {
      print "mkdir          : $wrkdir/$storage/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/POOL: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/LUN" ) {
      mkdir( "$wrkdir/$storage/LUN", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/LUN: $!";
      print "mkdir          : $wrkdir/$storage/LUN\n" if $DEBUG;
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NAS" ) {
      print "mkdir          : $wrkdir/$storage/NAS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NAS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NAS: $!";
    }
  } ## end if ( $st_type =~ m/^NETAPP$/)

  if ( $st_type =~ m/^DS8K$/ ) {
    if ( !-d "$wrkdir/$storage/RANK" ) {
      print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
  } ## end if ( $st_type =~ m/^DS8K$/)

  if ( $st_type =~ m/^SWIZ$/ ) {
    if ( !-d "$wrkdir/$storage/RANK" ) {
      print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/VOLUME" ) {
      print "mkdir          : $wrkdir/$storage/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-NODE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-NODE: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$storage/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE-CACHE: $!";
    }
    if ( !-d "$wrkdir/$storage/IO-GROUP" ) {
      print "mkdir          : $wrkdir/$storage/IO-GROUP\n" if $DEBUG;
       mkdir( "$wrkdir/$storage/IO-GROUP", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/IO-GROUP: $!";
    }
    if ( !-l "$wrkdir/$storage/MDISK" ) {
      print "ln -s          : $wrkdir/$storage/RANK $wrkdir/$storage/MDISK\n" if $DEBUG;
      symlink( "$wrkdir/$storage/RANK", "$wrkdir/$storage/MDISK" ) || die "$act_time: Cannot ln -s $wrkdir/$storage/MDISK: $!";
    }
  } ## end if ( $st_type =~ m/^SWIZ$/)

  if ( $st_type =~ m/^VSPG$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/HOST-PORT" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST-PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST-PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
    }
    if ( !-d "$wrkdir/$storage/RANK" ) {
      print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$storage/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE-CACHE: $!";
    }
    if ( !-l "$wrkdir/$storage/RG" ) {
      print "ln -s          : $wrkdir/$storage/RANK $wrkdir/$storage/RG\n" if $DEBUG;
      symlink( "$wrkdir/$storage/RANK", "$wrkdir/$storage/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$storage/RG: $!";
    }
  } ## end if ( $st_type =~ m/^VSPG$/)
  if ( $st_type =~ m/^VNXF$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
    }
    if ( !-d "$wrkdir/$storage/FS" ) {
      print "mkdir          : $wrkdir/$storage/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FS: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$storage/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE-CACHE: $!";
    }
  } ## end if ( $st_type =~ m/^VNXF$/)

  if ( $st_type =~ m/^3PAR$/ ) {
    if ( !-d "$wrkdir/$storage/TIER" ) {
      print "mkdir          : $wrkdir/$storage/TIER\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/TIER", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/TIER: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/CPU-CORE" ) {
      print "mkdir          : $wrkdir/$storage/CPU-CORE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/CPU-CORE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/CPU-CORE: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/VVOL" ) {
      mkdir( "$wrkdir/$storage/VVOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VVOL: $!";
      print "mkdir          : $wrkdir/$storage/VVOL\n" if $DEBUG;
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^3PAR$/)
  if ( $st_type =~ m/^VNX$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
    }
  } ## end if ( $st_type =~ m/^VNX$/)
  if ( $st_type =~ m/^VMAX$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/TIER" ) {
      print "mkdir          : $wrkdir/$storage/TIER\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/TIER", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/TIER: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE-CACHE" ) {
      print "mkdir          : $wrkdir/$storage/NODE-CACHE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE-CACHE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE-CACHE: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
    }
    if ( !-d "$wrkdir/$storage/FAST-SG" ) {
      print "mkdir          : $wrkdir/$storage/FAST-SG\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FAST-SG", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FAST-SG: $!";
    }
    if ( !-d "$wrkdir/$storage/SLO" ) {
      print "mkdir          : $wrkdir/$storage/SLO\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/SLO", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/SLO: $!";
    }
  } ## end if ( $st_type =~ m/^VMAX$/)
  if ( $st_type =~ m/^VPLEX$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
    }
  } ## end if ( $st_type =~ m/^VPLEX$/)
  if ( $st_type =~ m/^UNITY$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/FS" ) {
      print "mkdir          : $wrkdir/$storage/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FS: $!";
    }
    if ( !-d "$wrkdir/$storage/NAS" ) {
      print "mkdir          : $wrkdir/$storage/NAS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NAS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NAS: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
    }
    if ( !-d "$wrkdir/$storage/NASPOOL" ) {
      print "mkdir          : $wrkdir/$storage/NASPOOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NASPOOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NASPOOL: $!";
    }
  } ## end if ( $st_type =~ m/^UNITY$/)
  if ( $st_type =~ m/^NIMBLE$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
  }
  if ( $st_type =~ m/^VNXE$/ ) {
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
    }
    if ( !-d "$wrkdir/$storage/FS" ) {
      print "mkdir          : $wrkdir/$storage/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FS: $!";
    }
    if ( !-d "$wrkdir/$storage/NAS" ) {
      print "mkdir          : $wrkdir/$storage/NAS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NAS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NAS: $!";
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
    }
    if ( !-d "$wrkdir/$storage/NASPOOL" ) {
      print "mkdir          : $wrkdir/$storage/NASPOOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NASPOOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NASPOOL: $!";
    }
  } ## end if ( $st_type =~ m/^VNXE$/)
  if ( $st_type =~ m/^DOTHILL$/ ) {
    if ( !-d "$wrkdir/$storage/RANK" ) {
      print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    }
    if ( !-l "$wrkdir/$storage/RG" ) {
      print "ln -s          : $wrkdir/$storage/RANK $wrkdir/$storage/RG\n" if $DEBUG;
      symlink( "$wrkdir/$storage/RANK", "$wrkdir/$storage/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$storage/RG: $!";
    }
    if ( !-d "$wrkdir/$storage/VOLUME" ) {
      print "mkdir          : $wrkdir/$storage/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/POOL" ) {
      print "mkdir          : $wrkdir/$storage/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^DOTHILL$/)

  if ( $st_type =~ m/^COMPELLENT$/ ) {
    if ( !-d "$wrkdir/$storage/VOLUME" ) {
      print "mkdir          : $wrkdir/$storage/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/POOL" ) {
      print "mkdir          : $wrkdir/$storage/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/VOLUME-FOLDER" ) {
      print "mkdir          : $wrkdir/$storage/VOLUME-FOLDER\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/VOLUME-FOLDER", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME-FOLDER: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^COMPELLENT$/)
  if ( $st_type =~ m/^OCEANSTOR$/ ) {
    if ( !-d "$wrkdir/$storage/VOLUME" ) {
      print "mkdir          : $wrkdir/$storage/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/POOL" ) {
      print "mkdir          : $wrkdir/$storage/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/PORT" ) {
      print "mkdir          : $wrkdir/$storage/PORT\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/PORT", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/PORT: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/DRIVE" ) {
      print "mkdir          : $wrkdir/$storage/DRIVE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/DRIVE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/DRIVE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/NODE" ) {
      print "mkdir          : $wrkdir/$storage/NODE\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/NODE", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/NODE: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/FS" ) {
      print "mkdir          : $wrkdir/$storage/FS\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/FS", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/FS: $!";
      LoadDataModule::touch();
    }
  } ## end if ( $st_type =~ m/^OCEANSTOR$/)
  if ( $st_type =~ m/^PURE$/ ) {
    if ( !-d "$wrkdir/$storage/VOLUME" ) {
      print "mkdir          : $wrkdir/$storage/VOLUME\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$storage/VOLUME" ) {
      print "mkdir          : $webdir/$storage/VOLUME\n" if $DEBUG;
      mkdir( "$webdir/$storage/VOLUME", 0755 ) || die "$act_time: Cannot mkdir $webdir/$storage/VOLUME: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/POOL" ) {
      print "mkdir          : $wrkdir/$storage/POOL\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$storage/POOL" ) {
      print "mkdir          : $webdir/$storage/POOL\n" if $DEBUG;
      mkdir( "$webdir/$storage/POOL", 0755 ) || die "$act_time: Cannot mkdir $webdir/$storage/POOL: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/HOST" ) {
      print "mkdir          : $wrkdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$storage/HOST" ) {
      print "mkdir          : $webdir/$storage/HOST\n" if $DEBUG;
      mkdir( "$webdir/$storage/HOST", 0755 ) || die "$act_time: Cannot mkdir $webdir/$storage/HOST: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$wrkdir/$storage/HOST-GROUP" ) {
      print "mkdir          : $wrkdir/$storage/HOST-GROUP\n" if $DEBUG;
      mkdir( "$wrkdir/$storage/HOST-GROUP", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/HOST-GROUP: $!";
      LoadDataModule::touch();
    }
    if ( !-d "$webdir/$storage/HOST-GROUP" ) {
      print "mkdir          : $webdir/$storage/HOST-GROUP\n" if $DEBUG;
      mkdir( "$webdir/$storage/HOST-GROUP", 0755 ) || die "$act_time: Cannot mkdir $webdir/$storage/HOST-GROUP: $!";
      LoadDataModule::touch();
    }
    #if ( !-d "$wrkdir/$storage/RANK" ) {
    #  print "mkdir          : $wrkdir/$storage/RANK\n" if $DEBUG;
    #  mkdir( "$wrkdir/$storage/RANK", 0755 ) || die "$act_time: Cannot mkdir $wrkdir/$storage/RANK: $!";
    #}
    #if ( !-l "$wrkdir/$storage/RG" ) {
    #  print "ln -s          : $wrkdir/$storage/RANK $wrkdir/$storage/RG\n" if $DEBUG;
    #  symlink( "$wrkdir/$storage/RANK", "$wrkdir/$storage/RG" ) || die "$act_time: Cannot ln -s $wrkdir/$storage/RG: $!";
    #  LoadDataModule::touch();
    #}
  }

  print "data_load      : start : $act_time : $STEP\n" if $DEBUG;

  if ( $st_type =~ m/^SWIZ$/ ) {
    LoadDataModule::load_data_svc_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^DS8K$/ ) {
    if ( $ds8k_cap == 1 ) {

      # custom capacity report
      $no_time = 45000;    # 12 hours, it is collected every 2 hour usually
                           # $STEP=3600; --> no, no use 5 mins
      LoadDataModule::load_data_ds8k_all_cap( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
    } ## end if ( $ds8k_cap == 1 )
    else {
      LoadDataModule::load_data_ds8k_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
    }
  } ## end if ( $st_type =~ m/^DS8K$/)
  if ( $st_type =~ m/^XIV$/ ) {
    $no_time = 1980;       # XIV heart beat time increase due to bigger step (15 minutes)
    LoadDataModule::load_data_xiv_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^DS5K$/ ) {
    LoadDataModule::load_data_ds5_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^HUS$/ ) {
    LoadDataModule::load_data_hus_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^NETAPP$/ ) {
    LoadDataModule::load_data_netapp_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^VSPG$/ ) {
    LoadDataModule::load_data_vspg_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^VNXF$/ ) {
    LoadDataModule::load_data_vnxf_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^3PAR/ ) {
    LoadDataModule::load_data_3par_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^VNX$/ ) {
    LoadDataModule::load_data_vnxb_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^VMAX$/ ) {
    LoadDataModule::load_data_vmax_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^VPLEX$/ ) {
    LoadDataModule::load_data_vplex_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^UNITY$/ ) {
    LoadDataModule::load_data_unity_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^NIMBLE$/ ) {
    LoadDataModule::load_data_nimble_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^VNXE$/ ) {
    LoadDataModule::load_data_vnxe_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^DOTHILL$/ ) {
    LoadDataModule::load_data_dothill_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^COMPELLENT$/ ) {
    LoadDataModule::load_data_compellent_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^OCEANSTOR$/ ) {
    LoadDataModule::load_data_oceanstor_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }
  if ( $st_type =~ m/^PURE$/ ) {
    LoadDataModule::load_data_pure_all( $storage, $wrkdir, $webdir, $act_time, $STEP, $DEBUG, $no_time, $st_type );
  }

  $act_time = localtime();
  print "data_load      : end   : $act_time\n" if $DEBUG;
} ## end sub load_storage

sub error {
  my $text     = shift;
  my $act_time = localtime();

  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( !defined($digit) ) {
    return 0;
  }
  if ( $digit eq '' ) {
    return 0;
  }
  if ( $digit eq 'U' ) {
    return 1;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

# if digit or hexa then true
sub ishexa {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  # hexa filter
  $digit_work =~ s/[a|b|c|d|e|f|A|B|C|D|E|F]//g;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  #main::error ("there was expected a digit but a string is there, field: , value: $digit");
  return 0;
} ## end sub ishexa

