#!/usr/bin/perl

use strict;
use warnings;
use Time::Local;
use Data::Dumper;
use Xorux_lib;

my $storage_name;
my $sample_rate;
my $user_name;
my $user_pw;

my $SMcli         = "SMcli";
my $authorization = "";

if ( defined $ENV{DS5_CLIDIR} && -f "$ENV{DS5_CLIDIR}/SMcli" ) { $SMcli = "$ENV{DS5_CLIDIR}/SMcli"; }
if ( defined $ENV{SAMPLE_RATE} ) { $sample_rate = $ENV{SAMPLE_RATE} }
if ( defined $ENV{DS5K_USER} )   { $user_name   = $ENV{DS5K_USER} }
if ( defined $ENV{DS5K_PW} )     { $user_pw     = $ENV{DS5K_PW} }
if ( defined $ENV{STORAGE_NAME} ) {
  $storage_name = $ENV{STORAGE_NAME};
}
else {
  error( "ds5perf.pl: DS5k storage name alias is required! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( defined $user_name && $user_name ne '' && defined $user_pw && $user_pw ne '' ) {
  $authorization = " -R $user_name -p $user_pw";
}

my $inputdir   = $ENV{INPUTDIR};
my $webdir     = $ENV{WEBDIR};
my $wrkdir     = "$inputdir/data";
my $tmpdir     = "$inputdir/tmp";
my $output_dir = "$wrkdir/$storage_name";
my $timeout    = $sample_rate * 3;          #alarm timeout

my $storage_type = "";
my $version      = "";
my $perf_out     = "";
my $conf_out     = "";
my $start_time   = "";
my $end_time     = "";
my $interval     = "";

#
# Set right command parameters!
#
my $SMcli_cmd = "$SMcli -n $storage_name -e$authorization -c";
set_params();

#
# Set right commands!
#
my @show_cmds;
my $cmd_show_all = "$SMcli_cmd \"show;\" 2>/dev/null";
@show_cmds = run_test_cmd($cmd_show_all);

my $all_volumes_to_cmd = "";
my $volumes_to_cmd     = "";
my $st_system_to_cmd   = "";
set_cmds();

#
# Run commands
#
my %data;
my @storage_items;
my @logicalDrives;

my $cmd_performanceStats = "$SMcli_cmd \"set session performanceMonitorInterval=$sample_rate performanceMonitorIterations=1 ; show $all_volumes_to_cmd performanceStats;\" 2>/dev/null";
my $cmd_logicalDrives    = "$SMcli_cmd \"show $volumes_to_cmd;\" 2>/dev/null";

@storage_items = run_cmd($cmd_performanceStats);
@logicalDrives = run_cmd($cmd_logicalDrives);

parse_logicaldrives();
parse_storage_items();

# Pools
if ( defined $data{POOL} ) {
  foreach my $pool ( keys( %{ $data{POOL} } ) ) {
    if ( defined $data{POOL}{$pool}{CMD_TYPE} && $data{POOL}{$pool}{CMD_TYPE} ne '' ) {
      my @pool_data;
      my $cmd_type = "";
      $cmd_type = $data{POOL}{$pool}{CMD_TYPE};

      my $cmd_pool = "$SMcli_cmd \"show $cmd_type [\\\"$pool\\\"] ;\" 2>/dev/null";
      @pool_data = run_cmd_pool( $cmd_pool, 0 );

      my $count_of_lines = @pool_data;
      if ( $count_of_lines eq 0 ) {
        $cmd_pool = "$SMcli_cmd \"show $cmd_type [\"$pool\"] ;\" 2>/dev/null";
        @pool_data = run_cmd_pool( $cmd_pool, 1 );
      }

      parse_pool_data( $pool, $cmd_type, \@pool_data );
    } ## end if ( defined $data{POOL...})
    else {
      error( "CMD_type for pool \"$pool\" not found! : " . __FILE__ . ":" . __LINE__ );
    }
  } ## end foreach my $pool ( keys( %{...}))
} ## end if ( defined $data{POOL...})

#
# print PERF file
#
print_perf();

#
# Configuration
#
my $config_html   = "$wrkdir/$storage_name/config.html";
my $cfg_time_diff = file_time_diff($config_html);
my $stor_sub_sys  = 0;
if ( !-f $config_html || $cfg_time_diff > 3599 ) {

  # firmware info
  my @show_summary;
  my $cmd_summary = "$SMcli_cmd \"show $st_system_to_cmd summary;\" 2>/dev/null";
  @show_summary = run_cmd($cmd_summary);
  parse_show_summary( \@show_summary );

  # host part
  my $hosts_count = grep {/Accessible By:/} @logicalDrives;
  if ( isdigit($hosts_count) && $hosts_count == 0 ) {    # run lunmapping cmd
    my @lunmappings;
    my $cmd_lunmappings = "$SMcli_cmd \"show $st_system_to_cmd lunMappings;\" 2>/dev/null";
    @lunmappings = run_cmd($cmd_lunmappings);
    parse_lunmappings( \@lunmappings );
  } ## end if ( isdigit($hosts_count...))

  my @host_topology;
  my $cmd_host_topology = "$SMcli_cmd \"show hostTopology;\" 2>/dev/null";
  @host_topology = run_cmd($cmd_host_topology);

  # test hostTopology cmd
  if ( "@host_topology" =~ /The command at line 1 is deprecated\. Use the command \"show storageArray hostTopology;\"/ ) {
    $stor_sub_sys      = 0;
    $cmd_host_topology = "$SMcli_cmd \"show storageArray hostTopology;\" 2>/dev/null";
    @host_topology     = run_cmd($cmd_host_topology);

    if ( isdigit($stor_sub_sys) && $stor_sub_sys eq "1" ) {
      $cmd_host_topology = "$SMcli_cmd \"show storageSubsystem hostTopology;\" 2>/dev/null";
      @host_topology     = run_cmd($cmd_host_topology);
    }
  } ## end if ( "@host_topology" ...)

  parse_host_topology( \@host_topology );

  #
  # print CONF file
  #
  print_conf();
} ## end if ( !-f $config_html ...)

#
# Health status
#
my $health_status_html = "$webdir/$storage_name/health_status.html";
my $hs_time_diff       = file_time_diff($health_status_html);
if ( !-f $health_status_html || $hs_time_diff > 180 ) {
  health_status();
}

#print Dumper(\%data);
#print Dumper(\%{$data{HOST}});
#print Dumper(\%{$data{'HOST_GROUP'}});

exit;

sub set_params {
  my $cmd = "$SMcli_cmd \"show hostTopology;\" 2>/dev/null";
  my @out = run_cmd_simple($cmd);

  my $succes_line = grep {/SMcli completed successfully/} @out;
  if ( defined $succes_line && isdigit($succes_line) && $succes_line > 0 ) {
    return 1;
  }

  my $error_text = grep {/The operation cannot be completed because an invalid/} @out;
  if ( defined $authorization && $authorization ne '' ) {
    if ( defined $error_text && isdigit($error_text) && $error_text > 0 ) { # change parameter "-R" by "-u"
      $cmd           = "$SMcli -n $storage_name -e -u $user_name -p $user_pw -c \"show hostTopology;\" 2>/dev/null";
      @out           = run_cmd_simple($cmd);

      $succes_line = grep {/SMcli completed successfully/} @out;
      if ( defined $succes_line && isdigit($succes_line) && $succes_line > 0 ) {
        $authorization = " -u $user_name -p $user_pw";
        $SMcli_cmd     = "$SMcli -n $storage_name -e$authorization -c";

        return 1;
      }

      $error_text = grep {/command line option to allow an insecure connection/} @out;
      if ( defined $error_text && isdigit($error_text) && $error_text > 0 ) { # add "-k" parameter
        $cmd = "$SMcli -n $storage_name -e -u $user_name -p $user_pw -k -c \"show hostTopology;\" 2>/dev/null";
        @out = run_cmd_simple($cmd);

        $succes_line = grep {/SMcli completed successfully/} @out;
        if ( defined $succes_line && isdigit($succes_line) && $succes_line > 0 ) {
          $authorization = " -u $user_name -p $user_pw";
          $SMcli_cmd     = "$SMcli -n $storage_name -e$authorization -k -c";

          return 1;
        }
      }
    }
  }

  $error_text = grep {/command line option to allow an insecure connection/} @out;
  if ( defined $error_text && isdigit($error_text) && $error_text > 0 ) { # add "-k" parameter
    $cmd = "$SMcli -n $storage_name -e$authorization -k -c \"show hostTopology;\" 2>/dev/null";
    @out = run_cmd_simple($cmd);

    $succes_line = grep {/SMcli completed successfully/} @out;
    if ( defined $succes_line && isdigit($succes_line) && $succes_line > 0 ) {
      $SMcli_cmd = "$SMcli -n $storage_name -e$authorization -k -c";

      return 1;
    }
  }


  error( "SMcli cmd not work! \"$SMcli -n $storage_name -e$authorization -c \"show hostTopology;\" 2>/dev/null\" : " . __FILE__ . ":" . __LINE__ );
  return 0;
}

sub set_cmds {

  # IBM
  # show allLogicalDrives performanceStats;
  # show logicalDrives;
  # show storagesubsystem summary;
  # show storageSubsystem lunMappings;
  # show hostTopology;
  # show storageSubsystem healthStatus;
  # save storageSubsystem criticalEvents file

  # NETAPP
  # show allVolumes performanceStats;
  # show volumes;
  # show storageArray summary;
  # show storageArray lunMappings;
  # show hostTopology;
  # show storageArray healthStatus;
  # save storageArray criticalEvents file

  foreach my $line (@show_cmds) {
    chomp $line;

    # IBM
    if ( $line =~ /"allLogicalDrives"/ ) { $all_volumes_to_cmd = "allLogicalDrives"; }
    if ( $line =~ /"logicalDrives"/ )    { $volumes_to_cmd     = "logicalDrives"; }
    if ( $line =~ /"storagesubsystem"/ ) { $st_system_to_cmd   = "storagesubsystem"; }
    if ( $line =~ /"storageSubsystem"/ ) { $st_system_to_cmd   = "storageSubsystem"; }

    # NETAPP
    if ( $line =~ /"allVolumes"/ )   { $all_volumes_to_cmd = "allVolumes"; }
    if ( $line =~ /"volumes"/ )      { $volumes_to_cmd     = "volumes"; }
    if ( $line =~ /"storageArray"/ ) { $st_system_to_cmd   = "storageArray"; }

    # DELL
    if ( $line =~ /"allVirtualDisks"/ ) { $all_volumes_to_cmd = "allVirtualDisks"; }
    if ( $line =~ /"virtualDisks"/ )    { $volumes_to_cmd     = "virtualDisks"; }
    if ( $line =~ /"storageArray"/ )    { $st_system_to_cmd   = "storageArray"; }
  } ## end foreach my $line (@show_cmds)

  if ( !defined $all_volumes_to_cmd || $all_volumes_to_cmd eq '' ) {
    error( "CMD not found! \"allLogicalDrives\" or \"allVolumes\" or \"allVirtualDisks\" : " . __FILE__ . ":" . __LINE__ );
    exit;
  }
  if ( !defined $volumes_to_cmd || $volumes_to_cmd eq '' ) {
    error( "CMD not found! \"logicalDrives\" or \"volumes\" or \"virtualDisks\" : " . __FILE__ . ":" . __LINE__ );
    exit;
  }
  if ( !defined $st_system_to_cmd || $st_system_to_cmd eq '' ) {
    error( "CMD not found! \"storagesubsystem\" or \"storageArray\" : " . __FILE__ . ":" . __LINE__ );
    exit;
  }

  return (1);
} ## end sub set_cmds

sub parse_host_topology {
  my $data          = shift;
  my @host_topology = @{$data};

  my $default_group = "";
  my $host_group    = "";
  my $host_name     = "";
  foreach my $line (@host_topology) {
    chomp $line;
    if ( $line =~ /^\s+Default Group\s+$/ ) {
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $default_group = "$line";
    }
    if ( $line =~ /Host Group:/ ) {
      $line =~ s/Host Group://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $host_group = "$line";
    } ## end if ( $line =~ /Host Group:/)
    if ( $line =~ /^         Host:/ ) {
      $host_group    = "";
      $default_group = "";
    }
    if ( $line =~ /Host:/ ) {
      $line =~ s/Host://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $host_name = "$line";

      if ( defined $host_group && $host_group ne '' ) {
        $data{HOST_GROUP}{$host_group}{HOSTS}{$host_name} = $host_name;
      }
      if ( defined $default_group && $default_group ne '' ) {
        $data{HOST_GROUP}{$default_group}{HOSTS}{$host_name} = $host_name;
      }
    } ## end if ( $line =~ /Host:/ )
  } ## end foreach my $line (@host_topology)

  # add Hosts from Host groups
  if ( defined $data{HOST_GROUP} ) {
    foreach my $group ( keys( %{ $data{HOST_GROUP} } ) ) {
      if ( defined $data{HOST_GROUP}{$group}{VOLUMES} && defined $data{HOST_GROUP}{$group}{HOSTS} ) {
        foreach my $host ( keys( %{ $data{HOST_GROUP}{$group}{HOSTS} } ) ) {
          foreach my $volume ( keys( %{ $data{HOST_GROUP}{$group}{VOLUMES} } ) ) {
            $data{HOST}{$host}{VOLUMES}{$volume} = $volume;
          }
        }
      } ## end if ( defined $data{HOST_GROUP...})
    } ## end foreach my $group ( keys( %...))
  } ## end if ( defined $data{HOST_GROUP...})

  return (1);
} ## end sub parse_host_topology

sub parse_lunmappings {
  my $data        = shift;
  my @lunmappings = @{$data};

  foreach my $volume_name ( keys( %{ $data{VOLUME} } ) ) {
    my $host = "";
    ($host) = grep {/^\s+$volume_name\s+/} @lunmappings;
    if ( defined $host && $host ne '' ) {
      chomp $host;
      $host =~ s/^\s+$volume_name.+  Host /Host /g;
      $host =~ s/  .+//g;

      if ( $host =~ /^Host Group\s+/ ) {
        $host =~ s/^Host Group\s+//;
        $data{VOLUME}{$volume_name}{'HOST_GROUP'} = $host;
        $data{HOST_GROUP}{$host}{VOLUMES}{$volume_name} = $volume_name;
      }
      else {
        $host =~ s/^Host\s+//;
        $data{VOLUME}{$volume_name}{'HOST'} = $host;
        $data{HOST}{$host}{VOLUMES}{$volume_name} = $volume_name;
      }
    } ## end if ( defined $host && ...)
  } ## end foreach my $volume_name ( keys...)

  return (1);
} ## end sub parse_lunmappings

sub parse_show_summary {
  my $data         = shift;
  my @show_summary = @{$data};

  my $found_idx       = 0;
  my $package_version = "";
  my $firmware_vers   = "";
  foreach my $line (@show_summary) {
    chomp $line;
    $line =~ s/^\s+//g;
    if ( $line =~ "^Chassis Serial Number:" ) {
      $found_idx++;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      my ( $title, $variable ) = split( ":", $line );
      $title =~ s/^\s+//g;
      $title =~ s/\s+$//g;
      $variable =~ s/^\s+//g;
      $variable =~ s/\s+$//g;

      #print "$title: $variable\n";
      $data{CONFIGURATION}{HEADER}{$found_idx} = "$title: $variable";
    } ## end if ( $line =~ "^Chassis Serial Number:")
    if ( $line =~ "^Current Package Version:" ) {
      if   ( $package_version =~ "Current Package Version:" ) { next; }
      else                                                    { $package_version = $line; }
      $found_idx++;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      my ( $title, $variable ) = split( ":", $line );
      $title =~ s/^\s+//g;
      $title =~ s/\s+$//g;
      $variable =~ s/^\s+//g;
      $variable =~ s/\s+$//g;

      #print "$title: $variable\n";
      $data{CONFIGURATION}{HEADER}{$found_idx} = "$title: $variable";
    } ## end if ( $line =~ "^Current Package Version:")
    if ( $line =~ "^SMW Version:" ) {
      $found_idx++;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      my ( $title, $variable ) = split( ":", $line );
      $title =~ s/^\s+//g;
      $title =~ s/\s+$//g;
      $variable =~ s/^\s+//g;
      $variable =~ s/\s+$//g;

      #print "$title: $variable\n";
      $data{CONFIGURATION}{HEADER}{$found_idx} = "$title: $variable";
    } ## end if ( $line =~ "^SMW Version:")
    if ( $line =~ "^Firmware version:" ) {
      if   ( $firmware_vers =~ "Firmware version:" ) { next; }
      else                                           { $firmware_vers = $line; }
      $found_idx++;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      my ( $title, $variable ) = split( ":", $line );
      $title =~ s/^\s+//g;
      $title =~ s/\s+$//g;
      $variable =~ s/^\s+//g;
      $variable =~ s/\s+$//g;

      #print "$title: $variable\n";
      $data{CONFIGURATION}{HEADER}{$found_idx} = "$title: $variable";
    } ## end if ( $line =~ "^Firmware version:")
    if ( $line =~ "^Feature pack:" ) {
      $found_idx++;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      my ( $title, $variable ) = split( ":", $line );
      $title =~ s/^\s+//g;
      $title =~ s/\s+$//g;
      $variable =~ s/^\s+//g;
      $variable =~ s/\s+$//g;

      #print "$title: $variable\n";
      $data{CONFIGURATION}{HEADER}{$found_idx} = "$title: $variable";
    } ## end if ( $line =~ "^Feature pack:")
    if ( $line =~ "^Feature pack submodel ID:" ) {
      $found_idx++;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      my ( $title, $variable ) = split( ":", $line );
      $title =~ s/^\s+//g;
      $title =~ s/\s+$//g;
      $variable =~ s/^\s+//g;
      $variable =~ s/\s+$//g;

      #print "$title: $variable\n";
      $data{CONFIGURATION}{HEADER}{$found_idx} = "$title: $variable";
    } ## end if ( $line =~ "^Feature pack submodel ID:")
  } ## end foreach my $line (@show_summary)

  return (1);
} ## end sub parse_show_summary

sub parse_pool_data {
  my $pool_name = shift;
  my $cmd_type  = shift;
  my $data      = shift;
  my @pool_data = @{$data};

  my $capacity      = "";
  my $used_capacity = "";
  my $free_capacity = "";

  my $free_cap_idx = 0;
  foreach my $line_k (@pool_data) {
    chomp $line_k;
    $line_k =~ s/^\s+//g;

    if ( $line_k =~ "^Usable capacity:" || $line_k =~ "^Capacity:" || $line_k =~ /^Capacity\s+/ ) {
      $line_k =~ s/Usable capacity://g;
      $line_k =~ s/Capacity://g;
      $line_k =~ s/Capacity//g;
      $line_k =~ s/,//g;
      $line_k =~ s/^\s+//g;
      $line_k =~ s/\s+$//g;
      my ( $cap_num, $cap_size ) = split( " ", $line_k );
      if ( $cap_size =~ /^B/ ) { $capacity = $cap_num / 1024 / 1024 / 1024 / 1024; }
      if ( $cap_size =~ /kB/ ) { $capacity = $cap_num / 1024 / 1024 / 1024; }
      if ( $cap_size =~ /MB/ ) { $capacity = $cap_num / 1024 / 1024; }
      if ( $cap_size =~ /GB/ ) { $capacity = $cap_num / 1024; }
      if ( $cap_size =~ /TB/ ) { $capacity = $cap_num; }
      if ( $cap_size =~ /PB/ ) { $capacity = $cap_num * 1024; }
    } ## end if ( $line_k =~ "^Usable capacity:"...)
    if ( $line_k =~ "^Used capacity:" ) {
      $line_k =~ s/Used capacity://g;
      $line_k =~ s/,//g;
      $line_k =~ s/^\s+//g;
      $line_k =~ s/\s+$//g;
      my ( $cap_num, $cap_size ) = split( " ", $line_k );
      if ( $cap_size =~ /^B/ ) { $used_capacity = $cap_num / 1024 / 1024 / 1024 / 1024; }
      if ( $cap_size =~ /kB/ ) { $used_capacity = $cap_num / 1024 / 1024 / 1024; }
      if ( $cap_size =~ /MB/ ) { $used_capacity = $cap_num / 1024 / 1024; }
      if ( $cap_size =~ /GB/ ) { $used_capacity = $cap_num / 1024; }
      if ( $cap_size =~ /TB/ ) { $used_capacity = $cap_num; }
      if ( $cap_size =~ /PB/ ) { $used_capacity = $cap_num * 1024; }
    } ## end if ( $line_k =~ "^Used capacity:")
    if ( $line_k =~ "^Free Capacity" ) {
      $free_cap_idx++;
      my $act_free_capacity = "";

      $line_k =~ s/Free Capacity//g;
      $line_k =~ s/://g;
      $line_k =~ s/\([0-9]+\),//g;
      $line_k =~ s/,//g;
      $line_k =~ s/^\s+//g;
      $line_k =~ s/\s+$//g;
      my ( $cap_num, $cap_size ) = split( " ", $line_k );
      if ( $cap_size =~ /^B/ ) { $act_free_capacity = $cap_num / 1024 / 1024 / 1024 / 1024; }
      if ( $cap_size =~ /kB/ ) { $act_free_capacity = $cap_num / 1024 / 1024 / 1024; }
      if ( $cap_size =~ /MB/ ) { $act_free_capacity = $cap_num / 1024 / 1024; }
      if ( $cap_size =~ /GB/ ) { $act_free_capacity = $cap_num / 1024; }
      if ( $cap_size =~ /TB/ ) { $act_free_capacity = $cap_num; }
      if ( $cap_size =~ /PB/ ) { $act_free_capacity = $cap_num * 1024; }

      if ( $free_cap_idx == 1 ) { $free_capacity = $act_free_capacity; }
      if ( $free_cap_idx > 1 )  { $free_capacity = $free_capacity + $act_free_capacity; }
    } ## end if ( $line_k =~ "^Free Capacity")
  } ## end foreach my $line_k (@pool_data)

  if ( !defined $capacity || $capacity eq '' || !isdigit($capacity) ) {
    my $volumes_cap_idx  = 0;
    my $volumes_capacity = "";
    if ( defined $data{VOLUME} && $data{VOLUME} ne '' ) {
      foreach my $vol_name ( keys( %{ $data{VOLUME} } ) ) {
        if ( defined $data{VOLUME}{$vol_name}{POOL} && $data{VOLUME}{$vol_name}{POOL} eq "$pool_name" ) {
          my $cap_line = "";
          ($cap_line) = grep {/^$vol_name\s+/} @pool_data;
          if ( defined $cap_line && $cap_line ne '' ) {
            $cap_line =~ s/^$vol_name//;
            $cap_line =~ s/^\s+//g;
            $cap_line =~ s/\s+$//g;
            $cap_line =~ s/,//g;

            my $act_capacity = "";

            my ( $cap_num, $cap_size ) = split( /\s+/, $cap_line );
            if ( defined $cap_num && isdigit($cap_num) && defined $cap_size && $cap_size ne '' ) {
              $volumes_cap_idx++;
              if ( $cap_size =~ /^B/ ) { $act_capacity = $cap_num / 1024 / 1024 / 1024 / 1024; }
              if ( $cap_size =~ /kB/ ) { $act_capacity = $cap_num / 1024 / 1024 / 1024; }
              if ( $cap_size =~ /MB/ ) { $act_capacity = $cap_num / 1024 / 1024; }
              if ( $cap_size =~ /GB/ ) { $act_capacity = $cap_num / 1024; }
              if ( $cap_size =~ /TB/ ) { $act_capacity = $cap_num; }
              if ( $cap_size =~ /PB/ ) { $act_capacity = $cap_num * 1024; }

              if ( $volumes_cap_idx == 1 ) {
                $volumes_capacity = $act_capacity;
              }
              else {
                $volumes_capacity = $volumes_capacity + $act_capacity;
              }
            } ## end if ( defined $cap_num ...)
          } ## end if ( defined $cap_line...)
        } ## end if ( defined $data{VOLUME...})
        else {
          next;
        }
      } ## end foreach my $vol_name ( keys...)
    } ## end if ( defined $data{VOLUME...})

    if ( defined $volumes_capacity && isdigit($volumes_capacity) ) {
      if ( defined $free_capacity && isdigit($free_capacity) ) {
        $capacity = $volumes_capacity + $free_capacity;
      }
      else {
        $capacity = $volumes_capacity;
      }
    } ## end if ( defined $volumes_capacity...)
  } ## end if ( !defined $capacity...)

  if ( defined $capacity      && isdigit($capacity) )      { $data{POOL}{$pool_name}{CAPACITY}      = $capacity; }
  if ( defined $used_capacity && isdigit($used_capacity) ) { $data{POOL}{$pool_name}{CAPACITY_USED} = $used_capacity; }
  if ( defined $free_capacity && isdigit($free_capacity) ) { $data{POOL}{$pool_name}{CAPACITY_FREE} = $free_capacity; }

  return (1);
} ## end sub parse_pool_data

sub parse_storage_items {

  ### storage type and version for different outputs
  if ( "@storage_items" =~ /\"Objects\"/ ) {
    $storage_type = "new";
    $version      = "v2";
  }
  if ( "@storage_items" =~ /\"Storage Subsystems/ || "@storage_items" =~ /\"Storage Arrays/ || "@storage_items" =~ /Storage Subsystems/ || "@storage_items" =~ /Storage Arrays/ ) {
    $storage_type = "old";
    $version      = "v1";
  }

  ### search start date/time and interval
  my ($time_line) = grep {/Performance Monitor Statistics/} @storage_items;
  $time_line =~ s/\"//g;
  my ( undef, $time, $int ) = split( " - ", $time_line );
  if ( !defined $time || !defined $int ) {    # maybe diff output, try another way
    ($time) = grep {/Date\/Time:/} @storage_items;
    ($int)  = grep {/Polling interval in seconds:/} @storage_items;
  }

  $time =~ s/Date\/Time://g;
  $time =~ s/^\s+//g;
  $time =~ s/\s+$//g;
  $int =~ s/Polling interval in seconds://g;
  $int =~ s/^\s+//g;
  $int =~ s/\s+$//g;
  my ( $date,  $time_s, $part_of_the_day ) = split( " ", $time );
  my ( $month, $day,    $year )            = split( "/", $date );
  my ( $hour,  $min,    $sec )             = split( ":", $time_s );
  my $year_s = $year + 2000;
  my $hour_s = $hour;

  if ( $part_of_the_day =~ /PM/ && $hour !~ /12/ ) {
    $hour_s = $hour + 12;
  }
  if ( $part_of_the_day =~ /AM/ && $hour =~ /12/ ) {
    $hour_s = $hour - 12;
  }
  my $s_timestamp = timelocal( $sec, $min, $hour_s, $day, $month - 1, $year_s );
  my $s_date = sprintf( "%4d:%02d:%02d",  $year_s, $month, $day );
  my $s_time = sprintf( "%02d:%02d:%02d", $hour_s, $min,   $sec );
  my $e_timestamp = $s_timestamp + $int;
  my ( $sec_e, $min_e, $hour_e, $day_e, $month_e, $year_e, $wday_e, $yday_e, $isdst_e ) = localtime($e_timestamp);
  my $e_date = sprintf( "%4d:%02d:%02d",  $year_e + 1900, $month_e + 1, $day_e );
  my $e_time = sprintf( "%02d:%02d:%02d", $hour_e,        $min_e,       $sec_e );
  my $time_to_output_name = sprintf( "%4d%02d%02d_%02d%02d", $year_s, $month, $day, $hour_s, $min );

  ### set start time, end time, interval and output files
  $start_time = "$s_date" . "T" . "$s_time";
  $end_time   = "$e_date" . "T" . "$e_time";
  $interval   = "$int";
  $perf_out   = "$output_dir/$storage_name\_ds5perf_$time_to_output_name.out.tmp";
  $conf_out   = "$output_dir/$storage_name\_ds5conf_$time_to_output_name.out.tmp";

  ### parse volumes performance
  my @header;
  foreach my $line (@storage_items) {
    chomp $line;
    $line =~ s/"//g;

    if ( $line =~ /^Storage Subsystems|^Objects|^Storage Arrays/ ) {
      @header = split( ",", $line );
    }

    if ( $line =~ /^Thin Logical Drive/ || $line =~ /^Logical Drive/ || $line =~ /^Thin Volume/ || $line =~ /^Volume/ || $line =~ /^Virtual Disk/ ) {
      $line =~ s/^Logical Drive //g;
      $line =~ s/^Thin Volume //g;
      $line =~ s/^Volume //g;
      $line =~ s/^Virtual Disk //g;
      $line =~ s/^Thin Logical Drive //g;

      my @values      = split( ",", $line );
      my $value_idx   = 0;
      my $volume_name = "";
      foreach my $value (@values) {
        $header[$value_idx] =~ s/\s+$//;
        $value =~ s/\s+$//;

        #print "$header[$value_idx] = $value\n";
        if ( $value_idx == 0 ) { $volume_name = $value; }
        if ( defined $data{VOLUME}{$volume_name} ) {
          $data{VOLUME}{$volume_name}{PERF}{ $header[$value_idx] } = $value;
        }
        else {
          error( "Volume not found : \"$volume_name\" : " . __FILE__ . ":" . __LINE__ );
          last;
        }
        $value_idx++;
      } ## end foreach my $value (@values)
    } ## end if ( $line =~ /^Logical Drive/...)
  } ## end foreach my $line (@storage_items)

  return (1);
} ## end sub parse_storage_items

sub parse_logicaldrives {

  my $volume_name = "";
  foreach my $line (@logicalDrives) {
    chomp $line;
    if ( $line =~ "Logical Drive name:" || $line =~ "Volume name:" || $line =~ "Virtual Disk name:" || $line =~ "LOGICAL DRIVE NAME:" ) {

      #print "$line\n";
      $line =~ s/Logical Drive name://g;
      $line =~ s/Volume name://g;
      $line =~ s/Virtual Disk name://g;
      $line =~ s/LOGICAL DRIVE NAME://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $volume_name = $line;
      $data{VOLUME}{$volume_name}{NAME} = $volume_name;
    } ## end if ( $line =~ "Logical Drive name:"...)
    if ( $line =~ "Logical Drive ID:" || $line =~ "Volume world-wide identifier:" || $line =~ "Virtual Disk world-wide identifier:" || $line =~ "Logical Drive world-wide identifier:" ) {

      #print "$line\n";
      $line =~ s/Logical Drive ID://g;
      $line =~ s/Volume world-wide identifier://g;
      $line =~ s/Virtual Disk world-wide identifier://g;
      $line =~ s/Logical Drive world-wide identifier://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $line =~ s/://g;
      $data{VOLUME}{$volume_name}{ID} = $line;
    } ## end if ( $line =~ "Logical Drive ID:"...)
    if ( ($line =~ "Capacity:" || $line =~ "Virtual capacity:") && !$data{VOLUME}{$volume_name}{CAPACITY} ) {

      #print "$line\n";
      $line =~ s/Capacity://g;
      $line =~ s/Virtual capacity://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $line =~ s/,//g;
      $data{VOLUME}{$volume_name}{CAPACITY} = $line;
    } ## end if ( $line =~ "Capacity:"...)
    if ( $line =~ "Associated disk pool:" || $line =~ "Associated array:" || $line =~ "Associated volume group:" || $line =~ "Associated disk group:" ) {

      #print "$line\n";
      my $type = "";
      if ( $line =~ "Associated disk pool:" )    { $type = "diskPool"; }
      if ( $line =~ "Associated array:" )        { $type = "array"; }
      if ( $line =~ "Associated volume group:" ) { $type = "volumeGroup"; }
      if ( $line =~ "Associated disk group:" )   { $type = "diskGroup"; }
      $line =~ s/Associated disk pool://g;
      $line =~ s/Associated array://g;
      $line =~ s/Associated volume group://g;
      $line =~ s/Associated disk group://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $data{VOLUME}{$volume_name}{POOL} = $line;
      $data{POOL}{$line}{NAME}          = $line;
      $data{POOL}{$line}{CMD_TYPE}      = $type;
    } ## end if ( $line =~ "Associated disk pool:"...)
    if ( $line =~ "Current owner:" ) {

      #print "$line\n";
      $line =~ s/Current owner://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $data{VOLUME}{$volume_name}{CONTROLLER} = $line;
    } ## end if ( $line =~ "Current owner:")
    if ( $line =~ "Interface type:" || $line =~ "Drive interface type:" || $line =~ "Drive type:" || $line =~ "Physical Disk type:" ) {

      #print "$line\n";
      $line =~ s/Interface type://g;
      $line =~ s/Drive interface type://g;
      $line =~ s/Drive type://g;
      $line =~ s/Physical Disk type://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      $data{VOLUME}{$volume_name}{INTERFACE_TYPE} = $line;
    } ## end if ( $line =~ "Interface type:"...)
    if ( $line =~ "Accessible By:" ) {

      #print "$line\n";
      $line =~ s/Accessible By://g;
      $line =~ s/^\s+//g;
      $line =~ s/\s+$//g;
      if ( $line =~ /^Host Group\s+/ ) {
        $line =~ s/^Host Group\s+//;
        $data{VOLUME}{$volume_name}{'HOST_GROUP'} = $line;
        $data{HOST_GROUP}{$line}{VOLUMES}{$volume_name} = $volume_name;
      }
      elsif ( $line =~ /^Default Group$/ ) {
        $data{VOLUME}{$volume_name}{'HOST_GROUP'} = $line;
        $data{HOST_GROUP}{$line}{VOLUMES}{$volume_name} = $volume_name;
      }
      elsif ( $line =~ /Host\s+/ ) {
        $line =~ s/^Host\s+//;
        $data{VOLUME}{$volume_name}{'HOST'} = $line;
        $data{HOST}{$line}{VOLUMES}{$volume_name} = $volume_name;
      }
    } ## end if ( $line =~ "Accessible By:")
  } ## end foreach my $line (@logicalDrives)

  return (1);
} ## end sub parse_logicaldrives

sub print_conf {
  open( CONF, ">$conf_out" ) || error( "Couldn't open file $conf_out $!" . __FILE__ . ":" . __LINE__ ) && exit;

  # Configuration data main header
  print CONF "Configuration Data\n";
  print CONF "------------------\n";
  print CONF "DS5K type : $version\n";
  if ( defined $data{CONFIGURATION}{HEADER} ) {
    foreach my $idx ( sort { $a <=> $b } keys( %{ $data{CONFIGURATION}{HEADER} } ) ) {
      if ( defined $data{CONFIGURATION}{HEADER}{$idx} && $data{CONFIGURATION}{HEADER}{$idx} ne '' ) {
        print CONF "$data{CONFIGURATION}{HEADER}{$idx}\n";
      }
    }
  } ## end if ( defined $data{CONFIGURATION...})
  print CONF "\nVolume Level Configuration\n";
  print CONF "--------------------------\n";
  print CONF "volume_id,id,name,,,,pool_id,pool_name,capacity (MB),,,,,,vdisk_UID,,,,,,interface_type\n";

  foreach my $volume_name ( keys( %{ $data{VOLUME} } ) ) {
    my $volume_id          = "";
    my $volume_pool        = "";
    my $vol_interface_type = "";
    my $capacity           = "";

    if ( defined $data{VOLUME}{$volume_name}{ID}             && $data{VOLUME}{$volume_name}{ID} ne '' )             { $volume_id          = $data{VOLUME}{$volume_name}{ID}; }
    if ( defined $data{VOLUME}{$volume_name}{POOL}           && $data{VOLUME}{$volume_name}{POOL} ne '' )           { $volume_pool        = $data{VOLUME}{$volume_name}{POOL}; }
    if ( defined $data{VOLUME}{$volume_name}{INTERFACE_TYPE} && $data{VOLUME}{$volume_name}{INTERFACE_TYPE} ne '' ) { $vol_interface_type = $data{VOLUME}{$volume_name}{INTERFACE_TYPE}; }
    if ( defined $data{VOLUME}{$volume_name}{CAPACITY}       && $data{VOLUME}{$volume_name}{CAPACITY} ne '' ) {
      my $cap_line = $data{VOLUME}{$volume_name}{CAPACITY};
      my ( $cap_num, $cap_size ) = split( /\s+/, $cap_line );
      if ( isdigit($cap_num) ) {
        if ( $cap_size =~ /MB/ ) { $capacity = $cap_num; }
        if ( $cap_size =~ /GB/ ) { $capacity = $cap_num * 1024; }
        if ( $cap_size =~ /TB/ ) { $capacity = $cap_num * 1024 * 1024; }
        if ( $cap_size =~ /PB/ ) { $capacity = $cap_num * 1024 * 1024 * 1024; }
      } ## end if ( isdigit($cap_num))
    } ## end if ( defined $data{VOLUME...})

    if ( defined $data{VOLUME}{$volume_name}{PERF} ) {
      print CONF "$volume_id,,$volume_name,,,,,$volume_pool,$capacity,,,,,,,,,,,,$vol_interface_type\n";
    }
    else {
      print CONF "$volume_id,,$volume_name,,,,,$volume_pool,$capacity,,,,,,,,,,,,$vol_interface_type,no_perf_data\n";
    }
  } ## end foreach my $volume_name ( keys...)

  print CONF "\nHost Level Configuration\n";
  print CONF "--------------------------\n";
  print CONF "host_id,id,name,port_count,Type,volume_count,WWPN,Volume IDs,Volume Names\n";

  foreach my $host_name ( keys( %{ $data{HOST} } ) ) {
    if ( defined $host_name && $host_name ne '' ) {
      my $vol_ids_line   = "";
      my $vol_names_line = "";
      if ( defined $data{HOST}{$host_name}{VOLUMES} ) {
        my $vol_idx = 0;
        foreach my $vol_name ( keys( %{ $data{HOST}{$host_name}{VOLUMES} } ) ) {
          if ( defined $data{VOLUME}{$vol_name}{ID} && $data{VOLUME}{$vol_name}{ID} ne '' ) {
            my $vol_id = $data{VOLUME}{$vol_name}{ID};
            $vol_idx++;
            $vol_name =~ s/\s/=====space=====/g;
            $vol_id =~ s/\s/=====space=====/g;
            if ( isdigit($vol_idx) && $vol_idx == 1 ) {
              $vol_ids_line   = "$vol_id";
              $vol_names_line = "$vol_name";
            }
            else {
              $vol_ids_line   .= " $vol_id";
              $vol_names_line .= " $vol_name";
            }
          } ## end if ( defined $data{VOLUME...})
        } ## end foreach my $vol_name ( keys...)
      } ## end if ( defined $data{HOST...})
      print CONF ",,$host_name,,,,,$vol_ids_line,$vol_names_line\n";
    } ## end if ( defined $host_name...)
  } ## end foreach my $host_name ( keys...)

  # Pool conf part
  print CONF "\nPool Level Configuration\n";
  print CONF "------------------------\n";
  print CONF "name,id,status,mdisk_count,vdisk_count,capacity (GB),extent_size,free_capacity (GB),virtual_capacity,used_capacity (GB),real_capacity,overallocation,warning,easy_tier,easy_tier_status,compression_active,compression_virtual_capacity,compression_compressed_capacity,compression_uncompressed_capacity\n";

  foreach my $pool_name ( keys( %{ $data{POOL} } ) ) {
    my $capacity      = "";
    my $used_capacity = "";
    my $free_capacity = "";

    if ( defined $data{POOL}{$pool_name}{CAPACITY}      && isdigit( $data{POOL}{$pool_name}{CAPACITY} ) )      { $capacity      = $data{POOL}{$pool_name}{CAPACITY} * 1024; }
    if ( defined $data{POOL}{$pool_name}{CAPACITY_USED} && isdigit( $data{POOL}{$pool_name}{CAPACITY_USED} ) ) { $used_capacity = $data{POOL}{$pool_name}{CAPACITY_USED} * 1024; }
    if ( defined $data{POOL}{$pool_name}{CAPACITY_FREE} && isdigit( $data{POOL}{$pool_name}{CAPACITY_FREE} ) ) { $free_capacity = $data{POOL}{$pool_name}{CAPACITY_FREE} * 1024; }

    print CONF "$pool_name,,,,,$capacity,,$free_capacity,,$used_capacity,,,,,,,,,,,,,,,,,,,,,\n";
  } ## end foreach my $pool_name ( keys...)

  close(CONF);

  return (1);
} ## end sub print_conf

sub print_perf {
  open( PERF, ">$perf_out" ) || error( "Couldn't open file $perf_out $!" . __FILE__ . ":" . __LINE__ ) && exit;

  print PERF "Volume Level Statistics\n";
  print PERF "  Interval Start:   $start_time\n";
  print PERF "  Interval End:     $end_time\n";
  print PERF "  Interval Length:  $interval seconds\n";
  print PERF "---------------------\n";
  print PERF "Volume ID,Time,Interval (s),Volume Name,Pool Name,Controler,Total IOs,Total IO Rate (IO/s),Total Data Rate (KB/s),Read Hits,Write Hits,Cache read %,SSD Read Cache Hit %,IO Latency,Cache hits,Capacity (MB)\n";

  foreach my $volume_name ( keys( %{ $data{VOLUME} } ) ) {
    my $id             = "";
    my $pool           = "";
    my $vol_controller = "";
    my $capacity       = "";

    if ( defined $data{VOLUME}{$volume_name}{ID}         && $data{VOLUME}{$volume_name}{ID} ne '' )         { $id             = $data{VOLUME}{$volume_name}{ID}; }
    if ( defined $data{VOLUME}{$volume_name}{POOL}       && $data{VOLUME}{$volume_name}{POOL} ne '' )       { $pool           = $data{VOLUME}{$volume_name}{POOL}; }
    if ( defined $data{VOLUME}{$volume_name}{CONTROLLER} && $data{VOLUME}{$volume_name}{CONTROLLER} ne '' ) { $vol_controller = $data{VOLUME}{$volume_name}{CONTROLLER}; }
    if ( defined $data{VOLUME}{$volume_name}{CAPACITY}   && $data{VOLUME}{$volume_name}{CAPACITY} ne '' ) {
      my $cap_line = $data{VOLUME}{$volume_name}{CAPACITY};
      my ( $cap_num, $cap_size ) = split( /\s+/, $cap_line );
      if ( isdigit($cap_num) ) {
        if ( $cap_size =~ /MB/ ) { $capacity = $cap_num; }
        if ( $cap_size =~ /GB/ ) { $capacity = $cap_num * 1024; }
        if ( $cap_size =~ /TB/ ) { $capacity = $cap_num * 1024 * 1024; }
        if ( $cap_size =~ /PB/ ) { $capacity = $cap_num * 1024 * 1024 * 1024; }
      } ## end if ( isdigit($cap_num))
    } ## end if ( defined $data{VOLUME...})
    $vol_controller =~ s/Controller in slot//g;
    $vol_controller =~ s/RAID Controller Module in slot//g;
    $vol_controller =~ s/^\s+//g;
    $vol_controller =~ s/\s+$//g;

    my $total_IOs                   = "";    # Total IOs
    my $Read_Percentage             = "";    # Read Percentage
    my $Cache_Hit_Percentage        = "";    # Cache Hit Percentage
    my $Current_KBsec               = "";    # Current KB/second
    my $Current_MBsec               = "";    # Current MB/second
    my $Maximum_KBsec               = "";    # Maximum KB/second
    my $Maximum_MBsec               = "";    # Maximum MB/second
    my $Current_IOsec               = "";    # Current IO/second
    my $Maximum_IOsec               = "";    # Maximum IO/second
    my $Read_pct                    = "";    # Read %
    my $Primary_Read_Cache_Hit_pct  = "";    # Primary Read Cache Hit %
    my $Primary_Write_Cache_Hit_pct = "";    # Primary Write Cache Hit %
    my $SSD_Read_Cache_Hit_pct      = "";    # SSD Read Cache Hit %
    my $Current_MBs_sec             = "";    # Current MBs/sec
    my $Maximum_MBs_sec             = "";    # Maximum MBs/sec
    my $Current_IOs_sec             = "";    # Current IOs/sec
    my $Maximum_IOs_sec             = "";    # Maximum IOs/sec
    my $Minimum_IOs_sec             = "";    # Minimum IOs/sec
    my $Average_IOs_sec             = "";    # Average IOs/sec
    my $Minimum_MBs_sec             = "";    # Minimum MBs/sec
    my $Average_MBs_sec             = "";    # Average MBs/sec
    my $Current_IO_Latency          = "";    # Current IO Latency
    my $Maximum_IO_Latency          = "";    # Maximum IO Latency
    my $Minimum_IO_Latency          = "";    # Minimum IO Latency
    my $Average_IO_Latency          = "";    # Average IO Latency

    if ( defined $data{VOLUME}{$volume_name}{PERF} ) {
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Total IOs'}                 && $data{VOLUME}{$volume_name}{PERF}{'Total IOs'} ne '' )                 { $total_IOs                   = $data{VOLUME}{$volume_name}{PERF}{'Total IOs'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Read Percentage'}           && $data{VOLUME}{$volume_name}{PERF}{'Read Percentage'} ne '' )           { $Read_Percentage             = $data{VOLUME}{$volume_name}{PERF}{'Read Percentage'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Cache Hit Percentage'}      && $data{VOLUME}{$volume_name}{PERF}{'Cache Hit Percentage'} ne '' )      { $Cache_Hit_Percentage        = $data{VOLUME}{$volume_name}{PERF}{'Cache Hit Percentage'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Current KB/second'}         && $data{VOLUME}{$volume_name}{PERF}{'Current KB/second'} ne '' )         { $Current_KBsec               = $data{VOLUME}{$volume_name}{PERF}{'Current KB/second'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Current MB/second'}         && $data{VOLUME}{$volume_name}{PERF}{'Current MB/second'} ne '' )         { $Current_MBsec               = $data{VOLUME}{$volume_name}{PERF}{'Current MB/second'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Maximum KB/second'}         && $data{VOLUME}{$volume_name}{PERF}{'Maximum KB/second'} ne '' )         { $Maximum_KBsec               = $data{VOLUME}{$volume_name}{PERF}{'Maximum KB/second'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Maximum MB/second'}         && $data{VOLUME}{$volume_name}{PERF}{'Maximum MB/second'} ne '' )         { $Maximum_MBsec               = $data{VOLUME}{$volume_name}{PERF}{'Maximum MB/second'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Current IO/second'}         && $data{VOLUME}{$volume_name}{PERF}{'Current IO/second'} ne '' )         { $Current_IOsec               = $data{VOLUME}{$volume_name}{PERF}{'Current IO/second'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Maximum IO/second'}         && $data{VOLUME}{$volume_name}{PERF}{'Maximum IO/second'} ne '' )         { $Maximum_IOsec               = $data{VOLUME}{$volume_name}{PERF}{'Maximum IO/second'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Read %'}                    && $data{VOLUME}{$volume_name}{PERF}{'Read %'} ne '' )                    { $Read_pct                    = $data{VOLUME}{$volume_name}{PERF}{'Read %'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Primary Read Cache Hit %'}  && $data{VOLUME}{$volume_name}{PERF}{'Primary Read Cache Hit %'} ne '' )  { $Primary_Read_Cache_Hit_pct  = $data{VOLUME}{$volume_name}{PERF}{'Primary Read Cache Hit %'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Primary Write Cache Hit %'} && $data{VOLUME}{$volume_name}{PERF}{'Primary Write Cache Hit %'} ne '' ) { $Primary_Write_Cache_Hit_pct = $data{VOLUME}{$volume_name}{PERF}{'Primary Write Cache Hit %'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'SSD Read Cache Hit %'}      && $data{VOLUME}{$volume_name}{PERF}{'SSD Read Cache Hit %'} ne '' )      { $SSD_Read_Cache_Hit_pct      = $data{VOLUME}{$volume_name}{PERF}{'SSD Read Cache Hit %'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Current MBs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Current MBs/sec'} ne '' )           { $Current_MBs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Current MBs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Maximum MBs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Maximum MBs/sec'} ne '' )           { $Maximum_MBs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Maximum MBs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Current IOs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Current IOs/sec'} ne '' )           { $Current_IOs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Current IOs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Maximum IOs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Maximum IOs/sec'} ne '' )           { $Maximum_IOs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Maximum IOs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Minimum IOs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Minimum IOs/sec'} ne '' )           { $Minimum_IOs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Minimum IOs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Average IOs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Average IOs/sec'} ne '' )           { $Average_IOs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Average IOs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Minimum MBs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Minimum MBs/sec'} ne '' )           { $Minimum_MBs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Minimum MBs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Average MBs/sec'}           && $data{VOLUME}{$volume_name}{PERF}{'Average MBs/sec'} ne '' )           { $Average_MBs_sec             = $data{VOLUME}{$volume_name}{PERF}{'Average MBs/sec'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Current IO Latency'}        && $data{VOLUME}{$volume_name}{PERF}{'Current IO Latency'} ne '' )        { $Current_IO_Latency          = $data{VOLUME}{$volume_name}{PERF}{'Current IO Latency'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Maximum IO Latency'}        && $data{VOLUME}{$volume_name}{PERF}{'Maximum IO Latency'} ne '' )        { $Maximum_IO_Latency          = $data{VOLUME}{$volume_name}{PERF}{'Maximum IO Latency'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Minimum IO Latency'}        && $data{VOLUME}{$volume_name}{PERF}{'Minimum IO Latency'} ne '' )        { $Minimum_IO_Latency          = $data{VOLUME}{$volume_name}{PERF}{'Minimum IO Latency'}; }
      if ( defined $data{VOLUME}{$volume_name}{PERF}{'Average IO Latency'}        && $data{VOLUME}{$volume_name}{PERF}{'Average IO Latency'} ne '' )        { $Average_IO_Latency          = $data{VOLUME}{$volume_name}{PERF}{'Average IO Latency'}; }
    } ## end if ( defined $data{VOLUME...})

    # Total Data Rate (KB/s)
    my $total_data_rate_KBs = "";
    if ( defined $Average_MBs_sec && isdigit($Average_MBs_sec) ) { $total_data_rate_KBs = $Average_MBs_sec * 1024; }
    if ( defined $Current_KBsec   && isdigit($Current_KBsec) )   { $total_data_rate_KBs = $Current_KBsec; }
    if ( defined $Current_MBsec   && isdigit($Current_MBsec) )   { $total_data_rate_KBs = $Current_MBsec * 1024; }

    # Total IO Rate (IO/s)
    my $total_IO_rate_IOs = "";
    if ( defined $Average_IOs_sec && isdigit($Average_IOs_sec) ) { $total_IO_rate_IOs = $Average_IOs_sec; }
    if ( defined $Current_IOsec   && isdigit($Current_IOsec) )   { $total_IO_rate_IOs = $Current_IOsec; }

    # Cache read %
    my $cache_read_pct = "";
    if ( defined $Read_pct             && isdigit($Read_pct) )             { $cache_read_pct = $Read_pct; }
    if ( defined $Cache_Hit_Percentage && isdigit($Cache_Hit_Percentage) ) { $cache_read_pct = $Cache_Hit_Percentage; }

    # "Storage Subsystems ","Total IOs ","Read Percentage ","Cache Hit Percentage ","Current KB/second ","Maximum KB/second ","Current IO/second ","Maximum IO/second"
    # "Objects","Total IOs","Read %","Primary Read Cache Hit %","Primary Write Cache Hit %","SSD Read Cache Hit %","Current MBs/sec","Maximum MBs/sec","Current IOs/sec","Maximum IOs/sec","Minimum IOs/sec","Average IOs/sec","Minimum MBs/sec","Average MBs/sec","Current IO Latency","Maximum IO Latency","Minimum IO Latency","Average IO Latency"

    print PERF "$id,$start_time,$interval,$volume_name,$pool,$vol_controller,$total_IOs,$total_IO_rate_IOs,$total_data_rate_KBs,$Primary_Read_Cache_Hit_pct,$Primary_Write_Cache_Hit_pct,$cache_read_pct,$SSD_Read_Cache_Hit_pct,$Average_IO_Latency,$Read_Percentage,$capacity\n";
  } ## end foreach my $volume_name ( keys...)

  print PERF "\nPool Capacity Statistics\n";
  print PERF "  Interval Start:   $start_time\n";
  print PERF "  Interval End:     $end_time\n";
  print PERF "  Interval Length:  $interval seconds\n";
  print PERF "---------------------\n";
  print PERF "name,id,status,mdisk count,volume count,capacity (TB),extent size,free capacity (TB),virtual capacity (TB),used capacity (TB),real capacity (TB),overallocation,warning (%),easy tier,easy tier status,TIER-0 type,TIER-0 mdisk count,TIER-0 capacity (TB),TIER-0 free capacity (TB),TIER-1 type,TIER-1 mdisk count,TIER-1 capacity (TB),TIER-1 free capacity (TB),TIER-2 type,TIER-2 mdisk count,TIER-2 capacity (TB),TIER-2 free capacity (TB),compression active,compression virtual capacity (TB),compression compressed capacity (TB),compression uncompressed capacity (TB)\n";

  foreach my $pool_name ( keys( %{ $data{POOL} } ) ) {
    my $capacity      = "";
    my $used_capacity = "";
    my $free_capacity = "";

    if ( defined $data{POOL}{$pool_name}{CAPACITY}      && isdigit( $data{POOL}{$pool_name}{CAPACITY} ) )      { $capacity      = $data{POOL}{$pool_name}{CAPACITY}; }
    if ( defined $data{POOL}{$pool_name}{CAPACITY_USED} && isdigit( $data{POOL}{$pool_name}{CAPACITY_USED} ) ) { $used_capacity = $data{POOL}{$pool_name}{CAPACITY_USED}; }
    if ( defined $data{POOL}{$pool_name}{CAPACITY_FREE} && isdigit( $data{POOL}{$pool_name}{CAPACITY_FREE} ) ) { $free_capacity = $data{POOL}{$pool_name}{CAPACITY_FREE}; }

    print PERF "$pool_name,,,,,$capacity,,$free_capacity,,$used_capacity,,,,,,,,,,,,,,,,,,,,,\n";
  } ## end foreach my $pool_name ( keys...)

  close(PERF);

  return (1);
} ## end sub print_perf

sub run_cmd_simple {
  my $cmd     = shift;
  my $cmd_out = "";
  my @out;

  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    $cmd_out = `$cmd`;
    @out = split( "\n", $cmd_out );

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      $cmd =~ s/-p $user_pw/-p <XXXXX>/;
      error("SMcli command timed out after : $timeout seconds");
      error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      exit(0);
    } ## end if ( $@ =~ /died in SIG ALRM/)
  } ## end if ($@)

  if ( "@out" =~ /error code 12/ ) {
    $cmd_out =~ s/\n//g;
    $cmd_out = substr $cmd_out, -512;
    $cmd =~ s/-p $user_pw/-p <XXXXX>/;
    error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
    error( "$cmd_out : $!" . __FILE__ . ":" . __LINE__ );
    error("Most probably permission issue, STOR2RRD cannot run SMcli command. Problem description and fix: http://www.stor2rrd.com/SMclierror.htm");
    exit;
  }

  return @out;
}

sub run_cmd {
  my $cmd     = shift;
  my $cmd_out = "";
  my @out;

  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    $cmd_out = `$cmd`;
    @out = split( "\n", $cmd_out );

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      $cmd =~ s/-p $user_pw/-p <XXXXX>/;
      error("SMcli command timed out after : $timeout seconds");
      error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      exit(0);
    } ## end if ( $@ =~ /died in SIG ALRM/)
  } ## end if ($@)
  if ( "@out" !~ /SMcli completed successfully/ ) {
    if ( $cmd =~ /show storageArray hostTopology/ && ( $cmd_out =~ /storageSubsystem/ || $cmd_out =~ /storagesubsystem/ ) ) {
      $stor_sub_sys = 1;
      @out          = ();
      return @out;
    }

    $cmd_out =~ s/\n//g;
    $cmd_out = substr $cmd_out, -512;
    $cmd =~ s/-p $user_pw/-p <XXXXX>/;
    if ( "@out" =~ /error code 12/ ) {
      error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      error( "$cmd_out : $!" . __FILE__ . ":" . __LINE__ );
      error("Most probably permission issue, STOR2RRD cannot run SMcli command. Problem description and fix: http://www.stor2rrd.com/SMclierror.htm");
      exit;
    } ## end if ( "@out" =~ /error code 12/)
    else {
      error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      error( "$cmd_out : $!" . __FILE__ . ":" . __LINE__ );
      exit;
    }
  } ## end if ( "@out" !~ /SMcli completed successfully/)

  return @out;
} ## end sub run_cmd

sub run_cmd_pool {
  my $cmd       = shift;
  my $pool_rank = shift;
  my $cmd_out   = "";
  my @out;

  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    $cmd_out = `$cmd`;
    @out = split( "\n", $cmd_out );

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      $cmd =~ s/-p $user_pw/-p <XXXXX>/;
      error("SMcli command timed out after : $timeout seconds");
      error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      exit(0);
    } ## end if ( $@ =~ /died in SIG ALRM/)
  } ## end if ($@)
  if ( "@out" !~ /SMcli completed successfully/ ) {
    $cmd_out =~ s/\n//g;
    $cmd_out = substr $cmd_out, -512;
    $cmd =~ s/-p $user_pw/-p <XXXXX>/;

    if ( defined $pool_rank && $pool_rank eq "1" ) {
      if ( "@out" =~ /error code 12/ ) {
        error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
        error( "$cmd_out : $!" . __FILE__ . ":" . __LINE__ );
        error("Most probably permission issue, STOR2RRD cannot run SMcli command. Problem description and fix: http://www.stor2rrd.com/SMclierror.htm");
        exit;
      } ## end if ( "@out" =~ /error code 12/)
      else {
        error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
        error( "$cmd_out : $!" . __FILE__ . ":" . __LINE__ );
        exit;
      }
    } ## end if ( defined $pool_rank...)
    else {
      @out = ();
    }
  } ## end if ( "@out" !~ /SMcli completed successfully/)

  return @out;
} ## end sub run_cmd_pool

sub run_test_cmd {
  my $cmd     = shift;
  my $cmd_out = "";
  my @out;

  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    $cmd_out = `$cmd`;
    @out = split( "\n", $cmd_out );

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      $cmd =~ s/-p $user_pw/-p <XXXXX>/;
      error("SMcli command timed out after : $timeout seconds");
      error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      exit(0);
    } ## end if ( $@ =~ /died in SIG ALRM/)
  } ## end if ($@)
  if ( "@out" =~ /error code 12/ ) {
    $cmd_out =~ s/\n//g;
    $cmd_out = substr $cmd_out, -512;
    error( "SMcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
    error( "$cmd_out : $!" . __FILE__ . ":" . __LINE__ );
    error("Most probably permission issue, STOR2RRD cannot run SMcli command. Problem description and fix: http://www.stor2rrd.com/SMclierror.htm");
    exit;
  } ## end if ( "@out" =~ /error code 12/)

  return @out;
} ## end sub run_test_cmd

sub health_status {
  my $act_time    = localtime();
  my $last_update = $act_time;

  my @health_status_arr;
  my @HS_data_out;

  ### SMcli commands
  my $cmd_healthStatus = "$SMcli_cmd \"show $st_system_to_cmd healthStatus;\" 2>/dev/null";
  @health_status_arr = run_cmd($cmd_healthStatus);

  my $storage_status = "";
  push( @HS_data_out, "<br>$last_update" );

  foreach my $line (@health_status_arr) {
    chomp $line;
    if ( $line =~ "^Warning: The monitor" || $line =~ "^Executing script" || $line =~ "^The controller clocks" || $line =~ "^Controller in Slot" || $line =~ "^Storage Management Station:" || $line =~ "^Script execution complete" || $line =~ "^SMcli completed successfully" || $line eq '' || $line =~ "^Warning! No Monitor" ) {
      next;
    }
    else {
      push( @HS_data_out, "<br>$line" );
    }
    if ( $line =~ "Storage Subsystem health status =" || $line =~ "Storage subsystem health status =" || $line =~ "Storage array health status =" ) {
      $storage_status = $line;
      $storage_status =~ s/Storage Subsystem health status =//g;
      $storage_status =~ s/Storage subsystem health status =//g;
      $storage_status =~ s/Storage array health status =//g;
      $storage_status =~ s/^\s+//g;
      $storage_status =~ s/\s+$//g;
    } ## end if ( $line =~ "Storage Subsystem health status ="...)
  } ## end foreach my $line (@health_status_arr)
  push( @HS_data_out, "<br><br>" );

  # Global health check
  my $act_timestamp  = time();
  my $main_state     = "OK";
  my $state_suffix   = "ok";
  my $component_name = $storage_name;

  $component_name =~ s/\s+//g;
  my $alert_status = 0;

  if ( $storage_status !~ "optimal" ) { $main_state = "NOT_OK"; $state_suffix = "nok"; }

  if ( !-d "$inputdir/tmp/health_status_summary" ) {
    mkdir( "$inputdir/tmp/health_status_summary", 0755 ) || error( "$act_time: Cannot mkdir $inputdir/tmp/health_status_summary: $!" . __FILE__ . ":" . __LINE__ ) && exit;

    if ( !-f "$inputdir/tmp/health_status_summary/$component_name.ok" && !-f "$inputdir/tmp/health_status_summary/$component_name.nok" ) {
      $alert_status = 1;
    }

  } ## end if ( !-d "$inputdir/tmp/health_status_summary")
  if ( -f "$inputdir/tmp/health_status_summary/$component_name.ok" ) { unlink("$inputdir/tmp/health_status_summary/$component_name.ok"); $alert_status = 1; }
  if ( -f "$inputdir/tmp/health_status_summary/$component_name.nok" ) { unlink("$inputdir/tmp/health_status_summary/$component_name.nok"); }
  if ( $state_suffix eq "nok" && $alert_status == 1 ) {
    ### alert health status
    if ( exists &Xorux_lib::health_status_alerting ) {
      Xorux_lib::health_status_alerting($component_name);
    }
  } ## end if ( $state_suffix eq ...)

  open( MAST, ">$inputdir/tmp/health_status_summary/$component_name.$state_suffix" ) || error( "Couldn't open file $inputdir/tmp/health_status_summary/$component_name.$state_suffix $!" . __FILE__ . ":" . __LINE__ ) && exit;
  print MAST "STORAGE : $storage_name : $main_state : $act_timestamp\n";
  close(MAST);

  #
  # get events logs
  #

  ### allevents

  my $cmd_allevent;
  my $cmd_allevent_to_errorlog;
  my @cmd_allevent_output;
  my $allevent_txt = $output_dir . "/allevent.txt";
  my @event_lines;
  my @allevent_data_out;

  my $seq_num         = "";
  my $seq_num_last    = "";
  my $take_new_events = 0;

  if ( -f $allevent_txt ) {
    open( AE, "<$allevent_txt" ) || die "Couldn't open file $allevent_txt";
    my @allevent_lines = <AE>;
    close(AE);

    my @sequence_number = grep {/^Sequence number:/} @allevent_lines;
    if ( defined $sequence_number[0] && $sequence_number[0] ne "" ) {
      $sequence_number[0] =~ s/^Sequence number://g;
      $sequence_number[0] =~ s/^\s+//g;
      $sequence_number[0] =~ s/\s+$//g;
      chomp $sequence_number[0];
      $seq_num = $sequence_number[0];
    } ## end if ( defined $sequence_number...)

    # last one eventlog
    my $cmd_last_event;
    my @cmd_last_event_output;
    my $last_event_txt = $output_dir . "/last_event.txt";
    $cmd_last_event        = "$SMcli_cmd 'save $st_system_to_cmd criticalEvents file=\"$last_event_txt\" count=1 ;' 2>/dev/null";
    @cmd_last_event_output = run_cmd($cmd_last_event);

    open( LE, "<$last_event_txt" ) || die "Couldn't open file $last_event_txt";
    my @last_event_lines = <LE>;
    close(LE);

    my @last_sequence_number = grep {/^Sequence number:/} @last_event_lines;
    if ( defined $last_sequence_number[0] && $last_sequence_number[0] ne "" ) {
      $last_sequence_number[0] =~ s/^Sequence number://g;
      $last_sequence_number[0] =~ s/^\s+//g;
      $last_sequence_number[0] =~ s/\s+$//g;
      chomp $last_sequence_number[0];
      $seq_num_last = $last_sequence_number[0];
    } ## end if ( defined $last_sequence_number...)

    # diff lasr old and new sequence number
    if ( $seq_num eq $seq_num_last ) {
      @event_lines = @allevent_lines;
    }
    else {
      $take_new_events = 1;
    }
  } ## end if ( -f $allevent_txt )

  if ( !-f $allevent_txt || $take_new_events == 1 ) {
    $cmd_allevent        = "$SMcli_cmd 'save $st_system_to_cmd criticalEvents file=\"$allevent_txt\";' 2>/dev/null";
    @cmd_allevent_output = run_cmd($cmd_allevent);

    if ( -f $allevent_txt ) {
      open( AE, "<$allevent_txt" ) || die "Couldn't open file $allevent_txt";
      my @allevent_lines = <AE>;
      close(AE);
      @event_lines = @allevent_lines;
    } ## end if ( -f $allevent_txt )
    else {
      error( "$allevent_txt not found! : $!" . __FILE__ . ":" . __LINE__ );
    }
  } ## end if ( !-f $allevent_txt...)

  #my $events_header = "<table class =\"tabcfgsum\"><thead><tr><th>Date/Time</th><th>Sequence number</th><th>Event type</th><th>Event category</th><th>Priority</th><th>Event needs attention</th><th>Event send alert</th><th>Event visibility</th><th>Description</th><th>Event specific codes</th><th>Component type</th><th>Component location</th><th>Logged by</th></tr></thead><tbody>";

  my $events_header = "<table class =\"tabcfgsum tablesorter tablesortercfgsum\"><thead><tr>
<th class = \"sortable\" title=\"Date/Time\" nowrap=\"\">Date/Time&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Sequence number\" nowrap=\"\">id&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Event type\" nowrap=\"\">type&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Event category\" nowrap=\"\">cat.&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Priority\" nowrap=\"\">priority&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Event needs attention\" nowrap=\"\">attn.&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Event send alert\" nowrap=\"\">alert&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Event visibility\" nowrap=\"\">visibility&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Description\" nowrap=\"\">description&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Event specific codes\" nowrap=\"\">code&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Component type\" nowrap=\"\">com. type&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Component location\" nowrap=\"\">com. location&nbsp;&nbsp;&nbsp;&nbsp;</th>
<th class = \"sortable\" title=\"Logged by\" nowrap=\"\">logged by&nbsp;&nbsp;&nbsp;&nbsp;</th></tr>
</thead><tbody>";

  push( @allevent_data_out, "$events_header\n" );

  my $Date_Time             = "";
  my $Sequence_number       = "";
  my $Event_type            = "";
  my $Event_category        = "";
  my $Priority              = "";
  my $Event_needs_attention = "";
  my $Event_send_alert      = "";
  my $Event_visibility      = "";
  my $Description           = "";
  my $Event_specific_codes  = "";
  my $Component_type        = "";
  my $Component_location    = "";
  my $Logged_by             = "";

  foreach my $line (@event_lines) {
    chomp $line;
    if ( $line =~ "^Date/Time:" )             { $line =~ s/^Date\/Time: //g;            $Date_Time             = $line; }
    if ( $line =~ "^Sequence number:" )       { $line =~ s/^Sequence number: //g;       $Sequence_number       = $line; }
    if ( $line =~ "^Event type:" )            { $line =~ s/^Event type: //g;            $Event_type            = $line; }
    if ( $line =~ "^Event category:" )        { $line =~ s/^Event category: //g;        $Event_category        = $line; }
    if ( $line =~ "^Priority:" )              { $line =~ s/^Priority: //g;              $Priority              = $line; }
    if ( $line =~ "^Event needs attention:" ) { $line =~ s/^Event needs attention: //g; $Event_needs_attention = $line; }
    if ( $line =~ "^Event send alert:" )      { $line =~ s/^Event send alert: //g;      $Event_send_alert      = $line; }
    if ( $line =~ "^Event visibility:" )      { $line =~ s/^Event visibility: //g;      $Event_visibility      = $line; }
    if ( $line =~ "^Description:" )           { $line =~ s/^Description: //g;           $Description           = $line; }
    if ( $line =~ "^Event specific codes:" )  { $line =~ s/^Event specific codes: //g;  $Event_specific_codes  = $line; }
    if ( $line =~ "^Component type:" )        { $line =~ s/^Component type: //g;        $Component_type        = $line; }
    if ( $line =~ "^Component location:" )    { $line =~ s/^Component location: //g;    $Component_location    = $line; }
    if ( $line =~ "^Logged by:" )             { $line =~ s/^Logged by: //g;             $Logged_by             = $line; }

    if ( $line =~ "^Raw data:" ) {
      push( @allevent_data_out, "<tr><td nowrap=\"\">$Date_Time</td><td nowrap=\"\">$Sequence_number</td><td nowrap=\"\">$Event_type</td><td nowrap=\"\">$Event_category</td><td nowrap=\"\">$Priority</td><td nowrap=\"\">$Event_needs_attention</td><td nowrap=\"\">$Event_send_alert</td><td nowrap=\"\">$Event_visibility</td><td nowrap=\"\">$Description</td><td nowrap=\"\">$Event_specific_codes</td><td nowrap=\"\">$Component_type</td><td nowrap=\"\">$Component_location</td><td nowrap=\"\">$Logged_by</td></tr>\n" );
    }
  } ## end foreach my $line (@event_lines)
  push( @allevent_data_out, "</tbody></table>\n" );

  open( HS, ">$health_status_html" ) || error( "Couldn't open file $health_status_html $!" . __FILE__ . ":" . __LINE__ ) && exit;

  print HS "<HTML> <HEAD> <TITLE>STOR2RRD</TITLE> </HEAD>\n";
  print HS "<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 >\n";
  print HS "@HS_data_out";
  print HS "@allevent_data_out";
  print HS "</body></html>\n";
  print HS "</pre></body></html>\n";

  close(HS);

  return (1);
} ## end sub health_status

sub file_time_diff {
  my $file = shift;

  my $act_time  = time();
  my $file_time = $act_time;
  my $time_diff = 0;

  if ( -f $file ) {
    $file_time = ( stat($file) )[9];
    $time_diff = $act_time - $file_time;
  }

  return ($time_diff);
} ## end sub file_time_diff

sub isdigit {
  my $digit = shift;

  if ( $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  #main::error ("there was expected a digit but a string is there, field: , value: $digit");
  return 0;
} ## end sub isdigit

### ERROR HANDLING
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);
  print STDERR "$act_time: $text : $!\n";
  return 1;
} ## end sub error
