#!/usr/bin/perl
# generates JSON data structures
use strict;
use warnings;

use CGI::Carp qw(fatalsToBrowser);

# use Data::Dumper;

my $DEBUG    = $ENV{DEBUG}    ||= 0;
my $GUIDEBUG = $ENV{GUIDEBUG} ||= "";
my $DEMO     = $ENV{DEMO}     ||= "";
my $BETA     = $ENV{BETA}     ||= "";
my $WLABEL   = $ENV{WLABEL}   ||= "";
my $version  = $ENV{version};
my $errlog   = $ENV{ERRLOG};
my $basedir  = $ENV{INPUTDIR};
my $webdir   = $ENV{WEBDIR};
my $inputdir = $ENV{INPUTDIR};
my $uid             = $ENV{REMOTE_USER} ||= "";
my $grpnames        = "";
my $dashb_rrdheight = $ENV{DASHB_RRDHEIGHT};
my $dashb_rrdwidth  = $ENV{DASHB_RRDWIDTH};
my $legend_height   = $ENV{LEGEND_HEIGHT};
my $jump_to_rank    = $ENV{JUMP_TO_RANK};
my $isAdmin = 0;
my $showSAN = 0;
my $hasSAN = 0;
my $hasBNA = 0;

my $sdel = "";

my $prodname = "STOR2RRD";

if ($WLABEL) {
  $prodname = $WLABEL;
}
if ($jump_to_rank) {
  $jump_to_rank = "true";
}
else {
  $jump_to_rank = "false";
}

my $md5module = 1;
eval "use Digest::MD5 qw(md5_hex); 1" or $md5module = 0;

if ( !$md5module ) {    # binary MD5 module not found, use bundled pure Perl one
  use Digest::Perl::MD5 qw(md5_hex);

  # print STDERR "Bundled MD5\n";
}

use JSON qw(encode_json decode_json);

use lib "../bin";

use CustomGroups;
use Alerting;
use Users;
use ACL;

my $useacl        = ACL::useACL;
my $aclAdminGroup = ACL::getAdminGroup;

if ($useacl) {
  $grpnames = ACL::getGroupsHtml();
  $isAdmin  = ACL::isAdmin();
  $showSAN  = ACL::showSAN();
} ## end if ($useacl)

my @gtree;     # Globals
my @ctree;     # Customs
my %ctreeh;    # Customs
my @ftree;     # Favourites
my %htree;     # HMCs
my @ttree;     # Tail
my %stree;     # Servers
my %sstree;    # SAN switch tree
my %types;     # storage types
my %rtree;     # Removed LPARs
my %utree;     # unused SAN ports
my %lstree;    # LPARs by Server
my %lhtree;    # LPARs by HMC
my %qtree;     # Qtree
my @caps;      # Capacity
my @lnames;    # LPAR name (for autocomplete)
my %fleet;     # server / type / name
my %times;     # server timestamps
my $free;      # 1 -> free / 0 -> full

# set unbuffered stdout
#$| = 1;

open( OUT, ">> $errlog" ) if $DEBUG == 2;

# get QUERY_STRING
use Env qw(QUERY_STRING);
print OUT "-- $QUERY_STRING\n" if $DEBUG == 2;

#`echo "QS $QUERY_STRING " >> /tmp/xx32`;
my ( $jsontype, $par1, $par2, $par3 ) = split( /&/, $QUERY_STRING );

if ( $jsontype eq "" ) {
  if (@ARGV) {
    $jsontype = "jsontype=" . $ARGV[0];
    $basedir  = "..";
  }
  else {
    $jsontype = "jsontype=dump";
  }
} ## end if ( $jsontype eq "" )

$jsontype =~ s/jsontype=//;

if ( $jsontype eq "test" ) {
  $basedir = "..";
  &test();
  exit;
}

if ( $jsontype eq "dump" ) {
  &dumpHTML();
  exit;
}

# CGI-BIN HTML header
if ( !@ARGV ) {
  print "Content-type: application/json\n\n";
}

if ( $jsontype eq "menu" ) {
  &mainMenu();
  exit;
}

if ( $jsontype eq "menuh" ) {
  &mainMenuHmc();
  exit;
}
elsif ( $jsontype eq "lparsel" ) {
  &lparSelect();
  exit;
}
elsif ( $jsontype eq "hmcsel" ) {
  &hmcSelect();
  exit;
}
elsif ( $jsontype eq "times" ) {
  &times();
  exit;
}
elsif ( $jsontype eq "powersel" ) {
  &print_all_models();
  exit;
}
elsif ( $jsontype eq "pools" ) {
  &genPools();
  exit;
}

elsif ( $jsontype eq "lparnames" ) {
  &readMenu();
  $par1 =~ s/term=//;
  &lparNames($par1);
  exit;
} ## end elsif ( $jsontype eq "lparnames")
elsif ( $jsontype eq "histrep" ) {
  &readMenu();
  $par1 =~ s/hmc=//;
  $par2 =~ s/managedname=//;
  if ( $par3 =~ "san=" ) {
    $par3 =~ s/san=//;
  }
  else {
    $par3 = "";
  }
  $par1 = urldecode($par1);
  $par2 = urldecode($par2);
  &histReport( $par1, $par2, $par3 );
  exit;
} ## end elsif ( $jsontype eq "histrep")
elsif ( $jsontype eq "env" ) {
  &readMenu();
  &sysInfo();
  exit;
}
elsif ( $jsontype eq "pre" ) {
  &readMenu();
  &genPredefined();
  exit;
}
elsif ( $jsontype eq "cust" ) {
  &readMenu();
  &custGroupsSelect();
  exit;
}
elsif ( $jsontype eq "aclgrp" ) {
  &readMenu();
  &aclGroups();
  exit;
}
elsif ( $jsontype eq "fleet" ) {
  &readMenu();
  &genFleet();
  exit;
}
elsif ( $jsontype eq "fleetree" ) {
  &readMenu();
  &genFleetTree();
  exit;
}
elsif ( $jsontype eq "custgrps" ) {
  &genCustGrps();
  exit;
}
elsif ( $jsontype eq "alrttree" ) {
  &genAlertTree();
  exit;
}
elsif ( $jsontype eq "alrtgrptree" ) {
  &genAlertGroupTree();
  exit;
}
elsif ( $jsontype eq "alrttimetree" ) {
  &genAlertTimeTree();
  exit;
}
elsif ( $jsontype eq "isok" ) {
  &checkHealth();
  exit;
}
elsif ( $jsontype eq "caps" ) {
  &genCaps();
  exit;
}
elsif ( $jsontype eq "alrtcfg" ) {
  &getAlertConfig();
  exit;
}

sub sysInfo() {
  my $sideMenuWidth = "";
  my $vmImage;
  if ( $ENV{'SIDE_MENU_WIDTH'} ) {
    $sideMenuWidth = $ENV{'SIDE_MENU_WIDTH'};
  }
  if ( $ENV{'VM_IMAGE'} ) {
    $vmImage = 1;
  }
  print "{\n";    # envelope begin
  print "\"version\":\"$version\",\n";
  print "\"free\":\"$free\",\n";
  print "\"dashb_rrdheight\":\"$dashb_rrdheight\",\n";
  print "\"dashb_rrdwidth\":\"$dashb_rrdwidth\",\n";
  print "\"legend_height\":\"$legend_height\",\n";
  print "\"jump_to_rank\":$jump_to_rank,\n";
  print "\"guidebug\":\"$GUIDEBUG\",\n";
  print "\"basename\":" . &boolean($free) . ",\n";
  print "\"wlabel\":\"$WLABEL\",\n";
  print "\"vmImage\":" . &boolean($vmImage) . ",\n";
  print "\"uid\":\"$uid\",\n";
  print "\"gid\":\"$grpnames\",\n";
  print "\"sideMenuWidth\":\"$sideMenuWidth\",\n";
  print "\"isAdmin\":\"" . $isAdmin . "\",\n";
  print "\"useACL\":\"" . $useacl . "\",\n";
  print "\"aclAdminGroup\":\"$aclAdminGroup\",\n";
  print "\"showSAN\":\"" . $showSAN . "\",\n";
  print "\"beta\":\"$BETA\",\n";
  print "\"demo\":\"$DEMO\"\n";
  print "}\n";    # envelope end
} ## end sub sysInfo

sub dumpHTML() {
  print "Content-type: application/octet-stream\n";
  print("Content-Disposition:attachment;filename=debug.txt\n\n");
  my $buffer;
  read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
  my @pairs = split( /&/, $buffer );

  my @q = split( /=/, $pairs[1] );
  my $html = urldecode( $q[1] );
  print $html;

  #use CGI;
  #use CGI('header');
  #print header(-type=>'application/octet-stream',
  #       -attachment=>'debug.txt');
  #my $q = new CGI;
  #print $q->param('tosave');

} ## end sub dumpHTML

sub test () {
  print "Content-type: text/plain\n\n";

  #use Data::Dumper;
  #&readMenu();
  # print Dumper \%types;
  #print Dumper \%lhtree;
  #print Dumper \%sstree;
  # my %alerts = Alerting::getAlerts();
  #print Dumper \%alerts;
  #print Dumper \@caps;
  # &genHMCs;
  #print Dumper \%fleet;
  #print Dumper \%qtree;
  #print Dumper \%types;
  print "\n";
} ## end sub test

sub genCaps () {

  # use Data::Dumper;
  &readMenu();
  print "[\n";    # envelope begin
  if (@caps) {
    my $delim = "";
    foreach my $toAsk ( sort { lc( $a->[0] ) cmp lc( $b->[0] ) } @caps ) {
      print $delim . "{\"storage\": \"@$toAsk[0]\", \"subsys\": \"@$toAsk[1]\", \"url\": \"@$toAsk[2]\"}";
      $delim = "\n,";
    }
  } ## end if (@caps)

  # $Data::Dumper::Terse = 1;
  # $Data::Dumper::Useqq = 1;
  print "\n]\n";    # envelope end
} ## end sub genCaps

sub mainMenu () {
  &readMenu();
  my $hash = substr( md5_hex("DASHBOARD"), 0, 7 );
  ### Generate JSON
  print "[\n";      # envelope begin
  print "{\"title\":\"DASHBOARD\",\"extraClasses\":\"boldmenu\",\"href\":\"dashboard.html\",\"hash\":\"$hash\"}\n";
  &globalWoTitle();

  &genServersReduced();    # List by Servers

  #	&genHMCs ();  # List by HMCs

  &tail();
  print "\n]\n";           # envelope end
  ### End of JSON
} ## end sub mainMenu

sub mainMenuHmc () {
  &readMenu();
  my $hash = substr( md5_hex("DASHBOARD"), 0, 7 );
  ### Generate JSON
  print "[\n";             # envelope begin
  print "{\"title\":\"DASHBOARD\",\"extraClasses\":\"boldmenu\",\"href\":\"dashboard.html\",\"hash\":\"$hash\"},\n";
  &globalWoTitle();

  &genHMCs();              # List by HMCs

  &tail();
  print "\n]\n";           # envelope end
  ### End of JSON
} ## end sub mainMenuHmc

sub lparSelect() {
  &readMenu();
  ### Generate JSON
  print "[\n";             # envelope begin
  &genLpars();             # List by Servers
  print "\n]\n";           # envelope end
  ### End of JSON
} ## end sub lparSelect

sub hmcSelect() {
  &readMenu();
  ### Generate JSON
  print "[\n";             # envelope begin
  &genHmcSelect();         # List by HMCs
  print "\n]\n";           # envelope end
  ### End of JSON
} ## end sub hmcSelect

sub times () {
  &readMenu();
  my @sorted = sort { $times{$b} <=> $times{$a} } keys %times;

  print Dumper \%times;

  #	print Dumper \@sorted;
  for my $srv ( sort keys %times ) {
    for my $hmc (
      sort { $times{$srv}{$b} <=> $times{$srv}{$a} }
      keys %{ $times{$srv} }
      )
    {
      print Dumper $hmc;
    } ## end for my $hmc ( sort { $times...})
  } ## end for my $srv ( sort keys...)
} ## end sub times

sub genPools {
  &readMenu();
  ### Generate JSON
  print "[\n";    # envelope begin
  my $n1 = "";
  for my $hmc ( sort keys %fleet ) {
    if ( $fleet{$hmc}{POOL} ) {
      print $n1 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      $n1 = ",";
      my $n2 = "";
      foreach my $pool ( sort @{ $fleet{$hmc}{POOL} } ) {

        # if (exists $lhtree{$hmc}{$srv} ) {
        print $n2 . "{\"title\":\"$pool\"}\n";
        $n2 = ",";
      } ## end foreach my $pool ( sort @{ ...})
      print "\n]}\n";    # envelope end
    } ## end if ( $fleet{$hmc}{POOL...})
  } ## end for my $hmc ( sort keys...)
  print "\n]\n";         # envelope end
  ### End of JSON
} ## end sub genPools

sub custGroupsSelect {
  ### Generate JSON
  print "[\n";           # envelope begin
  &genCusts();           #
  print "\n]\n";         # envelope end
  ### End of JSON
} ## end sub custGroupsSelect

sub checkHealth {

  # print "[\n";      # envelope begin
  my @files = glob "${basedir}/tmp/health_status_summary/*.nok";
  my $status = '{"status": "OK"}';
  if (@files) {
    if ( !( $useacl && !$isAdmin) ) {
      $status = '{"status": "NOK"}';
    }
    else {
      foreach my $file ( @files ) {
        my @st = split(" : ", file_read($file));
        if ( ( $st[0] eq "STORAGE" && ACL::canShow( "S", "", $st[1], "" ) )
          || ( $st[0] eq "SAN-SWITCH" && ACL::canShow( "W" ) ) ) {
          $status = '{"status": "NOK"}';
          last;
        }
      }
    }
  }
  print $status;

  # print "\n]\n";    # envelope end
} ## end sub checkHealth

sub readMenu {
  my $skel = "$basedir/tmp/menu.txt";

  open( SKEL, $skel ) or die "Cannot open file: $!\n";

  while ( my $line = <SKEL> ) {
    my ( $hmc, $srv, $txt, $url );
    chomp $line;
    my @val = split( ':', $line );
    for (@val) {
      &collons($_);
    }
    {
      "O" eq $val[0] && do {
        $free = ( $val[1] == 1 ) ? 1 : 0;
        last;
      };
      "G" eq $val[0] && do {
        my ( $txt, $url ) = ( $val[2], $val[3] );
        if ( $txt eq "HMC totals" ) { last; }
        push @gtree, [ $txt, $url ];
        last;
      };
      "F" eq $val[0] && do {
        my ( $txt, $url ) = ( $val[2], $val[3] );
        push @ftree, [ $txt, $url ];
        last;
      };
      "C" eq $val[0] && do {
        my ( $txt, $url ) = ( $val[2], $val[3] );
        if ( $txt ne "<b>Configuration</b>" ) {
          push @ctree, [ $txt, $url ];
          $ctreeh{$txt} = $url;
        }
        last;
      };
      "H" eq $val[0] && do {
        my ( $hmc, $txt, $url, $subtype ) = ( $val[1], $val[2], $val[3], $val[4] );
        if ( $txt || $url ) {
          $htree{$hmc}{$txt} = $url;
        }
        if ($subtype) {
          $types{$hmc} = $subtype;
        }
        last;
      };
      "W" eq $val[0] && do {
        ( $hmc, $txt, $url ) = ( $val[1], $val[2], $val[3] );
        if ( $txt || $url ) {
          $sstree{$hmc}{$txt} = $url;
        }
        if ( $hmc =~ /-BNA$/ ) {
          $hasBNA ||= 1;
        }
        else {
          $hasSAN ||= 1;
        }
        if ( !exists $types{$hmc} ) {
          if ($url) {
            $url =~ /.*&storage=([^&]*)/;
            if ($1) {
              $types{$hmc} = $1;
            }
          } ## end if ($url)
        } ## end if ( !exists $types{$hmc...})
        last;
      };

      # aggregates
      #A:DS05:POOL:read_io:Read IO:/stor2rrd-cgi/detail.sh?host=DS05&type=POOL&name=read_io&storage=DS8K&item=sum&gui=1&none=none::
      "A" eq $val[0] && do {
        my ( $storage, $subsys, $agg, $txt, $url, $timestamp ) =
          ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7] );
        if ($timestamp) {
          $times{$subsys}{$storage} = $timestamp;
        }
        $subsys =~ s/&nbsp;/ /g;
        my $type = ( $url =~ /type=([^&]*)/ )[0];
        push @{ $stree{$subsys}{$storage} }, [ $txt, $url, $agg, $type ];
        if ( !exists $types{$storage} ) {
          $url =~ /.*&storage=([^&]*)/;
          $types{$storage} = $1;
        }
        if ( $subsys eq "HOST" ) {
          if ( $types{$storage} ne "VSPG" && $types{$storage} ne "VMAX" && $types{$storage} ne "3PAR" && $types{$storage} ne "COMPELLENT" && $types{$storage} ne "OCEANSTOR"&& $types{$storage} ne "PURE" ) {
            push @{ $lhtree{$storage}{$subsys} }, [ $txt => $url, $agg, $type ];
            push @{ $fleet{$storage}{$subsys} }, $txt;
          }
        } ## end if ( $subsys eq "HOST")

        #if ($agg eq "cap") {
        #push @caps, [$storage, $subsys, $url];
        #}
        last;
      };
      "L" eq $val[0] && do {
        my ( $storage, $subsys, $atxt, $txt, $url ) = ( $val[1], $val[2], $val[3], $val[4], $val[5] );
        $subsys =~ s/&nbsp;/ /g;
        $txt =~ s/\r//g;
        my $type = ( $url =~ /type=([^&]*)/ )[0];
        push @{ $lhtree{$storage}{$subsys} }, [ $txt => $url, $atxt, $type ];
        if ( $subsys eq "Managed disk" || $subsys eq "RAID GROUP" ) {
          $subsys = 'RANK';
        }
        if ( $subsys eq "STORAGE GROUP" ) {
          $subsys = 'VOLUME';
        }
        elsif ( $subsys eq "CPU" ) {
          $subsys = $type;
        }
        push @lnames, $txt;
        push @{ $fleet{$storage}{$subsys} }, $txt;
        last;
      };

      #R:ahmc11:BSRV21:BSRV21LPAR5:BSRV21LPAR5:/lpar2rrd-cgi/detail.sh?host=ahmc11&server=BSRV21&lpar=BSRV21LPAR5&item=lpar&entitle=0&gui=1&none=none::
      "R" eq $val[0] && do {
        my ( $hmc, $srv, $txt, $url ) = ( $val[1], $val[2], $val[4], $val[5] );
        push @{ $rtree{$hmc}{$srv} }, [ $txt => $url ];

        # push @lnames, $txt;
        last;
      };
      "U" eq $val[0] && do {
        my ( $hmc, $srv, $txt, $url ) = ( $val[1], $val[2], $val[4], $val[5] );
        push @{ $utree{$hmc}{$srv} }, [ $txt => $url ];

        # push @lnames, $txt;
        last;
      };
      "T" eq $val[0] && do {
        my ( $txt, $url ) = ( $val[2], $val[3] );
        push @ttree, [ $txt, $url ];
        last;
      };
      "X" eq $val[0] && do {
        my ( $storage, $subsys, $url ) = ( $val[1], $val[2], $val[6] );
        push @caps, [ $storage, $subsys, $url ];
        last;
      };
      "Q" eq $val[0] && do {    # product version
        $version = $val[1];
        last;
      };
      "E" eq $val[0] && do {
        my ( $storage, $subsys, $qtree, $url ) =
          ( $val[1], $val[2], $val[3], $val[5] );

        if ($qtree !~ /.*@.*/) {   # ignore line when Qtree name doesn't contain @
          next;
        }

        my ( $vol, $qtr ) = ( split "@", $qtree );

        # push @{ $lhtree{$storage}{$subsys} }, [ ($vol . "/" . $qtr) => $url, $qtr, $subsys ];
        $qtree{$storage}{$vol}{$qtr}{url}  = $url;
        $qtree{$storage}{$vol}{$qtr}{text} = $vol . "/" . $qtr;
        last;
      };
    };
  } ## end while ( my $line = <SKEL>)

  close(SKEL);
} ## end sub readMenu

### Generate STORAGE submenu
sub genHMCs {
  if (%htree) {
    print "{\"title\":\"STORAGE\",\"folder\":\"true\",\"expanded\":true,\"children\":[\n";
    $sdel = ",";
    my $n1 = "";
    for my $hmc ( sort keys %lhtree ) {
      if ( !exists $sstree{$hmc} ) {
        my $onList = ACL::canShow( "S", "", $hmc, "" );
        if ( $useacl && !$isAdmin && !$onList ) {
          next;
        }
        my $stype = $types{$hmc};
        print $n1 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"type\":\"$stype\",\"obj\":\"S\",\"children\":[\n";
        hmcTotals($hmc);
        $n1 = ",";
        my $n2 = "";
        my @subSysList = ( 'CPU-NODE', 'CPU', 'DIRECTOR', 'NODE', 'SLO', 'TIER', 'POOL', 'IO GROUP', 'RANK', 'Managed disk', 'RAID GROUP', "SVM", 'VOLUME', 'VOLUME-FOLDER', 'VVOL', 'STORAGE GROUP', 'LUN', 'DRIVE', 'DISK GROUP', 'PORT', 'HOST', 'HOST-PORT', 'CACHE', "NAS POOL", 'NAS', 'FS', 'HOST-GROUP');
        foreach my $srv (@subSysList) {
          if ( exists $stree{$srv}->{$hmc} || exists $lhtree{$hmc}{$srv} ) {
            my $backlink = $srv;
            my $title    = $srv;
            if ( $srv eq "Managed disk" ) {
              $backlink = 'RANK';
            }
            elsif ( $stype eq "VMAX" ) {
              if ( $srv eq "STORAGE GROUP" ) {
                $backlink = "VOLUME";
              }
              elsif ( $srv eq "DIRECTOR" ) {
                $backlink = "NODE";
              }
              elsif ( $srv eq "DISK GROUP" ) {
                $backlink = "DRIVE";
              }
              elsif ( $srv eq "CACHE" ) {
                $backlink = "NODE-CACHE";
              }
            } ## end elsif ( $stype eq "VMAX" )
            elsif ( $stype eq "3PAR" ) {
              if ( $srv eq "POOL" ) {
                $title = "POOL (CPG)";
              }
              elsif ( $srv eq "CACHE" ) {
                $backlink = "NODE";
              }
            } ## end elsif ( $stype eq "3PAR" )
            elsif ( $stype eq "DOTHILL" ) {
              if ( $srv eq "RAID GROUP" ) {
                $backlink = "RANK";
              }
            }
            elsif ( $stype eq "NETAPP" ) {
              if ( $srv eq "POOL" ) {
                $title = "POOL (AGGREGATE)";
              }
              elsif ( $srv eq "SVM" ) {
                $backlink = "NAS";
              }
            } ## end elsif ( $stype eq "NETAPP")
            if ( $srv eq "IO GROUP" ) {
              $backlink = "IO-GROUP";
            }
            # if (exists $lhtree{$hmc}{$srv} ) {
            print $n2 . "{\"title\":\"$title\",\"bl\":\"$backlink\",\"folder\":\"true\",\"children\":[\n";
            $n2 = ",";
            if ( exists $stree{$srv}->{$hmc} ) {
              server( $hmc, $srv, $stype );
              if ( ( $srv ne "HOST" || ( $srv eq "HOST" && ( $stype eq "VSPG" || $stype eq "VMAX" || $stype eq "3PAR" || $stype eq "COMPELLENT" || $stype eq "OCEANSTOR" || $stype eq "PURE" ) ) ) && exists $lhtree{$hmc}{$srv} ) {
                print ",\n";
              }
            } ## end if ( exists $stree{$srv...})
            if ( ( $srv ne "HOST" || ( $srv eq "HOST" && ( $stype eq "VSPG" || $stype eq "VMAX" || $stype eq "3PAR" || $stype eq "COMPELLENT" || $stype eq "OCEANSTOR" || $stype eq "PURE" ) ) ) && exists $lhtree{$hmc}{$srv} ) {
              print "{\"title\":\"Items\",\"folder\":\"true\",\"children\":[\n";
              my $n3 = "";
              for my $lpar ( @{ $lhtree{$hmc}->{$srv} } ) {

                #print Dumper $lpar;
                my $alt  = @$lpar[2];
                my $type = @$lpar[3];

                # my $alt = "";
                # if ($srv eq 'POOL' || $srv eq 'RANK' || $srv eq 'Managed disk') {
                #  $alt = @$lpar[2];
                #}
                print $n3 . &fullNode( @$lpar[0], @$lpar[1], $hmc, $srv, 1, $alt, $type );
                $n3 = ",";
              }    # L3 END
              if ( exists $rtree{$hmc}->{$srv} ) {
                print $n3 . "\n{\"title\":\"Removed\",\"folder\":\"true\",\"children\":[\n";
                my $n4 = "";
                for my $removed ( @{ $rtree{$hmc}->{$srv} } ) {
                  print $n4 . &fullNode( @$removed[0], @$removed[1], $hmc, $srv, 1 );
                  $n4 = ",\n";
                }
                print "]}";
              } ## end if ( exists $rtree{$hmc...})
              print "]}";
            } ## end if ( ( $srv ne "HOST" ...))
            if ( $srv eq "VOLUME" && exists $qtree{$hmc} ) {
              print ",{\"title\":\"Qtree\",\"folder\":\"true\",\"children\":[\n";
              my $n3 = "";
              for my $vol ( keys %{ $qtree{$hmc} } ) {
                for my $qtr ( keys %{ $qtree{$hmc}{$vol} } ) {
                  my $rec = $qtree{$hmc}{$vol}{$qtr};

                  my $alt  = "";
                  my $type = "QTREE";

                  # my $alt = "";
                  # if ($srv eq 'POOL' || $srv eq 'RANK' || $srv eq 'Managed disk') {
                  #  $alt = @$lpar[2];
                  #}
                  print $n3 . &fullNode( $qtree{$hmc}{$vol}{$qtr}{text}, $qtree{$hmc}{$vol}{$qtr}{url}, $hmc, $srv, 1, $alt, $type );
                  $n3 = ",";
                } ## end for my $qtr ( keys %{ $qtree...})
              }    # L3 END
              print "]}";
            } ## end if ( $srv eq "VOLUME" ...)
            print "]}\n";
          }    # L2 END
        } ## end foreach my $srv (@subSysList)
        print "]}\n";
      } ## end if ( !exists $sstree{$hmc...})
    } ## end for my $hmc ( sort keys...)
    print "]}\n";
  } ## end if (%htree)
  my $onList = ACL::canShow( "W" );
  if ($hasSAN && %sstree && !( $useacl && !$isAdmin && !$onList ) ) {    ###  SAN SWITCH
    print $sdel . "{\"title\":\"SAN SWITCH\",\"folder\":\"true\",\"expanded\":true,\"children\":[\n";
    $sdel = ",";
    my $n1 = "";
    if ( exists $sstree{"Totals"} ) {
      print $n1 . "{\"title\":\"Totals\",\"folder\":\"true\",\"children\":[\n";
      hmcTotals("Totals");
      $n1 = ",";
      print "]}\n";
    } ## end if ( exists $sstree{"Totals"...})
    for my $hmc ( sort keys %sstree ) {
      if ( $hmc =~ /-BNA$/ ) {
        next;
      }
      if ( $hmc ne "Totals" ) {
        print $n1 . "{\"title\":\"$hmc\",\"type\":\"SAN\",\"folder\":\"true\",\"obj\":\"W\",\"children\":[\n";
        hmcTotals($hmc);
        if ( exists $lhtree{$hmc} ) {
          print ",";
        }
        $n1 = ",";
        my $n2 = "";
        foreach my $srv ('SANPORT') {
          if ( exists $lhtree{$hmc}{$srv} ) {
            print $n2 . "{\"title\":\"PORT\",\"folder\":\"true\",\"type\":\"SAN\",\"children\":[\n";
            $n2 = ",";
            my $delim = '';
            for ( @{ $lhtree{$hmc}{$srv} } ) {
              print $delim . &fullNode( @$_[0], @$_[1], $hmc, $srv, 1, @$_[2] );
              $delim = ",\n";
            }
            if ( exists $utree{$hmc}->{$srv} ) {
              print $delim . "\n{\"title\":\"Unused\",\"folder\":\"true\",\"children\":[\n";
              my $n3 = "";
              for my $removed ( @{ $utree{$hmc}->{$srv} } ) {
                print $n3 . &fullNode( @$removed[0], @$removed[1], $hmc, $srv, 1 );
                $n3 = ",\n";
              }
              print "]}";
            } ## end if ( exists $utree{$hmc...})
            print "]}\n";
          } ## end if ( exists $lhtree{$hmc...})
        } ## end foreach my $srv ('SANPORT')
        print "]}\n";
      } ## end if ( $hmc ne "Totals" )
    } ## end for my $hmc ( sort keys...)
    print "]}\n";
  } ## end if (%sstree)
  if ($hasBNA && %sstree && !( $useacl && !$isAdmin && !$onList ) ) {    ###  SAN SWITCH BNA
    print $sdel . "{\"title\":\"SAN SWITCH BNA\",\"folder\":\"true\",\"expanded\":true,\"children\":[\n";
    $sdel = ",";
    my $n1 = "";
    if ( exists $sstree{"Totals-BNA"} ) {
      print $n1 . "{\"title\":\"Totals\",\"folder\":\"true\",\"children\":[\n";
      hmcTotals("Totals-BNA");
      $n1 = ",";
      print "]}\n";
    } ## end if ( exists $sstree{"Totals"...})
    for my $hmc ( sort keys %sstree ) {
      if ( $hmc !~ /-BNA$/ ) {
        next;
      }
      if ( $hmc ne "Totals-BNA" ) {
        my $title = substr $hmc, 0, -4;
        print $n1 . "{\"title\":\"$title\",\"type\":\"SAN\",\"altname\":\"$hmc\",\"folder\":\"true\",\"obj\":\"W\",\"children\":[\n";
        hmcTotals($hmc);
        if ( exists $lhtree{$hmc} ) {
          print ",";
        }
        $n1 = ",";
        my $n2 = "";
        foreach my $srv ('SANPORT') {
          if ( exists $lhtree{$hmc}{$srv} ) {
            print $n2 . "{\"title\":\"PORT\",\"folder\":\"true\",\"type\":\"SAN\",\"children\":[\n";
            $n2 = ",";
            my $delim = '';
            for ( @{ $lhtree{$hmc}{$srv} } ) {
              print $delim . &fullNode( @$_[0], @$_[1], $hmc, $srv, 1, @$_[2] );
              $delim = ",\n";
            }
            if ( exists $utree{$hmc}->{$srv} ) {
              print $delim . "\n{\"title\":\"Unused\",\"folder\":\"true\",\"children\":[\n";
              my $n3 = "";
              for my $removed ( @{ $utree{$hmc}->{$srv} } ) {
                print $n3 . &fullNode( @$removed[0], @$removed[1], $hmc, $srv, 1 );
                $n3 = ",\n";
              }
              print "]}";
            } ## end if ( exists $utree{$hmc...})
            print "]}\n";
          } ## end if ( exists $lhtree{$hmc...})
        } ## end foreach my $srv ('SANPORT')
        print "]}\n";
      } ## end if ( $hmc ne "Totals" )
    } ## end for my $hmc ( sort keys...)
    print "]}\n";
  } ## end if (%sstree)
} ## end sub genHMCs

### Generate HMC select tree
sub genHmcSelect {
  my $n1 = "";
  for my $hmc ( sort keys %lhtree ) {
    print $n1 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";
    for my $srv ( sort keys %{ $lhtree{$hmc} } ) {
      print $n2 . "{\"title\":\"$srv\",\"folder\":\"true\",\"children\":[\n";
      $n2 = ",";
      print "{\"title\":\"LPAR\",\"folder\":\"true\",\"children\":[\n";
      my $n3 = "";
      for my $lpar ( @{ $lhtree{$hmc}->{$srv} } ) {
        my $value = "$hmc|$srv|@$lpar[0]";
        print $n3 . "{\"title\":\"@$lpar[0]\",\"icon\":false,\"key\":\"$value\"}";
        $n3 = ",";
      }    # L3 END
      if ( exists $rtree{$hmc}->{$srv} ) {
        for my $removed ( @{ $rtree{$hmc}->{$srv} } ) {
          my $value = "$hmc|$srv|@$removed[0]";
          print $n3 . "\n" . "{\"title\":\"@$removed[0]\",\"icon\":false,\"extraClasses\":\"removed\",\"key\":\"$value\"}";
        }
      } ## end if ( exists $rtree{$hmc...})
      print "]}]}";
    }    # L2 END
    print "]}\n";
  } ## end for my $hmc ( sort keys...)
} ## end sub genHmcSelect

### Generate Custom groups list for select inputs
sub genCusts {
  my $delim = '';
  for (@ctree) {
    if ( @$_[0] ne "Custom groups" && @$_[0] ne "<b>Configuration</b>" ) {
      my $egrp = urlencode( @$_[0] );
      print $delim . "{\"title\":\"@$_[0]\",\"obj\":\"C\",\"key\":\"$egrp\"}";
      $delim = ",\n";
    }
  } ## end for (@ctree)
} ## end sub genCusts

sub histReport {
  my ( $hmc, $server, $isSAN ) = @_;
  print "[\n";
  my $n3 = "";

  # print Dumper "$hmc, $server, $isSAN\n";
  # print "{\"title\":\"SELECT ALL\",\"folder\":\"true\",\"expanded\":true,\"children\":[\n";
  if ( $server eq "RANK" ) {
    if ( exists $lhtree{$hmc}{"Managed disk"} ) {
      $server = "Managed disk";
    }
    elsif ( exists $lhtree{$hmc}{"RAID GROUP"} ) {
      $server = "RAID GROUP";
    }
  } ## end if ( $server eq "RANK")
  if ( $server eq "VOLUME" ) {
    if ( exists $lhtree{$hmc}{"STORAGE GROUP"} ) {
      $server = "STORAGE GROUP";
    }
  }
  if ($isSAN) {
    $server = "SANPORT";
    my $n2 = "";
    for my $hmc ( sort keys %sstree ) {
      if ( $hmc eq "Totals" || $hmc eq "Totals-BNA" ) {
        next;
      }
      print $n2 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      $n2 = ",";
      $n3 = "";
      for my $lpar ( @{ $lhtree{$hmc}->{$server} } ) {

        #print Dumper $lpar;
        my $value = "@$lpar[2]";

        #if ($server eq 'POOL' || $server eq 'RANK') {
        # 	$value = "@$lpar[2]";
        #}
        print $n3 . "{\"title\":\"@$lpar[0]\",\"icon\":false,\"key\":\"$hmc|$value\"}";
        $n3 = ",\n";
      } ## end for my $lpar ( @{ $lhtree...})
      if ( exists $utree{$hmc}->{$server} ) {
        for my $removed ( @{ $utree{$hmc}->{$server} } ) {
          print $n3 . "\n" . "{\"title\":\"@$removed[0]\",\"icon\":false,\"extraClasses\":\"removed\",\"key\":\"$hmc|@$removed[0]\"}";
        }
      }
      print "]}";
    } ## end for my $hmc ( sort keys...)
  } ## end if ($isSAN)
  else {
    for my $lpar ( @{ $lhtree{$hmc}->{$server} } ) {

      #print Dumper $lpar;
      my $value = "@$lpar[2]";

      #if ($server eq 'POOL' || $server eq 'RANK') {
      # 	$value = "@$lpar[2]";
      #}
      print $n3 . "{\"title\":\"@$lpar[0]\",\"icon\":false,\"key\":\"$value\"}";
      $n3 = ",\n";
    } ## end for my $lpar ( @{ $lhtree...})
    if ( exists $rtree{$hmc}->{$server} ) {
      for my $removed ( @{ $rtree{$hmc}->{$server} } ) {
        print $n3 . "\n" . "{\"title\":\"@$removed[0]\",\"icon\":false,\"extraClasses\":\"removed\",\"key\":\"@$removed[0]\"}";
      }
    }
  } ## end else [ if ($isSAN) ]

  #print "]}";

  print "]";

} ## end sub histReport

sub genPredefined() {
  print "[\n";    # envelope begin
  my $delim = "";
  my $hash  = "";
  for my $srv ( sort keys %types ) {
    if ( $types{$srv} && $types{$srv} ne 'SAN-BRCD' ) {
      $hash = substr( md5_hex( $srv . "POOLio" . "SubSys_SUM" ), 0, 7 );
      if ( $types{$srv} =~ "DS5K" ) {
        if ($DEMO) {
          next;    # skip DS5K total on DEMO website
        }
        print $delim . "\"" . $hash . "xOd\"";    # agg IO total
        $delim = ",\n";
      } ## end if ( $types{$srv} =~ "DS5K")
      else {
        print $delim . "\"" . $hash . "xgd\"";    # POOL IO sum
        $delim = ",\n";
      }
    } ## end if ( $types{$srv} && $types...)
  } ## end for my $srv ( sort keys...)
  for (@ctree) {
    if ( lc @$_[0] ne "custom groups" && @$_[0] ne "<b>Configuration</b>" ) {
      my $grp = @$_[0];
      my $hash = substr( md5_hex( "nana" . $grp ), 0, 7 );
      @$_[1] =~ /type=([^&]*)/i;
      my $type = $1;
      if ( $type eq "VOLUME" || $type eq "HOST") {

        # my @files = glob "'${basedir}/tmp/custom-group/${grp}-io_rate-d.cmd'";
        my $filetochk = "${basedir}/tmp/custom-group/${grp}-io_rate-d.cmd";

        # print STDERR "\nGLOB RES: " . Dumper $filetochk;
        # $egrp = @$_[1];
        if ( -f $filetochk ) {
          print $delim . "\"" . $hash . "x1d\"";    # custom-group-io_rate
          $delim = ",\n";
        }
        else {
          print $delim . "\"" . $hash . "x3d\"";    # custom-group-read_io
          $delim = ",\n";
          print $delim . "\"" . $hash . "x4d\"";    # custom-group-write_io
        }
      } ## end if ( $type eq "VOLUME")
      else {
        print $delim . "\"" . $hash . "aqd\"";
        $delim = ",\n";
        print $delim . "\"" . $hash . "ard\"";
      }
    } ## end if ( lc @$_[0] ne "custom groups"...)
  } ## end for (@ctree)
  print "\n]\n";                                    # envelope end
} ## end sub genPredefined

sub urlencode {
  my $s = shift;
  $s =~ s/ /+/g;
  $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  $s =~ s/\+/ /g;
  return $s;
} ## end sub urldecode

### Global section
sub global {
  my $fsub = 0;
  my $csub = 0;
  my $hsub = 0;

  print "{\"title\":\"GLOBAL\",\"folder\":\"true\",\"children\":[\n";
  if ( @ftree > 0 ) {
    $fsub = 1;
  }
  if ( @ctree > 0 ) {
    $csub = 1;
  }
  my $delim = '';
  for (@gtree) {
    my ( $t, $u ) = ( @$_[0], @$_[1] );
    {
      print $delim;
      ( ( lc $t eq "favourites" )    && $fsub ) && do { &favs();  last; };
      ( ( lc $t eq "custom groups" ) && $csub ) && do { &custs(); last; };
      if ( lc $t eq "health status" || lc $t eq "heatmap" || lc $t eq "mapping" ) {
        print &txthrefbold( $t, $u );
      }
      else {
        print &txthref( $t, $u );
      }
    }
    $delim = ",\n";
  } ## end for (@gtree)
  print "]},";
} ## end sub global

### Global section without Title
sub globalWoTitle {
  my $fsub = 0;
  my $csub = 0;
  my $hsub = 0;

  if ( @ftree > 0 ) {
    $fsub = 1;
  }
  if ( @ctree > 0 ) {
    $csub = 1;
  }
  my $delim = '';
  for (@gtree) {
    my ( $t, $u ) = ( @$_[0], @$_[1] );
    {
      if ( lc $t eq "custom groups" ) {
        print $delim;
        &custs();
        $delim = ",\n";
      }
      elsif ( lc $t eq "health status"){
        print $delim . &txthrefbold( $t, $u );
        $delim = ",\n";
      }
      elsif (!( $useacl && !$isAdmin)) {
        print $delim;
        if ( ( lc $t eq "favourites" ) && $fsub ) {
          &favs();
        }
        elsif ( lc $t eq "cpu workload estimator"
          || lc $t eq "alerting"
          || lc $t eq "volumes top"
          || lc $t eq "heatmap"
          || lc $t eq "mapping"
          || lc $t eq "hosts top"
          || lc $t eq "capacity" )
        {
          print &txthrefbold( $t, $u );
        } ## end elsif ( lc $t eq "cpu workload estimator"...)
        else {
          print &txthref( $t, $u );
        }
        $delim = ",\n";
      }
    }
  } ## end for (@gtree)
  print $delim;
} ## end sub globalWoTitle

### Favourites
sub favs {
  print "{\"title\":\"FAVOURITES\",\"folder\":\"true\",\"children\":[\n";
  my $delim = '';
  for (@ftree) {
    print $delim . &txthref( @$_[0], @$_[1] );
    $delim = ",\n";
  }
  print "]}";
} ## end sub favs
### Custom Groups
sub custs {
  print "{\"title\":\"CUSTOM GROUPS\",\"folder\":\"true\",\"children\":[\n";
  my $delim = "";
  if (!( $useacl && !$isAdmin)) {
    print &fullNode( "<b>Configuration</b>", "/stor2rrd-cgi/cgrps.sh", "na", "na", 0 );
    $delim = ",\n";
  }
  my %coll  = CustomGroups::getCollections();
  if ( $coll{collection} ) {
    for my $collname ( sort keys %{ $coll{collection} } ) {
      print $delim . "{\"title\":\"$collname\",\"folder\":\"true\",\"children\":[\n";
      $delim = ",\n";
      my $n2 = "";
      for my $cgrp ( sort keys %{ $coll{collection}{$collname} } ) {
        my $onList = ACL::canShow( "C", $cgrp );
        if ( $useacl && !$isAdmin && !$onList  ) {
          next;
        }
        print $n2 . &fullNode( $cgrp, $ctreeh{$cgrp}, "na", "na", 1, $cgrp, "", "C" );
        $n2 = ",\n";
      }

      print "]}";
    } ## end for my $collname ( sort...)
  } ## end if ( $coll{collection})
  if ( $coll{nocollection} ) {

    # print Dumper \%ctreeh;
    for my $cgrp ( sort keys %{ $coll{nocollection} } ) {
      my $onList = ACL::canShow( "C", $cgrp );
      if ( $useacl && !$isAdmin && !$onList  ) {
        next;
      }
      print $delim . &fullNode( $cgrp, $ctreeh{$cgrp}, "na", "na", 1, $cgrp, "", "C" );
      $delim = ",\n";
    }
  } ## end if ( $coll{nocollection...})
  print "]}";
} ## end sub custs

### HMC submenu
sub hmcs {

  #	print "{\"title\":\"HMC totals\",\"folder\":\"true\",\"children\":[\n";
  my $delim = '';
  for (%htree) {
    print $delim . &txthref( @$_[0], @$_[1] );
    $delim = ",\n";
  }

  #	print "]}";
} ## end sub hmcs

### single HMC Totals
sub hmcTotals {
  my ($hmc) = @_;

  my $delim = '';

  #	print Dumper \%htree;
  if ( exists $htree{$hmc} ) {
    for my $ss ( sort keys %{ $htree{$hmc} } ) {
      if ( $ss eq "Capacity" && $htree{$hmc}{$ss} =~ "cap_pool" ) {
        print $delim . fullNode( $ss, $htree{$hmc}{$ss}, $hmc, $ss, 0, "cap_pool" );
      }
      else {
        print $delim . &txthref( $ss, $htree{$hmc}{$ss} );
      }
      $delim = ",\n";
    } ## end for my $ss ( sort keys ...)
    if ( exists $lhtree{$hmc} ) {
      print "$delim";
    }
  } ## end if ( exists $htree{$hmc...})
  if ( exists $sstree{$hmc} ) {
    foreach my $srv ( 'Data', 'IO', 'Frame', 'CPU/Memory', 'Fabric', "ISL", 'Heatmap', 'Health status', 'Configuration', 'Historical reports', 'Zone Configuration' ) {
      if ( exists $sstree{$hmc}->{$srv} ) {
        print $delim . &fullNode( $srv, $sstree{$hmc}{$srv}, $hmc, $srv, 0 );
        $delim = ",\n";
      }
    } ## end foreach my $srv ( 'Data', 'IO'...)
  } ## end if ( exists $sstree{$hmc...})

  #print "$delim";
} ## end sub hmcTotals

### Tail menu section
sub tail {
  my $freeOrFull = ( $free == 1 ? "free" : "full" );
  print $sdel . "{\"title\":\"$prodname <span style='font-weight: normal'>($version $freeOrFull)</span>\",\"folder\":\"true\",\"children\":[\n";
  my $delim = '';
  if (!( $useacl && !$isAdmin)) {
    for (@ttree) {
      print $delim . &txthref( @$_[0], @$_[1] );
      $delim = ",\n";
    }
    print $delim . &txthref( "User management", "/stor2rrd-cgi/users.sh?cmd=form" );
    if ($GUIDEBUG) {
      print $delim . &txthref( "Load debug content", "debug.txt" );
    }
  }
  else {
    print $delim . &txthref( "Documentation", "gui-help.html" );
  }
  # print $delim . &txthref( "FAST SG", "/stor2rrd-cgi/fast-sg.sh?host=EMC-VMAX-111&type=VOLUME&name=fast_sg&item=sum&time=d" );
  print "]}";
} ## end sub tail

### Single Server menu
# params: (hmc, srv)
sub server {
  my ( $h, $s, $t ) = @_;
  my $delim  = '';
  my $isLpar = 0;
  if ( $s eq "HOST" && $t ne "VSPG" && $t ne "VMAX" && $t ne "3PAR" && $t ne "COMPELLENT" && $t ne "OCEANSTOR" && $t ne "PURE" ) {
    $isLpar = 1;
  }

  my @totarr;
  if ( $s eq "PORT" && $t eq "UNITY" ) {
    @totarr = sort { $b->[0] cmp $a->[0] } @{ $stree{$s}->{$h} };
  }
  else {
    @totarr = @{ $stree{$s}->{$h} };
  }
  if ( $s eq "SVM" ) {
    $s = "NAS";
  }

  # print STDERR "@totarr\n";
  for (@totarr) {
    print $delim . &fullNode( @$_[0], @$_[1], $h, $s, $isLpar, @$_[2], @$_[3] );
    $delim = ",\n";
  }
} ## end sub server

sub lparNames () {
  my @unique = uniq(@lnames);
  print "[";
  if (@_) {
    @unique = grep( {/@_/i} @unique );
  }
  my $delim = '';
  for (@unique) {

    #		print Dumper $_;
    print $delim . "{\"value\":\"$_\"}";
    $delim = ",\n";
  } ## end for (@unique)
  print "]";
} ## end sub lparNames

sub genFleetTree {
  print "[";
  my $n1 = "";
  for my $srv ( sort keys %fleet ) {
    if ( $times{$srv}{"removed"} == 1 ) {
      next;
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",\"type\":\"$types{$srv}\",\"children\":[\n";
    $n1 = ",\n";
    my $n2 = "";
    for my $type ( sort keys %{ $fleet{$srv} } ) {
      print $n2 . "{\"title\":\"$type\",\"folder\":\"true\",\"children\":[";
      $n2 = ",\n";
      my $n3  = "";
      my @uni = uniq( @{ $fleet{$srv}{$type} } );
      foreach my $name (@uni) {
        $name =~ s{\\}{\\\\}g;
        print $n3 . "{\"title\":\"$name\"}";
        $n3 = ",";
      }    # L3 END
      print "]}";
    }    # L2 END
    print "]}";
  } ## end for my $srv ( sort keys...)
  print "]\n";
} ## end sub genFleetTree

sub genFleet {
  print "{";
  my $n1 = "";
  for my $srv ( sort keys %fleet ) {
    print $n1 . "\"$srv\":{";
    print "\"STORTYPE\":\"$types{$srv}\",";
    $n1 = ",\n";
    my $n2 = "";
    for my $type ( sort keys %{ $fleet{$srv} } ) {
      if ( $type eq "POOL" && $types{$srv} eq "VMAX" ) {
        next;
      }
      print $n2 . "\"$type\":[";
      $n2 = ",\n";
      my $n3  = "";
      my @uni = uniq( @{ $fleet{$srv}{$type} } );

      # print Dumper \@uni;
      foreach my $name (@uni) {
        $name =~ s{\\}{\\\\}g;
        print $n3 . "\"$name\"";
        $n3 = ",";
      }    # L3 END
      print "]";
    }    # L2 END
    print "}";
  } ## end for my $srv ( sort keys...)
  print "}\n";
} ## end sub genFleet

sub genCustGrps {
  print "[";
  my $n1    = "";
  my %cgrps = CustomGroups::getGrp();
  for my $cgrp ( sort keys %cgrps ) {
    my $collection = "";
    if ( $cgrps{$cgrp}{'collection'} ) {
      $collection = ",\"collection\":\"" . $cgrps{$cgrp}{'collection'} . "\"";
    }
    print $n1 . "{\"title\":\"$cgrp\",\"folder\":\"true\",\"type\":\"" . $cgrps{$cgrp}{'type'} . "\"$collection,\"loaded\":\"true\",\"children\":[\n";
    $n1 = "\n,";
    my $n2 = "";
    for my $type ( sort keys %{ $cgrps{$cgrp}{'children'} } ) {
      print $n2 . "{\"title\":\"$type\",\"folder\":\"true\",\"children\":[\n";
      $n2 = "\n,";
      my $n3 = "";
      for my $name ( @{ $cgrps{$cgrp}{'children'}{$type} } ) {
        print $n3 . "{\"title\":\"$name\"}";
        $n3 = "\n,";
      }    # L3 END
      print "]}";
    }    # L2 END
    print "]}";
  } ## end for my $cgrp ( sort keys...)
  print "]\n";
} ## end sub genCustGrps

sub genAlertTree {
  print "[";
  my $n1     = "";
  my %alerts = Alerting::getAlerts();

  # print Dumper \%alerts;
  foreach my $alert ( sort keys %alerts ) {
    print $n1 . "{\"title\":\"$alert\",\"folder\":\"true\",\"children\":[\n";
    $n1 = "\n,";
    my $n2 = "";
    foreach my $subsys ( sort keys %{ $alerts{$alert} } ) {
      foreach my $name ( sort keys %{ $alerts{$alert}{$subsys} } ) {
        if ( $subsys eq "POOL-ALL" ) {
          print $n2 . "{\"title\":\"All pools\",\"subsys\":\"$subsys\",\"folder\":\"true\",\"children\":[\n";
        }
        else {
          print $n2 . "{\"title\":\"$name\",\"subsys\":\"$subsys\",\"folder\":\"true\",\"children\":[\n";
        }
        $n2 = "\n,";
        my $n3      = "";
        my $ruleidx = 1;
        foreach my $metric ( %{ $alerts{$alert}{$subsys}{$name} } ) {
          foreach my $vals ( @{ $alerts{$alert}{$subsys}{$name}{$metric} } ) {
            no warnings 'uninitialized';

            # print $vals->{limit};
            # $my ( $limit, $peak, $repeat, $exclude, $mailgrp ) = (%{\$val{limit}, \$val{limit}, \$val{limit}, \$val{limit}, \$val{limit});
            #print $n3 . "{\"title\":\"Rule #$ruleidx\",\"metrics\":\"$metric\",\"limit\":\"$vals->{limit}\",\"peak\":\"$vals->{peak}\",\"repeat\":\"$vals->{repeat}\",\"exclude\":\"$vals->{exclude}\",\"mailgrp\":\"$vals->{mailgrp}\"}";
            print $n3 . "{\"title\":\"\",\"metric\":\"$metric\",\"limit\":\"$vals->{limit}\",\"peak\":\"$vals->{peak}\",\"repeat\":\"$vals->{repeat}\",\"exclude\":\"$vals->{exclude}\",\"mailgrp\":\"$vals->{mailgrp}\"}";
            $n3 = "\n,";
            $ruleidx++;
          } ## end foreach my $vals ( @{ $alerts...})
        } ## end foreach my $metric ( %{ $alerts...})
        print "]}\n";
      } ## end foreach my $name ( sort keys...)
    } ## end foreach my $subsys ( sort keys...)
    print "]}\n";
  } ## end foreach my $alert ( sort keys...)
  print "]\n";
} ## end sub genAlertTree

sub genAlertGroupTree {
  print "[";
  my $n1 = "";

  foreach my $grp ( Alerting::getDefinedGroups() ) {
    print $n1 . "{\"title\":\"$grp\",\"folder\":\"true\",\"children\":[\n";
    $n1 = "\n,";
    my $n2 = "";

    # my @members = Alerting::getGroupMembers($grp);
    # print Dumper @members;
    foreach ( @{ Alerting::getGroupMembers($grp) } ) {

      # my $user = Alerting::getUserDetails($grp, $_);
      print $n2 . "{\"title\":\"$_\"}";
      $n2 = "\n,";
    } ## end foreach ( @{ Alerting::getGroupMembers...})
    print "]}\n";
  } ## end foreach my $grp ( Alerting::getDefinedGroups...)
  print "]\n";
} ## end sub genAlertGroupTree

sub getAlertConfig {
  print "[";
  my $n1  = "";
  my %cfg = Alerting::getConfig();

  # print Dumper %cfg;
  while ( my ( $key, $val ) = each %cfg ) {
    print $n1 . "{\"$key\":\"$val\"}";
    $n1 = "\n,";
  }
  print "]\n";
} ## end sub getAlertConfig

sub txthref {
  my $hash = substr( md5_hex( $_[0] . $_[1] ), 0, 7 );
  return "{\"title\":\"$_[0]\",\"href\":\"$_[1]\",\"hash\":\"$hash\",\"str\":\"$_[0]\"}";
}

sub fullNode {
  my ( $title, $href, $hmc, $srv, $islpar, $altname, $type, $obj ) = @_;
  $href ||= "";
  $title =~ s{\\}{\\\\}g;
  my $key = ( $srv eq "na" ? "" : $srv ) . " " . $title;
  if ( $srv eq "Managed disk" || $srv eq "RAID GROUP" ) {
    $srv = "RANK";
  }
  elsif ( $srv eq "SVM" ) {
    $srv = "NAS";
  }
  elsif ( $srv eq "DIRECTOR" ) {
    $srv = "NODE";
  }
  elsif ( $srv eq "STORAGE GROUP" ) {
    $srv = "VOLUME";
  }
  elsif ( $srv eq "DISK GROUP" ) {
    $srv = "DRIVE";
  }
  elsif ( $srv eq "CPU" && $type ) {
    $srv = "$type";
  }
  elsif ( $srv eq "IO GROUP" && $type ) {
    $srv = "IO-GROUP";
  }
  $altname = '' unless defined $altname;
  $obj ||= "";
  if ( !$islpar ) {
    my $hashstr = $hmc . $srv . $altname . "SubSys_SUM";
    if ( $srv eq $altname ) {
      $hashstr = $hmc . $srv . "SubSys_SUM";
    }
    my $hash = substr( md5_hex($hashstr), 0, 7 );

    return "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"altname\":\"$altname\",\"agg\":true,\"str\":\"$key\",\"hashstr\":\"$hashstr\",\"obj\": \"$obj\"}";

    #  return
    #       "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"altname\":\"$altname\",\"agg\":true,\"str\":\"$key\"}";
  } ## end if ( !$islpar )
  else {
    my $hashstr = $hmc . $srv . $altname;
    my $hash = substr( md5_hex($hashstr), 0, 7 );

    return "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"altname\":\"$altname\",\"str\":\"$key\",\"hashstr\":\"$hashstr\",\"obj\": \"$obj\"}";

    # return
    #      "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"altname\":\"$altname\",\"str\":\"$key\"}";
  } ## end else [ if ( !$islpar ) ]
} ## end sub fullNode

sub txthrefbold {
  my $hash = substr( md5_hex( $_[0] ), 0, 7 );
  return "{\"title\":\"$_[0]\",\"extraClasses\":\"boldmenu\",\"href\":\"$_[1]\",\"hash\":\"$hash\"}";
}

sub txthref_wchld {
  return "{\"title\":\"$_[0]\",\"icon\":false,\"href\":\"$_[1]\",\"children\":[";
}

sub txtkey {
  return "{\"title\":\"$_[0]\",\"icon\":false,\"key\":\"$_[1]\"}";
}

sub txtkeysel {
  return "{\"title\":\"$_[0]\",\"icon\":false,\"selected\":true,\"key\":\"$_[1]\"}";
}

sub collons {
  return s/===double-col===/:/g;
}

sub uniq {
  my %seen;
  return grep { !$seen{$_}++ } @_;
}

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

sub boolean {
  my $val = shift;
  if ($val) {
    return "true";
  }
  else {
    return "false";
  }
} ## end sub boolean

sub file_read {
  my $file = shift;
  open IO, $file or die "Cannot open $file for input: $!\n";
  my @data = <IO>;
  close IO;
  wantarray ? @data : join( '' => @data );
} ## end sub file_read
