#!/bin/ksh
#set -x

PATH=$PATH:/usr/bin:/bin
export PATH
umask 0022

if [ ! -d $WEBDIR ]; then
   echo "WEBDIR does not exist, supply it as the first parametr"
   exit 1
fi
if [ ! -d $INPUTDIR ]; then
   echo "Source does not exist, supply it as the second parametr"
   exit 1
fi
if [ ! -f $PERL ]; then
   echo "perl path is invalid, correct it and re-run the tool"
   exit 1
fi


type_amenu="A" # aggregated items under subsystem menu
type_lmenu="L" # item under subsystem
type_removed="R"
type_gmenu="G" # global menu
type_cmenu="C" # custom group menu
type_fmenu="F" # favourites menu
type_smenu="S" # subsystem menu
type_tmenu="T" # tail menu
type_hmenu="H" # storage menu
type_wmenu="W" # SAN switch  menu
type_xmenu="X" # capacity overview
type_xmenu="X" # capacity overview
type_version="O" # free(open)/full version (1/0)
type_qmenu="Q" # version
type_emenu="E" # sub section qtree under volume

gmenu_created=0
gmenu_storage_created=0


if [ "$TMPDIR_STOR"x = "x" ]; then
  TMPDIR_STOR="$INPUTDIR/tmp"
fi
MENU_OUT=$TMPDIR_STOR/menu.txt-tmp
rm -f $MENU_OUT
MENU_OUT_FINAL=$TMPDIR_STOR/menu.txt

if [ ! -f "$MENU_OUT_FINAL" ]; then
  # force to create tmp/menu.txt when it does not exist
  if [ $DEBUG ]; then echo "load.sh html   : tmp/menu.txt is missing, force it to create a new one"; fi
  touch "$TMPDIR_STOR/$version-run"
fi



ALIAS_CFG="$INPUTDIR/etc/alias.cfg"
ALIAS=0
CGI_DIR="stor2rrd-cgi"
AGG_LIST="sum_io_total sum_data_totam sum_io io_rate read_ops write_ops ops_rate sum_data data_rate packet_rate io_rate_b data_rate_b read_io write_io read write sum_resp_t resp resp_t resp_t_r resp_t_w resp_t_b resp_t_r_b resp_t_w_b sum_capacity real used  read_error write_error cache_hit r_cache_hit w_cache_hit r_cache_usage w_cache_usage read_io_b write_io_b read_b write_b sys busy compress tier0 tier0-nl tier1 tier2 pprc_rio pprc_wio pprc_data_r pprc_data_w pprc_rt_r pprc_rt_w read_pct r_transfer_size w_transfer_size top write_pend clean_usage middle_usage phys_usage operating_rate wafl_exempt-kahuna-exempt-hostOS-nwk_legacy dnlc_hit buffer_hit calls_rate read_calls write_calls prct_d_cache_p b2b crc WP tier_alloc_prct alloc_vols io_rate_frontend read_io_frontend write_io_frontend data_rate_frontend read_frontend write_frontend resp_t_r_frontend resp_t_w_frontend sum_busy_frontend io_rate_backend read_io_backend write_io_backend data_rate_backend read_backend write_backend sum_busy_backend io_rate_rdf read_io_rdf write_io_rdf data_rate_rdf read_rdf write_rdf sum_busy_rdf mapped_vols cfc_dirty cfc_max del_ack compress_cap uncompress_cap compress_virt_cap queue"
SUBSYSTEM_LIST="POOL RANK MDISK VOLUME DRIVE PORT CPU-NODE HOST CPU-CORE NODE-CACHE LUN NODE FS NAS NASPOOL TIER SLO VVOL VOLUME-FOLDER HOST-GROUP IO-GROUP"
# avoid CPU-CORE on purpose
if [ -L "$INPUTDIR/data/$hmc/MDISK" ]; then
  # SVC
  FIRST_GLOBAL="sum_io"
  FIRST_GLOBAL_SUB="MDISK"
  #FIRST="sum_data" # POOL, RANK
  #FIRST_SECOND="data_rate" # PORT
else
  if [ -L "$INPUTDIR/data/$hmc/UNITY" -o  -L "$INPUTDIR/data/$hmc/VNX" -o  -L "$INPUTDIR/data/$hmc/VNXE" -o -L "$INPUTDIR/data/$hmc/VNXF" -o -L "$INPUTDIR/data/$hmc/DS5K" -o -L "$INPUTDIR/data/$hmc/HUS" -o -L "$INPUTDIR/data/$hmc/DOTHILL" -o -L "$INPUTDIR/data/$hmc/NETAPP" -o -L "$INPUTDIR/data/$hmc/3PAR" -o -L "$INPUTDIR/data/$hmc/VSPG" -o -L "$INPUTDIR/data/$hmc/COMPELLENT" -o -L "$INPUTDIR/data/$hmc/OCEANSTOR" -o -L "$INPUTDIR/data/$hmc/PURE" ]; then
    FIRST_GLOBAL="sum_io_total"
    FIRST_GLOBAL_SUB="POOL"
  else
    # DS8K
    FIRST_GLOBAL="sum_io"
    FIRST_GLOBAL_SUB="RANK"
    #FIRST="read" # POOL, RANK
    #FIRST_SECOND="data_rate" # PORT
  fi
fi
TMPDIR_STOR=$INPUTDIR/tmp
INDEX=$INPUTDIR/html
pwd=`pwd`
TIME=`$PERL -e '$v = time(); print "$v\n"'`
# 10 days in secs`
TEND=864000

if [ "$ACCOUTING"x = "x" ]; then
  ACCOUTING=0
fi

if [ $DEBUG ]; then echo "installing WWW : install-html.sh "; fi
if [ $DEBUG ]; then echo "Host identif   : $UNAME "; fi

if [ -f $BINDIR/premium.sh ]; then
  . $BINDIR/premium.sh
fi

if [ -f "$ALIAS_CFG" ]; then
  ALIAS=1
fi

# create skeleton of menu
menu () {
  a_type=$1
  a_hmc=`echo "$2"|sed 's/:/===double-col===/g'`
  a_server=`echo "$3"|sed 's/:/===double-col===/g'`
  a_lpar=`echo "$4"|sed 's/:/===double-col===/g'`
  a_text=`echo "$5"|sed 's/:/===double-col===/g'`
  a_url=`echo "$6"|sed -e 's/:/===double-col===/g' -e 's/ /%20/g'`
  a_lpar_wpar=`echo "$7"|sed 's/:/===double-col===/g'` # lpar name when wpar is passing
  a_last_time=$8


  #if [ ! "$LPARS_EXCLUDE"x = "x" -a `echo "$4"|egrep "$LPARS_EXCLUDE"|wc -l|sed 's/ //g'` -eq 1 ]; then
  #  # excluding some LPARs based on a string : LPARS_EXCLUDE --> etc/.magic
  #  echo "lpar exclude   : $2:$2:$4 - exclude string: $LPARS_EXCLUDE"
  #  return 1
  #fi

  if [ "$a_type" = "$type_gmenu" -a $gmenu_created -eq 1 -a $gmenu_storage_created -eq 1 ]; then
    return # print global menu once
  fi

  echo "$a_type:$a_hmc:$a_server:$a_lpar:$a_text:$a_url:$a_lpar_wpar:$a_last_time" >> $MENU_OUT

}

# create custm group list
custom_group() {
  CUSTOM_CFG="$INPUTDIR/etc/web_config/custom_groups.cfg"
  inventory_custom_group=""
  if [ -f "$CUSTOM_CFG" ]; then
    for line_space in `egrep -v "#" $CUSTOM_CFG 2>/dev/null|sed -e 's/ *$//g' -e 's/\\\:/===colon===/g' |cut -d : -f 1,2,4|sed -e 's/===colon===/\:/g' -e 's/ *$//g' -e 's/ /=====space=====/g'|sort|uniq`
    do
      line=`echo $line_space|sed 's/=====space=====/ /g'`
      cgroup=`echo "$line"|cut -d : -f 3`
      cgroup_space=`echo "$line"|cut -d : -f 3|sed 's/ /%20/g'`
      type=`echo "$line"| cut -d : -f 1`
      storage_name=`echo "$line"| cut -d : -f 2`

      st_type_act="na"
      if [ -f "$INPUTDIR/data/$storage_name/DS8K" ]; then
        st_type_act="DS8K"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/SWIZ" ]; then
        st_type_act="SWIZ"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/XIV" ]; then
        st_type_act="XIV"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/DS5K" ]; then
        st_type_act="DS5K"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/HUS" ]; then
        st_type_act="HUS"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/DOTHILL" ]; then
        st_type_act="DOTHILL"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/VNXF" ]; then
        st_type_act="VNXF"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/VNX" ]; then
        st_type_act="VNX"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/NETAPP" ]; then
        st_type_act="NETAPP"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/3PAR" ]; then
        st_type_act="3PAR"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/VSPG" ]; then
        st_type_act="VSPG"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/UNITY" ]; then
        st_type_act="UNITY"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/NIMBLE" ]; then
        st_type_act="NIMBLE"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/VNXE" ]; then
        st_type_act="VNXE"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/VMAX" ]; then
        st_type_act="VMAX"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/VPLEX" ]; then
        st_type_act="VPLEX"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/COMPELLENT" ]; then
        st_type_act="COMPELLENT"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/OCEANSTOR" ]; then
        st_type_act="OCEANSTOR"
      fi
      if [ -f "$INPUTDIR/data/$storage_name/PURE" ]; then
        st_type_act="PURE"
      fi


      if [ "$cgroup"x = "x" -o "$type"x = "x" ]; then
        echo "error cgroup   : $type_cmenu : CUSTOM GROUPS : $type:$cgroup : something is null: $line, skipping"
        continue
      fi
      if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_cmenu : CUSTOM GROUPS : $type:$cgroup"; fi
      check=`echo "$inventory_custom_group"|grep "$cgroup" | wc -l`
      if [ $check -eq 0 ]; then
        inventory_custom_group="$inventory_custom_group $cgroup"
      else
        continue
      fi
      menu "$type_cmenu" "$cgroup_space" "$cgroup" "/$CGI_DIR/detail.sh?host=custom-group&type=$type&name=$cgroup_space&storage=$st_type_act&item=custom&gui=1&none=none"
    done
  fi
}



is_alive() {
  if [ ! -f "$1" ]; then
    return 1
  fi

  access_t=`$PERL -e '$inp=shift;$m=shift;$h=shift;$v = (stat("$inp"))[9]; print "$v\n";' "$1"`
  tot=`expr $TEND + $access_t`

  if [ $tot -lt $TIME ]; then
    # it is death, more than 10 day old data
    return 1
  fi
  return 0
}

print_hoststop (){

echo "<div  id=\"tabs\"> <ul>"

if [ -f "$WEBDIR/glob-HOST-top-avrg-d.html-tmp" ]; then
  mv "$WEBDIR/glob-HOST-top-avrg-d.html-tmp" "$WEBDIR/glob-HOST-top-avrg-d.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-HOST-top-avrg-d.html"
  echo  "<li><a href=\"glob-HOST-top-avrg-d.html\">daily</a></li>"
fi
if [ -f "$WEBDIR/glob-HOST-top-avrg-w.html-tmp" ]; then
  mv "$WEBDIR/glob-HOST-top-avrg-w.html-tmp" "$WEBDIR/glob-HOST-top-avrg-w.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-HOST-top-avrg-w.html"
  echo  "<li><a href=\"glob-HOST-top-avrg-w.html\">weekly</a></li>"
fi

if [ -f "$WEBDIR/glob-HOST-top-avrg-m.html-tmp" ]; then
  mv "$WEBDIR/glob-HOST-top-avrg-m.html-tmp" "$WEBDIR/glob-HOST-top-avrg-m.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-HOST-top-avrg-m.html"
  echo  "<li><a href=\"glob-HOST-top-avrg-m.html\">monthly</a></li>"
fi

if [ -f "$WEBDIR/glob-HOST-top-max-d.html-tmp" ]; then
  mv "$WEBDIR/glob-HOST-top-max-d.html-tmp" "$WEBDIR/glob-HOST-top-max-d.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-HOST-top-max-d.html"
  echo  "<li><a href=\"glob-HOST-top-max-d.html\">daily max</a></li>"
fi

if [ -f "$WEBDIR/glob-HOST-top-max-w.html-tmp" ]; then
  mv "$WEBDIR/glob-HOST-top-max-w.html-tmp" "$WEBDIR/glob-HOST-top-max-w.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-HOST-top-max-w.html"
  echo  "<li><a href=\"glob-HOST-top-max-w.html\">weekly max</a></li>"
fi

if [ -f "$WEBDIR/glob-HOST-top-max-m.html-tmp" -a "$TOP_MAX_MONTHLY" = "1" ]; then
  mv "$WEBDIR/glob-HOST-top-max-m.html-tmp" "$WEBDIR/glob-HOST-top-max-m.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-HOST-top-max-m.html"
  echo  "<li><a href=\"glob-HOST-top-max-m.html\">monthly max</a></li>"
fi

echo "   </ul>"

}

print_voltop () {

echo "<div  id=\"tabs\"> <ul>"
echo "<li><a href='#tabs-0'>custom</a></li>";
if [ -f "$WEBDIR/glob-VOLUME-top-avrg-d.html-tmp" ]; then
  mv "$WEBDIR/glob-VOLUME-top-avrg-d.html-tmp" "$WEBDIR/glob-VOLUME-top-avrg-d.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-VOLUME-top-avrg-d.html"
  echo  "<li><a href=\"glob-VOLUME-top-avrg-d.html\">daily</a></li>"
fi
if [ -f "$WEBDIR/glob-VOLUME-top-avrg-w.html-tmp" ]; then
  mv "$WEBDIR/glob-VOLUME-top-avrg-w.html-tmp" "$WEBDIR/glob-VOLUME-top-avrg-w.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-VOLUME-top-avrg-w.html"
  echo  "<li><a href=\"glob-VOLUME-top-avrg-w.html\">weekly</a></li>"
fi
if [ -f "$WEBDIR/glob-VOLUME-top-avrg-m.html-tmp" ]; then
  mv "$WEBDIR/glob-VOLUME-top-avrg-m.html-tmp" "$WEBDIR/glob-VOLUME-top-avrg-m.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-VOLUME-top-avrg-m.html"
  echo  "<li><a href=\"glob-VOLUME-top-avrg-m.html\">monthly</a></li>"
fi
if [ -f "$WEBDIR/glob-VOLUME-top-max-d.html-tmp" ]; then
  mv "$WEBDIR/glob-VOLUME-top-max-d.html-tmp" "$WEBDIR/glob-VOLUME-top-max-d.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-VOLUME-top-max-d.html"
  echo  "<li><a href=\"glob-VOLUME-top-max-d.html\">daily max</a></li>"
fi
if [ -f "$WEBDIR/glob-VOLUME-top-max-w.html-tmp" ]; then
  mv "$WEBDIR/glob-VOLUME-top-max-w.html-tmp" "$WEBDIR/glob-VOLUME-top-max-w.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-VOLUME-top-max-w.html"
  echo  "<li><a href=\"glob-VOLUME-top-max-w.html\">weekly max</a></li>"
fi
if [ -f "$WEBDIR/glob-VOLUME-top-max-m.html-tmp" -a "$TOP_MAX_MONTHLY" = "1" ]; then
  mv "$WEBDIR/glob-VOLUME-top-max-m.html-tmp" "$WEBDIR/glob-VOLUME-top-max-m.html"
  echo "</tbody></table>" >> "$WEBDIR/glob-VOLUME-top-max-m.html"
  echo  "<li><a href=\"glob-VOLUME-top-max-m.html\">monthly max</a></li>"
fi
echo "   </ul>"
echo "<div id='tabs-0'>
  <center>
    <div style='font-size: 0.8rem; margin-top: 1em'>
    <form method='get' action='/stor2rrd-cgi/detail-graph.sh' id='volcustomform'>
      <label for='from'>From</label>
      <input type='text' id='fromTime' size='14'>
      <label for='to'>to</label>
      <input type='text' id='toTime' size='14'>
      <select>
        <option value='AVG'>AVG</option>
        <option value='MAX'>MAX</option>
      </select>
      <input type='submit' style='font-weight: bold;' id='showvolumes' value='Show top volumes'>
    </form>
    </div>
    <div id='volresults'></div>
  </center>
</div>
</div>"
}


#
# real start of the script
#

#if [ -f "$WEBDIR/glob-VOLUME-top-avrg-d.html-tmp" ]; then
#  print_voltop > "$WEBDIR/glob-volumes-top.html"
#fi

#if [ -f "$WEBDIR/glob-HOST-top-avrg-d.html-tmp" ]; then
#  print_hoststop > "$WEBDIR/glob-hosts-top.html"
#fi

if [ -f $INPUTDIR/bin/premium.pl ]; then # BINDIR cannot be used here as it is relative
  menu "$type_version" "0" "" "" "" "" ""
else
  menu "$type_version" "1" "" "" "" "" ""
fi

if [ $UPGRADE -eq 0 ]; then
  if [ ! -f $TMPDIR_STOR/$version-run ]; then
    if [ $DEBUG -eq 1 ]; then echo "Apparently nothing new, install_html.sh exiting"; fi
    exit 0
  else
    if [ $DEBUG -eq 1 ]; then echo "Apparently some changes in the env, install_html.sh continuing"; fi
  fi
else
  if [ $DEBUG -eq 1 ]; then echo "Looks like there was an upgrade, re-newing web pages"; fi
  touch $TMPDIR_STOR/$version
fi

version_act=$version
if [ -f $INPUTDIR/etc/version.txt ]; then
  # actually installed version include patch level (in $version is not patch level)
  version_act=`cat $INPUTDIR/etc/version.txt|tail -1`
fi
menu "$type_qmenu" "$version_act"

cp "$INPUTDIR/html/not_implemented.html" "$WEBDIR/"

if [ ! -f "$WEBDIR/gui-help.html" ]; then
  # copy of help file
  cp "$INPUTDIR/html/gui-help.html" "$WEBDIR/"
fi
if [ $UPGRADE -eq 1 ]; then
  cp "$INPUTDIR/html/gui-help.html" "$WEBDIR/"
fi

###
#Volume and Host top empty html pages

if [ ! -f "$WEBDIR/glob-hosts-top.html" ]; then
  cp "$INPUTDIR/html/glob-hosts-top.html" "$WEBDIR/"
fi

if [ ! -f "$WEBDIR/glob-volumes-top.html" ]; then
  cp "$INPUTDIR/html/glob-volumes-top.html" "$WEBDIR/"
fi


###



if [ $UPGRADE -eq 1 -o ! -f "$WEBDIR/hist_reports.html" -o ! -f $INPUTDIR/html/noscript.html -o ! -f $INPUTDIR/html/wipecookies.html -o ! -f "$WEBDIR/index.html" -o ! -f "$WEBDIR/gui-help.html" -o ! -f "$WEBDIR/hist_reports.html" -o ! -f "$WEBDIR/dashboard.html" ]; then
  cp "$INPUTDIR/html/dashboard.html" "$WEBDIR/"
  cp "$INPUTDIR/html/gui-help.html" "$WEBDIR/"
  cp "$INPUTDIR/html/index.html" "$WEBDIR/"
  cp "$INPUTDIR/html/wipecookies.html" "$WEBDIR/"
  cp "$INPUTDIR/html/noscript.html" "$WEBDIR/"
  cp "$INPUTDIR/html/hist_reports.html" "$WEBDIR/"
  cp "$INPUTDIR/html/test.html" "$WEBDIR/"
  cp "$INPUTDIR/html/robots.txt" "$WEBDIR/"
  cp "$INPUTDIR/html/favicon.ico" "$WEBDIR/"
fi

if [ ! -d "$WEBDIR/jquery" -o $UPGRADE -eq 1 -o ! -d "$WEBDIR/css" ]; then
  if [ ! -L "$WEBDIR/jquery" ]; then
    # skip that in case of internal development
    cd $INPUTDIR/html
    tar cf - jquery | (cd $WEBDIR ; tar xf - )
    cd - >/dev/null
  fi
fi

if [ ! -d "$WEBDIR/css" -o $UPGRADE -eq 1 ]; then
  if [ ! -L "$WEBDIR/css" ]; then
    # skip that in case of internal development
    cd $INPUTDIR/html
    tar cf - css | (cd $WEBDIR ; tar xf - )
    cd - >/dev/null
  fi
fi

#
# Main section
#


# workaround for sorting
HLIST=`for m in $INPUTDIR/data/*; do echo "$m"|sed 's/ /====spacce====/g'; done|sort -fr|xargs -n 1024`
for dir1 in $HLIST
do

  # workaround for managed names with a space inside
  dir1_space=`echo "$dir1"|sed 's/====spacce====/ /g'`
  hmc=`basename "$dir1_space"`
  # exclude sym links
  if [ -L "$dir1_space" ]; then
    continue
  fi

  if [ ! -d "$dir1_space" ]; then
    continue
  fi

  st_type="DS8K"
  if [ -f "$INPUTDIR/data/$hmc/SWIZ" ]; then
    st_type="SWIZ"
  fi
  if [ -f "$INPUTDIR/data/$hmc/XIV" ]; then
    st_type="XIV"
  fi
  if [ -f "$INPUTDIR/data/$hmc/DS5K" ]; then
    st_type="DS5K"
  fi
  if [ -f "$INPUTDIR/data/$hmc/HUS" ]; then
    st_type="HUS"
  fi
  if [ -f "$INPUTDIR/data/$hmc/DOTHILL" ]; then
    st_type="DOTHILL"
  fi
  if [ -f "$INPUTDIR/data/$hmc/VNXF" ]; then
    st_type="VNXF"
  fi
  if [ -f "$INPUTDIR/data/$hmc/VNX" ]; then
    st_type="VNX"
  fi
  if [ -f "$INPUTDIR/data/$hmc/NETAPP" ]; then
    st_type="NETAPP"
  fi
  if [ -f "$INPUTDIR/data/$hmc/3PAR" ]; then
    st_type="3PAR"
  fi
  if [ -f "$INPUTDIR/data/$hmc/VSPG" ]; then
    st_type="VSPG"
  fi
  if [ -f "$INPUTDIR/data/$hmc/UNITY" ]; then
    st_type="UNITY"
  fi
  if [ -f "$INPUTDIR/data/$hmc/NIMBLE" ]; then
    st_type="NIMBLE"
  fi
  if [ -f "$INPUTDIR/data/$hmc/VNXE" ]; then
    st_type="VNXE"
  fi
  if [ -f "$INPUTDIR/data/$hmc/VMAX" ]; then
    st_type="VMAX"
  fi
  if [ -f "$INPUTDIR/data/$hmc/VPLEX" ]; then
    st_type="VPLEX"
  fi
  if [ -f "$INPUTDIR/data/$hmc/COMPELLENT" ]; then
    st_type="COMPELLENT"
  fi
  if [ -f "$INPUTDIR/data/$hmc/OCEANSTOR" ]; then
    st_type="OCEANSTOR"
  fi
  if [ -f "$INPUTDIR/data/$hmc/PURE" ]; then
    st_type="PURE"
  fi



  if [ $gmenu_created -eq 0 ]; then
    # Global health status
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : HEALTH STATUS"; fi
    menu "$type_gmenu" "glob_hs" "HEALTH STATUS" "/$CGI_DIR/glob_hs.sh"

    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : Alerting"; fi
    menu "$type_gmenu" "alerting" "Alerting"  "/$CGI_DIR/alcfg.sh?cmd=form"

    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : CUSTOM GROUPS"; fi
    menu "$type_gmenu" "cgroups" "CUSTOM GROUPS" "whatever"

    # add Custom groups
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_cmenu : CUSTOM GROUPS : Configuration"; fi
    menu "$type_cmenu" "group_config" "<b>Configuration</b>" "/$CGI_DIR/cgrps.sh"
    custom_group # print custom group list

    gmenu_created=1 # it will not print global menu items
  fi

  # It must be behing dlobal section, in case there is not any storage configured to put there
  configured=`egrep "^$hmc:" $INPUTDIR/etc/storage-list.cfg|sed 's/^ *//g'|wc -l|sed 's/ //g'`
  if [ $configured -eq 0 ]; then
    # already unconfigured, then skip it
    continue
  fi

  if [ $gmenu_storage_created -eq 0 ]; then
    #if [ $ACCOUTING -eq 1 ]; then # accounting for DHL
    #  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : Accounting "; fi
    #  menu "$type_gmenu" "accounting" "Accounting" "/$CGI_DIR/acc.sh?sort=server"
    #fi

    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : Capacity"; fi
    menu "$type_gmenu" "capacity" "Capacity"  "capacity.html"

    if [ -f "$WEBDIR/glob-volumes-top.html" ]; then
      if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : Global Volumes TOP"; fi
      menu "$type_gmenu" "voltop" "Volumes TOP"  "glob-volumes-top.html"
    fi

    if [ -f "$WEBDIR/glob-hosts-top.html" ]; then
      if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : Global Hosts TOP"; fi
      menu "$type_gmenu" "hoststop" "Hosts TOP"  "glob-hosts-top.html"
    fi

    if [ -f "$WEBDIR/heatmap.html" ]; then
      if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_gmenu : Heatmap"; fi
      menu "$type_gmenu" "heatmap" "Heatmap"  "heatmap.html"
    fi

    host_exist=`ls -l $INPUTDIR/data/*/HOST/hosts.cfg 2>/dev/null|wc -l`
    if [ $host_exist -ne 0 ]; then
      echo "adding to menu : $type_gmenu : Mapping"
      menu "$type_gmenu" "mapping" "Mapping" "/$CGI_DIR/mapping.sh?host=ALL&storage=ALL"
    fi
    gmenu_storage_created=1 # it will not print global menu items
  fi


  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_hmenu : $hmc"; fi
  menu "$type_hmenu" "$hmc"

  hmc_space=`echo "$hmc"|sed -e 's/:/===double-col===/g' -e 's/ /%20/g'`

  # Pool capacity overview for selected storage platforms
  if [ ! "$st_type" = "XIV" -a  ! "$st_type" = "DS8K" -a  ! "$st_type" = "VPLEX" ]; then
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_hmenu : $hmc Capacity"; fi
    menu "$type_hmenu" "$hmc" "Capacity" "/$CGI_DIR/detail-graph.sh?host=$hmc_space&type=POOL&name=cap_pool&item=sum&time=d"
  fi

  if [ "$st_type" = "VMAX" ]; then
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_hmenu : $hmc Heatmap"; fi
     menu "$type_hmenu" "$hmc" "Heatmap"  "$hmc/heatmap.html"
  fi

  if [ "$st_type" = "DS5K" ]; then
    if [ -f "$INPUTDIR/data/$hmc/DS5K-v2" ]; then
      # pass to the GUI what kind of DS5k it is, it is used for alerting purposes (DS5k-v1 does not have response time metric)
      menu "$type_hmenu" "$hmc" "Configuration" "$hmc/gui-config-detail.html" "DS5K-v2"
    else
      menu "$type_hmenu" "$hmc" "Configuration" "$hmc/gui-config-detail.html"
    fi
  else
    if [ ! "$st_type" = "VMAX" -a $st_type != "VPLEX" ]; then
      if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_hmenu : $hmc Configuration"; fi
      menu "$type_hmenu" "$hmc" "Configuration" "$hmc_space/gui-config-detail.html"
    fi
  fi

  #menu "$type_hmenu" "$hmc" "Historical reports" "hist_reports.html"

  if [ \( $DEBUG -eq 1 -a -f "$INPUTDIR/data/$hmc/HOST/hosts.cfg" \) -a \( $st_type = "HUS" -o $st_type = "VSPG" -o $st_type = "DOTHILL" -o $st_type = "OCEANSTOR" -o $st_type = "COMPELLENT" \) ]; then
    echo "adding to menu : $type_hmenu : $hmc Mapping"
    menu "$type_hmenu" "$hmc" "Mapping" "$hmc_space/mapping.html"
  fi
  if [ $st_type != "HUS" -a $st_type != "VSPG" -a $st_type != "DOTHILL" -a $st_type != "COMPELLENT" -a $st_type != "OCEANSTOR" -a -f "$INPUTDIR/data/$hmc/HOST/hosts.cfg" -a -s "$INPUTDIR/data/$hmc/HOST/hosts.cfg" ]; then
    echo "adding to menu : $type_hmenu : $hmc Mapping"
    menu "$type_hmenu" "$hmc" "Mapping" "/$CGI_DIR/mapping.sh?host=$hmc_space&storage=$st_type"
    #menu "$type_hmenu" "$hmc" "Mapping" "$hmc/mapping.html"
  fi

  ### local heatmap per storage
  if [ -f "$WEBDIR/$hmc/heatmap.html" ]; then
    echo "adding to menu : $type_hmenu : $hmc Heatmap"
    menu "$type_hmenu" "$hmc" "Heatmap" "$hmc_space/heatmap.html"
  fi

  # Historical report form are different per each storage type
  hreport_form="hist_reports-$st_type.html"
  if [ ! -f "$INPUTDIR/html/$hreport_form" ]; then
    hreport_form="hist_reports.html"
  fi
  if [ "$st_type" = "DS5K" ]; then
    hreport_form="hist_reports-$st_type-v1.html"
    if [ -f "$INPUTDIR/data/$hmc/DS5K-v2" ]; then
      hreport_form="hist_reports-$st_type-v2.html"
    fi
  fi
  menu "$type_hmenu" "$hmc" "Historical reports" "$hreport_form"
  if [ $UPGRADE -eq 1 -o ! -f "$WEBDIR/$hreport_form" ]; then
    cp "$INPUTDIR/html/$hreport_form" "$WEBDIR"
  fi
  if [ "$st_type" = "DS5K" -o "$st_type" = "HUS" -o "$st_type" = "3PAR" -o "$st_type" = "VNX" -o "$st_type" = "SWIZ" -o "$st_type" = "UNITY" -o "$st_type" = "VNXE" -o "$st_type" = "VNXF" -o "$st_type" = "DOTHILL" -o "$st_type" = "COMPELLENT" -o $st_type = "OCEANSTOR" -o $st_type = "VPLEX" -o $st_type = "PURE" ]; then
    health_status="health_status.html"
    if [ -f "$WEBDIR/$hmc/$health_status" ]; then
      menu "$type_hmenu" "$hmc" "Health status" "$hmc_space/$health_status"
    fi
  fi
  if [ "$st_type" = "NETAPP" ]; then
      menu "$type_hmenu" "$hmc" "Health status" "/stor2rrd-cgi/netapp-ems.sh?storname=$hmc"
  fi

  # capacity overview for selected storage platforms
  #if [ "$st_type" = "3PAR" -o "$st_type" = "NETAPP" -o "$st_type" = "HUS" -o "$st_type" = "DS5K" -o "$st_type" = "SWIZ" -o "$st_type" = "DS8K" -o "$st_type" = "XIV" -o "$st_type" = "VSPG" -o "$st_type" = "VNXF" -o "$st_type" = "VNX" -o "$st_type" = "UNITY" -o "$st_type" = "VNXE" -o "$st_type" = "DOTHILL" ]; then
      menu "$type_xmenu" "$hmc" "POOL" "" "" "" "/$CGI_DIR/detail-graph.sh?host=$hmc&type=POOL&name=cap_total&item=sum&time=d"
  #fi



  for subs in $SUBSYSTEM_LIST
  do
    cd $INPUTDIR/data/"$hmc"

    if [ ! -d "$subs" -o -L "$subs" ]; then
      continue
    fi

    # workaround for managed names with a space inside
    #dir2_space=`echo "$dir2"|sed 's/====spacce====/ /g'`
    #managedname=`basename "$dir2_space"`
    managedname=$subs

    if [ "$managedname" = "iostats" -o "$managedname" = "tmp" ]; then
      continue
    fi

    if [ `echo "$SUBSYSTEM_LIST" | egrep " $managedname|$managedname "|wc -l|sed 's/ //g'` -eq 0 ]; then
      continue
    fi

    managedname_head=$managedname

    if [ -f "$INPUTDIR/data/$hmc/3PAR"  -a "$managedname" = "RANK" ]; then
      continue # skip RG (RANK, Logical Drive for 3PARs)
    fi

    if [ -f "$INPUTDIR/data/$hmc/SWIZ"  -a "$managedname" = "RANK" ]; then
      managedname_head="Managed&nbsp;disk"
    fi

    if [ -f "$INPUTDIR/data/$hmc/3PAR" -a "$managedname" = "RANK" ]; then
      managedname_head="RAID&nbsp;GROUP"
    fi

    if [ -f "$INPUTDIR/data/$hmc/HUS" -a "$managedname" = "RANK" ]; then
      managedname_head="RAID&nbsp;GROUP"
    fi

    if [ -f "$INPUTDIR/data/$hmc/VSPG" -a "$managedname" = "RANK" ]; then
      managedname_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/DOTHILL" -a "$managedname" = "RANK" ]; then
      managedname_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/3PAR" -a "$managedname" = "NODE" ]; then
      managedname_head="CACHE"
    fi

    if [ -f "$INPUTDIR/data/$hmc/NETAPP" -a "$managedname" = "RANK" ]; then
      managedname_head="RAID&nbsp;GROUP"
    fi

    if [ -f "$INPUTDIR/data/$hmc/PURE" -a "$managedname" = "RANK" ]; then
      managedname_head="RAID&nbsp;GROUP"
    fi

    if [ -f "$INPUTDIR/data/$hmc/UNITY" -a "$managedname" = "NASPOOL" ]; then
      managedname_head="NAS POOL"
    fi

    if [ -f "$INPUTDIR/data/$hmc/VNXE" -a "$managedname" = "NASPOOL" ]; then
      managedname_head="NAS POOL"
    fi

    if [ -f "$INPUTDIR/data/$hmc/NETAPP" -a "$managedname" = "NAS" ]; then
      managedname_head="SVM"
    fi

    if [ -f "$INPUTDIR/data/$hmc/VMAX" -a "$managedname" = "VOLUME" ]; then
      managedname_head="STORAGE GROUP"
    fi

    if [ -f "$INPUTDIR/data/$hmc/VMAX" -a "$managedname" = "DRIVE" ]; then
      managedname_head="DISK GROUP"
    fi

    if [ -f "$INPUTDIR/data/$hmc/VMAX" -a "$managedname" = "NODE" ]; then
      managedname_head="DIRECTOR"
    fi
    if [ -f "$INPUTDIR/data/$hmc/VPLEX" -a "$managedname" = "NODE" ]; then
      managedname_head="DIRECTOR"
    fi

    if [ -f "$INPUTDIR/data/$hmc/SWIZ"  -a "$managedname" = "CPU-CORE" ]; then
      # avoid CPU_CORE for SVC
      continue
    fi

    if [ -f "$INPUTDIR/data/$hmc/SWIZ" -a "$managedname" = "IO-GROUP" ]; then
      managedname_head="IO GROUP"
    fi

    if [ "$managedname" = "CPU-NODE" ]; then
      managedname_head="CPU"
    fi
    if [ "$managedname" = "CPU-CORE" ]; then
      managedname_head="CPU"
    fi
    if [ "$managedname" = "NODE-CACHE" ]; then
      managedname_head="CACHE"
    fi
    if [ "$managedname" = "NODE-CACHE"  -a  ! -f "$INPUTDIR/data/$hmc/VSPG" -a ! -f "$INPUTDIR/data/$hmc/HUS" -a ! -f "$INPUTDIR/data/$hmc/VNXF" -a ! -f "$INPUTDIR/data/$hmc/VMAX"  ]; then
      continue # NODE-CACHE only for HUS,VSPG,VNXF,VMAX
    fi

    drive_exist=`ls -l $INPUTDIR/data/$hmc/DRIVE/*rr* 2>/dev/null|wc -l|sed 's/ //g'`
    nnodecache_exist=`ls -l $INPUTDIR/data/$hmc/NODE-CACHE/*rr* 2>/dev/null|wc -l|sed 's/ //g'`
    raid_exist=`ls -l $INPUTDIR/data/$hmc/RANK/*rr* 2>/dev/null|wc -l|sed 's/ //g'`

    # workaround for sorting
    # here must be a list with predefined sorting and then check if it exist
    for managed1base in $SUBSYSTEM_LIST
    do
      if [ ! -d "../$managed1base" -o -L "../$managed1base" ]; then
        continue # exclude other stuff
      fi

      managed1base_head=$managed1base
      if [ -f "$INPUTDIR/data/$hmc/SWIZ" -a "$managed1base" = "RANK" ]; then
        managed1base_head="Managed&nbsp;disk"
      fi
      if [ -f "$INPUTDIR/data/$hmc/HUS" -a "$managed1base" = "RANK" ]; then
        managed1base_head="RAID&nbsp;GROUP"
      fi
      if [ -f "$INPUTDIR/data/$hmc/VSPG" -a "$managedname" = "RANK" ]; then
        managedname_head="RAID&nbsp;GROUP"
      fi
      if [ -f "$INPUTDIR/data/$hmc/DOTHILL" -a "$managedname" = "RANK" ]; then
        managedname_head="RAID&nbsp;GROUP"
      fi
      if [ -f "$INPUTDIR/data/$hmc/3PAR" -a "$managedname" = "NODE" ]; then
        managedname_head="CACHE"
      fi
      if [ -f "$INPUTDIR/data/$hmc/3PAR" -a "$managed1base" = "RANK" ]; then
        managed1base_head="RAID&nbsp;GROUP"
      fi
      if [ -f "$INPUTDIR/data/$hmc/NETAPP" -a "$managed1base" = "RANK" ]; then
        managed1base_head="RAID&nbsp;GROUP"
      fi
      if [ -f "$INPUTDIR/data/$hmc/PURE" -a "$managed1base" = "RANK" ]; then
        managed1base_head="RAID&nbsp;GROUP"
      fi
      if [ -f "$INPUTDIR/data/$hmc/SWIZ" -a "$managed1base" = "IO-GROUP" ]; then
        managed1base_head="IO&nbsp;GROUP"
      fi
      if [ "$managed1base" = "DRIVE" -a $drive_exist -eq 0 ]; then
        continue  # SVC and DS8k do not have DRIVES stats
      fi
      if [ "$managed1base" = "CPU-NODE" ]; then
        managed1base_head="CPU"
      fi
      if [ "$managed1base" = "CPU-CORE" ]; then
        managed1base_head="CPU"
      fi
      if [ "$managed1base" = "NODE-CACHE" -a $nnodecache_exist -gt 0 ]; then
        managed1base_head="CACHE"
      fi
    done

    if [ "$managedname" = "DRIVE" -a $drive_exist -eq 0 ]; then
      continue  # SVC and DS8k do not have DRIVES stats
    fi
    if [ "$managedname" = "NODE-CACHE" -a $nnodecache_exist -eq 0 ]; then
      continue  #
    fi
    if [ "$managedname" = "RANK" -a $raid_exist -eq 0 -a -f "$INPUTDIR/data/$hmc/DOTHILL" ]; then
     continue
    fi

    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_smenu : $hmc $managedname_head"; fi
    menu "$type_smenu" "$hmc" "$managedname_head"
    if [ "$managedname" = "NODE-CACHE" -a -f "$INPUTDIR/data/$hmc/VSPG" ]; then
      menu "$type_amenu" "$hmc" "$managedname_head" "cache-node" "Total" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=cache-node&storage=$st_type&item=cache&gui=1&none=none"
      continue
    fi

    ### QTREE FOR NETTAP
      if [ "$managedname" = "VOLUME" -a "$st_type" = "NETAPP" ]; then
        cd  $INPUTDIR/data/$hmc/$managedname
        for ii_space in `find . -print |egrep "\.rrq$"|sed -e 's/\.\///g' -e 's/\.rrq//' -e 's/ $//' -e 's/ /+===============+/g'`
        do
          ii=`echo "$ii_space" | sed 's/+===============+/ /g'`
          ii_url=`$PERL -e '$s=shift;$s=~s/ /+/g;;$s=~s/([^A-Za-z0-9\+-])/sprintf("%%%02X",ord($1))/seg;print "$s\n";' "$ii"`
          if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_emenu : $hmc:QTREE:$ii_url - $ii"; fi
          menu "$type_emenu" "$hmc" "QTREE" "$ii" "$ii" "/$CGI_DIR/detail.sh?host=$hmc&type=QTREE&name=$ii_url&storage=$st_type&item=lpar&gui=1&none=none"
        done
      fi

    ###

    c_error=0
    c_packet=0
    c_io=0
    c_data=0
    c_resp=0
    c_capacity=0
    c_cache=0
    c_cache_node=0 # HUS
    c_sys=0
    c_pprc=0
    c_top=0
    c_operating=0
    c_kahuna=0
    c_calls=0 # VNX
    c_b2b=0 #SWIZ
    c_tier=0 #3PAR NODE
    c_comp=0 #SWIZ
    c_sys_node=0 #COMPELLENT
    c_queue=0 #OCEANSTOR
    c_ops=0 #OCEANSTOR
    c_size=0

    host=0
    if [ "$managedname" = "HOST" -a "$st_type" = "VSPG" ]; then
      host=`ls "$INPUTDIR/data/$hmc/$managedname" 2>/dev/null| egrep "\.rrd$" | wc -l`
    fi

    for lpar in $AGG_LIST
    do
      if [ "$managedname" = "HOST" -a $st_type != "VMAX" -a $st_type != "3PAR" -a $st_type != "COMPELLENT" -a $st_type != "OCEANSTOR" -a $st_type != "PURE" ]; then
        if [ $host -eq 0 ]; then
          break # HOST does not have aggregates (except VSPG which has direct host data, but even here it can be skipped especially for old HUS-VM where are problems
        fi
        if [ ! "$VSPG_NATIVE_HOSTS"x = "x" ]; then
          if [ $VSPG_NATIVE_HOSTS -eq 0 ]; then
            if [ ! "$VSPG_NATIVE_HOSTS_EXCLUDED_LIST"x = "x" ]; then
              VSPG_EXCLUDE=0
              # list of exluded host is defined, go through
              for hmc_act_vspg in `echo $VSPG_NATIVE_HOSTS_EXCLUDED_LIST`
              do
                if [ "$hmc_act_vspg" = "$hmc" ]; then
                  VSPG_EXCLUDE=1
                  break
                fi
              done
              if [ $VSPG_EXCLUDE -eq 1 ]; then
                break # forced to use volume stats in etc/.magic
              fi
            else
              VSPG_EXCLUDE=1
              break # forced to use volume stats in etc/.magic
            fi
          fi
        fi
      fi

      item_sum="sum"
      if [ -f "$TMPDIR_STOR/$hmc/$managedname-$lpar-d.cmd" ]; then
      if [ "$lpar" = "cache_hit" -a "$st_type" = "VNXF" ]; then
        continue ### exeption for VNXF
      fi

        if [ "$lpar" = "sum_resp_t" -o "$lpar" = "sum_resp_t_b" -o "$lpar" = "resp_t_b" -o "$lpar" = "resp" -o "$lpar" = "resp_t" -o "$lpar" = "resp_t_r" -o "$lpar" = "resp_t_w" -o "$lpar" = "resp_t_r_frontend" -o "$lpar" = "resp_t_w_frontend" -o "$lpar" = "resp_t_r_b" -o "$lpar" = "resp_t_w_b" ]; then
          if [ $c_resp -gt 0 ]; then
            continue
          fi
          c_resp=1
          lpar_name="Response time"
          lpar="resp"
        fi
        if [ "$lpar" = "wafl_exempt-kahuna-exempt-hostOS-nwk_legacy" ]; then
          if [ $c_kahuna -gt 0 ]; then
            continue
          fi
          c_kahuna=1
          lpar_name="CPU"
          lpar="wafl_exempt-kahuna-exempt-hostOS-nwk_legacy"
        fi
        if [ "$lpar" = "b2b" ]; then
          if [ $c_b2b   -gt 0 ]; then
            continue
          fi
          c_b2b=1
          lpar_name="Other"
          lpar="other"
        fi
        if [ "$lpar" = "crc" ]; then
          if [ $c_b2b -gt 0 ]; then
            continue
          fi
          c_b2b=1
          lpar_name="Other"
          lpar="other"
        fi
        if [ "$lpar" = "read_error" -o "$lpar" = "write_error" ]; then
          if [ $c_error -gt 0 ]; then
            continue
          fi
          c_error=1
          lpar_name="Error"
          lpar="error"
        fi
        if [ "$lpar" = "packet_rate" -o "$lpar" = "read_packet" -o "$lpar" = "write_packet" ]; then
          if [ $c_packet -gt 0 ]; then
            continue
          fi
          c_packet=1
          lpar_name="Packet"
          lpar="packet"
        fi
        if [ "$lpar" = "sum_io_total" -o "$lpar" = "sum_io" -o "$lpar" = "sum_io_b" -o "$lpar" = "io" -o "$lpar" = "io_rate" -o "$lpar" = "read_io" -o "$lpar" = "write_io" -o "$lpar" = "read_io_b" -o "$lpar" = "write_io_b" -o "$lpar" = "io_rate_frontend" -o "$lpar" = "io_rate_backend" -o "$lpar" = "io_rate_rdf" -o "$lpar" = "read_io_frontend" -o "$lpar" = "read_io_backend" -o "$lpar" = "read_io_rdf" -o "$lpar" = "write_io_frontend" -o "$lpar" = "write_io_backend" -o "$lpar" = "write_io_rdf" -o "$lpar" = "io_rate_b" ]; then
          if [ $c_io -gt 0 ]; then
            continue
          fi
          if [ "$managedname" = "NODE" -a "$st_type" = "VNX" ]; then
            continue
          fi
          c_io=1
          lpar_name="IO"
          lpar="io"
        fi
        if [ "$lpar" = "calls_rate" -o "$lpar" = "read_calls" -o "$lpar" = "write_calls" ]; then
          if [ $c_calls -gt 0 ]; then
            continue
          fi
          c_calls=1
          lpar_name="Calls"
          lpar="calls"
        fi
        if [ "$lpar" = "sum_data_total" -o "$lpar" = "sum_data" -o "$lpar" = "sum_data_b" -o "$lpar" = "data" -o "$lpar" = "data_rate" -o "$lpar" = "read" -o "$lpar" = "write" -o "$lpar" = "read_b" -o "$lpar" = "write_b" -o "$lpar" = "data_rate_frontend" -o "$lpar" = "data_rate_backend" -o "$lpar" = "data_rate_rdf" -o "$lpar" = "read_frontend" -o "$lpar" = "read_backend" -o "$lpar" = "read_rdf" -o "$lpar" = "write_frontend" -o "$lpar" = "write_backend" -o "$lpar" = "write_rdf" -o "$lpar" = "data_rate_b" ]; then
          if [ $c_data -gt 0 ]; then
            continue
          fi
          c_data=1
          lpar_name="Data"
          lpar="data"
        fi
        if [ "$lpar" = "phys_usage" -o "$lpar" = "middle_usage" -o "$lpar" = "clean_usage" -o "$lpar" = "write_pend" -o "$lpar" = "dnlc_hit" -o "$lpar" = "buffer_hit" ]; then
          if [ $c_cache_node -gt 0 ]; then
            continue
          fi
          c_cache_node=1
          lpar_name="Total"
          lpar="cache-node"
        fi
        if [ "$lpar" = "operating_rate" ]; then
          if [ $c_operating -gt 0 ]; then
            continue
          fi
          c_operating=1
          lpar_name="Operating"
          lpar="operating"
        fi
        if [ "$lpar" = "queue" -a  -f "$INPUTDIR/data/$hmc/OCEANSTOR" -a \( "$managedname" = "NODE" -o "$managedname" = "DRIVE" -o "$managedname" = "VOLUME" \) ]; then
          if [ $c_queue -gt 0 ]; then
            continue
          fi
          c_queue=1
          lpar_name="Queue"
          lpar="queue"
        fi
        if [  \( "$lpar" = "ops_rate" -o "$lpar" = "read_ops" -o "$lpar" = "write_ops" \) -a  -f "$INPUTDIR/data/$hmc/OCEANSTOR" -a "$managedname" = "FS" ]; then
          if [ $c_ops -gt 0 ]; then
            continue
          fi
          c_ops=1
          lpar_name="OPS"
          lpar="ops"
       fi
        if [ "$lpar" = "sys" -a \( -f "$INPUTDIR/data/$hmc/COMPELLENT" -o -f "$INPUTDIR/data/$hmc/OCEANSTOR" \) -a "$managedname" = "NODE" ]; then
          if [ $c_sys_node -gt 0 ]; then
            continue
          fi
          c_sys_node=1
          lpar_name="CPU"
          lpar="cpu"
        fi
        #if [ "$lpar" = "sys" -o "$lpar" = "compress" ]; then
        if [ "$lpar" = "sys" -o "$lpar" = "compress" -o "$lpar" = "prct_d_cache_p" ]; then
          if [ $c_sys -gt 0 ]; then
            continue
          fi
          c_sys=1
          lpar_name="Total"
          lpar="cpu"
        fi
        if [ "$lpar" = "r_transfer_size" -o "$lpar" = "w_transfer_size"  ]; then
          if [ $c_size -gt 0 ]; then
            continue
          fi
          c_size=1
          lpar_name="Transfer size"
          lpar="transfer_size"
        fi
        if [ "$lpar" = "busy" -o "$lpar" = "sum_busy" -o "$lpar" = "sum_busy_backend" -o "$lpar" = "sum_busy_frontend" -o "$lpar" = "sum_busy_rdf" ]; then
          if [ $c_sys -gt 0 ]; then
            continue
          fi
          c_sys=1
          lpar_name="Total"
          lpar="busy"
          if [ -f "$INPUTDIR/data/$hmc/VNXF" -o -f "$INPUTDIR/data/$hmc/VMAX" -o -f "$INPUTDIR/data/$hmc/VPLEX" ]; then
            lpar_name="Busy"
          fi
        fi
        if [ "$lpar" = "sum_capacity" -o "$lpar" = "used" -o "$lpar" = "real" -o "$lpar" = "tier0" -o "$lpar" = "tier0-nl" -o "$lpar" = "tier2" -o "$lpar" = "tier1" -o "$lpar" = "alloc" ]; then
          if [ $c_capacity -gt 0 ]; then
            continue
          fi
          c_capacity=1
          lpar_name="Capacity"
          lpar="cap"
        fi
        if [ "$lpar" = "compress_cap" -o "$lpar" = "uncompress_cap" -o "$lpar" = "compress_virt_cap" ]; then
          if [ $c_comp -gt 0 ]; then
            continue
          fi
          c_comp=1
          lpar_name="Compression"
          lpar="comp"
        fi
        if [ "$lpar" = "cache_hit" -o "$lpar" = "read_pct" -o "$lpar" = "r_cache_usage" -o "$lpar" = "w_cache_usage" -o "$lpar" = "r_cache_hit" -o "$lpar" = "w_cache_hit" ]; then
          if [ $c_cache -gt 0 ]; then
            continue
          fi
          c_cache=1
          lpar_name="Cache"
          lpar="cache"
        fi
        if [ "$lpar" = "cfc_dirty" -o "$lpar" = "cfc_max" -o "$lpar" = "del_ack" ]; then
          if [ $c_tier -gt 0 ]; then
            continue
          fi
          c_tier=1
          lpar_name="Tier"
          lpar="tier"
        fi
        if [ "$lpar" = "pprc_rio" -o "$lpar" = "pprc_wio" -o "$lpar" = "pprc_data_r" -o "$lpar" = "pprc_data_w" -o "$lpar" = "pprc_rt_r" -o "$lpar" = "pprc_rt_w" ]; then
          if [ $c_pprc -gt 0 ]; then
            continue
          fi
          c_pprc=1
          lpar_name="PPRC"
          lpar="pprc"
        fi
        if [ "$lpar" = "top" ]; then
          if [ $c_top  -gt 0 ]; then
            continue
          fi
          if [ "$managedname" = "HOST" ]; then
            continue
          fi
          c_top=1
          lpar_name="Top"
          lpar="top"
          item_sum="all"
        fi
        if [ "$lpar" = "WP" ]; then
          lpar_name="Write Pending"
        fi

        # UNITY FC, ETHERNET, ISCSI PORTs
        if [ "$st_type" = "UNITY" -a "$managedname" = "PORT" ]; then
          if [ "$lpar" = "packet" ]; then
            if [ `ls -l $TMPDIR_STOR/$hmc/ETH$managedname-$lpar*-d.cmd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
              menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "Eth $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=ETH$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
            fi
            continue
          fi

          if [ `ls -l $INPUTDIR/data/$hmc/PORT/*_fc*.rrd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
            menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "FC $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
          fi

          if [ `ls -l $TMPDIR_STOR/$hmc/ETH$managedname-$lpar*-d.cmd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
            menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "Eth $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=ETH$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
          fi
          if [ `ls -l $TMPDIR_STOR/$hmc/ISCSI$managedname*-d.cmd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
            menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "iSCSI $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=ISCSI$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
          fi
          continue
        fi
        # VNXE FC, ETHERNET, ISCSI PORTs
        if [ "$st_type" = "VNXE" -a "$managedname" = "PORT" ]; then
          if [ "$lpar" = "packet" ]; then
            if [ `ls -l $TMPDIR_STOR/$hmc/ETH$managedname-$lpar*-d.cmd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
              menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "Eth $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=ETH$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
            fi
            continue
          fi

          if [ `ls -l $INPUTDIR/data/$hmc/PORT/*_fc*.rrd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
            menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "FC $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
          fi
          if [ `ls -l $TMPDIR_STOR/$hmc/ETH$managedname-$lpar*-d.cmd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
            menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "Eth $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=ETH$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
          fi
          if [ `ls -l $TMPDIR_STOR/$hmc/ISCSI$managedname*-d.cmd 2>/dev/null | wc -l |sed 's/ //g'` -gt 0 ]; then
            menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "iSCSI $lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=ISCSI$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"
          fi
          continue
        fi


        if [ "$st_type" = "VMAX" -a "$managedname" = "VOLUME" -a "$lpar" = "tier_alloc_prct" ]; then
          menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "FAST SG" "/$CGI_DIR/detail-graph.sh?host=$hmc&type=VOLUME&name=fast_sg&item=sum&time=d"
          continue
        fi
        if [ "$st_type" = "VMAX" -a "$managedname" = "VOLUME" -a "$lpar" = "alloc_vols" ]; then
          menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "Allocated to Tiers" "$hmc/sg_alloc.html"
          continue
        fi
        if [ "$st_type" = "VMAX" -a "$managedname" = "PORT" -a "$lpar" = "mapped_vols" ]; then
          menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "Mapped volumes" "$hmc/port_map.html"
          continue
        fi

        menu "$type_amenu" "$hmc" "$managedname_head" "$lpar" "$lpar_name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$lpar&storage=$st_type&item=$item_sum&gui=1&none=none"

      fi
    done

    managed1base_head=$managedname
    if [ -f "$INPUTDIR/data/$hmc/SWIZ" -a "$managedname" = "RANK" ]; then
      # for SWIZ
      managed1base_head="Managed&nbsp;disk"
    fi
    if [ -f "$INPUTDIR/data/$hmc/HUS" -a "$managedname" = "RANK" ]; then
      managed1base_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/3PAR" -a "$managedname" = "RANK" ]; then
      managed1base_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/VSPG" -a "$managedname" = "RANK" ]; then
      managed1base_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/DOTHILL" -a "$managedname" = "RANK" ]; then
      managed1base_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/3PAR" -a "$managedname" = "NODE" ]; then
      managed1base_head="CACHE"
    fi
    if [ -f "$INPUTDIR/data/$hmc/NETAPP" -a "$managedname" = "RANK" ]; then
      managed1base_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/PURE" -a "$managedname" = "RANK" ]; then
      managed1base_head="RAID&nbsp;GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/UNITY" -a "$managedname" = "NASPOOL" ]; then
      managed1base_head="NAS POOL"
    fi
    if [ -f "$INPUTDIR/data/$hmc/VMAX" -a "$managedname" = "DRIVE" ]; then
      managed1base_head="DISK GROUP"
    fi
    if [ -f "$INPUTDIR/data/$hmc/VMAX" -a "$managedname" = "NODE" ]; then
      managed1base_head="DIRECTOR"
    fi
    if [ -f "$INPUTDIR/data/$hmc/VPLEX" -a "$managedname" = "NODE" ]; then
      managed1base_head="DIRECTOR"
    fi
    if [ -f "$INPUTDIR/data/$hmc/VNXE" -a "$managedname" = "NASPOOL" ]; then
      managed1base_head="NAS POOL"
    fi
    if [ -f "$INPUTDIR/data/$hmc/NETAPP" -a "$managedname" = "NAS" ]; then
      managed1base_head="SVM"
    fi
    if [ -f "$INPUTDIR/data/$hmc/SWIZ" -a "$managedname" = "IO-GROUP" ]; then
      managed1base_head="IO&nbsp;GROUP"
    fi
    if [ "$managedname" = "CPU-NODE" ]; then
      managed1base_head="CPU"
    fi
    if [ "$managedname" = "CPU-CORE" ]; then
      managed1base_head="CPU"
    fi
    if [ "$managedname" = "NODE-CACHE" ]; then
      managed1base_head="CACHE"
    fi

    #
    # Hosts
    if [ "$managedname" = "HOST" -a -f "$INPUTDIR/data/$hmc/HOST/hosts.cfg" ]; then
      host=`ls "$INPUTDIR/data/$hmc/HOST" | egrep "\.rrd$" | wc -l`
      if [ $host -eq 0 -o "$VSPG_EXCLUDE" = "1" ]; then
        for host_space in `cut -f1 -d ":" "$INPUTDIR/data/$hmc/HOST/hosts.cfg"| sed -e 's/ $//' -e 's/ /+===============+/g'| sort -fn `
        do
          host=`echo "$host_space" | sed 's/+===============+/ /g'`
          host_url=`$PERL -e '$s=shift;$s=~s/ /+/g;;$s=~s/([^A-Za-z0-9\+-])/sprintf("%%%02X",ord($1))/seg;print "$s\n";' "$host"`
          if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_amenu : $hmc:$managedname:$host_url - $host"; fi
          menu "$type_amenu" "$hmc" "$managed1base_head" "$host" "$host" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$host_url&storage=$st_type&item=host&gui=1&none=none"
        done
      fi
    fi

    if [ "$managedname" = "VOLUME" ]; then
      continue
    fi

    #
    # LUN
    if [ "$managedname" = "LUN" ]; then
     # special loop per volumes as there will not be volumes in menu just volumes grouped per nicks
     if [ ! -f "$INPUTDIR/data/$hmc/$managedname/luns.cfg" ]; then
        if [ $DEBUG -eq 1 ]; then echo "volume no exist: $hmc:$managedname:$INPUTDIR/data/$hmc/$managedname/luns.cfg "; fi
        continue # lun stats do not exist there, at least cfg file
     fi
     for ii_space in `awk -F: '{print $1}' "$INPUTDIR/data/$hmc/$managedname/luns.cfg"|sed -e 's/ $//' -e 's/ /+===============+/g'| sort -fn `
     do
       ii=`echo "$ii_space" | sed 's/+===============+/ /g'`
       ii_url=`$PERL -e '$s=shift;$s=~s/ /+/g;;$s=~s/([^A-Za-z0-9\+-])/sprintf("%%%02X",ord($1))/seg;print "$s\n";' "$ii"`
       #ii_url=`echo $ii| sed -e 's/%\([0-9A-F][0-9A-F]\)/\\\\\x\1/g'`
       al=`egrep "^$managedname:$hmc:$ii:" $ALIAS_CFG 2>/dev/null|awk -F: '{print $4}'`
       if [ ! "$al"x =  "x" ]; then
         if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$ii_url - $ii - alias: $al"; fi
         menu "$type_lmenu" "$hmc" "$managed1base_head" "$ii" "$al" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$ii_url&storage=$st_type&item=lpar&gui=1&none=none"
       else
         if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$ii_url - $ii"; fi
         menu "$type_lmenu" "$hmc" "$managed1base_head" "$ii" "$ii" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$ii_url&storage=$st_type&item=lpar&gui=1&none=none"
       fi
     done
    else

      #
      # POOLs
      if [ "$managedname" = "POOL" -o "$managedname" = "TIER" ]; then
        pwd=`pwd`
        #if [ "$st_type" = "3PAR" -a "$managedname" = "POOL" ]; then
        #  continue # do not print POOL ites for 3PAR
        #fi

        #cd  $INPUTDIR/data/$hmc/POOL # since 1.35-2 all storage have POOL there
        cd  "$INPUTDIR/data/$hmc/$managedname" # since 1.35-2 all storage have POOL there

        managedname_cfg="pool.cfg"
        if [ "$managedname" = "TIER" ]; then
          managedname_cfg="tier.cfg"
        fi

        for i in `find . -print |egrep "\.rrd$"|egrep -v -- "-cap\.rrd$|-comp\.rrd$|nl-cap\.$rrd$"| sed -e 's/\.\///g' -e 's/\.rrd//' -e 's/^.*-P//g'| sort -fn|uniq`
        do
         menu_type_def="$type_lmenu"
         cd $pwd
           if [ $ALIAS -eq 1 ]; then
             al=`egrep "^$managedname:$hmc:$i:" $ALIAS_CFG|awk -F: '{print $4}'`
             #echo "003: $managedname:$hmc:$i: - $al"
             if [ ! "$al"x =  "x" ]; then
               if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$i - alias: $al"; fi
               menu "$type_lmenu" "$hmc" "$managed1base_head" "$i" "$al" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$i&storage=$st_type&item=lpar&gui=1&none=none"
             else
               name=$i
               # translate index into name
               #if [ -f "$INPUTDIR/data/$hmc/pool.cfg" ]; then
               if [ -f "$INPUTDIR/data/$hmc/$managedname_cfg" ]; then
                 #name=`egrep "^$i:" "$INPUTDIR/data/$hmc/pool.cfg"|awk -F: '{print $2}'`
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname_cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old POOL
                   name="$i"
                   menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               if [ $DEBUG -eq 1 ]; then echo "adding to menu : $menu_type_def : $hmc:$managedname:$i - $name"; fi
               menu "$menu_type_def" "$hmc" "$managed1base_head" "$i" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$i&storage=$st_type&item=lpar&gui=1&none=none"
             fi
           else
             name=$i
             # translate index into name
             #if [ -f "$INPUTDIR/data/$hmc/pool.cfg" ]; then
             if [ -f "$INPUTDIR/data/$hmc/$managedname_cfg" ]; then
               #name=`egrep "^$i:" "$INPUTDIR/data/$hmc/pool.cfg"|awk -F: '{print $2}'`
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname_cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old POOL
	         name="$i"
                 menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             if [ $DEBUG -eq 1 ]; then echo "adding to menu : $menu_type_def : $hmc:$managedname:$i "; fi
             menu "$menu_type_def" "$hmc" "$managed1base_head" "$i" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$i&storage=$st_type&item=lpar&gui=1&none=none"
           fi
        done
      else

        #
        # everything rest as PORT, RANKs, ....

        if [ "$managed1base_head" = "DRIVE" ]; then
          continue
        fi
        if [ "$managed1base_head" = "VVOL" ]; then
          continue
        fi

        pwd=`pwd`
        cd  "$INPUTDIR/data/$hmc/$managedname"

        if [ "$managedname" = "HOST" -a -f "$INPUTDIR/data/$hmc/VSPG" -a "$VSPG_EXCLUDE" = "1" ]; then
          continue
        fi

        for ii in `find . -print |egrep "\.rrd$"|sed -e 's/\.\///g' -e 's/\.rrd//'|sed "s/-/ /g" | sort -n -k1 -k2 | sed "s/ /-/g"`  # trick for sorting 1-1 at the end
        do

         menu_type_def="$type_lmenu"
         cd $pwd

         if [ "$managedname" = "RANK" -a ! -f "$INPUTDIR/data/$hmc/NETAPP" -a ! -f "$INPUTDIR/data/$hmc/HUS" -a ! -f "$INPUTDIR/data/$hmc/VSPG" -a ! -f "$INPUTDIR/data/$hmc/DOTHILL" -a ! -f "$INPUTDIR/data/$hmc/PURE" ]; then
           if [ "$managedname" = "RANK" -o "$managedname" = "VOLUME" ]; then
             i_pool=`echo $ii|grep -- "-P"|wc -l|sed 's/ //g'`
             if [ $i_pool -eq 0 ]; then
               continue # it is something wrong, could not found pool name
             fi
           fi
         fi
         #if [ "$managedname" = "DRIVE" -a  -f "$INPUTDIR/data/$hmc/NETAPP" -a -f "$INPUTDIR/data/$hmc/drive.cfg" ]; then
         #  i=`egrep "^$ii," $INPUTDIR/data/$hmc/drive.cfg|awk -F, '{print $2}'`
         #  if [ "$i"x = "x" ]; then
         #    menu_type_def="$type_removed" # place into removed menu
         #      i="$ii"
         #  fi
         #else
           if [ "$managedname" = "RANK" -o "$managedname" = "VOLUME" ]; then
             i=`echo $ii|sed 's/-P.*//'` # filter pool info for Ranks
           else
             i="$ii"
           fi
         #fi

         #if [ "$managedname" = "DRIVE" -a  -f "$INPUTDIR/data/$hmc/DOTHILL" -a -f "$INPUTDIR/data/$hmc/$managedname/drive.cfg" ]; then
         #  i=`egrep "^$ii," $INPUTDIR/data/$hmc/$managedname/drive.cfg|awk -F, '{print $2}'`
         #  if [ "$i"x = "x" ]; then
         #    menu_type_def="$type_removed" # place into removed menu
         #    i="$ii"
         #  fi
         #fi

           if [ $ALIAS -eq 1 ]; then
             al=`egrep "^$managedname:$hmc:$i:" $ALIAS_CFG|awk -F: '{print $4}'`
             #echo "003: $managedname:$hmc:$i: - $al"
             if [ ! "$al"x =  "x" ]; then
               if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$ii - $i - alias: $al"; fi
               menu "$type_lmenu" "$hmc" "$managed1base_head" "$ii" "$al" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$ii&storage=$st_type&item=lpar&gui=1&none=none"
             else
               name="$i"
               # translate index into name for MDISK
               if [ -f "$INPUTDIR/data/$hmc/mdisk.cfg" -a $managedname = "RANK" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/mdisk.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
	           name="$i"
                   menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               # translate index into name for FILESYSTEM
               if [ -f "$INPUTDIR/data/$hmc/$managedname/fs.cfg" -a $managedname = "FS" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/fs.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               # translate index into name for VOLUME-FOLDER
               if [ -f "$INPUTDIR/data/$hmc/$managedname/volume_folder.cfg" -a $managedname = "VOLUME-FOLDER" -a -f "$INPUTDIR/data/$hmc/COMPELLENT" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/volume_folder.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               # translate index into name for IO-GROUP
               if [ -f "$INPUTDIR/data/$hmc/$managedname/io_group.cfg" -a $managedname = "IO-GROUP" -a -f "$INPUTDIR/data/$hmc/SWIZ" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/io_group.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               # translate index into name for NODE
               if [ -f "$INPUTDIR/data/$hmc/$managedname/node.cfg" -a $managedname = "NODE" -a -f "$INPUTDIR/data/$hmc/COMPELLENT" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/node.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               if [ -f "$INPUTDIR/data/$hmc/$managedname/node.cfg" -a $managedname = "NODE" -a -f "$INPUTDIR/data/$hmc/OCEANSTOR" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/node.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               # translate index into name for NAS server
               if [ -f "$INPUTDIR/data/$hmc/$managedname/nas.cfg" -a $managedname = "NAS" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/nas.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               # translate index into name for PORT (3PAR & SWIZ so far)
               if [ -f "$INPUTDIR/data/$hmc/$managedname/port.cfg" -a $managedname = "PORT" ]; then
                 if [ -f "$INPUTDIR/data/$hmc/3PAR" -o -f "$INPUTDIR/data/$hmc/SWIZ" -o -f "$INPUTDIR/data/$hmc/COMPELLENT" -o -f "$INPUTDIR/data/$hmc/OCEANSTOR" ]; then
                   name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/port.cfg"|awk -F: '{print $1,$2}'`
                   if [ "$name"x = "x" ]; then
                     name="$i"
                   fi
                 else
                   name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/port.cfg"|awk -F: '{print $2}'`
                   if [ "$name"x = "x" ]; then
                     name="$i"
                   fi
                 fi
               fi
               #if [ "$managedname" = "VVOL" -a -f "$INPUTDIR/data/$hmc/$managedname/volumes.cfg" ]; then
               #  pom=$i
               #  vol_id=`echo "$pom"|sed 's/-.*//g'`
               #  name=`egrep ": $vol_id;" "$INPUTDIR/data/$hmc/$managedname/volumes.cfg"|awk -F ' : ' '{print $1}'`
               #  if [ "$name"x = "x" ]; then
               #    name=`echo "$i" | sed 's/-P[0-9]*$//'`
               #    menu_type_def="$type_removed" # place into removed menu
               #  fi
               #  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $menu_type_def : $hmc:$managedname:$ii - $i - $name "; fi
               #  menu "$menu_type_def" "$hmc" "$managed1base_head" "$ii" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$name&storage=$st_type&item=lpar&gui=1&none=none"
               #  continue
               #fi
               # translate index into name for NAS pool
               if [ -f "$INPUTDIR/data/$hmc/pool.cfg" -a $managedname = "NASPOOL" ]; then
                 name=`egrep "^$i:" "$INPUTDIR/data/$hmc/pool.cfg"|awk -F: '{print $2}'`
                 if [ "$name"x = "x" ]; then
                   # name has not been found --> looks like an old RANK
                   name="$i"
                   #menu_type_def="$type_removed" # place into removed menu
                 fi
               fi
               if [ $DEBUG -eq 1 ]; then echo "adding to menu : $menu_type_def : $hmc:$managedname:$ii - $i - $name"; fi
               menu "$menu_type_def" "$hmc" "$managed1base_head" "$ii" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$ii&storage=$st_type&item=lpar&gui=1&none=none"
             fi
           else
             name=$i
             # translate index into name for MDISK
             if [ -f "$INPUTDIR/data/$hmc/mdisk.cfg" -a $managedname = "RANK" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/mdisk.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
	         name="$i"
                 menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             if [ -f "$INPUTDIR/data/$hmc/$managedname/node.cfg" -a $managedname = "NODE" -a -f "$INPUTDIR/data/$hmc/COMPELLENT" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/node.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             # translate index into name for VOLUME-FOLDER
             if [ -f "$INPUTDIR/data/$hmc/$managedname/volume_folder.cfg" -a $managedname = "VOLUME-FOLDER" -a -f "$INPUTDIR/data/$hmc/COMPELLENT" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/volume_folder.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             # translate index into name for IO-GROUP
             if [ -f "$INPUTDIR/data/$hmc/$managedname/io_group.cfg" -a $managedname = "IO-GROUP" -a -f "$INPUTDIR/data/$hmc/SWIZ" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/io_group.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             if [ -f "$INPUTDIR/data/$hmc/$managedname/node.cfg" -a $managedname = "NODE" -a -f "$INPUTDIR/data/$hmc/OCEANSTOR" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/node.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             # translate index into name for FILESYSTEM
             if [ -f "$INPUTDIR/data/$hmc/$managedname/fs.cfg" -a $managedname = "FS" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/fs.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             # translate index into name for NAS server
             if [ -f "$INPUTDIR/data/$hmc/$managedname/nas.cfg" -a $managedname = "NAS" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/nas.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             # translate index into name for PORT (3PAR so far)
             if [ -f "$INPUTDIR/data/$hmc/$managedname/port.cfg" -a $managedname = "PORT" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/$managedname/port.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             # translate index into name for NAS pool
             if [ -f "$INPUTDIR/data/$hmc/pool.cfg" -a $managedname = "NASPOOL" ]; then
               name=`egrep "^$i:" "$INPUTDIR/data/$hmc/pool.cfg"|awk -F: '{print $2}'`
               if [ "$name"x = "x" ]; then
                 # name has not been found --> looks like an old RANK
                 name="$i"
                 #menu_type_def="$type_removed" # place into removed menu
               fi
             fi
             if [ $DEBUG -eq 1 ]; then echo "adding to menu : $menu_type_def : $hmc:$managedname:$ii - $i "; fi
             menu "$menu_type_def" "$hmc" "$managed1base_head" "$ii" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$ii&storage=$st_type&item=lpar&gui=1&none=none"
           fi
        done
        cd $pwd
      fi
    fi
  done
  hmc_space=`echo $hmc | sed 's/ /===space===/g'`
  drive_perl=`$PERL $INPUTDIR/bin/qs_volume.pl DRIVE $hmc_space`
  if [ ! "$drive_perl" = "" -a ! -f "$INPUTDIR/data/$hmc/VMAX" ]; then #  VMAX has DISK GROUPS
    echo "Adding drive :"
    echo "$drive_perl" >> $MENU_OUT
  fi
  volume_perl=`$PERL $INPUTDIR/bin/qs_volume.pl VOLUME $hmc_space`
  if [ ! "$volume_perl" = "" ]; then
    echo "Adding volumes :"
    echo "$volume_perl" >> $MENU_OUT
  fi
done



# SAN
count_of_switches=`ls -l $INPUTDIR/data/*/SAN-BRCD $INPUTDIR/data/*/SAN-CISCO 2>/dev/null| wc -l | sed -e 's/^[ \t]*//g;s/[ \t]*$//'`
count_of_switches_bna=`ls -l $INPUTDIR/data/*/SAN-BNA 2>/dev/null| wc -l | sed -e 's/^[ \t]*//g;s/[ \t]*$//'`
count_of_switches_diff=`expr $count_of_switches - $count_of_switches_bna`

# SAN Totals
if [ $count_of_switches_diff -gt 1 ]; then
  menu "$type_wmenu" "Totals"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals "; fi
  menu "$type_wmenu" "Totals" "Heatmap" "heatmap-port.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Heatmap "; fi
  menu "$type_wmenu" "Totals" "Data" "/$CGI_DIR/detail.sh?host=totals&type=DATA&name=totals&storage=SAN-BRCD&item=san_data_sum_tot&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Data "; fi
  menu "$type_wmenu" "Totals" "Frame" "/$CGI_DIR/detail.sh?host=totals&type=IO&name=totals&storage=SAN-BRCD&item=san_io_sum_tot&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Frame "; fi
  menu "$type_wmenu" "Totals" "Fabric" "/$CGI_DIR/detail.sh?host=totals&type=IO&name=totals&storage=SAN-BRCD&item=san_fabric&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Fabric "; fi

  if [ -f "$INPUTDIR/tmp/SAN-totals/san_isl_data_in-d.cmd" ]; then
    menu "$type_wmenu" "Totals" "ISL" "/$CGI_DIR/detail.sh?host=totals&type=ISL&name=totals&storage=SAN-BRCD&item=san_isl&gui=1&none=none"
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:ISL "; fi
  fi

  menu "$type_wmenu" "Totals" "Health status" "/$CGI_DIR/san_health_status.sh?host=total&type=HS-total&name=total&storage=SAN-BRCD&item=HS&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Health status "; fi
  menu "$type_wmenu" "Totals" "Configuration" "san_configuration.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Configuration "; fi
  menu "$type_wmenu" "Totals" "Historical reports" "hist_reports-SAN.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals:Historical reports "; fi
fi

# BNA Totals
if [ $count_of_switches_bna -gt 1 ]; then
  menu "$type_wmenu" "Totals-BNA"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA "; fi
  menu "$type_wmenu" "Totals-BNA" "Heatmap" "heatmap-port.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Heatmap "; fi
  menu "$type_wmenu" "Totals-BNA" "Data" "/$CGI_DIR/detail.sh?host=totals&type=DATA&name=totals&storage=SAN-BRCD&item=san_data_sum_tot_bna&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Data "; fi
  menu "$type_wmenu" "Totals-BNA" "Frame" "/$CGI_DIR/detail.sh?host=totals&type=IO&name=totals&storage=SAN-BRCD&item=san_io_sum_tot_bna&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Frame "; fi
  menu "$type_wmenu" "Totals-BNA" "CPU/Memory" "/$CGI_DIR/detail.sh?host=totals&type=CPU_MEMORY&name=totals&storage=SAN-BRCD&item=san_cpu_mem_tot&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:CPU/Memory "; fi
  menu "$type_wmenu" "Totals-BNA" "Fabric" "/$CGI_DIR/detail.sh?host=totals&type=IO&name=totals&storage=SAN-BRCD&item=san_fabric_bna&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Fabric "; fi

  if [ -f "$INPUTDIR/tmp/SAN-totals/san_isl_data_in_bna-d.cmd" ]; then
    menu "$type_wmenu" "Totals-BNA" "ISL" "/$CGI_DIR/detail.sh?host=totals&type=ISL&name=totals&storage=SAN-BRCD&item=san_isl_bna&gui=1&none=none"
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:ISL "; fi
  fi

  menu "$type_wmenu" "Totals-BNA" "Health status" "/$CGI_DIR/san_health_status.sh?host=total-BNA&type=HS-total&name=total&storage=SAN-BRCD&item=HS&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Health status "; fi
  menu "$type_wmenu" "Totals-BNA" "Configuration" "san_configuration_bna.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Configuration "; fi
  menu "$type_wmenu" "Totals-BNA" "Historical reports" "hist_reports-SAN.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Historical reports "; fi
  menu "$type_wmenu" "Totals-BNA" "Zone Configuration" "BNA/zones_cfg.html"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : Totals-BNA:Zone Configuration "; fi
fi


# SAN - Brocade
# workaround for sorting
HLIST=`for m in $INPUTDIR/data/*; do echo "$m"|sed 's/ /====spacce====/g'; done|sort -fr|xargs -n 1024`
for dir1 in $HLIST
do

  # workaround for managed names with a space inside
  dir1_space=`echo "$dir1"|sed 's/====spacce====/ /g'`
  hmc=`basename "$dir1_space"`
  # exclude sym links
  if [ -L "$dir1" ]; then
    continue
  fi

  if [ ! -d "$dir1" ]; then
    continue
  fi

  san_config_html_file="config.html"
  if [ -f "$INPUTDIR/data/$hmc/SAN-BRCD" ]; then
    switch_type="BRCD"
  else
    if [ -f "$INPUTDIR/data/$hmc/SAN-CISCO" ]; then
      switch_type="CISCO"
      san_config_html_file="glob_config.html"
    else
      continue
    fi
  fi

  is_alive "$INPUTDIR/data/$hmc/PORTS.cfg"
  if [ $? -eq 1 ] ; then
    # not updated more than 10 days --> death
    if [ $DEBUG -eq 1 ]; then echo "SAN old        : $hmc - skipped"; fi
    continue
  fi

  menu "$type_wmenu" "$hmc"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc "; fi

  menu "$type_wmenu" "$hmc" "Configuration" "$hmc/$san_config_html_file"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc:Configuration "; fi

  menu "$type_wmenu" "$hmc" "Data" "/$CGI_DIR/detail.sh?host=$hmc&type=Data&name=$hmc&storage=SAN-$switch_type&item=san_data_sum&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc:Data "; fi
  menu "$type_wmenu" "$hmc" "Frame" "/$CGI_DIR/detail.sh?host=$hmc&type=Frame&name=$hmc&storage=SAN-$switch_type&item=san_io_sum&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc:Frame "; fi

  if [ -f "$INPUTDIR/data/$hmc/SAN-BNA" ]; then
    menu "$type_wmenu" "$hmc" "CPU/Memory" "/$CGI_DIR/detail.sh?host=$hmc&type=CPU_MEMORY&name=$hmc&storage=SAN-$switch_type&item=san_cpu_mem&gui=1&none=none"
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc:CPU/Memory "; fi
  fi

  #menu "$type_wmenu" "$hmc" "Health status" "$hmc/health_status.html"
  #menu "$type_wmenu" "$hmc" "Health status" "/$CGI_DIR/detail.sh?host=$hmc&type=HS&name=$hmc&storage=SAN-$switch_type&item=HS&gui=1&none=none"
  menu "$type_wmenu" "$hmc" "Health status" "/$CGI_DIR/san_health_status.sh?host=$hmc&type=HS&name=$hmc&storage=SAN-$switch_type&item=HS&gui=1&none=none"
  if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc:Health status "; fi

  if [ $count_of_switches -eq 1 ]; then
    menu "$type_wmenu" "$hmc" "Historical reports" "hist_reports-SAN.html"
    if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_wmenu : $hmc:Historical reports "; fi
  fi

  echo "Adding SAN port:"
  volume_perl=`$PERL $INPUTDIR/bin/qs_volume.pl SAN $hmc`
  echo "$volume_perl" >> $MENU_OUT

  continue  # skip old method

  cd $INPUTDIR/data/"$hmc"

  managedname="SANPORT"
  st_type=SAN-CISCO
  item=san

  if [ ! -f "$INPUTDIR/data/$hmc/PORTS.cfg" ]; then
    continue
  fi

  #ports=`ls port*.rrd | sed "s/^port//g" | sort -n`
  #ports=`ls port*.rrd | sed "s/^port[a-zA-Z]*//g" | sed "s/-/ /g" | sort -n -k1 -k2 | sed "s/ /-/g"`

  ports=`ls port*.rrd | sed "s/^port\([a-zA-Z]*\)/\1 /g" \
        | sed "s/-/ /g" \
        | sed "s/^fc /111 /g" | sed "s/^Ethernet /222 /g" | sed "s/^GigabitEthernet /333 /g" | sed "s/^fcip /444 /g" | sed "s/^portChannel /555 /g" \
        | sort -n -k1 -k2 -k3 \
        | sed "s/^111 /fc/g" | sed "s/^222 /Ethernet/g" | sed "s/^333 /GigabitEthernet/g" | sed "s/^444 /fcip/g" | sed "s/^555 /portChannel/g" \
        | sed "s/ /-/g"`


  for lpar in $ports
  do
    first_name=`echo port$lpar | sed -e "s/.rrd//g"`
    second_name=`grep "^$first_name :" "$INPUTDIR/data/$hmc/PORTS.cfg" | awk -F' : ' '{print $3}'`
    if [ -z "$second_name" ]; then
      lpar_name=$first_name
    else
      lpar_name="$first_name [$second_name]"
    fi
    name=`echo $lpar_name | sed -e "s/^port//g"`

    if [ $ALIAS -eq 1 ]; then
      lpar_n=`echo $first_name | sed -e "s/^port//g"`
      port_al=`egrep "^SANPORT:$hmc:$first_name:" $ALIAS_CFG|awk -F: '{print $4}'`
      port_al2=`egrep "^SANPORT:$hmc:$lpar_n:" $ALIAS_CFG|awk -F: '{print $4}'`
      if [ "$port_al"x = "x" ] && [ "$port_al2"x != "x" ]; then
        port_al="$port_al2"
      fi

      if [ ! "$port_al"x =  "x" ]; then
        menu "$type_lmenu" "$hmc" "$managedname" "$first_name" "$port_al" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$first_name&storage=$st_type&item=$item&gui=1&none=none"
        if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$first_name:$port_al "; fi
      else
        menu "$type_lmenu" "$hmc" "$managedname" "$first_name" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$first_name&storage=$st_type&item=$item&gui=1&none=none"
        if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$first_name "; fi
      fi
    else
      menu "$type_lmenu" "$hmc" "$managedname" "$first_name" "$name" "/$CGI_DIR/detail.sh?host=$hmc&type=$managedname&name=$first_name&storage=$st_type&item=$item&gui=1&none=none"
      if [ $DEBUG -eq 1 ]; then echo "adding to menu : $type_lmenu : $hmc:$managedname:$first_name "; fi
    fi
  done
done
# End of SAN



menu "$type_tmenu" "doc" "Documentation" "gui-help.html"
menu "$type_tmenu" "maincfg" "Main configuration cfg"  "/$CGI_DIR/log-cgi.sh?name=maincfg&gui=1"
# do not print storage/SAN configuration as there could be visible passowrds for XIV and DCS3700, it will be without passwords in 1.30+
#menu "$type_tmenu" "scfg" "Storage configuration"  "/$CGI_DIR/log-cgi.sh?name=stcfg&gui=1"
#menu "$type_tmenu" "sancfg" "SAN configuration"  "/$CGI_DIR/log-cgi.sh?name=sancfg&gui=1"
menu "$type_tmenu" "acfg" "Alias configuration" "/$CGI_DIR/log-cgi.sh?name=aliascfg&gui=1"
menu "$type_tmenu" "elog" "Error log" "/$CGI_DIR/log-cgi.sh?name=errlog&gui=1"
menu "$type_tmenu" "errcgi" "Error log cgi-bin" "/$CGI_DIR/log-cgi.sh?name=errcgi&gui=1"

if [ ! "$VI_IMAGE"x = "x" ]; then
  if [ $VI_IMAGE -eq 1 ]; then
    menu "$type_tmenu" "update" "Product upgrade" "/$CGI_DIR/upgrade.sh?cmd=form"
  fi
fi

cp $MENU_OUT $MENU_OUT_FINAL

# to get some basic view into system for troubleshooting and healthcheck activities
df $INPUTDIR/data > $INPUTDIR/logs/sys.log 2>/dev/null
echo "========" >> $INPUTDIR/logs/sys.log
uname -a >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "========" >> $INPUTDIR/logs/sys.log
ls -l $INPUTDIR/bin >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "========" >> $INPUTDIR/logs/sys.log
ps -ef |egrep "2rrd|svc|xiv|ds8|ds5|hus|vspg|netapp|horcm" >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "========" >> $INPUTDIR/logs/sys.log
crontab -l >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "========" >> $INPUTDIR/logs/sys.log
ulimit -a >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "========" >> $INPUTDIR/logs/sys.log
free 2>/dev/null >> $INPUTDIR/logs/sys.log
echo "========" >> $INPUTDIR/logs/sys.log
svmon -G -O unit=MB 2>/dev/null >> $INPUTDIR/logs/sys.log
echo "========" >> $INPUTDIR/logs/sys.log
lsattr -El sys0 2>/dev/null |grep maxuproc  >> $INPUTDIR/logs/sys.log
echo "========" >> $INPUTDIR/logs/sys.log
vmstat 3 2 2>/dev/null >> $INPUTDIR/logs/sys.log
echo "========" >> $INPUTDIR/logs/sys.log
lparstat -i 2>/dev/null| grep "Online Virtual" >> $INPUTDIR/logs/sys.log
echo "========" >> $INPUTDIR/logs/sys.log
ls -l $INPUTDIR/data >> $INPUTDIR/logs/sys.log





# back up GUI config files
cd $INPUTDIR/etc
if [ ! -d .web_config ]; then
  mkdir .web_config
fi
if [ ! -f .web_config/Readme.txt ]; then
  echo "here is backup of all modification in GUI configuration files done through the web" > .web_config/Readme.txt
fi

cfg_backup ()
{
  cfg_file=$1
  date_act=`date "+%Y-%m-%d_%H:%M"`
  if [ -f web_config/$cfg_file ]; then
    if [ ! -f .web_config/$cfg_file ]; then
      cp -p web_config/$cfg_file .web_config/$cfg_file
      cp -p web_config/$cfg_file .web_config/$cfg_file-$date_act
    else
      if [ `diff web_config/$cfg_file .web_config/$cfg_file 2>/dev/null| wc -l|sed 's/ //g'` -gt 0 ]; then
        cp -p web_config/$cfg_file .web_config/$cfg_file
        cp -p web_config/$cfg_file .web_config/$cfg_file-$date_act
      fi
    fi
  fi
}

cfg_backup custom_groups.cfg
cfg_backup alerting.cfg

# VSP-G: removing old output files from Export Tool (data/storage/log/export0509012506.log)
for vsp in $INPUTDIR/data/*/VSP*
do
  if [ -f "$vsp" ]; then
    vsp_dir=`dirname "$vsp"`
    if [ -d "$vsp_dir/log" ]; then
      if [ $DEBUG -eq 1 ]; then echo "VSPG cleaning  : $vsp_dir/log -mtime +2 -exec rm -f {}; "; fi
      find $vsp_dir/log -mtime +2 -exec rm -f {} 2>/dev/null \;
    fi
  fi
done

rm -f $TMPDIR_STOR/$version-run
