#!/usr/bin/perl
# vim: set filetype=perl :

# Modules
use strict;
use warnings;
use Storable;
use Data::Dumper;

# $Data::Dumper::Terse = 1;
use Date::Parse;
use POSIX qw(strftime);
use XML::Simple;
use LWP::UserAgent;
use HTTP::Request;
use Sys::Hostname;
use Xorux_lib;

my $md5module = 1;
eval "use Digest::MD5 qw(md5_hex); 1" or $md5module = 0;

if ( !$md5module ) {    # binary MD5 module not found, use bundled pure Perl one
  use Digest::Perl::MD5 qw(md5_hex);

  # print STDERR "Bundled MD5\n";
}
my $hostname = hostname;
my $cntr_prefix = substr( md5_hex($hostname), 0, 4 ) . "_";

$XML::Simple::PREFERRED_PARSER = 'XML::Parser';

# Options and their default value
my $mininterval          = 300;     # minimal interval time for data collection (sec)
my $latency_io_threshold = 10;      # minimal IOs count to collect latency
my $time_to_cleanup      = 3600;    # minutes to delete * statistics collections, not just the latest (to remove possible orphans)
my $dir                  = "..";    # Data directory
my $now;

my $ssh = $ENV{SSH};
if ( !defined($ssh) || $ssh eq '' ) {
  $ssh = "ssh -o ConnectTimeout=15 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -o SendEnv=no ";
}

my $debug             = 0;          # Debug mode 0:off, 1:on
my $debug_full        = 0;          # Debug mode 0:off, 1:on
my $read_ext_file     = 0;
my $volume_data_limit = 3000000;    # default data rate limit for corrupted data detection - can be changed via ENV NETAPP_VOL_DATA_LIMIT
my $storname          = "";
my $storip            = "";
my $stormode          = "";
my $apiport           = 80;
my $apiproto          = "http";
my $statsFile;
my $cacheFile;
my $user = '';
my $pass = '';

# chomp(my $HOSTNAME = `hostname -s`);

my %netapp;
my $filer;

# required counters for 7-mode
my @volcounters   = qw(read_ops read_data read_latency write_ops write_data write_latency total_ops avg_latency node_uuid node_name);                                                                                                                                                                                       # volume
my @aggcounters   = qw(user_reads user_writes user_read_blocks user_write_blocks total_transfers total_transfers_hdd total_transfers_ssd user_reads_hdd user_writes_hdd user_read_blocks_hdd user_write_blocks_hdd user_reads_ssd user_writes_ssd user_read_blocks_ssd user_write_blocks_ssd node_uuid node_name);          # aggregate
my @dskcounters   = qw(user_reads read_ops read_data write_ops write_data user_writes user_read_blocks user_read_latency user_write_blocks user_write_latency total_transfers disk_speed disk_capacity disk_busy display_name raid_name);                                                                                   # disk
my @syscounters   = qw(system_id system_model serial_no ontap_version);                                                                                                                                                                                                                                                     # system
my @hacounters    = qw(total_reads total_writes bytes_read bytes_written);                                                                                                                                                                                                                                                  # hostadapter
my @ifcounters    = qw(recv_packets recv_errors send_packets send_errors collisions recv_data send_data recv_drop_packets);                                                                                                                                                                                                 # ifnet
my @luncounters   = qw(read_ops write_ops other_ops read_data write_data queue_full avg_latency total_ops avg_read_latency avg_write_latency avg_other_latency queue_depth_lun display_name);                                                                                                                               # LUN
my @nodecounters  = qw(read_ops write_ops total_ops read_data write_data total_data read_latency write_latency total_latency total_processor_busy cpu_busy sys_read_latency sys_write_latency sys_avg_latency disk_data_read disk_data_written);                                                                            # node
my @cpucounters   = qw(processor_busy sk_switches);                                                                                                                                                                                                                                                                         # CPU
my @cifscounters  = qw(cifs_latency cifs_ops cifs_read_ops cifs_write_ops connections established_sessions open_files path_based_ops persistent_opens read_data write_data);                                                                                                                                                # CIFS
my @nfs3counters  = qw(nfsv3_ops read_avg_latency nfsv3_read_ops write_avg_latency nfsv3_write_ops latency nfsv3_throughput nfsv3_read_throughput nfsv3_write_throughput link_avg_latency nfsv3_read_latency nfsv3_write_latency nfsv3_avg_op_latency);                                                                     # NFSv3
my @nfs4counters  = qw(compound_total null_total nfsv4_ops latency read_total read_avg_latency write_total write_avg_latency nfs4_throughput nfs4_read_throughput nfs4_write_throughput link_avg_latency nfsv4_calls nfsv4_ops nfsv4_ops_latency nfsv4_read_ops nfsv4_read_latency nfsv4_write_ops nfsv4_write_latency);    # NFSv4 - 7mode and cmode metrics have different names :-/
my @iscsicounters = qw(iscsi_read_ops iscsi_write_ops read_data write_data avg_read_latency avg_write_latency avg_latency iscsi_ops iscsi_latency);                                                                                                                                                                         # iSCSI
my @portcounters  = qw(recv-data recv-packets recv-errors sent-data sent-packets sent-errors);                                                                                                                                                                                                                              # Port
my @fcpcounters   = qw(read_ops write_ops total_ops read_data write_data avg_read_latency avg_write_latency avg_latency fcp_ops fcp_latency fcp_read_data fcp_write_data);                                                                                                                                                  # FCP
my @qtreecounters = qw(cifs_ops nfs_ops);

my $bindir  = $ENV{BINDIR};
my $webdir  = $ENV{WEBDIR};
my $basedir = $ENV{INPUTDIR};
if ( !$basedir ) {
  $basedir       = ".";
  $read_ext_file = 1;
}
if ( defined $ENV{DEBUG} )                 { $debug             = $ENV{DEBUG} }
if ( defined $ENV{DEBUG_FULL} )            { $debug_full        = $ENV{DEBUG_FULL} }
if ( defined $ENV{NETAPP_VOL_DATA_LIMIT} ) { $volume_data_limit = $ENV{NETAPP_VOL_DATA_LIMIT} }
if ( defined $ENV{NA_DIR} )                { $dir               = $ENV{NA_DIR} }

my $tmpdir = "$dir/";    # Directory for temporary files

my $out_perf_file;       #
my $out_conf_file;       #

sub file_write {
  my $file = shift;
  open IO, ">$file" or die "Cannot open $file for output: $!\n";
  print IO @_;
  close IO;
} ## end sub file_write

sub file_write_append {
  my $file = shift;
  if ( -f $file ) {
    open IO, ">>$file" or die "Cannot open $file for output: $!\n";
  }
  else {
    open IO, ">$file" or die "Cannot open $file for output: $!\n";
  }
  print IO @_;
  close IO;
} ## end sub file_write_append

sub file_read {
  my $file = shift;
  open IO, $file or die "Cannot open $file for input: $!\n";
  my @data = <IO>;
  close IO;
  wantarray ? @data : join( '' => @data );
} ## end sub file_read

sub trim {
  my $s = shift;
  if ($s) {
    $s =~ s/^\s+|\s+$//g;
  }
  return $s;
} ## end sub trim

sub crlf {
  my $s = shift;
  if ($s) {
    $s =~ s/\r\n/ /g;
    $s =~ s/\n\r/ /g;
  }
  return $s;
} ## end sub crlf

my $storage;
my $cache;
my $data;
my $config;
my $tzoffset;
my $baddata = 0;    # flag for incorrect volume data sent from NetApp 9.0

sub message {
  my $msg = shift;
  my $tm  = localtime();
  print( "INFO " . $tm . " naperf.pl: " . $msg . "\n" );
}

sub warning {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( "WARNING " . $tm . " naperf.pl: " . $msg . "\n" );
}

sub error {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( "ERROR " . $tm . " naperf.pl: " . $msg . "\n" );
}

sub writeStats {
  if ( !defined $cache->{'timestamp'} ) {
    store( \%netapp, $cacheFile );
    return;
  }
  $data->{'IntervalStartTime'} = $cache->{'timestamp'};

  # Open output file
  $out_perf_file = $tmpdir . $storname . "_netappperf_" . &fileextTime($now) . ".out";
  open( PERFOUT, ">>${out_perf_file}.tmp" ) || die "Couldn't open file ${out_perf_file}.tmp.";

  #
  my $elementType = shift;
  if ( $elementType eq "volume" ) {
    &writeVolumeStats;
  }
  elsif ( $elementType eq "pool" ) {
    &writePoolStats;
  }
  elsif ( $elementType eq "disk" ) {
    &writeDiskStats;
  }
  elsif ( $elementType eq "lun" ) {
    &writeLUNStats;
  }
  elsif ( $elementType eq "ha" ) {
    &writeHAStats;
  }
  elsif ( $elementType eq "port" ) {
    &writePortStats;
  }
  elsif ( $elementType eq "if" ) {
    &writeIFStats;
  }
  elsif ( $elementType eq "node" ) {
    &writeNodeStats;
  }
  elsif ( $elementType eq "cpu" ) {
    &writeCPUStats;
  }
  elsif ( $elementType eq "qtree" ) {
    &writeQtreeStats;
  }
  close(PERFOUT);
} ## end sub writeStats

sub writeVolumeStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nVolume Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  print PERFOUT "volume_id,name,total_ops,avg_latency,read_ops,read_data,read_latency,write_ops,write_data,write_latency,size-total,size-used,containing-aggregate-uuid,owning-vserver-uuid\n";
  foreach my $node ( sort keys %{ $netapp{volume} } ) {
    foreach my $dup ( @{ $netapp{volume}{$node} } ) {

      # print Dumper $dup;
      my %t = %{$dup};

      # print Dumper %t;
      # if (! $t{node_name}) {
      #   next;
      # }
      if ( $t{'containing-aggregate'} ) {
        $t{'containing-aggregate-uuid'} ||= $netapp{aggregate}{ $t{'containing-aggregate'} }{uuid};
      }

      #print Dumper @volcounters;
      my %r;
      @r{@volcounters} = @t{@volcounters};
      if ( $ENV{NETAPP_MICROS_TO_MILLIS} && $stormode eq "CMODE" ) {    # bad units fix (ONTAP Release 8.3.1)
        $r{avg_latency} =~ s/us/ms/;
        $r{read_latency} =~ s/us/ms/;
        $r{write_latency} =~ s/us/ms/;
      }
      &reCalc( \%r );

      # Discard latency metrics if IOs is too low
      $r{read_ops}  ||= 0;
      $r{write_ops} ||= 0;
      $r{total_ops} ||= 0;
      delete $r{read_latency}  if ( $r{read_ops} < $latency_io_threshold );
      delete $r{write_latency} if ( $r{write_ops} < $latency_io_threshold );
      delete $r{avg_latency}   if ( $r{total_ops} < $latency_io_threshold );
      $t{'size-total'} ||= $t{'size'};
      $t{'size-used'}  ||= $t{'used'};

      # print "$node: " . Dumper \%r;

      {
        no warnings 'uninitialized';
        print PERFOUT "$t{uuid},$node,$r{total_ops},$r{avg_latency},$r{read_ops},$r{read_data},$r{read_latency},$r{write_ops},$r{write_data},$r{write_latency}," . &ConvSizeUnits( 'GB', $t{'size-total'} ) . "," . &ConvSizeUnits( 'GB', $t{'size-used'} ) . ",$t{'containing-aggregate-uuid'},$t{'owning-vserver-uuid'}\n";
      }
    } ## end foreach my $dup ( @{ $netapp...})
  } ## end foreach my $node ( sort keys...)
} ## end sub writeVolumeStats

sub writePoolStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nPool Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  print PERFOUT "instance_uuid,instance_name,total_transfers,user_reads,user_writes,user_read_blocks,user_write_blocks,cap,used_cap,,,,,user_reads_hdd,user_reads_ssd,user_writes_hdd,user_writes_ssd\n";
  foreach my $node ( sort keys %{ $netapp{aggregate} } ) {
    my %t = %{ $netapp{aggregate}{$node} };
    if ( $t{uuid} ) {
      my %r;
      @r{@aggcounters} = @t{@aggcounters};
      &reCalc( \%r );
      $t{'size-total'} ||= $t{'size'};
      $t{'size-used'}  ||= $t{'used'};
      {
        no warnings 'uninitialized';
        print PERFOUT "$t{uuid},$node,$r{total_transfers},$r{user_reads},$r{user_writes},$r{user_read_blocks},$r{user_write_blocks}," . &ConvSizeUnits( 'TB', $t{'size-total'} ) . "," . &ConvSizeUnits( 'TB', $t{'size-used'} ) . ",,,,,$r{user_reads_hdd},$r{user_reads_ssd},$r{user_writes_hdd},$r{user_writes_ssd}\n";
      }
    } ## end if ( $t{uuid} )
  } ## end foreach my $node ( sort keys...)
} ## end sub writePoolStats

sub writeDiskStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nDrive Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  my @metrics = qw(disk-uid name read_ops write_ops read_data write_data user_read_ios user_write_ios user_read_latency user_write_latency user_read_data user_write_data total_transfers);
  print PERFOUT join( ",", @metrics ) . "\n";
  foreach my $node ( sort keys %{ $netapp{disk} } ) {
    my %t = %{ $netapp{disk}{$node} };
    if ( $stormode eq "CMODE" && !$t{plex} ) {
      next;
    }

    #print Dumper %t;
    my %r;
    @r{@dskcounters} = @t{@dskcounters};
    &reCalc( \%r );

    # print "$node: " . Dumper \%r;

    {
      $r{user_read_blocks}  ||= 0;
      $r{user_write_blocks} ||= 0;
      $r{user_read_blocks}  *= 4;
      $r{user_write_blocks} *= 4;
      no warnings 'uninitialized';
      if ( $stormode eq "7MODE" ) {
        print PERFOUT "$node,$t{'display_name'},$r{user_reads},$r{user_writes},$r{user_read_blocks},$r{user_write_blocks},$r{user_reads},$r{user_writes},$r{user_read_latency},$r{user_write_latency},$r{user_read_blocks},$r{user_write_blocks},$r{total_transfers}\n";
      }
      else {
        print PERFOUT "$t{'disk-uid'},$node,$r{read_ops},$r{write_ops},$r{read_data},$r{write_data},$r{user_reads},$r{user_writes},$r{user_read_latency},$r{user_write_latency},$r{user_read_blocks},$r{user_write_blocks},$r{total_transfers}\n";
      }
    }
  } ## end foreach my $node ( sort keys...)
} ## end sub writeDiskStats

sub writeLUNStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nLUN Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  my @metrics = qw(uuid name node_name read_ops write_ops read_data write_data avg_read_latency avg_write_latency avg_latency total_ops other_ops size size_used);
  print PERFOUT join( ",", @metrics ) . "\n";
  foreach my $lun ( sort keys %{ $netapp{lun} } ) {
    my %t = %{ $netapp{lun}{$lun} };

    #print Dumper %t;
    my %r;
    @r{@luncounters} = @t{@luncounters};
    &reCalc( \%r );

    # Discard latency metrics if IOs is too low
    $r{read_ops}  ||= 0;
    $r{write_ops} ||= 0;
    $r{total_ops} ||= 0;
    delete $r{avg_read_latency}  if ( $r{read_ops} < $latency_io_threshold );
    delete $r{avg_write_latency} if ( $r{write_ops} < $latency_io_threshold );
    delete $r{avg_latency}       if ( $r{total_ops} < $latency_io_threshold );

    #  message("size: $t->{'size'}, size-used: $t->{'size-used'}");

    # print "$node: " . Dumper \%r;

    {
      no warnings 'uninitialized';
      print PERFOUT "$t{uuid},$lun,$r{node_name},$r{read_ops},$r{write_ops},$r{read_data},$r{write_data},$r{avg_read_latency},$r{avg_write_latency},$r{avg_latency},$r{total_ops},$r{other_ops},$r{size},$r{size_used}\n";
    }
  } ## end foreach my $lun ( sort keys...)
} ## end sub writeLUNStats

sub writeHAStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nHostadapter Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  my @metrics = qw(name node_name total_reads total_writes kbytes_read kbytes_written);
  print PERFOUT join( ",", @metrics ) . "\n";

  if ( $stormode eq "7MODE" ) {
    foreach my $ha ( sort keys %{ $netapp{hostadapter} } ) {
      my %t = %{ $netapp{hostadapter}{$ha} };

      #print Dumper %t;
      my %r;
      @r{@hacounters} = @t{@hacounters};
      &reCalc( \%r );

      # print "$node: " . Dumper \%r;

      {
        $r{bytes_read}    ||= 0;
        $r{bytes_written} ||= 0;
        $r{bytes_read}    = sprintf( "%.3f", $r{bytes_read} / 1024 );
        $r{bytes_written} = sprintf( "%.3f", $r{bytes_written} / 1024 );
        no warnings 'uninitialized';
        print PERFOUT "$ha,$r{node_name},$r{total_reads},$r{total_writes},$r{bytes_read},$r{bytes_written}\n";
      }
    } ## end foreach my $ha ( sort keys ...)
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{hostadapter} } ) {
      foreach my $ha ( sort keys %{ $netapp{hostadapter}{$node} } ) {
        my %t = %{ $netapp{hostadapter}{$node}{$ha} };

        #print Dumper %t;
        my %r;
        @r{@hacounters} = @t{@hacounters};
        &reCalc( \%r );

        # print "$node: " . Dumper \%r;

        {
          $r{bytes_read}    ||= 0;
          $r{bytes_written} ||= 0;
          $r{bytes_read}    = sprintf( "%.3f", $r{bytes_read} / 1024 );
          $r{bytes_written} = sprintf( "%.3f", $r{bytes_written} / 1024 );
          no warnings 'uninitialized';
          print PERFOUT "$ha,$node,$r{total_reads},$r{total_writes},$r{bytes_read},$r{bytes_written}\n";
        }
      } ## end foreach my $ha ( sort keys ...)
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "7MODE")]
} ## end sub writeHAStats

sub writePortStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nPort Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  my @metrics = qw(name node_name recv-data recv-packets recv-errors sent-data sent-packets sent-errors);
  print PERFOUT join( ",", @metrics ) . "\n";

  if ( $stormode eq "7MODE" ) {
    foreach my $port ( sort keys %{ $netapp{ifnet} } ) {
      next if ( $port =~ /^lo.*/ );
      my %t = %{ $netapp{ifnet}{$port} };

      #print Dumper %t;
      my %r;
      my @portcounters = qw(recv_data recv_packets recv_errors send_data send_packets send_errors);    # Port
      @r{@portcounters} = @t{@portcounters};
      &reCalc( \%r );

      # print "$node: " . Dumper \%r;

      {
        no warnings 'uninitialized';
        print PERFOUT "$port,system,$r{'recv_data'},$r{'recv_packets'},$r{'recv_errors'},$r{'send_data'},$r{'send_packets'},$r{'send_errors'}\n";
      }
    } ## end foreach my $port ( sort keys...)
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{port} } ) {
      foreach my $port ( sort keys %{ $netapp{port}{$node} } ) {
        my %t = %{ $netapp{port}{$node}{$port} };
        my %c = %{ $cache->{port}{$node}{$port} };

        #print Dumper %t;
        my ( %rt, %rc );
        @rt{@portcounters} = @t{@portcounters};
        @rc{@portcounters} = @c{@portcounters};
        &reCalc( \%rt );
        &reCalc( \%rc );

        # print "$node: " . Dumper \%r;

        {
          my %d;
          no warnings 'uninitialized';
          foreach my $cntr (@portcounters) {
            $d{$cntr} = adjThroughput( $rt{$cntr} - $rc{$cntr} );
          }

          #$d{'recv-data'} = adjKB( $d{'recv-data'} );
          #$d{'sent-data'} = adjKB( $d{'sent-data'} );
          print PERFOUT "$port,$node,$d{'recv-data'},$d{'recv-packets'},$d{'recv-errors'},$d{'sent-data'},$d{'sent-packets'},$d{'sent-errors'}\n";
        }
      } ## end foreach my $port ( sort keys...)
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "7MODE")]
} ## end sub writePortStats

sub writeIFStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nIFNET Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  my @metrics = qw(name recv_packets recv_errors send_packets send_errors collisions recv_data send_data recv_drop_packets);
  print PERFOUT join( ",", @metrics ) . "\n";
  foreach my $node ( sort keys %{ $netapp{ifnet} } ) {
    my %t = %{ $netapp{ifnet}{$node} };

    #print Dumper %t;
    my %r;
    @r{@ifcounters} = @t{@ifcounters};
    &reCalc( \%r );

    # print "$node: " . Dumper \%r;

    {
      no warnings 'uninitialized';
      print PERFOUT "$node,$r{recv_packets},$r{recv_errors},$r{send_packets},$r{send_errors},$r{collisions},$r{recv_data},$r{send_data},$r{recv_drop_packets}\n";
    }
  } ## end foreach my $node ( sort keys...)
} ## end sub writeIFStats

sub writeNodeStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nNode Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  my @proto = qw(cifs_latency cifs_ops cifs_read_ops cifs_write_ops cifs_connections cifs_established_sessions cifs_open_files cifs_path_based_ops cifs_persistent_opens cifs_read_data cifs_write_data);
  push @proto, qw(nfsv3_ops nfsv3_read_latency nfsv3_read_ops nfsv3_write_latency nfsv3_write_ops nfsv3_avg_op_latency nfsv3_throughput nfsv3_read_throughput nfsv3_write_throughput nfsv3_link_avg_latency);
  push @proto, qw(nfsv4_calls nfsv4_ops nfsv4_ops_latency nfsv4_read_ops nfsv4_read_latency nfsv4_write_ops nfsv4_write_latency nfsv4_throughput nfsv4_read_throughput nfsv4_write_throughput nfsv4_link_avg_latency);
  push @proto, qw(iscsi_read_ops iscsi_write_ops iscsi_io_rate iscsi_read_data iscsi_write_data iscsi_data_rate iscsi_avg_read_latency iscsi_avg_write_latency iscsi_avg_latency);
  push @proto, qw(fcp_read_ops fcp_write_ops fcp_io_rate fcp_read_data fcp_write_data fcp_data_rate fcp_avg_read_latency fcp_avg_write_latency fcp_avg_latency);
  my @domain_busy = qw(wafl_exempt kahuna exempt hostOS nwk_legacy);

  print PERFOUT "node_name," . join( ",", @nodecounters, @proto, @domain_busy ) . "\n";
  if ( $stormode eq "CMODE" ) {
    foreach my $node ( sort keys %{ $netapp{'system:node'} } ) {
      my %t = %{ $netapp{'system:node'}{$node} };

      #print Dumper %t;
      my ( %r, %cifs, %nfs3, %nfs4, %iscsi, %fcp );
      @r{@nodecounters} = @t{@nodecounters};
      &reCalc( \%r );

      $r{read_data}  = adjKB( $r{read_data} );
      $r{write_data} = adjKB( $r{write_data} );
      $r{total_data} = adjKB( $r{total_data} );

      if ( exists $netapp{'cifs:node'}{$node} ) {
        my %cnode = %{ $netapp{'cifs:node'}{$node} };
        @cifs{@cifscounters} = @cnode{@cifscounters};
        reCalc( \%cifs );
      }

      if ( exists $netapp{'nfsv3:node'}{$node} ) {
        my %n3 = %{ $netapp{'nfsv3:node'}{$node} };

        # print "NFS3 $node: " . Dumper \%n3;
        @nfs3{@nfs3counters}        = @n3{@nfs3counters};
        $nfs3{nfsv3_read_latency}   = $nfs3{read_avg_latency};
        $nfs3{nfsv3_write_latency}  = $nfs3{write_avg_latency};
        $nfs3{nfsv3_avg_op_latency} = $nfs3{latency};
        ## bad unit fix
        if ( $ENV{NETAPP_MICROS_TO_MILLIS} ) {    # bad units fix (ONTAP Release 8.3.1)
          $nfs3{nfsv3_read_latency} =~ s/us/ms/;
          $nfs3{nfsv3_write_latency} =~ s/us/ms/;
          $nfs3{nfsv3_avg_op_latency} =~ s/us/ms/;
        }
        reCalc( \%nfs3 );
        $nfs3{nfsv3_throughput}       = adjKB( $nfs3{nfsv3_throughput} );
        $nfs3{nfsv3_read_throughput}  = adjKB( $nfs3{nfsv3_read_throughput} );
        $nfs3{nfsv3_write_throughput} = adjKB( $nfs3{nfsv3_write_throughput} );

        # print "NFS3 re $node: " . Dumper \%nfs3;
      } ## end if ( exists $netapp{'nfsv3:node'...})
      if ( exists $netapp{'nfsv4:node'}{$node} ) {
        my %n4 = %{ $netapp{'nfsv4:node'}{$node} };

        # print "NFS4 $node: " . Dumper \%n4;
        @nfs4{@nfs4counters}       = @n4{@nfs4counters};
        $nfs4{nfsv4_read_latency}  = $nfs4{read_avg_latency};
        $nfs4{nfsv4_write_latency} = $nfs4{write_avg_latency};
        $nfs4{nfsv4_ops_latency}   = $nfs4{latency};
        ## bad unit fix
        if ( $ENV{NETAPP_MICROS_TO_MILLIS} ) {    # bad units fix (ONTAP Release 8.3.1)
          $nfs4{nfsv4_read_latency} =~ s/us/ms/;
          $nfs4{nfsv4_write_latency} =~ s/us/ms/;
          $nfs4{nfsv4_ops_latency} =~ s/us/ms/;
        }

        reCalc( \%nfs4 );
        $nfs4{nfsv4_read_ops}  ||= int( $nfs4{read_total} / $data->{Interval} );
        $nfs4{nfsv4_write_ops} ||= int( $nfs4{write_total} / $data->{Interval} );
        $nfs4{nfsv4_calls}     ||= int( ( $nfs4{compound_total} + $nfs4{null_total} ) / $data->{Interval} );
        $nfs4{nfs4_throughput}       = adjKB( $nfs4{nfs4_throughput} );
        $nfs4{nfs4_read_throughput}  = adjKB( $nfs4{nfs4_read_throughput} );
        $nfs4{nfs4_write_throughput} = adjKB( $nfs4{nfs4_write_throughput} );

        # print "NFS4 re $node: " . Dumper \%nfs4;
      } ## end if ( exists $netapp{'nfsv4:node'...})
      if ( exists $netapp{'iscsi_lif:node'}{$node} ) {
        my %iscsin = %{ $netapp{'iscsi_lif:node'}{$node} };

        # print "NFS4 $node: " . Dumper \%n4;
        @iscsi{@iscsicounters} = @iscsin{@iscsicounters};

        # bad unit fix
        $iscsi{avg_read_latency} =~ s/us/ms/;
        $iscsi{avg_write_latency} =~ s/us/ms/;
        $iscsi{avg_latency} =~ s/us/ms/;

        reCalc( \%iscsi );
        $iscsi{iscsi_io_rate}   = $iscsi{iscsi_read_ops} + $iscsi{iscsi_write_ops};
        $iscsi{iscsi_data_rate} = $iscsi{read_data} + $iscsi{write_data};

        # $iscsi{iscsi_avg_latency} = wAvg($iscsi{avg_read_latency}, $iscsi{iscsi_read_ops}, $iscsi{avg_write_latency}, $iscsi{iscsi_write_ops});
        #print "iSCSI re $node: " . Dumper \%iscsi;
      } ## end if ( exists $netapp{'iscsi_lif:node'...})
      if ( exists $netapp{'fcp_lif:node'}{$node} ) {
        my %fcpn = %{ $netapp{'fcp_lif:node'}{$node} };

        # print "NFS4 $node: " . Dumper \%n4;
        @fcp{@fcpcounters} = @fcpn{@fcpcounters};
        ## bad unit fix
        if ( $ENV{NETAPP_MICROS_TO_MILLIS} ) {    # bad units fix (ONTAP Release 8.3.1)
          $fcp{avg_read_latency} =~ s/us/ms/;
          $fcp{avg_write_latency} =~ s/us/ms/;
          $fcp{avg_latency} =~ s/us/ms/;
        }

        reCalc( \%fcp );
        $fcp{fcp_data_rate} = $fcp{read_data} + $fcp{write_data};
      } ## end if ( exists $netapp{'fcp_lif:node'...})

      # print "$node: " . Dumper \%iscsi;

      {
        no warnings 'uninitialized';
        print PERFOUT "$node,$r{read_ops},$r{write_ops},$r{total_ops},$r{read_data},$r{write_data},$r{total_data},$r{read_latency},$r{write_latency},$r{total_latency},$r{total_processor_busy},$r{cpu_busy},";
        print PERFOUT "$cifs{cifs_latency},$cifs{cifs_ops},$cifs{cifs_read_ops},$cifs{cifs_write_ops},$cifs{connections},$cifs{established_sessions},$cifs{open_files},$cifs{path_based_ops},$cifs{persistent_opens},$cifs{read_data},$cifs{write_data},";
        print PERFOUT "$nfs3{nfsv3_ops},$nfs3{nfsv3_read_latency},$nfs3{nfsv3_read_ops},$nfs3{nfsv3_write_latency},$nfs3{nfsv3_write_ops},$nfs3{nfsv3_avg_op_latency},$nfs3{nfsv3_throughput},$nfs3{nfsv3_read_throughput},$nfs3{nfsv3_write_throughput},$nfs3{link_avg_latency},";
        print PERFOUT "$nfs4{nfsv4_calls},$nfs4{nfsv4_ops},$nfs4{nfsv4_ops_latency},$nfs4{nfsv4_read_ops},$nfs4{nfsv4_read_latency},$nfs4{nfsv4_write_ops},$nfs4{nfsv4_write_latency},$nfs4{nfs4_throughput},$nfs4{nfs4_read_throughput},$nfs4{nfs4_write_throughput},$nfs4{link_avg_latency},";
        print PERFOUT "$iscsi{iscsi_read_ops},$iscsi{iscsi_write_ops},$iscsi{iscsi_io_rate},$iscsi{read_data},$iscsi{write_data},$iscsi{iscsi_data_rate},$iscsi{avg_read_latency},$iscsi{avg_write_latency},$iscsi{avg_latency},";
        print PERFOUT "$fcp{read_ops},$fcp{write_ops},$fcp{total_ops},$fcp{read_data},$fcp{write_data},$fcp{fcp_data_rate},$fcp{avg_read_latency},$fcp{avg_write_latency},$fcp{avg_latency},";
        print PERFOUT "$t{wafl_exempt},$t{kahuna},$t{exempt},$t{hostOS},$t{nwk_legacy}\n";
      }
    } ## end foreach my $node ( sort keys...)
  } ## end if ( $stormode eq "CMODE")
  else {
    foreach my $node ( sort keys %{ $netapp{'system'} } ) {
      my %t = %{ $netapp{'system'}{$node} };

      #print Dumper %t;
      my ( %r, %cifs, %nfs3, %nfs4, %iscsi, %fcp );
      @r{@nodecounters} = @t{@nodecounters};
      &reCalc( \%r );

      # print Dumper %r;

      if ( exists $netapp{'cifs'}{cifs} ) {
        my %cnode = %{ $netapp{'cifs'}{cifs} };
        @cifs{@cifscounters} = @cnode{@cifscounters};
        reCalc( \%cifs );
      }

      if ( exists $netapp{'nfsv3'}{nfs} ) {
        my %n3 = %{ $netapp{'nfsv3'}{nfs} };

        # print "NFS3 $node: " . Dumper \%n3;
        @nfs3{@nfs3counters} = @n3{@nfs3counters};
        reCalc( \%nfs3 );
        $nfs3{nfsv3_read_latency}   ||= $nfs3{read_avg_latency};
        $nfs3{nfsv3_write_latency}  ||= $nfs3{write_avg_latency};
        $nfs3{nfsv3_avg_op_latency} ||= $nfs3{latency};

        #$nfs3{nfsv3_throughput} = adjThroughput($nfs3{nfsv3_throughput});
        #$nfs3{nfsv3_read_throughput} = adjThroughput($nfs3{nfsv3_read_throughput});
        #$nfs3{nfsv3_write_throughput} = adjThroughput($nfs3{nfsv3_write_throughput});
        # print "NFS3 re $node: " . Dumper \%nfs3;
      } ## end if ( exists $netapp{'nfsv3'...})
      if ( exists $netapp{'nfsv4'}{nfsv4} ) {
        my %n4 = %{ $netapp{'nfsv4'}{nfsv4} };

        # print "NFS4 $node: " . Dumper \%n4;
        @nfs4{@nfs4counters} = @n4{@nfs4counters};
        reCalc( \%nfs4 );
        $nfs4{nfsv4_read_latency}  ||= $nfs4{read_avg_latency};
        $nfs4{nfsv4_write_latency} ||= $nfs4{write_avg_latency};
        $nfs4{nfsv4_read_ops}      ||= $nfs4{read_total};
        $nfs4{nfsv4_write_ops}     ||= $nfs4{write_total};
        $nfs4{nfsv4_ops_latency}   ||= $nfs4{latency};
        if ( !exists $nfs4{nfsv4_calls} ) {
          $nfs4{nfsv4_calls} = $nfs4{compound_total} + $nfs4{null_total};
        }

        #$nfs3{nfs4_throughput} = adjThroughput($nfs4{nfs4_throughput});
        #$nfs4{nfs4_read_throughput} = adjThroughput($nfs4{nfs4_read_throughput});
        #$nfs4{nfs4_write_throughput} = adjThroughput($nfs4{nfs4_write_throughput});
        # print "NFS4 re $node: " . Dumper \%nfs4;
      } ## end if ( exists $netapp{'nfsv4'...})
      if ( exists $netapp{'iscsi'}{iscsi} ) {
        my %iscsin = %{ $netapp{'iscsi'}{iscsi} };
        @iscsi{@iscsicounters} = @iscsin{@iscsicounters};
        reCalc( \%iscsi );
      }
      if ( exists $netapp{'fcp'}{fcp} ) {
        my %fcpn = %{ $netapp{'fcp'}{fcp} };
        @fcp{@fcpcounters} = @fcpn{@fcpcounters};
        reCalc( \%fcp );
        $fcp{fcp_data_rate} = $fcp{fcp_read_data} + $fcp{fcp_write_data};
      } ## end if ( exists $netapp{'fcp'...})

      # print "$node: " . Dumper \%r;
      if ( exists $netapp{cpu}{elapsed_time} && exists $cache->{cpu}{elapsed_time} ) {

        # print Dumper $netapp{cpu}{domain_busy}, $cache->{cpu}{domain_busy};
        my $elapsed = $netapp{cpu}{elapsed_time} - $cache->{cpu}{elapsed_time};
        foreach my $param (qw(wafl_exempt kahuna exempt hostOS nwk_legacy)) {
          $t{$param} = sprintf( "%.1f", ( $netapp{cpu}{domain_busy}{$param} - $cache->{cpu}{domain_busy}{$param} ) / $elapsed * 100 );

          # print "$netapp{cpu}{domain_busy}{$param} - $cache->{cpu}{domain_busy}{$param} / $elapsed\n";
        }

        # print Dumper $elapsed;
      } ## end if ( exists $netapp{cpu...})
      {
        no warnings 'uninitialized';
        $r{total_ops}       = $r{read_ops} + $r{write_ops};
        $r{total_data}      = $r{disk_data_read} + $r{disk_data_written};
        $r{sys_avg_latency} = wAvg( $r{sys_read_latency}, $r{read_ops}, $r{sys_write_latency}, $r{write_ops} );
        print PERFOUT "$node,$r{read_ops},$r{write_ops},$r{total_ops},$r{disk_data_read},$r{disk_data_written},$r{total_data},$r{sys_read_latency},$r{sys_write_latency},$r{sys_avg_latency},$r{total_processor_busy},$r{cpu_busy},";
        print PERFOUT "$cifs{cifs_latency},$cifs{cifs_ops},$cifs{cifs_read_ops},$cifs{cifs_write_ops},$cifs{connections},$cifs{established_sessions},$cifs{open_files},$cifs{path_based_ops},$cifs{persistent_opens},$cifs{read_data},$cifs{write_data},";
        print PERFOUT "$nfs3{nfsv3_ops},$nfs3{nfsv3_read_latency},$nfs3{nfsv3_read_ops},$nfs3{nfsv3_write_latency},$nfs3{nfsv3_write_ops},$nfs3{nfsv3_avg_op_latency},$nfs3{nfsv3_throughput},$nfs3{nfsv3_read_throughput},$nfs3{nfsv3_write_throughput},$nfs3{link_avg_latency},";
        print PERFOUT "$nfs4{nfsv4_calls},$nfs4{nfsv4_ops},$nfs4{nfsv4_ops_latency},$nfs4{nfsv4_read_ops},$nfs4{nfsv4_read_latency},$nfs4{nfsv4_write_ops},$nfs4{nfsv4_write_latency},$nfs4{nfs4_throughput},$nfs4{nfs4_read_throughput},$nfs4{nfs4_write_throughput},$nfs4{link_avg_latency},";
        print PERFOUT "$iscsi{iscsi_read_ops},$iscsi{iscsi_write_ops},$iscsi{iscsi_ops},$iscsi{read_data},$iscsi{write_data},$iscsi{iscsi_data_rate},$iscsi{avg_read_latency},$iscsi{avg_write_latency},$iscsi{iscsi_latency},";
        print PERFOUT "$fcp{read_ops},$fcp{write_ops},$fcp{fcp_ops},$fcp{fcp_read_data},$fcp{fcp_write_data},$fcp{fcp_data_rate},$fcp{avg_read_latency},$fcp{avg_write_latency},$fcp{fcp_latency},";
        print PERFOUT "$t{wafl_exempt},$t{kahuna},$t{exempt},$t{hostOS},$t{nwk_legacy}\n";
      }
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "CMODE")]
} ## end sub writeNodeStats

sub writeCPUStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nCPU-core Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";

  # my @metrics = qw(name @nodecounters);
  print PERFOUT "cpu,node_name,processor_busy,sk_switches\n";
  if ( $stormode eq "7MODE" ) {
    foreach my $cpu ( sort keys %{ $netapp{processor} } ) {
      my %t = %{ $netapp{processor}{$cpu} };

      #print Dumper %t;
      my %r;
      @r{@cpucounters} = @t{@cpucounters};
      &reCalc( \%r );

      # print "$node: " . Dumper \%r;

      {
        no warnings 'uninitialized';
        print PERFOUT "$cpu,$r{node_name},$r{processor_busy},$r{sk_switches}\n";
      }
    } ## end foreach my $cpu ( sort keys...)
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{processor} } ) {
      foreach my $cpu ( sort keys %{ $netapp{processor}{$node} } ) {
        my %t = %{ $netapp{processor}{$node}{$cpu} };

        #print Dumper %t;
        my %r;
        @r{@cpucounters} = @t{@cpucounters};
        &reCalc( \%r );

        # print "$node: " . Dumper \%r;

        {
          no warnings 'uninitialized';
          print PERFOUT "$cpu,$node,$r{processor_busy},$r{sk_switches}\n";
        }
      } ## end foreach my $cpu ( sort keys...)
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "7MODE")]
} ## end sub writeCPUStats

sub writeQtreeStats {

  # print Dumper $volperf;
  #my %data;

  print PERFOUT "\nQtree Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";

  # my @metrics = qw(name @nodecounters);
  print PERFOUT "qtree,volume,cifs_ops,nfs_ops\n";
  if ( $stormode eq "7MODE" ) {
    foreach my $qtree ( sort keys %{ $netapp{qtree} } ) {
      my %t = %{ $netapp{qtree}{$qtree} };

      #print Dumper %t;
      my %r;
      @r{@qtreecounters} = @t{@qtreecounters};
      &reCalc( \%r );
      my ( $vol, $qtr ) = ( split "/", $qtree );

      # print "$node: " . Dumper \%r;

      {
        no warnings 'uninitialized';
        $qtr =~ s/,//g;
        $qtr =~ s/ /_/g;
        print PERFOUT "$qtr,$vol,$r{cifs_ops},$r{nfs_ops}\n";
      }
    } ## end foreach my $qtree ( sort keys...)
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{qtree} } ) {
      foreach my $qt ( sort keys %{ $netapp{qtree}{$node} } ) {
        my %t = %{ $netapp{qtree}{$node}{$qt} };

        #print Dumper %t;
        my %r;
        @r{@qtreecounters} = @t{@qtreecounters};
        &reCalc( \%r );
        my ( $vol, $qtr ) = ( split "/", $qt );
        {
          no warnings 'uninitialized';
          $qtr =~ s/,//g;
          $qtr =~ s/ /_/g;
          print PERFOUT "$qtr,$vol,$r{cifs_ops},$r{nfs_ops}\n";
        }
      } ## end foreach my $cpu ( sort keys...)
    } ## end foreach my $node ( sort keys...)
  }
} ## end sub writeQtreeStats

sub reCalc {
  my $vals = shift;
  while ( my ( $key, $value ) = each( %{$vals} ) ) {
    if ( !$value || $key =~ "_name" ) {
      next;
    }
    if ( $value eq "-" ) {
      $vals->{$key} = 0;
    }
    elsif ( $value =~ /KB/ ) {
      $vals->{$key} =~ s/KB//;
    }
    if ( $value =~ /.*\/s/ ) {
      $vals->{$key} =~ s/\/s//;
    }
    elsif ( $value =~ /us/ ) {
      $vals->{$key} =~ s/us//;
      $vals->{$key} = sprintf( "%.3f", $vals->{$key} / 1000 );
    }
    elsif ( $value =~ /ms/ ) {
      $vals->{$key} =~ s/ms//;
    }
    if ( $value =~ /[0-9]+b/ ) {
      $vals->{$key} =~ s/b//;
      $vals->{$key} = sprintf( "%.3f", $vals->{$key} / 1024 );
    }
    if ( $value =~ /%/ ) {
      $vals->{$key} =~ s/%//;
    }
  } ## end while ( my ( $key, $value...))

  # print Dumper \$vals;
} ## end sub reCalc

### adjust data Throughput: divide data by interval (seconds)
sub adjThroughput {
  my $bytes = shift;
  $bytes ||= 0;
  my $interval = $data->{'Interval'} || 300;
  return sprintf( "%.2f", $bytes / $interval );
} ## end sub adjThroughput

### divide data by 1024
sub adjKB {
  my $bytes = shift;
  $bytes ||= 0;
  return sprintf( "%.2f", $bytes / 1024 );
}

### calculates weighted average
sub wAvg {
  my ( $avg1, $cnt1, $avg2, $cnt2 ) = @_;
  no warnings 'uninitialized';
  my $result = 0;
  if ( $cnt1 + $cnt2 ) {
    $result = ( $avg1 * $cnt1 + $avg2 * $cnt2 ) / ( $cnt1 + $cnt2 );
  }
  return sprintf( "%.3f", $result );
} ## end sub wAvg

sub getCounters {
  my %counters;
  my @collectStart = `$ssh $user\@$storip "stats list counters" 2>&1`;
  my $object;
  foreach my $line (@collectStart) {
    $line = trim($line);
    if ( !$line ) {
      next;
    }
    if ( $line =~ "name:" ) {
      $object = ( split "name: ", $line )[1];
    }
    else {

      #if (! exists $counters{$object}) {
      #$counters{$object} = ();
      #}
      push @{ $counters{$object} }, $line;
    } ## end else [ if ( $line =~ "name:" )]
  } ## end foreach my $line (@collectStart)
  return \%counters;
} ## end sub getCounters

### save values to CSV
sub toCsv {
  if ( $stormode eq "CMODE" ) {
    no warnings 'uninitialized';
    my $csvFile = "$storname-protocols.csv";
    my $output;
    if ( !-f $csvFile ) {
      $output = "node,timestamp,nfsv3_read_ops,nfsv3_read_total,nfsv3_read_avg_latency,nfsv3_write_ops,nfsv3_write_total,nfsv3_write_avg_latency";
      $output .= ",nfsv4_read_ops,nfs4_read_throughput,nfsv4_read_avg_latency,nfsv4_write_ops,nfs4_write_throughput,nfsv4_write_avg_latency\n";
    }
    foreach my $node ( sort keys %{ $netapp{'nfsv3:node'} } ) {
      my %v3 = %{ $netapp{'nfsv3:node'}{$node} };
      my %v4 = %{ $netapp{'nfsv4:node'}{$node} };
      $output .= "$node,$data->{'IntervalEndTime'},$v3{'nfsv3_read_ops'},$v3{'read_total'},$v3{'read_avg_latency'},$v3{'nfsv3_write_ops'},$v3{'write_total'},$v3{'write_avg_latency'}";
      $output .= ",$v4{'read_total'},$v4{'nfs4_read_throughput'},$v4{'read_avg_latency'},$v4{'write_total'},$v4{'nfs4_write_throughput'},$v4{'write_avg_latency'}\n";
    } ## end foreach my $node ( sort keys...)
    file_write( $csvFile, $output );
  } ## end if ( $stormode eq "CMODE")
} ## end sub toCsv

sub writeConf {

  # Open output file
  if ( !defined $data->{'IntervalEndTime'} ) {
    return;
  }
  $out_conf_file = $tmpdir . $storname . "_netappconf_" . &fileextTime( $data->{'IntervalEndTime'} ) . ".out";
  open( PERFOUT, ">>${out_conf_file}.tmp" ) || die "Couldn't open file ${out_perf_file}.tmp.";

  #
  my $elementType = shift;
  &writeSystemConf if ( $elementType eq "system" );
  &writePoolConf   if ( $elementType eq "pool" );
  &writeVolumeConf if ( $elementType eq "volume" );
  &writeDiskConf   if ( $elementType eq "disk" );
  &writeLUNConf    if ( $elementType eq "lun" );
  &writeHAConf     if ( $elementType eq "ha" );
  &writeIFConf     if ( $elementType eq "if" );
  &writeNodeConf   if ( $elementType eq "node" );
  &writeCPUConf    if ( $elementType eq "cpu" );
  close(PERFOUT);
} ## end sub writeConf

sub writeSystemConf {
  print PERFOUT "\nConfiguration Data\n------------------\n";

  #print PERFOUT "\tMachine Name:".$config->{'system'}->{'Name'}."\t\n";
  print PERFOUT "\tMachine Type: NetApp\t\n";
  my $mode = "cluster";
  if ( $stormode eq "7MODE" ) {
    $mode = "system";
  }
  if ( exists $netapp{system}{$mode}{ontap_version} ) {
    $netapp{system}{$mode}{ontap_version} =~ s/\"//g;
  }
  {
    no warnings 'uninitialized';
    print PERFOUT "\tSystem ID: $netapp{system}{$mode}{system_id}\t\n";
    print PERFOUT "\tMachine Serial: $netapp{system}{$mode}{serial_no}\t\n";
    print PERFOUT "\tSystem Model: $netapp{system}{$mode}{system_model}\t\n";
    print PERFOUT "\tSystem Mode: $stormode\t\n";
    print PERFOUT "\tONTAP Version: $netapp{system}{$mode}{ontap_version}\t\n";
    print PERFOUT "\n";
  }
} ## end sub writeSystemConf

sub writePoolConf {
  print PERFOUT "\nPool Level Configuration\n------------------------\n";
  print PERFOUT "name,id,node_name,node_uuid,status,capacity,used_capacity\n";
  foreach my $aggr ( sort keys %{ $netapp{aggregate} } ) {
    my $t = \%{ $netapp{aggregate}{$aggr} };
    if ( $t->{uuid} ) {
      $t->{node_name}    ||= "";
      $t->{node_uuid}    ||= "";
      $t->{'size-total'} ||= $t->{'size'};
      $t->{'size-used'}  ||= $t->{'used'};

      # print Dumper $t;
      printf PERFOUT ( "%s,%s,%s,%s,%s,%.4f,%.4f\n", $aggr, $t->{uuid}, $t->{node_name}, $t->{node_uuid}, "", &ConvSizeUnits( "GB", $t->{'size-total'} ), &ConvSizeUnits( "GB", $t->{'size-used'} ) );
    } ## end if ( $t->{uuid} )
  } ## end foreach my $aggr ( sort keys...)
} ## end sub writePoolConf

sub writeVolumeConf {
  print PERFOUT "\nVolume Level Configuration\n--------------------------\n";
  my @metrics = qw(uuid name containing-aggregate-uuid containing-aggregate-name owning-vserver-uuid owning-vserver-name type instance-uuid state size-total size-available filesystem-size size-used percentage-used files-used files-total snapshot-percent-reserved node_name);
  print PERFOUT join( ",", @metrics ) . "\n";
  foreach my $node ( sort keys %{ $netapp{volume} } ) {
    foreach my $dup ( @{ $netapp{volume}{$node} } ) {
      my %t = %{$dup};

      # my $t = \%{ $netapp{volume}{$node} };
      if ( $t{'containing-aggregate'} ) {
        $t{'containing-aggregate-uuid'} ||= $netapp{aggregate}{ $t{'containing-aggregate'} }{uuid};
      }
      $t{'size-total'} ||= $t{'size'};
      $t{'size-used'}  ||= $t{'used'};
      {
        no warnings 'uninitialized';
        printf PERFOUT (
          "%s,%s,%s,%s,%s,%s,%s,%s,%s,%.4f,%.4f,%.4f,%.4f,%s,%s,%s,%s,%s\n", $t{uuid},
          $node,                                                             $t{'containing-aggregate-uuid'},
          $t{'containing-aggregate'},                                        $t{'owning-vserver-uuid'},
          $t{'owning-vserver-name'},                                         $t{'type'},
          $t{'instance-uuid'},                                               $t{'state'},
          &ConvSizeUnits( "GB", $t{'size-total'} ),      &ConvSizeUnits( "GB", $t{'size-available'} ),
          &ConvSizeUnits( "GB", $t{'filesystem-size'} ), &ConvSizeUnits( "GB", $t{'size-used'} ),
          $t{'percentage-used'}, $t{'files-used'},
          $t{'files-total'},     $t{'snapshot-percent-reserved'},
          $t{'node_name'}
        );
      }
    } ## end foreach my $dup ( @{ $netapp...})
  } ## end foreach my $node ( sort keys...)
} ## end sub writeVolumeConf

sub writeLUNConf {
  print PERFOUT "\nLUN Level Configuration\n--------------------------\n";
  my @metrics = qw(uuid name node_name serial-number block-size staging is-space-reservation-enabled alignment size size-used suffix-size multiprotocol-type share-state online prefix-size read-only mapped);
  print PERFOUT join( ",", @metrics ) . "\n";
  foreach my $lun ( sort keys %{ $netapp{lun} } ) {
    my $t = \%{ $netapp{lun}{$lun} };
    {
      no warnings 'uninitialized';
      printf PERFOUT (
        "%s,%s,%s,%s,%s,%s,%s,%s,%.4f,%.4f,%s,%s,%s,%s,%s,%s,%s\n", $t->{uuid},
        $lun,                                                       $t->{'node_name'},
        $t->{'serial-number'},                                      $t->{'block-size'},
        $t->{'staging'},                                            $t->{'is-space-reservation-enabled'},
        $t->{'alignment'}, &ConvSizeUnits( "GB", $t->{'size'} ),
        &ConvSizeUnits( "GB", $t->{'size-used'} ), $t->{'suffix-size'},
        $t->{'multiprotocol-type'}, $t->{'share-state'},
        $t->{'online'},             $t->{'prefix-size'},
        $t->{'read-only'},          $t->{'mapped'}
      );
    }
  } ## end foreach my $lun ( sort keys...)
} ## end sub writeLUNConf

sub writeDiskConf {
  print PERFOUT "\nDrive Level Configuration\n";
  print PERFOUT "-------------------------\n";
  print PERFOUT "disk-uid,name,id,port-name,raid-group,used-space,disk_capacity,port,raid-type,vendor-id,disk-type,node_name,plex,used-blocks,pool,aggregate,serial-number,disk-model,effective-disk-type,host-adapter,physical-space\n";
  foreach my $disk ( sort keys %{ $netapp{disk} } ) {
    my $t = \%{ $netapp{disk}{$disk} };
    if ( $stormode eq "CMODE" && !$t->{plex} ) {
      next;
    }

    #$t->{'disk-uid'} ||= $node;
    {
      no warnings 'uninitialized';
      if ( $stormode eq "7MODE" ) {
        printf PERFOUT (
          "%s,%s,%s,%s,%.4f,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%.4f\n", $disk,
          $t->{'display_name'},                                                $t->{'port-name'},
          $t->{'raid-group'},          &ConvSizeUnits( "GB", $t->{'used-space'} ),
          $t->{'disk_capacity'},       $t->{'port'},
          $t->{'raid-type'},           $t->{'vendor-id'},
          $t->{'disk-type'},           $t->{'node'},
          $t->{'plex'},                $t->{'used-blocks'},
          $t->{'pool'},                $t->{'aggregate'},
          $t->{'serial-number'},       $t->{'disk-model'},
          $t->{'effective-disk-type'}, $t->{'host-adapter'},
          &ConvSizeUnits( "GB", $t->{'physical-space'} )
        );
      } ## end if ( $stormode eq "7MODE")
      else {
        printf PERFOUT (
          "%s,%s,%s,%s,%.4f,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%.4f\n", $t->{'disk-uid'},
          $disk,                                                               $t->{'port-name'},
          $t->{'raid-group'},          &ConvSizeUnits( "GB", $t->{'used-space'} ),
          $t->{'disk_capacity'},       $t->{'port'},
          $t->{'raid-type'},           $t->{'vendor-id'},
          $t->{'disk-type'},           $t->{'node'},
          $t->{'plex'},                $t->{'used-blocks'},
          $t->{'pool'},                $t->{'aggregate'},
          $t->{'serial-number'},       $t->{'disk-model'},
          $t->{'effective-disk-type'}, $t->{'host-adapter'},
          &ConvSizeUnits( "GB", $t->{'physical-space'} )
        );
      } ## end else [ if ( $stormode eq "7MODE")]
    }
  } ## end foreach my $disk ( sort keys...)
} ## end sub writeDiskConf

sub writeHAConf {
  print PERFOUT "\nHostadapter Level Configuration\n------------------------\n";
  print PERFOUT "name,node_name,node_uuid\n";

  if ( $stormode eq "7MODE" ) {
    foreach my $ha ( sort keys %{ $netapp{hostadapter} } ) {
      my $t = \%{ $netapp{hostadapter}{$ha} };
      {
        no warnings 'uninitialized';
        printf PERFOUT ( "%s,%s,%s\n", $ha, $t->{node_name}, $t->{node_uuid} );
      }
    } ## end foreach my $ha ( sort keys ...)
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{hostadapter} } ) {
      foreach my $ha ( sort keys %{ $netapp{hostadapter}{$node} } ) {
        my $t = \%{ $netapp{$node}{hostadapter}{$ha} };
        {
          no warnings 'uninitialized';
          printf PERFOUT ( "%s,%s,%s\n", $ha, $node, $t->{node_uuid} );
        }
      } ## end foreach my $ha ( sort keys ...)
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "7MODE")]
} ## end sub writeHAConf

sub writeIFConf {
  print PERFOUT "\nIFNET Level Configuration\n------------------------\n";
  print PERFOUT "name,node_name,node_uuid\n";
  foreach my $node ( sort keys %{ $netapp{ifnet} } ) {
    my $t = \%{ $netapp{ifnet}{$node} };
    {
      no warnings 'uninitialized';
      printf PERFOUT ( "%s,%s,%s\n", $node, $t->{node_name}, $t->{node_uuid} );
    }
  } ## end foreach my $node ( sort keys...)
} ## end sub writeIFConf

sub writeNodeConf {
  print PERFOUT "\nNode Level Configuration\n------------------------\n";
  print PERFOUT "node_name,node_uuid\n";
  if ( $stormode eq "7MODE" ) {
    my $t = \%{ $netapp{'system'}{'system'} };
    {
      no warnings 'uninitialized';
      printf PERFOUT "$t->{instance_name},$t->{system_id}\n";
    }
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{'system:node'} } ) {
      my $t = \%{ $netapp{'system:node'}{$node} };
      {
        no warnings 'uninitialized';
        printf PERFOUT ( "%s,%s\n", $node, $t->{node_uuid} );
      }
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "7MODE")]
} ## end sub writeNodeConf

sub writeCPUConf {
  print PERFOUT "\nCPU-core Level Configuration\n------------------------\n";
  print PERFOUT "cpu,node_name,node_uuid\n";
  if ( $stormode eq "7MODE" ) {
    foreach my $cpu ( sort keys %{ $netapp{processor} } ) {
      my $t = \%{ $netapp{processor}{$cpu} };
      {
        no warnings 'uninitialized';
        printf PERFOUT ( "%s,%s,%s\n", $cpu, $t->{node_name}, $t->{node_uuid} );
      }
    } ## end foreach my $cpu ( sort keys...)
  } ## end if ( $stormode eq "7MODE")
  else {
    foreach my $node ( sort keys %{ $netapp{processor} } ) {
      foreach my $cpu ( sort keys %{ $netapp{processor}{$node} } ) {
        my $t = \%{ $netapp{$node}{processor}{$cpu} };
        {
          no warnings 'uninitialized';
          printf PERFOUT ( "%s,%s,%s\n", $cpu, $node, $t->{node_uuid} );
        }
      } ## end foreach my $cpu ( sort keys...)
    } ## end foreach my $node ( sort keys...)
  } ## end else [ if ( $stormode eq "7MODE")]
} ## end sub writeCPUConf

sub fileextTime {
  my $t = shift;
  if ( !defined $t ) { return "NA"; }
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime($t);
  my $y = $year + 1900;
  my $m = $mon + 1;
  return sprintf( "%d%02d%02d_%02d%02d", $y, $m, $mday, $hour, $min, $sec );
} ## end sub fileextTime

sub isotime {
  my $t = shift;
  if ( $t =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2}).(\d{6})([\+-])(\d+)$/ ) {
    my $tmod = $9 % 60;
    use integer;
    my $tdiv = $9 / 60;
    no integer;
    my $tz = sprintf( "%04d", ( $tdiv * 100 ) + $tmod );
    return sprintf( "%d:%02d:%02dT%02d:%02d:%02d.%06d%s%s", $1, $2, $3, $4, $5, $6, $7, $8, $tz );
  } ## end if ( $t =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2}).(\d{6})([\+-])(\d+)$/)
} ## end sub isotime

sub ConvSizeUnits {
  my ( $unit, $size ) = @_;
  if ( !$size ) {
    return 0;
  }
  if ( $unit eq "KB" ) { $size /= 1024 }
  if ( $unit eq "MB" ) { $size /= 1048576 }
  if ( $unit eq "GB" ) { $size /= 1073741824 }
  if ( $unit eq "TB" ) { $size /= 1099511627776 }

  return sprintf( "%.5f", $size );
} ## end sub ConvSizeUnits

sub tzoffset {
  return strftime( "%z", localtime );
}

sub epoch2isotime {

  # Output: 2015:02:05T19:54:07.000000+0100
  my ( $tm, $tz ) = @_;    # epoch, TZ offset (+0100)
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime($tm);
  my $y   = $year + 1900;
  my $m   = $mon + 1;
  my $mcs = 0;
  my $str = sprintf( "%4d:%02d:%02dT%02d:%02d:%02d.%06d%s", $y, $m, $mday, $hour, $min, $sec, $mcs, $tz );
  return ($str);
} ## end sub epoch2isotime

sub epoch2human {

  # Output: 2015:02:05T19:54:07.000000+0100
  my ( $tm, $tz ) = @_;    # epoch, TZ offset (+0100)
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime($tm);
  my $y   = $year + 1900;
  my $m   = $mon + 1;
  my $mcs = 0;
  my $str = sprintf( "%4d/%02d/%02d %02d:%02d:%02d", $y, $m, $mday, $hour, $min, $sec );
  return ($str);
} ## end sub epoch2human

sub obscure_password {
  my $string = shift;
  my $obscure = encode_base64( pack( "u", $string ), "" );
  return $obscure;
}

sub unobscure_password {
  my $string    = shift;
  my $unobscure = decode_base64($string);
  $unobscure = unpack( chr( ord("a") + 19 + print "" ), $unobscure );
  return $unobscure;
} ## end sub unobscure_password

sub encode_base64 {
  my $s = shift;
  my $r = '';
  while ( $s =~ /(.{1,45})/gs ) {
    chop( $r .= substr( pack( "u", $1 ), 1 ) );
  }
  my $pad = ( 3 - length($s) % 3 ) % 3;
  $r =~ tr|` -_|AA-Za-z0-9+/|;
  $r =~ s/.{$pad}$/"="x$pad/e if $pad;
  $r =~ s/(.{1,72})/$1\n/g;
  $r;
} ## end sub encode_base64

sub decode_base64 {
  my $d = shift;
  $d =~ tr!A-Za-z0-9+/!!cd;
  $d =~ s/=+$//;
  $d =~ tr!A-Za-z0-9+/! -_!;
  my $r = '';
  while ( $d =~ /(.{1,60})/gs ) {
    my $len = chr( 32 + length($1) * 3 / 4 );
    $r .= unpack( "u", $len . $1 );
  }
  $r;
} ## end sub decode_base64

sub health_status {
  my $alert_status = 0;
  if ( $netapp{health_status} ) {
    if ( !-d "$basedir/tmp/health_status_summary" ) {
      mkdir( "$basedir/tmp/health_status_summary", 0755 ) || error( "Cannot mkdir $basedir/tmp/health_status_summary: $!" . __FILE__ . ":" . __LINE__ ) && exit;
    }
    if ( !-f "$basedir/tmp/health_status_summary/$storname.ok" && !-f "$basedir/tmp/health_status_summary/$storname.nok" ) {
      $alert_status = 1;
    }
    my $health_status_file = "$basedir/tmp/health_status_summary/$storname";

    if ( lc( $netapp{health_status} ) eq "ok" ) {
      $health_status_file .= ".ok";
      my $status = "STORAGE : $storname : OK : $now";
      if ( -f "$basedir/tmp/health_status_summary/$storname.nok" ) {
        unlink("$basedir/tmp/health_status_summary/$storname.nok");
      }
      unlink($health_status_file);
      file_write( $health_status_file, $status );
    } ## end if ( lc( $netapp{health_status...}))
    else {
      $health_status_file .= ".nok";
      my $status = "STORAGE : $storname : NOT_OK : $now : $netapp{health_status}";
      if ( -f "$basedir/tmp/health_status_summary/$storname.ok" ) {
        unlink("$basedir/tmp/health_status_summary/$storname.ok");
        ### alert health status
        if ( exists &Xorux_lib::health_status_alerting ) {
          Xorux_lib::health_status_alerting($storname);
        }
      } ## end if ( -f "$basedir/tmp/health_status_summary/$storname.ok")
      if ( $alert_status == 1 ) {
        ### alert health status
        if ( exists &Xorux_lib::health_status_alerting ) {
          Xorux_lib::health_status_alerting($storname);
        }
      } ## end if ( $alert_status == ...)
      unlink($health_status_file);
      file_write( $health_status_file, $status );
    } ## end else [ if ( lc( $netapp{health_status...}))]

  } ## end if ( $netapp{health_status...})
} ## end sub health_status

### MAIN ##########################

$tzoffset = &tzoffset();

$now                       = time;
$data->{'IntervalEndTime'} = $now;
$netapp{timestamp}         = $now;

if ($read_ext_file) {
  $stormode = "CMODE";
}
else {
  if ( defined $ENV{STORAGE_NAME} ) {
    $storname = $ENV{STORAGE_NAME};
  }
  else {
    error("naperf.pl: NetApp storage name alias is required.\n");
    exit(1);
  }

  if ( defined $ENV{STORAGE_MODE} ) {
    $stormode = $ENV{STORAGE_MODE};
    $netapp{mode} = $stormode;
  }
  else {
    error("naperf.pl: NetApp storage mode is required.\n");
    exit(1);
  }

  if ( defined $ENV{NA_PORT_API} ) {
    $apiport = $ENV{NA_PORT_API};
  }

  if ( defined $ENV{NA_PROTO_API} && $ENV{NA_PROTO_API} ) {
    $apiproto .= "s";    # set HTTPS
  }

  if ( defined $ENV{NA_IP} ) {
    $storip = $ENV{NA_IP};
    $filer  = "$storip:$apiport";
  }
  else {
    error("naperf.pl: NetApp hostname/IP required.\n");
    exit(1);
  }

  if ( defined $ENV{NA_USER} ) {
    $user = "$ENV{NA_USER}";
  }
  else {
    error("naperf.pl: NetApp API username required.\n");
    exit(1);
  }

  if ( defined $ENV{NA_PASSWD} ) {
    $pass = unobscure_password("$ENV{NA_PASSWD}");
  }
  else {
    error("naperf.pl: NetApp API password required.\n");
    exit(1);
  }
} ## end else [ if ($read_ext_file) ]

my $cur_sampleid = $cntr_prefix . $netapp{timestamp};

$cacheFile = $tmpdir . "$storname.cache.file";

# Load cache from file
if ( -r $cacheFile ) {
  if ($debug) { message("Read cache file $cacheFile."); }
  $cache = retrieve($cacheFile);
}
else {
  my %cache_array;
  $cache = \%cache_array;
  store( \%netapp, $cacheFile );
}

if ( !$cache->{statscleaned} ) {
  $netapp{'statscleaned'} = $now;
  $cache->{'statscleaned'} = $now;
}
else {
  $netapp{'statscleaned'} = $cache->{'statscleaned'};
}
$data->{'Interval'} = 300;
if ( $cache->{timestamp} ) {
  $data->{'Interval'} = $now - $cache->{timestamp};
}

if ( $stormode eq "7MODE" ) {

  # 7-mode
  message("HOSTNAME: $hostname,  STATS PREFIX: $cntr_prefix,  NETAPP MODE: 7-mode");
  if ( $cache->{counters} ) {
    $netapp{counters} = $cache->{counters};
  }
  else {
    $netapp{counters} = getCounters;
  }

  #my $objects = "system ifnet";   # take all counters from this objects
  #$objects .= " " . join " ", map {"volume:*:" . $_} @volcounters;
  #$objects .= " " . join " ", map {"aggregate:*:" . $_} @aggcounters;
  #$objects .= " " . join " ", map {"disk:*:" . $_} @dskcounters;
  #$objects .= " " . join " ", map {"hostadapter:*:" . $_} @hacounters;
  #$objects .= " " . join " ", map {"lun:*:" . $_} @luncounters;

  my @wanted    = qw(system ifnet volume aggregate disk hostadapter lun processor cifs nfsv3 nfsv4 iscsi fcp qtree);    # objects we need
  my @available = ( keys %{ $netapp{counters} } );                                                                      # object we can use
  my @willuse   = ();
  my %count     = ();

  # intersection of wanted <-> available
  foreach my $element ( @wanted, @available ) { $count{$element}++ }
  foreach my $element ( keys %count ) {
    if ( $count{$element} > 1 ) {
      push @willuse, $element;
    }
  }
  my $objects = join " ", @willuse;                                                                                     # only use counters from these objects

  if ( ( $now - $cache->{statscleaned} ) < 86400 ) {
    my $collectStart = `$ssh $user\@$storip "stats start -I $cur_sampleid $objects" 2>&1`;
    chomp $collectStart;
    if ($collectStart) {
      &error("Statistics collection failed to start: $collectStart");
      exit(1);
    }
    else {
      &message("Statistics collection is being started for sample-id: $cur_sampleid");
    }
  } ## end if ( ( $now - $cache->...))

  # &message("Command: stats start -I $cur_sampleid $objects");

  if ( $cache->{timestamp} ) {
    my $old_sampleid   = $cntr_prefix . $cache->{timestamp};
    my @collectedData  = `$ssh $user\@$storip "stats stop -I $old_sampleid"`;
    my $collectedLines = scalar @collectedData;
    &message("Statistics collection is being stopped for sample-id: $old_sampleid ($collectedLines lines collected)");
    $statsFile = "stats-$storname-" . epoch2isotime( $cache->{timestamp}, $tzoffset );
    file_write( $tmpdir . $storname . "-" . &fileextTime($now) . ".cli.dump", @collectedData ) if $debug_full;

    # store(\@collectedData, $statsFile . ".txt");
    foreach my $line (@collectedData) {
      chomp $line;
      if ( length($line) > 5 ) {
        my @spline;
        if ( $line =~ "\.<[0-9]" ) {
          next;
        }
        if ( $line =~ "^disk" ) {
          if ( $line !~ "instance_name" ) {

            # @spline = $line =~ /([^:]+):(.*):(.*):(.*)/;
            @spline = $line =~ /([^:]+):((?>[^:]+:){9}[^:]+):([^:]+):(.*)/;
          }
          else {
            next;
          }
        } ## end if ( $line =~ "^disk" )
        else {

          #print $line . "\n";
          @spline = split( ":", $line, 4 );
        }
        if ( $spline[0] ne "StatisticsID" ) {
          $netapp{ $spline[0] }{ $spline[1] }{ $spline[2] } = $spline[3];
        }
        else {
          $netapp{ $spline[0] } = $spline[1];
        }
      } ## end if ( length($line) > 5)
    } ## end foreach my $line (@collectedData)

    #foreach my $diskid (keys %{ $netapp{disk} }) {
    #my $diskname = $netapp{disk}{$diskid}{display_name};
    #$netapp{disk}{$diskname} = delete $netapp{disk}{$diskid};
    #}
    foreach my $lun ( keys %{ $netapp{lun} } ) {
      my $lunname = $netapp{lun}{$lun}{display_name};
      $netapp{lun}{$lunname} = delete $netapp{lun}{$lun};
    }
    foreach my $vol ( keys %{ $netapp{volume} } ) {
      my $tmp = delete $netapp{volume}{$vol};
      $netapp{volume}{$vol} = [];
      push @{ $netapp{volume}{$vol} }, $tmp;
    }
    if ( ( $now - $cache->{statscleaned} ) >= 86400 ) {
      my $collectWipe = `$ssh $user\@$storip "stats stop -a" 2>&1`;
      chomp $collectWipe;
      $netapp{statscleaned} = $now;
      if ( $collectWipe =~ "No background stats" ) {
        &message("Samples cleanup not needed: $collectWipe");

        # exit(1);
      }
      else {
        &message("All background runs will be stopped.");
      }
      my $collectStart = `$ssh $user\@$storip "stats start -I $cur_sampleid $objects" 2>&1`;
      chomp $collectStart;
      if ($collectStart) {
        &error("Statistics collection failed to start: $collectStart");
        exit(1);
      }
      else {
        &message("Statistics collection is being started for sample-id: $cur_sampleid");
      }
    } ## end if ( ( $now - $cache->...))

    # store(\%coll, $statsFile . ".hsh");
    # file_write($statsFile . ".dump", Dumper \%netapp);
    # my $collectWipe = `ssh $storname "set -privilege advanced -confirmations off; statistics samples delete -sample-id $cache->{timestamp}"`;
    # &message($collectWipe);
  } ## end if ( $cache->{timestamp...})
  my @messages       = `$ssh $user\@$storip "rdfile /etc/messages" 2>&1`;
  my $collectedLines = scalar(@messages);
  &message("Getting event log... ($collectedLines lines collected)");

  # no warnings 'uninitialized';
  foreach my $msg (@messages) {
    my ( $date, $src, $event ) = ( $msg =~ /([^\[]*) \[([^\]]*)]: (.*)/ );
    $event = trim($event);
    $date  = str2time($date);

    # 7mode:wafl.scan.ownblocks.done:info
    if ($src) {
      my ( $node, $source, $severity ) = split ":", $src;
      if ( $severity && ( $severity ne "info" ) && ( $severity ne "notice" ) ) {
        $netapp{ems}{$date} = { node => $node, source => $source, severity => $severity, event => $event };
      }
    } ## end if ($src)
  } ## end foreach my $msg (@messages)

  ############## API part <<<<<<<<<<<<<<<<
  my $API = 'servlets/netapp.servlets.admin.XMLrequest_filer';
  my $url = "$apiproto://$filer/$API";
  &message("API URL: $url");

  my $xml_request = "<?xml version='1.0' encoding='utf-8'?>
  <!DOCTYPE netapp SYSTEM 'file:/etc/netapp_filer.dtd'>

  <netapp xmlns='http://www.netapp.com/filer/admin'><volume-list-info/><aggr-list-info/><disk-list-info/><lun-list-info/>
  <perf-object-get-instances>
    <objectname>processor</objectname>
  </perf-object-get-instances>
  <diagnosis-status-get/>
  </netapp>";

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( POST => $url );
  $request->content($xml_request);
  $request->authorization_basic( $user, $pass );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    error("Request error: $results->status_line");
    die;
  }

  my $xml = XMLin( $results->content );
  file_write( $tmpdir . $storname . "-" . &fileextTime($now) . ".api.dump", Dumper $xml ) if $debug_full == 2;
  my $vols        = $xml->{results}[0]{'volumes'}{'volume-info'};
  my $aggs        = $xml->{results}[1]{'aggregates'}{'aggr-info'};
  my $dsks        = $xml->{results}[2]{'disk-details'}{'disk-detail-info'};
  my $luns        = $xml->{results}[3]{'luns'}{'lun-info'};
  my $domain_busy = $xml->{results}[4]{'instances'}{'instance-data'};

  # Health Monitor Status Possible values: "ok" , "ok_with_suppressed" , "degraded" , "unreachable" , "unknown"
  my $status = $xml->{results}[5]{attributes}{'diagnosis-status'}{'status'};
  if ($status) {
    $netapp{health_status} = $status;
  }

  my %hluns;
  if ( ref($luns) eq 'HASH' ) {
    my $lunname = $luns->{path};
    $hluns{$lunname} = $luns;
  }
  else {
    foreach my $lun ( @{$luns} ) {

      #print Dumper $lun;
      my $lunname = $lun->{path};
      $hluns{$lunname} = $lun;
    } ## end foreach my $lun ( @{$luns} )
  } ## end else [ if ( ref($luns) eq 'HASH')]

  # print "XXXX hluns  " .  Dumper \%hluns;

  my %na;
  $na{volume}    = $vols;
  $na{aggregate} = $aggs;
  $na{disk}      = $dsks;
  $na{lun}       = \%hluns;

  #@{$netapp{volume}}{ keys %{$na{volume}} } = values %{$na{volume}};
  #@{$netapp{aggregate}}{ keys %{$na{aggregate}} } = values %{$na{aggregate}};
  #@{$netapp{disk}}{ keys %{$na{disk}} } = values %{$na{disk}};

  if ( exists $na{volume}{uuid} ) {
    my $key = $na{volume}{name};
    if ( !exists $netapp{volume}{$key} ) {
      $netapp{volume}{$key} = [];
    }
    foreach my $key1 ( keys %{ $na{volume} } ) {
      $netapp{volume}{$key}[0]{$key1} = trim( $na{volume}{$key1} );
    }
  } ## end if ( exists $na{volume...})
  else {
    foreach my $key ( keys %{ $na{volume} } ) {
      if ( !exists $netapp{volume}{$key} ) {
        $netapp{volume}{$key} = [];
      }
      foreach my $key1 ( keys %{ $na{volume}{$key} } ) {
        $netapp{volume}{$key}[0]{$key1} = trim( $na{volume}{$key}{$key1} );
      }
    } ## end foreach my $key ( keys %{ $na...})
  } ## end else [ if ( exists $na{volume...})]
      # UUID <--> volname
      #foreach my $vol ( keys %{ $netapp{volume} } ) {
      #  my $voluuid = $netapp{volume}{$vol}{uuid};
      #  $netapp{volume}{$voluuid} = delete $netapp{volume}{$vol};
      #  $netapp{volume}{$voluuid}{name} = $vol;
      #}

  if ( exists $na{aggregate}{uuid} ) {
    my $key = $na{aggregate}{name};
    foreach my $key1 ( keys %{ $na{aggregate} } ) {
      $netapp{aggregate}{$key}{$key1} = trim( $na{aggregate}{$key1} );
    }
  } ## end if ( exists $na{aggregate...})
  else {
    foreach my $key ( keys %{ $netapp{aggregate} } ) {
      foreach my $key1 ( keys %{ $na{aggregate}{$key} } ) {
        $netapp{aggregate}{$key}{$key1} = trim( $na{aggregate}{$key}{$key1} );
      }
    }
  } ## end else [ if ( exists $na{aggregate...})]
  foreach my $key ( keys %{ $netapp{disk} } ) {
    foreach my $key1 ( keys %{ $na{disk}{$key} } ) {
      $netapp{disk}{$key}{$key1} = trim( $na{disk}{$key}{$key1} );
    }
  }

  # print "XXXX netapp " .  Dumper $netapp{lun};
  # print "XXXX na     " .  Dumper $na{lun};
  foreach my $key ( keys %{ $netapp{lun} } ) {
    foreach my $key1 ( keys %{ $na{lun}{$key} } ) {
      $netapp{lun}{$key}{$key1} = trim( $na{lun}{$key}{$key1} );
    }
  }

  my %busy;
  my %helper;
  my @labels = qw(idle kahuna storage exempt raid raid_exempt target dnscache cifs wafl_exempt wafl_xcleaner sm_exempt cluster protocol nwk_exclusive nwk_exempt nwk_legacy hostOS ssan_exempt);
  if ( exists $domain_busy->{name} ) {
    my $cpu = $domain_busy->{name};
    @helper{@labels} = split ",", $domain_busy->{counters}{'counter-data'}{domain_busy}{value};
    foreach my $param ( keys %helper ) {
      $busy{domain_busy}{$param} += $helper{$param};
    }
    {
      no warnings 'uninitialized';
      $busy{elapsed_time} += $domain_busy->{counters}{'counter-data'}{processor_elapsed_time}{value};
    }
  } ## end if ( exists $domain_busy...)
  else {
    foreach my $cpu ( keys %{$domain_busy} ) {

      #print "$cpu: $domain_busy->{$cpu}{counters}{'counter-data'}{processor_elapsed_time}{value}\n";
      #$netapp{cpu}{$cpu}{elapsed_time} = $domain_busy->{$cpu}{counters}{'counter-data'}{processor_elapsed_time}{value};
      #$netapp{cpu}{$cpu}{domain_busy} = $domain_busy->{$cpu}{counters}{'counter-data'}{domain_busy}{value};

      @helper{@labels} = split ",", $domain_busy->{$cpu}{counters}{'counter-data'}{domain_busy}{value};
      foreach my $param ( keys %helper ) {
        $busy{domain_busy}{$param} += $helper{$param};
      }
      {
        no warnings 'uninitialized';
        $busy{elapsed_time} += $domain_busy->{$cpu}{counters}{'counter-data'}{processor_elapsed_time}{value};
      }
    } ## end foreach my $cpu ( keys %{$domain_busy...})
  } ## end else [ if ( exists $domain_busy...)]

  # print Dumper \%busy;
  # $netapp{cpu}{domain_busy} = $busy{domain_busy};
  $netapp{cpu} = \%busy;

  ############## API part >>>>>>>>>>>>>>>>
  #  file_write($statsFile . ".dump.txt", Dumper \%netapp) if $debug_full;

} ## end if ( $stormode eq "7MODE")
elsif ( $stormode eq "CMODE" ) {

  # C-mode
  ############## API part <<<<<<<<<<<<<<<<
  message("HOSTNAME: $hostname,  STATS PREFIX: $cntr_prefix,  NETAPP MODE: C-mode");

  my ( $xml, $results );
  if ($read_ext_file) {
    our $VAR1;
    my $apidump = file_read("/tmp/_api.dump");
    eval $apidump;
    $results = $VAR1;
  } ## end if ($read_ext_file)
  else {
    my $API = 'servlets/netapp.servlets.admin.XMLrequest_filer';
    my $url = "$apiproto://$filer/$API";
    &message("API URL: $url");

    my $volxml = "
    <volume-get-iter>
      <max-records>10000</max-records>
      <desired-attributes>
        <volume-attributes>
          <volume-id-attributes>
            <uuid />
            <name />
            <containing-aggregate-uuid />
            <containing-aggregate-name />
            <owning-vserver-uuid />
            <owning-vserver-name />
            <node />
            <reserve-used />
            <reserve />
            <disk-count />
            <instance-uuid />
            <style />
            <space-reserve-enabled />
            <raid-size />
          </volume-id-attributes>
          <volume-inode-attributes>
            <files-used />
            <files-total />
          </volume-inode-attributes>
          <volume-space-attributes>
            <filesystem-size />
            <size-total />
            <size-available />
            <size-used />
            <percentage-size-used />
            <percentage-snapshot-reserve />
          </volume-space-attributes>
          <volume-state-attributes>
            <state />
          </volume-state-attributes>
        </volume-attributes>
      </desired-attributes>
    </volume-get-iter>
    ";

    my $aggxml = "
    <aggr-get-iter>
      <max-records>10000</max-records>
      <desired-attributes>
        <aggr-attributes>
          <aggr-space-attributes>
            <size-total />
            <size-used />
          </aggr-space-attributes>
          <aggregate-uuid />
        </aggr-attributes>
      </desired-attributes>
    </aggr-get-iter>
    ";

    my $dskxml = "
    <storage-disk-get-iter>
      <max-records>10000</max-records>
      <desired-attributes>
        <storage-disk-info>
          <disk-inventory-info>
            <disk-type />
            <model />
            <capacity-sectors />
            <serial-number />
            <vendor />
            <disk-uid />
          </disk-inventory-info>
          <disk-ownership-info>
          </disk-ownership-info>
          <disk-paths>
            <disk-path-info>
              <disk-port-name />
              <node />
            </disk-path-info>
          </disk-paths>
          <disk-raid-info />
        </storage-disk-info>
      </desired-attributes>
    </storage-disk-get-iter>
    ";

    my $lunxml = "
    <lun-get-iter>
      <max-records>10000</max-records>
      <desired-attributes>
        <lun-info>
          <size />
          <size-used />
          <uuid />
          <serial-number />
        </lun-info>
      </desired-attributes>
    </lun-get-iter>
    ";

    my $emsxml = "
    <ems-message-get-iter>
      <max-records>100</max-records>
      <desired-attributes>
        <ems-message-info>
          <source />
          <time />
          <node />
          <event />
          <seq-num />
          <severity />
        </ems-message-info>
      </desired-attributes>
      <query>
        <ems-message-info>
          <severity>emergency | alert | critical | error</severity>
        </ems-message-info>
      </query>
    </ems-message-get-iter>
    ";

    my $xml_request = "<?xml version='1.0' encoding='utf-8'?>
    <!DOCTYPE netapp SYSTEM 'file:/etc/netapp_filer.dtd'>
    <netapp xmlns='http://www.netapp.com/filer/admin' version='1.10'>

    <diagnosis-status-get />
    $volxml
    $aggxml
    $dskxml
    $lunxml
    $emsxml

    </netapp>";

    my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

    my $request = HTTP::Request->new( POST => $url );
    $request->content($xml_request);
    $request->authorization_basic( $user, $pass );

    # print Dumper $request;
    $results = $agent->request($request);

    # print Dumper $results;

    if ( !$results->is_success ) {
      error("Request error: $results->status_line");
      die;
    }
  } ## end else [ if ($read_ext_file) ]

  $xml = $results->content;

  my $failstr = '<num-records>0</num-records></results>';
  $xml =~ s/\Q$failstr\E//;

  eval { $xml = XMLin($xml); };

  if ($@) {
    use File::Temp qw(tempfile);
    error("XML parsing error: $@");
    my ( $fh, $filename ) = tempfile( SUFFIX => '.xml' );
    file_write( $filename, $results->content );
    message("Trying to recover XML with xmllint... Invalid XML saved to $filename");
    eval {
      my $linted = `xmllint --recover $filename`;
      $xml = XMLin($linted);
    };
    unlink($filename) unless $debug_full == 2;

    #cleanup
  } ## end if ($@)

  # my $xml = XMLin($results->content);
  file_write( $tmpdir . $storname . "-" . &fileextTime($now) . ".api.dump", Dumper $results ) if $debug_full == 2;

  # print Dumper $xml;
  #$xml = $xml->{results};
  my $vols = $xml->{results}[1]{'attributes-list'}{'volume-attributes'};
  my $aggs = $xml->{results}[2]{'attributes-list'}{'aggr-attributes'};
  my $dsks = $xml->{results}[3]{'attributes-list'}{'storage-disk-info'};
  my $luns = $xml->{results}[4]{'attributes-list'}{'lun-info'};
  my $ems  = $xml->{results}[5]{'attributes-list'}{'ems-message-info'};

  # Health Monitor Status Possible values: "ok" , "ok_with_suppressed" , "degraded" , "unreachable" , "unknown"
  my $status = $xml->{results}[0]{attributes}{'diagnosis-status'}{'status'};
  if ($status) {
    $netapp{health_status} = $status;
  }

  if ( ref($vols) eq 'ARRAY' ) {
    foreach my $item ( @{$vols} ) {

      # print Dumper $item;
      my $id = $item->{'volume-id-attributes'};
      my $sz = $item->{'volume-space-attributes'};
      my $st = $item->{'volume-state-attributes'};
      my $in = $item->{'volume-inode-attributes'};
      my %v;

      # print Dumper $id;
      if ( $id->{'name'} ) {
        $v{'uuid'}                      = $id->{'uuid'};
        $v{'containing-aggregate-uuid'} = $id->{'containing-aggregate-uuid'};
        $v{'containing-aggregate'}      = $id->{'containing-aggregate-name'};
        $v{'owning-vserver-uuid'}       = $id->{'owning-vserver-uuid'};
        $v{'owning-vserver-name'}       = $id->{'owning-vserver-name'};
        $v{'type'}                      = $id->{'style'};
        $v{'instance-uuid'}             = $id->{'instance-uuid'};
        $v{'state'}                     = $st->{'state'};
        $v{'size-total'}                = $sz->{'size-total'};
        $v{'size-available'}            = $sz->{'size-available'};
        $v{'filesystem-size'}           = $sz->{'filesystem-size'};
        $v{'percentage-used'}           = $sz->{'percentage-size-used'};
        $v{'size-used'}                 = $sz->{'size-used'};
        $v{'files-used'}                = $in->{'files-used'};
        $v{'files-total'}               = $in->{'files-total'};
        $v{'snapshot-percent-reserved'} = $sz->{'percentage-snapshot-reserve'};
        if ( !exists $netapp{volume}{ $id->{name} } ) {
          $netapp{volume}{ $id->{name} } = [];
        }
        push @{ $netapp{volume}{ $id->{name} } }, \%v;
      } ## end if ( $id->{'name'} )
    } ## end foreach my $item ( @{$vols})
  } ## end if ( ref($vols) eq 'ARRAY')

  if ( ref($aggs) eq 'ARRAY' ) {
    foreach my $item ( @{$aggs} ) {

      # print Dumper $item;
      my $sz = $item->{'aggr-space-attributes'};

      # print Dumper $id;
      if ( $item->{'aggregate-name'} ) {
        $netapp{aggregate}{ $item->{'aggregate-name'} }{size} = $sz->{'size-total'};
        $netapp{aggregate}{ $item->{'aggregate-name'} }{used} = $sz->{'size-used'};
        $netapp{aggregate}{ $item->{'aggregate-name'} }{uuid} = $item->{'aggregate-uuid'};
      }
    } ## end foreach my $item ( @{$aggs})
  } ## end if ( ref($aggs) eq 'ARRAY')

  if ( ref($dsks) eq 'ARRAY' ) {
    foreach my $item ( @{$dsks} ) {

      # print Dumper $item;
      my $inv      = $item->{'disk-inventory-info'};
      my $raid     = $item->{'disk-raid-info'};
      my $path     = $item->{'disk-disk-paths'};
      my $name     = $item->{'disk-name'};
      my $uid      = $item->{'disk-uid'};
      my $sectsize = $inv->{'bytes-per-sector'};
      $sectsize ||= 512;

      if ( $raid->{'container-type'} ne 'unassigned' ) {
        $netapp{disk}{$name}{'disk-model'}     = $inv->{'model'};
        $netapp{disk}{$name}{'disk-type'}      = $inv->{'disk-type'};
        $netapp{disk}{$name}{'vendor-id'}      = $inv->{'vendor'};
        $netapp{disk}{$name}{'serial-number'}  = $inv->{'serial-number'};
        $netapp{disk}{$name}{'disk-uid'}       = $uid;
        $netapp{disk}{$name}{'physical-space'} = $raid->{'physical-blocks'} * $sectsize;
        if ( !$raid->{'used-blocks'} ) {
          $raid->{'used-blocks'} = 0;
        }
        $netapp{disk}{$name}{'used-space'}          = $raid->{'used-blocks'} * $sectsize;
        $netapp{disk}{$name}{'used-blocks'}         = $raid->{'used-blocks'};
        $netapp{disk}{$name}{'effective-disk-type'} = $raid->{'effective-disk-type'};
        $netapp{disk}{$name}{'aggregate'}           = $raid->{'disk-aggregate-info'}{'aggregate-name'};
        $netapp{disk}{$name}{'raid-group'}          = $raid->{'disk-aggregate-info'}{'raid-group-name'};
        $netapp{disk}{$name}{'plex'}                = $raid->{'disk-aggregate-info'}{'plex-name'};
        $netapp{disk}{$name}{'node'}                = $raid->{'active-node-name'};
        $netapp{disk}{$name}{'pool'}                = $raid->{'spare-pool'};
      } ## end if ( $raid->{'container-type'...})
    } ## end foreach my $item ( @{$dsks})
  } ## end if ( ref($dsks) eq 'ARRAY')

  if ( ref($luns) eq 'ARRAY' ) {
    foreach my $item ( @{$luns} ) {

      # print Dumper $item;
      my $sz = $item->{'aggr-space-attributes'};

      # print Dumper $id;
      if ( $item->{'uuid'} ) {
        $netapp{lun}{ $item->{'path'} }{volume}          = $item->{'volume'};
        $netapp{lun}{ $item->{'path'} }{uuid}            = $item->{'uuid'};
        $netapp{lun}{ $item->{'path'} }{'serial-number'} = $item->{'serial-number'};
        $netapp{lun}{ $item->{'path'} }{'size'}          = $item->{'size'};
        $netapp{lun}{ $item->{'path'} }{'size-used'}     = $item->{'size-used'};
      } ## end if ( $item->{'uuid'} )
    } ## end foreach my $item ( @{$luns})
  } ## end if ( ref($luns) eq 'ARRAY')
  elsif ( ref($luns) eq 'HASH' ) {
    my $item = $luns;
    if ( $item->{'uuid'} ) {
      $netapp{lun}{ $item->{'path'} }{volume}          = $item->{'volume'};
      $netapp{lun}{ $item->{'path'} }{uuid}            = $item->{'uuid'};
      $netapp{lun}{ $item->{'path'} }{'serial-number'} = $item->{'serial-number'};
      $netapp{lun}{ $item->{'path'} }{'size'}          = $item->{'size'};
      $netapp{lun}{ $item->{'path'} }{'size-used'}     = $item->{'size-used'};
    } ## end if ( $item->{'uuid'} )
  } ## end elsif ( ref($luns) eq 'HASH')

  if ( ref($ems) eq 'ARRAY' ) {
    foreach my $item ( @{$ems} ) {

      # print Dumper $item;
      if ( $item->{'time'} ) {
        $netapp{ems}{ $item->{'time'} }{source}    = $item->{'source'};
        $netapp{ems}{ $item->{'time'} }{'seq-num'} = $item->{'seq-num'};
        $netapp{ems}{ $item->{'time'} }{severity}  = $item->{'severity'};
        $netapp{ems}{ $item->{'time'} }{node}      = $item->{'node'};
        $netapp{ems}{ $item->{'time'} }{event}     = $item->{'event'};
      } ## end if ( $item->{'time'} )
    } ## end foreach my $item ( @{$ems} )
  } ## end if ( ref($ems) eq 'ARRAY')
  elsif ( ref($ems) eq 'HASH' ) {
    my $item = $ems;
    if ( $item->{'time'} ) {
      $netapp{ems}{ $item->{'time'} }{source}    = $item->{'source'};
      $netapp{ems}{ $item->{'time'} }{'seq-num'} = $item->{'seq-num'};
      $netapp{ems}{ $item->{'time'} }{severity}  = $item->{'severity'};
      $netapp{ems}{ $item->{'time'} }{node}      = $item->{'node'};
      $netapp{ems}{ $item->{'time'} }{event}     = $item->{'event'};
    } ## end if ( $item->{'time'} )
  } ## end elsif ( ref($ems) eq 'HASH')

  # print Dumper $netapp{disk};
  # print Dumper $vols;
  # print Dumper $aggs;
  # print Dumper $ems;
  ############## API part >>>>>>>>>>>>>>>>

  my $objects  = "system|lun|disk|volume|aggregate|hostadapter|lif|system:node|processor|cifs:node|nfsv3:node|nfsv4:node|iscsi_lif:node|port|fcp_lif:node|qtree";
  my $counters = "read_ops|read_data|read_latency|write_ops|write_data|write_latency|other_ops|other_latency|total_ops|avg_latency|total_transfers|user_reads|user_writes|user_read_blocks|user_write_blocks|user_reads_hdd|user_reads_ssd|user_writes_hdd|user_writes_ssd|vserver_name|vserver_uuid|node_name|node_uuid";
  $counters .= "|system_id|system_model|serial_no|ontap_version";                                                                                                                        # system counters
  $counters .= "|user_read_latency|user_write_latency";                                                                                                                                  # disk counters
  $counters .= "|total_reads|total_writes|bytes_read|bytes_written|rscn_count";                                                                                                          # hostadapter counters
  $counters .= "|recv_data|recv_errors|recv_packet|sent_data|sent_errors|sent_packet";                                                                                                   # lif counters
  $counters .= "|cpu_busy|total_processor_busy|total_data|total_latency|domain_busy";                                                                                                    # node counters
  $counters .= "|processor_busy|sk_switches";                                                                                                                                            # processor
  $counters .= "|avg_read_latency|avg_write_latency";                                                                                                                                    # LUN
  $counters .= "|cifs_latency|cifs_ops|cifs_read_ops|cifs_write_ops|connections|established_sessions|open_files|path_based_ops|persistent_opens";                                        # CIFS
  $counters .= "|latency|link_avg_latency|nfsv3_ops|nfsv3_read_ops|nfsv3_read_throughput|nfsv3_throughput|nfsv3_write_ops|nfsv3_write_throughput|read_avg_latency|write_avg_latency";    # NFSv3
  $counters .= "|nfsv4_ops|nfs4_read_throughput|nfs4_throughput|nfs4_write_throughput|compound_total|null_total|read_total|write_total";                                                 # NFSv4
  $counters .= "|iscsi_read_ops|iscsi_write_ops";                                                                                                                                        # iSCSI
  $counters .= "|recv-data|recv-packets|recv-errors|sent-data|sent-packets|sent-errors|node_name";                                                                                       # PORT
  $counters .= "|parent_vol|nfs_ops";                                                                                       # Qtree
  if ( !$read_ext_file ) {

    if ( ( $now - $cache->{statscleaned} ) < $time_to_cleanup ) {
      my $collectStart = `$ssh $user\@$storip "set -privilege advanced -confirmations off; statistics start -object $objects -counter $counters -sample-id $cur_sampleid"`;

      #chomp $collectStart;
      $collectStart = trim($collectStart);
      &message($collectStart);
    } ## end if ( ( $now - $cache->...))
  } ## end if ( !$read_ext_file )
  if ( $cache->{timestamp} ) {
    my $old_sampleid = $cntr_prefix . $cache->{timestamp};
    my $collectStop;
    if ( !$read_ext_file ) {
      $collectStop = `$ssh $user\@$storip "set -privilege advanced -confirmations off; statistics stop -sample-id $old_sampleid"`;

      #chomp $collectStop;
      $collectStop = trim($collectStop);
    } ## end if ( !$read_ext_file )
    my @collectedData;
    my $collectedLines;
    if ($read_ext_file) {
      @collectedData = file_read("/tmp/_cli.dump");
    }
    else {
      my $lCounter = 0;
      do {
        @collectedData  = `$ssh $user\@$storip "set -privilege advanced -confirmations off -showallfields true -showseparator :: -units KB -rows 0; statistics show -object $objects -sample-id $old_sampleid"`;
        $collectedLines = scalar(@collectedData);
        if ( $collectedLines < 5 ) {
          sleep(5);
        }
        $lCounter++;
      } until ( $collectedLines > 4 || $lCounter > 11 );
      &message("$collectStop ($collectedLines lines collected [readings: $lCounter])");
      my $statsFile = "stats-$storname-" . epoch2isotime( $cache->{timestamp}, $tzoffset );
      file_write( $tmpdir . $storname . "-" . &fileextTime($now) . ".cli.dump", @collectedData ) if $debug_full;

      # file_write($statsFile . ".dump", Dumper \@collectedData);
      # store(\@collectedData, $statsFile . ".txt");
    } ## end else [ if ($read_ext_file) ]
    my ( %cpu, %ha, %ports, %vols, %qtrees );
    my $valfield = 4;
    foreach my $line (@collectedData) {
      chomp $line;
      if ( length($line) > 6 ) {
        if ( $line =~ "^Error: " ) {
          &error($line);
        }
        else {

          # print $line . "\n";
          my @spline = split( "::", $line );

          # if (lc $spline[0] ne "object") {
          if ( scalar @spline > 3 ) {
            if ( $spline[0] !~ "system" && ( $spline[2] eq "read_data" || $spline[2] eq "write_data" ) ) {
              ( my $rate = $spline[$valfield] ) =~ s/KB//;
              if ( $rate > $volume_data_limit ) {
                $baddata = "Bad NetApp data detected! Collection ignored... SUBSYS: $spline[0]  NAME: $spline[1]  METRIC: $spline[2]  VALUE: $spline[$valfield]";
                last;
              }
            } ## end if ( $spline[0] !~ "system"...)
            if ( $spline[0] eq "object" ) {
              if ( $spline[4] ne "value" ) {
                $valfield = 3;
              }
            }
            elsif ( $spline[0] eq "processor" ) {
              if ( !exists $cpu{ $spline[1] }{ $spline[2] } ) {
                $cpu{ $spline[1] }{ $spline[2] } = [];
              }
              push @{ $cpu{ $spline[1] }{ $spline[2] } }, $spline[$valfield];
            } ## end elsif ( $spline[0] eq "processor")
            elsif ( $spline[0] eq "hostadapter" ) {
              if ( !exists $ha{ $spline[1] }{ $spline[2] } ) {
                $ha{ $spline[1] }{ $spline[2] } = [];
              }
              push @{ $ha{ $spline[1] }{ $spline[2] } }, $spline[$valfield];
            } ## end elsif ( $spline[0] eq "hostadapter")
            elsif ( $spline[0] eq "port" ) {
              if ( !exists $ports{ $spline[1] }{ $spline[2] } ) {
                $ports{ $spline[1] }{ $spline[2] } = [];
              }
              push @{ $ports{ $spline[1] }{ $spline[2] } }, $spline[$valfield];
            } ## end elsif ( $spline[0] eq "port")
            elsif ( $spline[0] eq "volume" ) {
              if ( !exists $vols{ $spline[1] }{ $spline[2] } ) {
                $vols{ $spline[1] }{ $spline[2] } = [];
              }
              push @{ $vols{ $spline[1] }{ $spline[2] } }, $spline[$valfield];
            } ## end elsif ( $spline[0] eq "volume")
            elsif ( $spline[0] eq "qtree" ) {
              if ( !exists $qtrees{ $spline[1] }{ $spline[2] } ) {
                $qtrees{ $spline[1] }{ $spline[2] } = $spline[$valfield];
              }
            } ## end elsif ( $spline[0] eq "qtree")
            elsif ( $spline[0] eq "system:node" && $spline[2] eq "domain_busy" ) {
              my @indexes = split ",", $spline[ $valfield + 1 ];
              @{ $netapp{ $spline[0] }{ $spline[1] } }{@indexes} = split ",", $spline[ $valfield + 2 ];
            }
            else {
              $netapp{ $spline[0] }{ $spline[1] }{ $spline[2] } = $spline[$valfield];
            }
          } ## end if ( scalar @spline > ...)
        } ## end else [ if ( $line =~ "^Error: ")]
      } ## end if ( length($line) > 6)
    } ## end foreach my $line (@collectedData)
    if (%cpu) {
      foreach my $processor ( keys %cpu ) {
        my $idx = 0;
        foreach my $node ( @{ $cpu{$processor}{node_name} } ) {
          $netapp{processor}{$node}{$processor}{sk_switches}    = $cpu{$processor}{sk_switches}[$idx];
          $netapp{processor}{$node}{$processor}{processor_busy} = $cpu{$processor}{processor_busy}[$idx];
          $idx++;
        }
      } ## end foreach my $processor ( keys...)
    } ## end if (%cpu)
    if (%ha) {
      foreach my $hostadapter ( keys %ha ) {
        my $idx = 0;
        foreach my $node ( @{ $ha{$hostadapter}{node_name} } ) {
          $netapp{hostadapter}{$node}{$hostadapter}{bytes_read}    = $ha{$hostadapter}{bytes_read}[$idx];
          $netapp{hostadapter}{$node}{$hostadapter}{bytes_written} = $ha{$hostadapter}{bytes_written}[$idx];
          $netapp{hostadapter}{$node}{$hostadapter}{total_reads}   = $ha{$hostadapter}{total_reads}[$idx];
          $netapp{hostadapter}{$node}{$hostadapter}{total_writes}  = $ha{$hostadapter}{total_writes}[$idx];
          $netapp{hostadapter}{$node}{$hostadapter}{rscn_count}    = $ha{$hostadapter}{rscn_count}[$idx];
          $idx++;
        } ## end foreach my $node ( @{ $ha{$hostadapter...}})
      } ## end foreach my $hostadapter ( keys...)
    } ## end if (%ha)
    if (%ports) {
      foreach my $port ( keys %ports ) {
        my $idx = 0;
        foreach my $node ( keys %{ $netapp{'system:node'} } ) {
          $netapp{port}{$node}{$port}{'recv-data'}    = $ports{$port}{'recv-data'}[$idx];
          $netapp{port}{$node}{$port}{'recv-packets'} = $ports{$port}{'recv-packets'}[$idx];
          $netapp{port}{$node}{$port}{'recv-errors'}  = $ports{$port}{'recv-errors'}[$idx];
          $netapp{port}{$node}{$port}{'sent-data'}    = $ports{$port}{'sent-data'}[$idx];
          $netapp{port}{$node}{$port}{'sent-packets'} = $ports{$port}{'sent-packets'}[$idx];
          $netapp{port}{$node}{$port}{'sent-errors'}  = $ports{$port}{'sent-errors'}[$idx];
          $idx++;
        } ## end foreach my $node ( keys %{ ...})
      } ## end foreach my $port ( keys %ports)
    } ## end if (%ports)
    if (%vols) {
      foreach my $vol ( keys %vols ) {
        my $idx = 0;

        # my $dups = scalar @{ $vols{$vol}{total_ops} };
        my $dups = scalar @{ $vols{$vol}{ ( keys %{ $vols{$vol} } )[0] } };

        # print $vsrvuid . "\n";
        my $voluuid = "";
        foreach my $vv ( @{ $netapp{volume}{$vol} } ) {

          # print Dumper $vv;
          my $uuid = $vv->{uuid};
          $voluuid = $uuid;
          if ( $dups > 1 ) {

            # print Dumper $netapp{volume};
            # print Dumper $vol;
            $netapp{volume}{$vol}[$idx]{name} .= " ($netapp{volume}{$vol}[$idx]{'owning-vserver-name'})";
          } ## end if ( $dups > 1 )

          # if ($voluuid) {
          foreach my $metric ( keys %{ $vols{$vol} } ) {
            $netapp{volume}{$vol}[$idx]{$metric} = $vols{$vol}{$metric}[$idx];
          }

          # }
          $idx++;
        } ## end foreach my $vv ( @{ $netapp...})
      } ## end foreach my $vol ( keys %vols)
    } ## end if (%vols)

    if (%qtrees) {
      foreach my $qt ( keys %qtrees ) {
          $netapp{qtree}{ $qtrees{$qt}{'node_name'} }{$qt}{'volume'}    = $qtrees{$qt}{'parent_vol'};
          $netapp{qtree}{ $qtrees{$qt}{'node_name'} }{$qt}{'nfs_ops'} = $qtrees{$qt}{'nfs_ops'};
          $netapp{qtree}{ $qtrees{$qt}{'node_name'} }{$qt}{'cifs_ops'}  = $qtrees{$qt}{'cifs_ops'};
      } ## end foreach my $qt ( keys %qtrees)
    } ## end if (%qtrees)

    # print Dumper \%ports;
    if ( !$read_ext_file ) {
      if ( ( $now - $cache->{statscleaned} ) >= $time_to_cleanup ) {
        my $collectWipe = `$ssh $user\@$storip "set -privilege advanced -confirmations off; statistics samples delete -vserver * -sample-id $cntr_prefix*"`;

        #chomp $collectWipe;
        $collectWipe = crlf( trim($collectWipe) );
        &message($collectWipe);
        $netapp{statscleaned} = $now;
        my $collectStart = `$ssh $user\@$storip "set -privilege advanced -confirmations off; statistics start -object $objects -counter $counters -sample-id $cur_sampleid"`;

        #chomp $collectStart;
        $collectStart = trim($collectStart);
        &message($collectStart);
      } ## end if ( ( $now - $cache->...))
      else {
        my $collectWipe = `$ssh $user\@$storip "set -privilege advanced -confirmations off; statistics samples delete -vserver * -sample-id $old_sampleid"`;

        #chomp $collectWipe;
        $collectWipe = crlf( trim($collectWipe) );
        &message($collectWipe);
      } ## end else [ if ( ( $now - $cache->...))]
    } ## end if ( !$read_ext_file )
  } ## end if ( $cache->{timestamp...})
} ## end elsif ( $stormode eq "CMODE")

# print Dumper \%netapp;
#toCsv();
if ($baddata) {
  &error($baddata);
  exit 1;
}
&message("Storing latest data to $cacheFile");
store( \%netapp, $cacheFile );

&message("Starting to create conf & perf files...");
&writeStats("volume");
&writeStats("pool");
&writeStats("disk");
&writeStats("lun");

#if ($stormode eq "CMODE") {
&writeStats("port");

#}
#&writeStats("if");
&writeStats("cpu");
&writeConf("system");
&writeConf("volume");
&writeConf("pool");
&writeConf("disk");
&writeConf("lun");
&writeConf("ha");

#&writeConf("if");
&writeConf("cpu");
&writeConf("node");
&writeStats("node");
&writeStats("qtree");
&health_status();

#&message("Done.");
exit 0;
