#!/usr/bin/perl
use strict;
use warnings;
use POSIX;
use Time::Local;
use File::Basename;
use File::Glob ':glob';
use Data::Dumper;

#`. /home/stor2rrd/stor2rrd/etc/stor2rrd.cfg`;

###### RUN SCRIPT WITHOUT ARGUMENTS (PRINT ALL SERVER STORAGE AND SAN SWITCHES):
######
###### .  /home/stor2rrd/stor2rrd/etc/stor2rrd.cfg ; $PERL ./qs_volume.pl
######
###### RUN SCRIPT WITH ARGUMENTS (PRINT ALL STORAGE):
######
###### . /home/stor2rrd/stor2rrd/etc/stor2rrd.cfg ; $PERL ./qs_volume.pl VOLUME
######
###### RUN SCRIPT WITH ARGUMENTS (PRINT ONLY ONE SERVER STORAGE):
######
###### . /home/stor2rrd/stor2rrd/etc/stor2rrd.cfg ; $PERL ./qs_volume.pl VOLUME name_server
######
###### RUN SCRIPT WITH ARGUMENTS (PRINT ONLY ONE SAN SWITCH):
######
###### . /home/stor2rrd/stor2rrd/etc/stor2rrd.cfg ; $PERL ./qs_volume.pl SAN name_server
######
###### RUN SCRIPT WITH ARGUMENTS (PRINT ALL SWITCHES):
######
###### . /home/stor2rrd/stor2rrd/etc/stor2rrd.cfg ; $PERL ./qs_volume.pl SAN
######

######################### PRINT HEADER IF PROGRAM STARTED FROM BROWSER
if ( defined $ENV{FROMWEB} && $ENV{FROMWEB} eq "1" ) {
  print "Content-type: text/plain\n\n";
}

my $basedir         = $ENV{INPUTDIR};
my $wrkdir          = (<$basedir/data>);
my @wrkdir_all      = (<$wrkdir/*>);
my $ab              = ":";
my $server_list_cfg = "";
my $alias_list_cfg  = "$basedir/etc/alias.cfg";
my $storage         = "";
my $primar_alias    = "";
my $primar_alias_ci = "";
my %hash;
my %hash_brcd;
my $port_alpha;
my %hash_brcd_report;
my $switch_test = "";
my @list_of_cisco;
my @list_of_brcd;
my @cisco_switch_all_sorted;
my $identity;
################ check alias.cfg
my @alias_servers;

if ( -f $alias_list_cfg ) {
  open( FC, "< $alias_list_cfg" ) || error( "Cannot read $alias_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @alias_list_config = <FC>;
  close(FC);
  @alias_servers = grep {/^VOLUME:/} @alias_list_config;
  my @help_array;
  foreach my $line (@alias_servers) {
    $line =~ s/\\:/===double-col===/g;
    push( @help_array, "$line" );
  }
  @alias_servers = @help_array;
} ## end if ( -f $alias_list_cfg)

my @alias_switch;
if ( -f $alias_list_cfg ) {
  open( FC, "< $alias_list_cfg" ) || error( "Cannot read $alias_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @alias_list_config = <FC>;
  close(FC);
  @alias_switch = grep {/^SANPORT:/} @alias_list_config;
} ## end if ( -f $alias_list_cfg)

my @alias_cisco_switch;
if ( -f $alias_list_cfg ) {
  open( FC, "< $alias_list_cfg" ) || error( "Cannot read $alias_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @alias_list_config = <FC>;
  close(FC);
  @alias_cisco_switch = grep {/^SANPORT:/} @alias_list_config;
} ## end if ( -f $alias_list_cfg)
################ read arguments (ONLY ONE SERVER!)

my ( $param_argument, $param_server ) = @ARGV;
if (defined $param_server){
  $param_server =~ s/===space===/ /g;
}
if ( defined $param_argument ) {
  if ( $param_argument eq "VOLUME" ) {
    if ( defined $param_server ) {
      @wrkdir_all = ();
      $wrkdir_all[0] = "$wrkdir/$param_server";
      my $result = print_all_server();
    }
    else {
      print_all_server();
    }
  } ## end if ( $param_argument eq...)
  elsif ( $param_argument eq "SAN" ) {
    if ( defined $param_server ) {
      @wrkdir_all = ();
      $wrkdir_all[0] = "$wrkdir/$param_server";
      sanswitch_all();
    }
    else {
      sanswitch_all();
    }
  } ## end elsif ( $param_argument eq...)
  elsif ( $param_argument eq "DRIVE" ) {
    if ( defined $param_server ) {
      @wrkdir_all = ();
      $wrkdir_all[0] = "$wrkdir/$param_server";
      drive_all();
    }
    else {
      drive_all();
    }
  } ## end elsif ( $param_argument eq...)
} ## end if ( defined $param_argument)
else {
  print "Must be defined parametr DRIVE,VOLUME or SAN!\n";
}
################ get URL parameters (could be GET or POST) and put them into hash %PAR
sub getURLparams {
  my ( $buffer, @pairs, $pair, $name, $value, %PAR );
  if ( defined $ENV{'REQUEST_METHOD'} ) {
    if ( lc $ENV{'REQUEST_METHOD'} eq "post" ) {
      read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
    }
    else {
      $buffer = $ENV{'QUERY_STRING'};
    }

    # Split information into name/value pairs
    @pairs = split( /&/, $buffer );
    foreach $pair (@pairs) {
      ( $name, $value ) = split( /=/, $pair );
      $value =~ tr/+/ /;
      $value =~ s/%(..)/pack("C", hex($1))/eg;
      $PAR{$name} = $value;
    } ## end foreach $pair (@pairs)
    return \%PAR;
  } ## end if ( defined $ENV{'REQUEST_METHOD'...})
  else {
    return 0;
  }
} ## end sub getURLparams
my $params = getURLparams;

if ($params) {    ### if defined server in qs print only one server
  print "SERVER: $params->{server}\n";
  if ( defined $params->{server} ) {
    my ($server_all) = "$wrkdir/$params->{server}";
    print_one_server($server_all);
  }
  if ( !defined $params->{server} ) {    ### if not defined server in qs print all servers
    my ($server_all) = @wrkdir_all;
    print_all_server($server_all);
  }
} ## end if ($params)

if ( !defined $ENV{INPUTDIR} ) {
  die "Not defined INPUTDIR, probably not read etc/storr2rrd.cfg";
}

exit;

############# PRINT ALL VOLUMES
sub print_all_server {
  foreach my $server_all (@wrkdir_all) {
    my ($server) = basename($server_all);
    if ( -l $server_all ) { next; }
    chomp $server_all;
    my $server_list_cfg = "$wrkdir/$server/VOLUME/volumes.cfg";
    my @server_list;
    my @path_storages = (<$wrkdir/$server/*>);
    my $storage = storage_type( "$basedir/data", $server );
    if ( -f $server_list_cfg ) {    #### READ volumes.cfg
      open( FC, "< $server_list_cfg" ) || error( "Cannot read $server_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
      @server_list = <FC>;
      close(FC);
    }
    my @sorted_server_list = sort @server_list;
    my $name_to_menu       = "";
    if ( -f "$wrkdir/$server/VMAX" ) {
      $name_to_menu = "STORAGE GROUP";
    }
    else {
      $name_to_menu = "VOLUME";
    }
    foreach my $volume_all (@sorted_server_list) {
      my $volume_path = basename($volume_all);
      ( my $lpar, my $id ) = split( / :/, $volume_path );
      chomp $lpar;
      chomp $id;
      $id =~ s/;//g;
      $id =~ s/^\s+|\s+$//g;
      my $lpar_enc      = urlencode($lpar);
      my $alias_storage = "";

      if ( -f "$wrkdir/$server/VSPG" && !-f "$wrkdir/$server/VOLUME/$id.rrd" ) {
        next;
      }
      $lpar =~ s/\:/===double-col===/g;

      #print "VOLUME:$server:$lpar\n";
      my ($find_alias) = grep {/^VOLUME:$server:$lpar:/} @alias_servers;
      if ( defined $find_alias && $find_alias ne '' ) {
        $alias_storage = $find_alias;

        #print "$alias_storage\n";
        ( undef, my $server_a, my $lpar_a, my $alias ) = split( /:/, $alias_storage, 4 );
        chomp $alias;
        $alias =~ s/\:/===double-col===/g;
        if ( $server_a eq $server ) {
          my $server_encode = urlencode($server);
          print "L:" . "$server" . "$ab" . "$name_to_menu" . "$ab" . "$lpar" . "$ab" . "$alias" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server_encode&type=VOLUME&name=$lpar_enc&storage=$storage&item=lpar&gui=1&none=none::\n";
        }
        else {
          my $server_encode = urlencode($server);
          print "L:" . "$server" . "$ab" . "$name_to_menu" . "$ab" . "$lpar" . "$ab" . "$lpar" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server_encode&type=VOLUME&name=$lpar_enc&storage=$storage&item=lpar&gui=1&none=none::\n";
        }
      } ## end if ( defined $find_alias...)
      else {
        my $server_encode = urlencode($server);
        print "L:" . "$server" . "$ab" . "$name_to_menu" . "$ab" . "$lpar" . "$ab" . "$lpar" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server_encode&type=VOLUME&name=$lpar_enc&storage=$storage&item=lpar&gui=1&none=none::\n";
      }
    } ## end foreach my $volume_all (@sorted_server_list)
  } ## end foreach my $server_all (@wrkdir_all)
  foreach my $server_all_vol (@wrkdir_all) {
    my ($server) = basename($server_all_vol);
    if ( -f "$wrkdir/$server/3PAR" ) {
      if ( -l $server_all_vol ) { next; }
      chomp $server_all_vol;
      my $server_list_cfg = "$wrkdir/$server/VVOL/volumes.cfg";
      my @server_list;
      my @path_storages = (<$wrkdir/$server/VVOL/*>);
      my $storage = storage_type( "$basedir/data", $server );
      if ( -f $server_list_cfg ) {    #### READ volumes.cfg
        open( FC, "< $server_list_cfg" ) || error( "Cannot read $server_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
        @server_list = <FC>;
        close(FC);
      }
      my @sorted_server_list = sort @server_list;
      my $name_to_menu       = "";
      foreach my $volume_all (@path_storages) {
        if ( $volume_all =~ /\.rrd/ ) {
          my $volume_path = basename($volume_all);
          my $volume_a    = $volume_path;
          $volume_a =~ s/-P\d//g;
          $volume_a =~ s/\.rrd//g;
          $volume_path =~ s/\.rrd//g;
          my $alias_storage = "";
          my ($find_vol) = grep {/$volume_a;/} @server_list;
          if ($find_vol) {
            my ( $alias, $id ) = split( /:/, $find_vol );
            chomp( $alias, $id );
            $alias =~ s/^\s+|\s+$//g;
            $alias =~ s/\:/===double-col===/g;
            print "L:" . "$server" . "$ab" . "VVOL" . "$ab" . "$volume_path" . "$ab" . "$alias" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=VVOL&name=$alias&storage=$storage&item=lpar&gui=1&none=none::\n";
          } ## end if ($find_vol)
        } ## end if ( $volume_all =~ /\.rrd/)
      } ## end foreach my $volume_all (@path_storages)
    } ## end if ( -f "$wrkdir/$server/3PAR")
  } ## end foreach my $server_all_vol ...
} ## end sub print_all_server

############### DRIVE ALL
sub drive_all {
  foreach my $drive_all (@wrkdir_all) {
    my ($server)        = basename($drive_all);
    my @drive_storages  = (<$wrkdir/$server/DRIVE/*>);
    my $drive_list_cfg1 = "$wrkdir/$server/drive.cfg";
    if ( !-f $drive_list_cfg1 ) {
      $drive_list_cfg1 = "$wrkdir/$server/DRIVE/drive.cfg";
      if ( !-f $drive_list_cfg1 ) {
        $drive_list_cfg1 = "no";
      }
    } ## end if ( !-f $drive_list_cfg1)
    my $storage = storage_type( "$basedir/data", $server );
    my @drive_list;
    if ( -f $drive_list_cfg1 ) {    #### READ volumes.cfg
      open( FC, "< $drive_list_cfg1" ) || error( "Cannot read $drive_list_cfg1: $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
      @drive_list = <FC>;
      close(FC);
    }

    #print "S:" . "$server" . "$ab" . "DRIVE" . "$ab" . "$ab" . "$ab" ."::\n";
    #print "A:" . "$server" . "$ab" . "DRIVE" . "$ab" . "io" . "$ab" . "IO" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=DRIVE&name=io&storage=$storage&item=sum&gui=1&none=none::\n";
    #print "A:" . "$server" . "$ab" . "DRIVE" . "$ab" . "data" . "$ab" . "Data" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=DRIVE&name=data&storage=$storage&item=sum&gui=1&none=none::\n";
    #print "A:" . "$server" . "$ab" . "DRIVE" . "$ab" . "resp" . "$ab" . "Response time" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=DRIVE&name=resp&storage=$storage&item=sum&gui=1&none=none::\n";
    foreach my $drive_path (@drive_storages) {
      if ( $drive_path =~ /\.rrd/ ) {
        my $server_encode = urlencode($server);
        my $drive = basename($drive_path);
        $drive =~ s/\.rrd//;
        if ( $drive_list_cfg1 ne "no" ) {
          my ($alias_line) = grep {/^$drive,/} @drive_list;
          if ($alias_line) {
            my ( undef, $alias ) = split( ",", $alias_line );
            chomp($alias);
            $drive =~ s/:/===double-col===/g;
            $alias =~ s/:/===double-col===/g;
            print "L:" . "$server" . "$ab" . "DRIVE" . "$ab" . "$drive" . "$ab" . "$alias" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server_encode&type=DRIVE&name=$drive&storage=$storage&item=lpar&gui=1&none=none::\n";
          } ## end if ($alias_line)
          else {
            print "L:" . "$server" . "$ab" . "DRIVE" . "$ab" . "$drive" . "$ab" . "$drive" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server_encode&type=DRIVE&name=$drive&storage=$storage&item=lpar&gui=1&none=none::\n";
          }
        } ## end if ( $drive_list_cfg1 ...)
        else {
          $drive =~ s/:/===double-col===/g;
          print "L:" . "$server" . "$ab" . "DRIVE" . "$ab" . "$drive" . "$ab" . "$drive" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server_encode&type=DRIVE&name=$drive&storage=$storage&item=lpar&gui=1&none=none::\n";
        }
      } ## end if ( $drive_path =~ /\.rrd/)
    } ## end foreach my $drive_path (@drive_storages)
  } ## end foreach my $drive_all (@wrkdir_all)
} ## end sub drive_all

############### SAN SWITCHES
sub sanswitch_all {
  foreach my $server_all (@wrkdir_all) {
    my $server         = basename($server_all);
    my @san_switch_all = glob "$wrkdir/$server/*";
    foreach my $unsorted_brcd (@san_switch_all) {
      my $unsort_brcd = basename($unsorted_brcd);
      if ( !defined $hash_brcd_report{$unsort_brcd} ) {
        $hash_brcd_report{$unsort_brcd} = "test";
      }
      else {
        next;
      }
      if ( $unsort_brcd =~ /\.rrd$/ ) {
        $unsort_brcd =~ s/\.rrd$//g;
        chomp $unsort_brcd;
        $unsort_brcd =~ s/\s+//g;

        #print "$server/$unsort_brcd\n";
        push @list_of_brcd, "$unsort_brcd\n";    ##### not sorted rrd files for ports
      } ## end if ( $unsort_brcd =~ /\.rrd$/)
    } ## end foreach my $unsorted_brcd (...)

    #my @san_brocade_all = sort { ( $a =~ /(\d+)/ )[0] <=> ( $b =~ /(\d+)/ )[0] } @list_of_brcd; ### sort rrd files for port
    #print "@list_of_brcd\n";
    my $ports_list_cfg = "$wrkdir/$server/PORTS.cfg";
    my @switch_ports;
    my @san_brocade_all;
    my $phys_port;
    my $phys_port_cfg = "$wrkdir/$server/PHYS_PORTS.cfg";
    my @grep_phys_port;
    my @grep_phys_slots;

    if ( -f $ports_list_cfg ) {
      open( FC, "< $ports_list_cfg" ) || error( "Cannot read $ports_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @switch_ports = <FC>;
      close(FC);

      #print "@switch_ports\n";
      #@switch_ports_cfg = grep {/^$switch :/} @switch_ports;
    } ## end if ( -f $ports_list_cfg)
    if ( -f $phys_port_cfg ) {
      open( FC, "< $phys_port_cfg" ) || error( "Cannot read $phys_port_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @grep_phys_port = <FC>;
      close(FC);
      foreach (@grep_phys_port) {
        chomp $_;
        my ( $name, $num ) = split( /:/, $_ );
        $name =~ s/^\s+|\s+$//g;
        $num =~ s/^\s+|\s+$//g;
        if ( $num =~ m/^[0-9]+$/ ) {
          $hash{test}{'XORUX-1'}{$num} = $num;
        }
        if ( $num =~ m/^[0-9]+-[0-9]+$/ ) {
          my ( $first_num, $second_num ) = split( /-/, $num );
          $hash{test}{'XORUX-2'}{$first_num}{$second_num} = $second_num;
        }
        if ( $num =~ m/^[0-9]+-[0-9]+-[0-9]+$/ ) {
          my ( $first_num, $second_num, $third_num ) = split( /-/, $num );
          $hash{test}{'XORUX-3'}{$first_num}{$second_num}{$third_num} = $third_num;
        }
      } ## end foreach (@grep_phys_port)

      #print Dumper \%hash;
      foreach my $numbers ( keys( %{ $hash{test} } ) ) {
        if ( $numbers eq "XORUX-1" ) {
          foreach my $f_num ( sort { $a <=> $b } keys( %{ $hash{test}{$numbers} } ) ) {
            my $port_name = $f_num;
            push( @grep_phys_slots, "$port_name\n" );
          }
        } ## end if ( $numbers eq "XORUX-1")
        if ( $numbers eq "XORUX-2" ) {
          foreach my $f_num ( sort { $a <=> $b } keys( %{ $hash{test}{$numbers} } ) ) {
            foreach my $s_num ( sort { $a <=> $b } keys( %{ $hash{test}{$numbers}{$f_num} } ) ) {
              my $port_name = $f_num . "-" . $s_num;
              push( @grep_phys_slots, "$port_name\n" );
            }
          } ## end foreach my $f_num ( sort { ...})
        } ## end if ( $numbers eq "XORUX-2")
        if ( $numbers eq "XORUX-3" ) {
          foreach my $f_num ( sort { $a <=> $b } keys( %{ $hash{test}{$numbers} } ) ) {
            foreach my $s_num ( sort { $a <=> $b } keys( %{ $hash{test}{$numbers}{$f_num} } ) ) {
              foreach my $t_num ( sort { $a <=> $b } keys( %{ $hash{test}{$numbers}{$f_num}{$s_num} } ) ) {
                my $port_name = $f_num . "-" . $s_num . "-" . $t_num;

                #print "$port_name\n";
                push( @grep_phys_slots, "$port_name\n" );
              } ## end foreach my $t_num ( sort { ...})
            } ## end foreach my $s_num ( sort { ...})
          } ## end foreach my $f_num ( sort { ...})
        } ## end if ( $numbers eq "XORUX-3")
      } ## end foreach my $numbers ( keys(...))
      foreach my $phys_slots (@grep_phys_slots) {
        chomp $phys_slots;

        #print "$phys_slots\n";
        my ($slot) = grep {/: $phys_slots$/} @grep_phys_port;
        my ( $switch_a, $slot_a ) = split( / : /, $slot );
        push( @san_brocade_all, $switch_a );
      } ## end foreach my $phys_slots (@grep_phys_slots)
    } ## end if ( -f $phys_port_cfg)
    else {
      @san_brocade_all = sort { ( $a =~ /(\d+)/ )[0] <=> ( $b =~ /(\d+)/ )[0] } @list_of_brcd;    ### sort rrd files for port
    }
    my $unused_txt = "$wrkdir/$server/unused_ports.txt";
    my $unused_port;
    my @unused_ports;
    if ( -f $unused_txt ) {
      open( FC, "< $unused_txt" ) || error( "Cannot read $unused_txt: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @unused_ports = <FC>;
      close(FC);
    }
    if ( -f "$server_all/SAN-BRCD" ) {
      my @switch_ports_a = "";

      #print "@san_brocade_all\n";
      foreach my $san_switch (@san_brocade_all) {
        my $switch      = basename($san_switch);
        my $switch_port = $switch;
        $switch_port =~ s/^port//g;
        chomp( $switch_port, $switch );
        my ($ports_string) = grep {/^$switch :/} @switch_ports;
        my $port_alias;
        if ( defined $ports_string && $ports_string ne '' ) {
          ( my $port, my $port_path, $port_alias, my $unknown ) = split( /:/, $ports_string );
          chomp( $port, $port_path, $port_alias, $unknown );
          $port_alias =~ s/\s+//g;
        }
        else { next; }
        my ($switch_alias) = grep {/:$switch:/} @alias_switch;
        my $no_primar_alias = "$server:SANPORT:$switch:$switch_port";

        #print "@grep_phys_port\n";
        my ( $port_a, $phys_port_a );
        if ( -f $phys_port_cfg ) {
          ($phys_port) = grep {/^$switch :/} @grep_phys_port;
          ( $port_a, $phys_port_a ) = split( / : /, $phys_port );
          chomp $phys_port_a;
        }
        ($unused_port) = grep {/^$switch :/} @unused_ports;
        my $port_u = "";
        if ( defined $unused_port && $unused_port ne '' ) { ( $port_u, undef, undef ) = split( /:/, $unused_port ); }
        if ( defined $port_u && $port_u ne '' ) {
          $identity = 'U:';
        }
        else {
          $identity = 'L:';
        }
        foreach my $switch_ports (@alias_switch) {
          ( undef, my $server_a, my $port_b, my $alias_sw ) = split( /:/, $switch_ports );
          $port_b =~ s/^port//g;
          if ( $server_a eq $server ) {
            chomp $alias_sw;
            if ( defined $phys_port_cfg && $phys_port_cfg ne '' ) {

              #print "PHYSPORT:$phys_port_a - $port_a, alias: $port_b\n";
              #print "$phys_port_a --- $port_b\n";
              if ( $phys_port_a eq $port_b ) {
                $primar_alias = "$server_a:SANPORT:$switch:$switch_port";
                print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$switch" . "$ab" . "$alias_sw" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$switch&storage=SAN-BRCD&item=san&gui=1&none=none::\n";
              }
            } ## end if ( defined $phys_port_cfg...)
            else {
              if ( $switch eq $port_a or $switch_port eq $port_a ) {
                $primar_alias = "$server_a:SANPORT:$switch:$switch_port";
                print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$switch" . "$ab" . "$alias_sw" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$switch&storage=SAN-BRCD&item=san&gui=1&none=none::\n";
              }
            } ## end else [ if ( defined $phys_port_cfg...)]
          } ## end if ( $server_a eq $server)
        } ## end foreach my $switch_ports (@alias_switch)
        if ( $no_primar_alias eq $primar_alias ) { next; }
        if ( defined $phys_port && $phys_port ne '' ) {
          if ( defined $port_alias && $port_alias ne '' ) {
            print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$switch" . "$ab" . "$phys_port_a" . " [$port_alias]" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$switch&storage=SAN-BRCD&item=san&gui=1&none=none::\n";
          }
          else {
            print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$switch" . "$ab" . "$phys_port_a" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$switch&storage=SAN-BRCD&item=san&gui=1&none=none::\n";
          }
        } ## end if ( defined $phys_port...)
        else {
          if ( defined $port_alias && $port_alias ne '' ) {
            print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$switch" . "$ab" . "$switch_port" . " [$port_alias]" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$switch&storage=SAN-BRCD&item=san&gui=1&none=none::\n";
          }
          else {
            print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$switch" . "$ab" . "$switch_port" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$switch&storage=SAN-BRCD&item=san&gui=1&none=none::\n";
          }
        } ## end else [ if ( defined $phys_port...)]

        #}
      } ## end foreach my $san_switch (@san_brocade_all)
    } ## end if ( -f "$server_all/SAN-BRCD")

######### ALL CISCO SWITCH
    if ( -f "$server_all/SAN-CISCO" ) {    ### find SAN-CISCO file
      my @cisco_switch_all = glob "$wrkdir/$server/*";
      foreach my $unsorted_cisco (@cisco_switch_all) {
        my $unsort_port = basename($unsorted_cisco);
        if ( $unsort_port =~ /\.rrd$/ ) {
          $unsort_port =~ s/\.rrd$//g;
          $unsort_port =~ s/^(port[a-zA-Z]*)/$1 /g;
          push @list_of_cisco, "$unsort_port\n";
        }
      } ## end foreach my $unsorted_cisco ...
      foreach (@list_of_cisco) {
        chomp $_;
        my ( $name, $num ) = split( " ", $_ );
        if ( $num =~ m/^[0-9]+$/ ) {
          $hash{test}{$name}{'XORUX-1'}{$num} = $num;
        }
        if ( $num =~ m/^[0-9]+-[0-9]+$/ ) {
          my ( $first_num, $second_num ) = split( /-/, $num );
          $hash{test}{$name}{'XORUX-2'}{$first_num}{$second_num} = $second_num;
        }
        if ( $num =~ m/^[0-9]+-[0-9]+-[0-9]+$/ ) {
          my ( $first_num, $second_num, $third_num ) = split( /-/, $num );
          $hash{test}{$name}{'XORUX-3'}{$first_num}{$second_num}{$third_num} = $third_num;
        }
      } ## end foreach (@list_of_cisco)
      foreach my $port_n ( keys( %{ $hash{test} } ) ) {
        foreach my $numbers ( keys( %{ $hash{test}{$port_n} } ) ) {
          if ( $numbers eq "XORUX-1" ) {
            foreach my $f_num ( sort { $a <=> $b } keys( %{ $hash{test}{$port_n}{$numbers} } ) ) {

              #print "$port_n - $f_num\n";
              my $port_name = $port_n . $f_num;
              push( @cisco_switch_all_sorted, "$port_name\n" );
            } ## end foreach my $f_num ( sort { ...})
          } ## end if ( $numbers eq "XORUX-1")
          if ( $numbers eq "XORUX-2" ) {
            foreach my $f_num ( sort { $a <=> $b } keys( %{ $hash{test}{$port_n}{$numbers} } ) ) {
              foreach my $s_num ( sort { $a <=> $b } keys( %{ $hash{test}{$port_n}{$numbers}{$f_num} } ) ) {
                my $port_name = $port_n . $f_num . "-" . $s_num;
                push( @cisco_switch_all_sorted, "$port_name\n" );
              }
            } ## end foreach my $f_num ( sort { ...})
          } ## end if ( $numbers eq "XORUX-2")
          if ( $numbers eq "XORUX-3" ) {
            foreach my $f_num ( sort { $a <=> $b } keys( %{ $hash{test}{$port_n}{$numbers} } ) ) {
              foreach my $s_num ( sort { $a <=> $b } keys( %{ $hash{test}{$port_n}{$numbers}{$f_num} } ) ) {
                foreach my $t_num ( sort { $a <=> $b } keys( %{ $hash{test}{$port_n}{$numbers}{$f_num}{$s_num} } ) ) {
                  my $port_name = $port_n . $f_num . "-" . $s_num . "-" . $t_num;
                  push( @cisco_switch_all_sorted, "$port_name\n" );
                }
              } ## end foreach my $s_num ( sort { ...})
            } ## end foreach my $f_num ( sort { ...})
          } ## end if ( $numbers eq "XORUX-3")
        } ## end foreach my $numbers ( keys(...))
      } ## end foreach my $port_n ( keys( ...))
      my $cisco_list_cfg = glob "$wrkdir/$server/PORTS.cfg";    #### list of cisco FCS with secondary ALIAS, or without alias.
      my @cisco_ports_cfg;                                      ### list of cisco_ports
      my @cisco_ports;
      if ( -f $cisco_list_cfg ) {
        open( FC, "< $cisco_list_cfg" ) || error( "Cannot read $cisco_list_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
        @cisco_ports = <FC>;
        close(FC);
      }
      my $unused_txt = "$wrkdir/$server/unused_ports.txt";
      my $unused_port;
      my @unused_ports;
      if ( -f $unused_txt ) {
        open( FC, "< $unused_txt" ) || error( "Cannot read $unused_txt: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
        @unused_ports = <FC>;
        close(FC);
      }
      foreach my $cisco_switch (@cisco_switch_all_sorted) {
        my $cisco = basename($cisco_switch);
        chomp $cisco;

        #if ($cisco =~ /\.rrd$/){ ### find only rrd file
        #$cisco =~ s/\.rrd$//g; ### delete .rrd
        my $cisco_fcs = $cisco;
        $cisco_fcs =~ s/^port//g;    ### cisco_fcs is value without port, only FCS
        @cisco_ports_cfg = grep {/^$cisco :/} @cisco_ports;    ### find cisco name in ports.cfg
        my ($cisco_ports_string) = grep {/^$cisco :/} @cisco_ports_cfg;    ### grep cisco name from list of ports.cfg
        my ( $port_c, $port_path_c, $cisco_alias, $unknown_c ) = "";
        if ( defined $cisco_ports_string && $cisco_ports_string ne '' ) {
          ( $port_c, $port_path_c, $cisco_alias, $unknown_c ) = split( /:/, $cisco_ports_string );
          chomp( $port_c, $port_path_c, $cisco_alias, $unknown_c );
          $cisco_alias =~ s/\s+//g;
        }
        my ($cisco_alias_string) = grep {/:$cisco:/} @alias_cisco_switch;
        ($unused_port) = grep {/^$cisco :/} @unused_ports;
        my $port_u = "";

        if ( defined $unused_port && $unused_port ne '' ) { ( $port_u, undef, undef ) = split( /:/, $unused_port ); }
        if ( defined $port_u && $port_u ne '' ) {
          $identity = 'U:';
        }
        else {
          $identity = 'L:';
        }
        foreach my $cisco_ports (@alias_cisco_switch) {
          ( undef, my $server_b, my $port_b, my $alias_cisco ) = split( /:/, $cisco_ports );
          if ( $server_b eq $server ) {
            chomp $alias_cisco;
            if ( $cisco eq $port_b or $cisco_fcs eq $port_b ) {
              $primar_alias_ci = "$server_b:SANPORT:$cisco:$cisco_fcs";
              print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$cisco" . "$ab" . "$alias_cisco" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$cisco&storage=SAN-CISCO&item=san&gui=1&none=none::\n";
            }
          } ## end if ( $server_b eq $server)
        } ## end foreach my $cisco_ports (@alias_cisco_switch)
        my $no_primar_alias_ci = "$server:SANPORT:$cisco:$cisco_fcs";
        if ( $no_primar_alias_ci eq $primar_alias_ci ) { next; }
        if ( defined $cisco_alias && $cisco_alias ne '' ) {
          print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$cisco" . "$ab" . "$cisco_fcs" . " [$cisco_alias]" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$cisco&storage=SAN-CISCO&item=san&gui=1&none=none::\n";
        }
        else {
          print "$identity" . "$server" . "$ab" . "SANPORT" . "$ab" . "$cisco" . "$ab" . "$cisco_fcs" . "$ab" . "/stor2rrd-cgi/detail.sh?host=$server&type=SANPORT&name=$cisco&storage=SAN-CISCO&item=san&gui=1&none=none::\n";
        }

        #}
      } ## end foreach my $cisco_switch (@cisco_switch_all_sorted)
    } ## end if ( -f "$server_all/SAN-CISCO")
  } ## end foreach my $server_all (@wrkdir_all)
} ## end sub sanswitch_all

#print Dumper \%hash_brcd_report;

sub storage_type {
  my $wrkdir  = shift;
  my $host    = shift;
  my $st_type = "";

  if ( -f "$wrkdir/$host/SWIZ" ) {
    $st_type = "SWIZ";
  }
  if ( -f "$wrkdir/$host/XIV" ) {
    $st_type = "XIV";
  }
  if ( -f "$wrkdir/$host/DS8K" ) {
    $st_type = "DS8K";
  }
  if ( -f "$wrkdir/$host/DS5K" ) {
    $st_type = "DS5K";
  }
  if ( -f "$wrkdir/$host/SAN-BRCD" ) {
    $st_type = "SAN-BRCD";
  }
  if ( -f "$wrkdir/$host/SAN-CISCO" ) {
    $st_type = "SAN-CISCO";
  }
  if ( -f "$wrkdir/$host/HUS" ) {
    $st_type = "HUS";
  }
  if ( -f "$wrkdir/$host/3PAR" ) {
    $st_type = "3PAR";
  }
  if ( -f "$wrkdir/$host/NETAPP" ) {
    $st_type = "NETAPP";
  }
  if ( -f "$wrkdir/$host/VSPG" ) {
    $st_type = "VSPG";
  }
  if ( -f "$wrkdir/$host/VNXF" ) {
    $st_type = "VNXF";
  }
  if ( -f "$wrkdir/$host/VNX" ) {
    $st_type = "VNX";
  }
  if ( -f "$wrkdir/$host/UNITY" ) {
    $st_type = "UNITY";
  }
  if ( -f "$wrkdir/$host/VNXE" ) {
    $st_type = "VNXE";
  }
  if ( -f "$wrkdir/$host/DOTHILL" ) {
    $st_type = "DOTHILL";
  }
  if ( -f "$wrkdir/$host/VMAX" ) {
    $st_type = "VMAX";
  }
  if ( -f "$wrkdir/$host/COMPELLENT" ) {
    $st_type = "COMPELLENT";
  }
  if ( -f "$wrkdir/$host/OCEANSTOR" ) {
    $st_type = "OCEANSTOR";
  }
  if ( -f "$wrkdir/$host/PURE" ) {
    $st_type = "PURE";
  }
  return $st_type;
} ## end sub storage_type

############### URL ENCODING
sub urlencode {
  my $s = shift;

  # $s =~ s/ /+/g;
  $s =~ s/([^a-zA-Z0-9!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

  # $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;

  # $s =~ s/\+/ /g;
  return $s;
} ## end sub urldecode

