#!/usr/bin/perl

use strict;
use warnings;

use Data::Dumper;
use LWP::UserAgent;
use HTTP::Request;
use Xorux_lib;
use JSON qw(encode_json decode_json);

my $storage_name = $ENV{'STORAGE_NAME'}    ||= "";
my $univmax_ip   = $ENV{'EMC_VMAX_IP'}     ||= "";
my $univmax_port = $ENV{'EMC_VMAX_PORT'}   ||= "";
my $symmetrix_id = $ENV{'EMC_VMAX_ID'}     ||= "";
my $user         = $ENV{'EMC_VMAX_USER'}   ||= "";
my $pass         = $ENV{'EMC_VMAX_PASSWD'} ||= "";

$pass = Xorux_lib::unobscure_password($pass);

if ( !defined $storage_name || $storage_name eq '' ) {
  error( "Storage NAME must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( !defined $univmax_ip || $univmax_ip eq '' ) {
  error( "UNIVMAX IP or hostname must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( !defined $symmetrix_id || $symmetrix_id eq '' ) {
  error( "SYMMETRIX ID must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( !defined $user || $user eq '' ) {
  error( "USER must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}
if ( !defined $pass || $pass eq '' ) {
  error( "PASSWORD must be set! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}

my $basedir = $ENV{INPUTDIR};
my $webdir  = $ENV{WEBDIR};
my $wrkdir  = "$basedir/data";
my $tmpdir  = "$basedir/tmp";

my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
my $time_to_output_file = sprintf( "%4d%02d%02d_%02d%02d", $year + 1900, $month + 1, $day, $hour, $min );

my $perf_out = "$wrkdir/$storage_name/$storage_name\_vmaxperf_$time_to_output_file.out.tmp";
my $conf_out = "$wrkdir/$storage_name/$storage_name\_vmaxconf_$time_to_output_file.out.tmp";

my $url = "https://$univmax_ip:$univmax_port/univmax";

my %parameters;
my %keys;
my %data;

my @fedirector_metrics     = ( "READ_HIT_PER_SEC",           "HIT_PER_SEC",              "RESPONSE_TIME_WRITE",   "RESPONSE_TIME_READ",     "WRITE_HIT_PER_SEC",     "PERCENT_READ_HIT",      "PERCENT_WRITE_HIT", "PERCENT_HIT", "PERCENT_BUSY", "WRITES", "READS", "IO_RATE" );
my @bedirector_metrics     = ( "IO_RATE",                    "MB_RATE",                  "MB_READ_PER_SEC",       "MB_WRITE_PER_SEC",       "PERCENT_BUSY",          "READS",                 "WRITES" );
my @rdfdirector_metrics    = ( "IO_RATE",                    "MB_RATE",                  "MB_READ_PER_SEC",       "MB_WRITE_PER_SEC",       "PERCENT_BUSY",          "READS",                 "WRITES" );
my @array_metrics          = ( "BE_IOS_PER_SEC",             "BE_READS",                 "BE_WRITES",             "HA_MB_PER_SEC",          "HIT_PER_SEC",           "IO_RATE",               "MB_READ_PER_SEC", "MB_WRITE_PER_SEC", "PERCENT_HIT", "PERCENT_READ", "PERCENT_WRITE", "READS", "READ_HIT_PER_SEC", "RESPONSE_TIME_READ", "RESPONSE_TIME_WRITE", "TOTAL_CACHE_UTILIZATION", "WRITE_HIT_PER_SEC", "WRITES" );
my @storagegroup_metrics   = ( "DEVICE_CAPACITY",            "IO_RATE",                  "MB_READ_PER_SEC",       "MB_WRITE_PER_SEC",       "PERCENT_HIT",           "PERCENT_READ",          "PERCENT_READ_HIT", "PERCENT_WRITE", "PERCENT_WRITE_HIT", "READ_HIT_PER_SEC", "READS", "WRITE_HIT_PER_SEC", "TOTAL_HIT_PER_SEC", "WRITES", "RESPONSE_TIME" );
my @thinpool_metrics       = ( "ALLOCATED_POOL_CAPACITY_GB", "ENABLED_POOL_CAPACITY_GB", "PERCENT_USED_CAPACITY", "TOTAL_POOL_CAPACITY_GB", "USED_POOL_CAPACITY_GB", "BE_MB_READ_RATE",       "BE_MB_TRANSFERED_PER_SEC", "BE_MB_WRITE_RATE", "BE_PERCENT_READ", "BE_PERCENT_WRITE", "BE_READS", "BE_WRITES", "BE_RESPONSE_TIME", "BE_RESPONSE_TIME_READ", "BE_RESPONSE_TIME_WRITE" );
my @diskgroup_metrics      = ( "IO_RATE",                    "MB_RATE",                  "MB_READ_PER_SEC",       "MB_WRITE_PER_SEC",       "PERCENT_DISK_BUSY",     "PERCENT_FREE_CAPACITY", "PERCENT_USED_CAPACITY", "READS", "RESPONSE_TIME_READ", "RESPONSE_TIME", "RESPONSE_TIME_WRITE", "USED_DISK_CAPACITY_GB", "WRITES", "TOTAL_DISK_CAPACITY_GB" );
my @cachepartition_metrics = ( "IO_RATE",                    "MB_RATE",                  "PERCENT_BUSY",          "PERCENT_HIT",            "SIZE",                  "WP",                    "WP_LIMIT" );
my @fedirectorbyport_metrics = ( "IO_RATE", "MB_RATE", "AVG_REQ_SIZE_KB", "UTILIZATION" );
my @thintier_metrics = ( "ALLOCATED_POOL_CAPACITY_GB", "AVG_BE_DISK_TIME", "BE_MB_READ_RATE", "BE_MB_TRANSFERED_PER_SEC", "BE_MB_WRITE_RATE", "BE_READS", "BE_WRITES", "BE_RESPONSE_TIME", "BE_RESPONSE_TIME_READ", "BE_RESPONSE_TIME_WRITE", "ENABLED_POOL_CAPACITY_GB", "PERCENT_USED_CAPACITY", "TOTAL_POOL_CAPACITY_GB", "USED_POOL_CAPACITY_GB" );
my @storagegroupbytier_metrics = ( "CAPACITY", "ALLOCATED_TRACKS" );

# start time must be in 5 min sample rate
my $act_timestamp      = time();
my $start_timestamp    = int( $act_timestamp / 300 ) * 300;
my $start_timestamp_ms = $start_timestamp * 1000;             # from sec to ms

perf_array();
perf_storagegroup();
perf_thinpool();
perf_thintier();
perf_fedirector();
perf_bedirector();
perf_rdfdirector();
perf_diskgroup();
perf_cachepartition();
perf_fedirectorbyport();
perf_storagegroupbytier();

#
# print PERF file
#
print_perf();

#
# print CONF file
#
my $config_html   = "$wrkdir/$storage_name/config.html";
my $cfg_time_diff = file_time_diff($config_html);

if ( !-f $config_html || $cfg_time_diff > 3599 ) {
  if ( defined $data{'ThinTier'} && $data{'ThinTier'} ne '' ) {
    conf_tierbystoragegroup();
  }

  print_conf();
} ## end if ( !-f $config_html ...)

#print Dumper \%data;
#print Dumper(\%{$data{'BEDirector'}});
exit;

sub print_perf {
  open( PERF, ">$perf_out" ) || error( "Couldn't open file $perf_out $!" . __FILE__ . ":" . __LINE__ ) && exit;

  foreach my $section ( keys %data ) {
    chomp $section;

    #Array Level Statistics
    if ( $section eq "Array" && $data{'Array'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,PERCENT_WRITE,PERCENT_READ,HIT_PER_SEC,READ_HIT_PER_SEC,READS,HA_MB_PER_SEC,MB_WRITE_PER_SEC (kB),WRITE_HIT_PER_SEC,TOTAL_CACHE_UTILIZATION,PERCENT_HIT,BE_READS,MB_READ_PER_SEC (kB),RESPONSE_TIME_READ,WRITES,BE_IOS_PER_SEC,RESPONSE_TIME_WRITE,BE_WRITES,IO_RATE,data_rate\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'Array'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp               = "";
        my $PERCENT_WRITE           = "";
        my $PERCENT_READ            = "";
        my $HIT_PER_SEC             = "";
        my $READ_HIT_PER_SEC        = "";
        my $HA_MB_PER_SEC           = "";
        my $WRITE_HIT_PER_SEC       = "";
        my $TOTAL_CACHE_UTILIZATION = "";
        my $BE_READS                = "";
        my $BE_IOS_PER_SEC          = "";
        my $BE_WRITES               = "";

        # Data
        my $data_rate        = "";
        my $MB_READ_PER_SEC  = "";
        my $MB_WRITE_PER_SEC = "";

        # IO
        my $IO_RATE = "";
        my $READS   = "";
        my $WRITES  = "";

        # resp_
        my $RESPONSE_TIME_READ  = "";
        my $RESPONSE_TIME_WRITE = "";

        # cache
        my $PERCENT_HIT = "";

        if ( defined $data{'Array'}{$id}{'timestamp'} && isdigit( $data{'Array'}{$id}{'timestamp'} ) ) { $timestamp = $data{'Array'}{$id}{'timestamp'} / 1000; }
        my $act_timestamp = time();
        my $time_diff     = $act_timestamp - $timestamp;
        if ( $time_diff > 43200 ) {    # data collection is 12 hours old
          error( "ERROR: Data collection is stopped in Unisphere. Restart Unisphere might fix it. " . __FILE__ . ":" . __LINE__ );
        }
        if ( defined $data{'Array'}{$id}{'PERCENT_WRITE'}           && isdigit( $data{'Array'}{$id}{'PERCENT_WRITE'} ) )           { $PERCENT_WRITE           = $data{'Array'}{$id}{'PERCENT_WRITE'}; }
        if ( defined $data{'Array'}{$id}{'PERCENT_READ'}            && isdigit( $data{'Array'}{$id}{'PERCENT_READ'} ) )            { $PERCENT_READ            = $data{'Array'}{$id}{'PERCENT_READ'}; }
        if ( defined $data{'Array'}{$id}{'HIT_PER_SEC'}             && isdigit( $data{'Array'}{$id}{'HIT_PER_SEC'} ) )             { $HIT_PER_SEC             = $data{'Array'}{$id}{'HIT_PER_SEC'}; }
        if ( defined $data{'Array'}{$id}{'READ_HIT_PER_SEC'}        && isdigit( $data{'Array'}{$id}{'READ_HIT_PER_SEC'} ) )        { $READ_HIT_PER_SEC        = $data{'Array'}{$id}{'READ_HIT_PER_SEC'}; }
        if ( defined $data{'Array'}{$id}{'READS'}                   && isdigit( $data{'Array'}{$id}{'READS'} ) )                   { $READS                   = $data{'Array'}{$id}{'READS'}; }
        if ( defined $data{'Array'}{$id}{'HA_MB_PER_SEC'}           && isdigit( $data{'Array'}{$id}{'HA_MB_PER_SEC'} ) )           { $HA_MB_PER_SEC           = $data{'Array'}{$id}{'HA_MB_PER_SEC'}; }
        if ( defined $data{'Array'}{$id}{'WRITE_HIT_PER_SEC'}       && isdigit( $data{'Array'}{$id}{'WRITE_HIT_PER_SEC'} ) )       { $WRITE_HIT_PER_SEC       = $data{'Array'}{$id}{'WRITE_HIT_PER_SEC'}; }
        if ( defined $data{'Array'}{$id}{'TOTAL_CACHE_UTILIZATION'} && isdigit( $data{'Array'}{$id}{'TOTAL_CACHE_UTILIZATION'} ) ) { $TOTAL_CACHE_UTILIZATION = $data{'Array'}{$id}{'TOTAL_CACHE_UTILIZATION'}; }
        if ( defined $data{'Array'}{$id}{'PERCENT_HIT'}             && isdigit( $data{'Array'}{$id}{'PERCENT_HIT'} ) )             { $PERCENT_HIT             = $data{'Array'}{$id}{'PERCENT_HIT'}; }
        if ( defined $data{'Array'}{$id}{'BE_READS'}                && isdigit( $data{'Array'}{$id}{'BE_READS'} ) )                { $BE_READS                = $data{'Array'}{$id}{'BE_READS'}; }
        if ( defined $data{'Array'}{$id}{'RESPONSE_TIME_READ'}      && isdigit( $data{'Array'}{$id}{'RESPONSE_TIME_READ'} ) )      { $RESPONSE_TIME_READ      = $data{'Array'}{$id}{'RESPONSE_TIME_READ'}; }
        if ( defined $data{'Array'}{$id}{'WRITES'}                  && isdigit( $data{'Array'}{$id}{'WRITES'} ) )                  { $WRITES                  = $data{'Array'}{$id}{'WRITES'}; }
        if ( defined $data{'Array'}{$id}{'BE_IOS_PER_SEC'}          && isdigit( $data{'Array'}{$id}{'BE_IOS_PER_SEC'} ) )          { $BE_IOS_PER_SEC          = $data{'Array'}{$id}{'BE_IOS_PER_SEC'}; }
        if ( defined $data{'Array'}{$id}{'RESPONSE_TIME_WRITE'}     && isdigit( $data{'Array'}{$id}{'RESPONSE_TIME_WRITE'} ) )     { $RESPONSE_TIME_WRITE     = $data{'Array'}{$id}{'RESPONSE_TIME_WRITE'}; }
        if ( defined $data{'Array'}{$id}{'BE_WRITES'}               && isdigit( $data{'Array'}{$id}{'BE_WRITES'} ) )               { $BE_WRITES               = $data{'Array'}{$id}{'BE_WRITES'}; }
        if ( defined $data{'Array'}{$id}{'IO_RATE'}                 && isdigit( $data{'Array'}{$id}{'IO_RATE'} ) )                 { $IO_RATE                 = $data{'Array'}{$id}{'IO_RATE'}; }
        if ( defined $data{'Array'}{$id}{'MB_READ_PER_SEC'}         && isdigit( $data{'Array'}{$id}{'MB_READ_PER_SEC'} ) ) {
          $MB_READ_PER_SEC = $data{'Array'}{$id}{'MB_READ_PER_SEC'} * 1024;    # from MB to kB
        }
        if ( defined $data{'Array'}{$id}{'MB_WRITE_PER_SEC'} && isdigit( $data{'Array'}{$id}{'MB_WRITE_PER_SEC'} ) ) {
          $MB_WRITE_PER_SEC = $data{'Array'}{$id}{'MB_WRITE_PER_SEC'} * 1024;    # from MB to kB
        }
        if ( defined $MB_READ_PER_SEC && isdigit($MB_READ_PER_SEC) && defined $MB_WRITE_PER_SEC && isdigit($MB_WRITE_PER_SEC) ) {
          $data_rate = $MB_READ_PER_SEC + $MB_WRITE_PER_SEC;
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$id,$stat_time,$timestamp,$PERCENT_WRITE,$PERCENT_READ,$HIT_PER_SEC,$READ_HIT_PER_SEC,$READS,$HA_MB_PER_SEC,$MB_WRITE_PER_SEC,$WRITE_HIT_PER_SEC,$TOTAL_CACHE_UTILIZATION,$PERCENT_HIT,$BE_READS,$MB_READ_PER_SEC,$RESPONSE_TIME_READ,$WRITES,$BE_IOS_PER_SEC,$RESPONSE_TIME_WRITE,$BE_WRITES,$IO_RATE,$data_rate\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print PERF "\n";
    } ## end if ( $section eq "Array"...)

    #StorageGroup Level Statistics
    if ( $section eq "StorageGroup" && $data{'StorageGroup'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "SG_ID,time,timestamp,DEVICE_CAPACITY (GB),IO_RATE,MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_HIT,PERCENT_READ,PERCENT_READ_HIT,PERCENT_WRITE,PERCENT_WRITE_HIT,READ_HIT_PER_SEC,READS,WRITE_HIT_PER_SEC,TOTAL_HIT_PER_SEC,WRITES,RESPONSE_TIME,data_rate\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'StorageGroup'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp         = "";
        my $DEVICE_CAPACITY   = "";
        my $PERCENT_READ      = "";
        my $PERCENT_WRITE     = "";
        my $READ_HIT_PER_SEC  = "";
        my $WRITE_HIT_PER_SEC = "";
        my $TOTAL_HIT_PER_SEC = "";

        my $IO_RATE = "";
        my $READS   = "";
        my $WRITES  = "";

        my $data_rate        = "";
        my $MB_READ_PER_SEC  = "";
        my $MB_WRITE_PER_SEC = "";

        my $PERCENT_HIT       = "";
        my $PERCENT_READ_HIT  = "";
        my $PERCENT_WRITE_HIT = "";

        my $RESPONSE_TIME = "";

        if ( defined $data{'StorageGroup'}{$id}{'timestamp'}         && isdigit( $data{'StorageGroup'}{$id}{'timestamp'} ) )         { $timestamp         = $data{'StorageGroup'}{$id}{'timestamp'} / 1000; }
        if ( defined $data{'StorageGroup'}{$id}{'DEVICE_CAPACITY'}   && isdigit( $data{'StorageGroup'}{$id}{'DEVICE_CAPACITY'} ) )   { $DEVICE_CAPACITY   = $data{'StorageGroup'}{$id}{'DEVICE_CAPACITY'}; }
        if ( defined $data{'StorageGroup'}{$id}{'IO_RATE'}           && isdigit( $data{'StorageGroup'}{$id}{'IO_RATE'} ) )           { $IO_RATE           = $data{'StorageGroup'}{$id}{'IO_RATE'}; }
        if ( defined $data{'StorageGroup'}{$id}{'PERCENT_HIT'}       && isdigit( $data{'StorageGroup'}{$id}{'PERCENT_HIT'} ) )       { $PERCENT_HIT       = $data{'StorageGroup'}{$id}{'PERCENT_HIT'}; }
        if ( defined $data{'StorageGroup'}{$id}{'PERCENT_READ'}      && isdigit( $data{'StorageGroup'}{$id}{'PERCENT_READ'} ) )      { $PERCENT_READ      = $data{'StorageGroup'}{$id}{'PERCENT_READ'}; }
        if ( defined $data{'StorageGroup'}{$id}{'PERCENT_READ_HIT'}  && isdigit( $data{'StorageGroup'}{$id}{'PERCENT_READ_HIT'} ) )  { $PERCENT_READ_HIT  = $data{'StorageGroup'}{$id}{'PERCENT_READ_HIT'}; }
        if ( defined $data{'StorageGroup'}{$id}{'PERCENT_WRITE'}     && isdigit( $data{'StorageGroup'}{$id}{'PERCENT_WRITE'} ) )     { $PERCENT_WRITE     = $data{'StorageGroup'}{$id}{'PERCENT_WRITE'}; }
        if ( defined $data{'StorageGroup'}{$id}{'PERCENT_WRITE_HIT'} && isdigit( $data{'StorageGroup'}{$id}{'PERCENT_WRITE_HIT'} ) ) { $PERCENT_WRITE_HIT = $data{'StorageGroup'}{$id}{'PERCENT_WRITE_HIT'}; }
        if ( defined $data{'StorageGroup'}{$id}{'READ_HIT_PER_SEC'}  && isdigit( $data{'StorageGroup'}{$id}{'READ_HIT_PER_SEC'} ) )  { $READ_HIT_PER_SEC  = $data{'StorageGroup'}{$id}{'READ_HIT_PER_SEC'}; }
        if ( defined $data{'StorageGroup'}{$id}{'READS'}             && isdigit( $data{'StorageGroup'}{$id}{'READS'} ) )             { $READS             = $data{'StorageGroup'}{$id}{'READS'}; }
        if ( defined $data{'StorageGroup'}{$id}{'WRITE_HIT_PER_SEC'} && isdigit( $data{'StorageGroup'}{$id}{'WRITE_HIT_PER_SEC'} ) ) { $WRITE_HIT_PER_SEC = $data{'StorageGroup'}{$id}{'WRITE_HIT_PER_SEC'}; }
        if ( defined $data{'StorageGroup'}{$id}{'TOTAL_HIT_PER_SEC'} && isdigit( $data{'StorageGroup'}{$id}{'TOTAL_HIT_PER_SEC'} ) ) { $TOTAL_HIT_PER_SEC = $data{'StorageGroup'}{$id}{'TOTAL_HIT_PER_SEC'}; }
        if ( defined $data{'StorageGroup'}{$id}{'WRITES'}            && isdigit( $data{'StorageGroup'}{$id}{'WRITES'} ) )            { $WRITES            = $data{'StorageGroup'}{$id}{'WRITES'}; }
        if ( defined $data{'StorageGroup'}{$id}{'RESPONSE_TIME'}     && isdigit( $data{'StorageGroup'}{$id}{'RESPONSE_TIME'} ) )     { $RESPONSE_TIME     = $data{'StorageGroup'}{$id}{'RESPONSE_TIME'}; }

        if ( defined $data{'StorageGroup'}{$id}{'MB_READ_PER_SEC'} && isdigit( $data{'StorageGroup'}{$id}{'MB_READ_PER_SEC'} ) ) {
          $MB_READ_PER_SEC = $data{'StorageGroup'}{$id}{'MB_READ_PER_SEC'} * 1024;    # from MB to kB
        }
        if ( defined $data{'StorageGroup'}{$id}{'MB_WRITE_PER_SEC'} && isdigit( $data{'StorageGroup'}{$id}{'MB_WRITE_PER_SEC'} ) ) {
          $MB_WRITE_PER_SEC = $data{'StorageGroup'}{$id}{'MB_WRITE_PER_SEC'} * 1024;    # from MB to kB
        }
        if ( defined $MB_READ_PER_SEC && isdigit($MB_READ_PER_SEC) && defined $MB_WRITE_PER_SEC && isdigit($MB_WRITE_PER_SEC) ) {
          $data_rate = $MB_READ_PER_SEC + $MB_WRITE_PER_SEC;
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$id,$stat_time,$timestamp,$DEVICE_CAPACITY,$IO_RATE,$MB_READ_PER_SEC,$MB_WRITE_PER_SEC,$PERCENT_HIT,$PERCENT_READ,$PERCENT_READ_HIT,$PERCENT_WRITE,$PERCENT_WRITE_HIT,$READ_HIT_PER_SEC,$READS,$WRITE_HIT_PER_SEC,$TOTAL_HIT_PER_SEC,$WRITES,$RESPONSE_TIME,$data_rate\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print PERF "\n";
    } ## end if ( $section eq "StorageGroup"...)

    #ThinPool Level Statistics
    if ( $section eq "ThinPool" && $data{'ThinPool'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,ALLOCATED_POOL_CAPACITY_GB (TB),ENABLED_POOL_CAPACITY_GB (TB),PERCENT_USED_CAPACITY,TOTAL_POOL_CAPACITY_GB (TB),USED_POOL_CAPACITY_GB (TB),BE_MB_READ_RATE (kB),BE_MB_TRANSFERED_PER_SEC (kB),BE_MB_WRITE_RATE (kB),BE_PERCENT_READ,BE_PERCENT_WRITE,BE_READS,BE_WRITES,BE_RESPONSE_TIME,BE_RESPONSE_TIME_READ,BE_RESPONSE_TIME_WRITE,io_rate_b,\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'ThinPool'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp                  = "";
        my $ALLOCATED_POOL_CAPACITY_GB = "";
        my $ENABLED_POOL_CAPACITY_GB   = "";
        my $PERCENT_USED_CAPACITY      = "";
        my $TOTAL_POOL_CAPACITY_GB     = "";
        my $USED_POOL_CAPACITY_GB      = "";
        my $BE_MB_READ_RATE            = "";
        my $BE_MB_TRANSFERED_PER_SEC   = "";
        my $BE_MB_WRITE_RATE           = "";
        my $BE_PERCENT_READ            = "";
        my $BE_PERCENT_WRITE           = "";
        my $io_rate_b                  = "";
        my $BE_READS                   = "";
        my $BE_WRITES                  = "";
        my $BE_RESPONSE_TIME           = "";
        my $BE_RESPONSE_TIME_READ      = "";
        my $BE_RESPONSE_TIME_WRITE     = "";

        if ( defined $data{'ThinPool'}{$id}{'timestamp'}              && isdigit( $data{'ThinPool'}{$id}{'timestamp'} ) )              { $timestamp              = $data{'ThinPool'}{$id}{'timestamp'} / 1000; }
        if ( defined $data{'ThinPool'}{$id}{'PERCENT_USED_CAPACITY'}  && isdigit( $data{'ThinPool'}{$id}{'PERCENT_USED_CAPACITY'} ) )  { $PERCENT_USED_CAPACITY  = $data{'ThinPool'}{$id}{'PERCENT_USED_CAPACITY'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_PERCENT_READ'}        && isdigit( $data{'ThinPool'}{$id}{'BE_PERCENT_READ'} ) )        { $BE_PERCENT_READ        = $data{'ThinPool'}{$id}{'BE_PERCENT_READ'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_PERCENT_WRITE'}       && isdigit( $data{'ThinPool'}{$id}{'BE_PERCENT_WRITE'} ) )       { $BE_PERCENT_WRITE       = $data{'ThinPool'}{$id}{'BE_PERCENT_WRITE'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_READS'}               && isdigit( $data{'ThinPool'}{$id}{'BE_READS'} ) )               { $BE_READS               = $data{'ThinPool'}{$id}{'BE_READS'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_WRITES'}              && isdigit( $data{'ThinPool'}{$id}{'BE_WRITES'} ) )              { $BE_WRITES              = $data{'ThinPool'}{$id}{'BE_WRITES'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME'}       && isdigit( $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME'} ) )       { $BE_RESPONSE_TIME       = $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME_READ'}  && isdigit( $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME_READ'} ) )  { $BE_RESPONSE_TIME_READ  = $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME_READ'}; }
        if ( defined $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME_WRITE'} && isdigit( $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME_WRITE'} ) ) { $BE_RESPONSE_TIME_WRITE = $data{'ThinPool'}{$id}{'BE_RESPONSE_TIME_WRITE'}; }

        if ( defined $data{'ThinPool'}{$id}{'ALLOCATED_POOL_CAPACITY_GB'} && isdigit( $data{'ThinPool'}{$id}{'ALLOCATED_POOL_CAPACITY_GB'} ) ) {
          $ALLOCATED_POOL_CAPACITY_GB = $data{'ThinPool'}{$id}{'ALLOCATED_POOL_CAPACITY_GB'} / 1024;    # from GB to TB
        }
        if ( defined $data{'ThinPool'}{$id}{'ENABLED_POOL_CAPACITY_GB'} && isdigit( $data{'ThinPool'}{$id}{'ENABLED_POOL_CAPACITY_GB'} ) ) {
          $ENABLED_POOL_CAPACITY_GB = $data{'ThinPool'}{$id}{'ENABLED_POOL_CAPACITY_GB'} / 1024;        # from GB to TB
        }
        if ( defined $data{'ThinPool'}{$id}{'TOTAL_POOL_CAPACITY_GB'} && isdigit( $data{'ThinPool'}{$id}{'TOTAL_POOL_CAPACITY_GB'} ) ) {
          $TOTAL_POOL_CAPACITY_GB = $data{'ThinPool'}{$id}{'TOTAL_POOL_CAPACITY_GB'} / 1024;            # from GB to TB
        }
        if ( defined $data{'ThinPool'}{$id}{'USED_POOL_CAPACITY_GB'} && isdigit( $data{'ThinPool'}{$id}{'USED_POOL_CAPACITY_GB'} ) ) {
          $USED_POOL_CAPACITY_GB = $data{'ThinPool'}{$id}{'USED_POOL_CAPACITY_GB'} / 1024;              # from GB to TB
        }
        if ( defined $data{'ThinPool'}{$id}{'BE_MB_READ_RATE'} && isdigit( $data{'ThinPool'}{$id}{'BE_MB_READ_RATE'} ) ) {
          $BE_MB_READ_RATE = $data{'ThinPool'}{$id}{'BE_MB_READ_RATE'} * 1024;                          # from MB to kB
        }
        if ( defined $data{'ThinPool'}{$id}{'BE_MB_TRANSFERED_PER_SEC'} && isdigit( $data{'ThinPool'}{$id}{'BE_MB_TRANSFERED_PER_SEC'} ) ) {
          $BE_MB_TRANSFERED_PER_SEC = $data{'ThinPool'}{$id}{'BE_MB_TRANSFERED_PER_SEC'} * 1024;        # from MB to kB
        }
        if ( defined $data{'ThinPool'}{$id}{'BE_MB_WRITE_RATE'} && isdigit( $data{'ThinPool'}{$id}{'BE_MB_WRITE_RATE'} ) ) {
          $BE_MB_WRITE_RATE = $data{'ThinPool'}{$id}{'BE_MB_WRITE_RATE'} * 1024;                        # from MB to kB
        }
        if ( defined $BE_READS && isdigit($BE_READS) && defined $BE_WRITES && isdigit($BE_WRITES) ) {
          $io_rate_b = $BE_READS + $BE_WRITES;
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$id,$stat_time,$timestamp,$ALLOCATED_POOL_CAPACITY_GB,$ENABLED_POOL_CAPACITY_GB,$PERCENT_USED_CAPACITY,$TOTAL_POOL_CAPACITY_GB,$USED_POOL_CAPACITY_GB,$BE_MB_READ_RATE,$BE_MB_TRANSFERED_PER_SEC,$BE_MB_WRITE_RATE,$BE_PERCENT_READ,$BE_PERCENT_WRITE,$BE_READS,$BE_WRITES,$BE_RESPONSE_TIME,$BE_RESPONSE_TIME_READ,$BE_RESPONSE_TIME_WRITE,$io_rate_b,\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print PERF "\n";
    } ## end if ( $section eq "ThinPool"...)

    #DiskGroup Level Statistics
    if ( $section eq "DiskGroup" && $data{'DiskGroup'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,IO_RATE,MB_RATE (kB),MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_DISK_BUSY,PERCENT_FREE_CAPACITY,PERCENT_USED_CAPACITY,READS,RESPONSE_TIME_READ,RESPONSE_TIME,RESPONSE_TIME_WRITE,USED_DISK_CAPACITY_GB (TB),WRITES,TOTAL_DISK_CAPACITY_GB (TB)\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'DiskGroup'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp              = "";
        my $IO_RATE                = "";
        my $MB_RATE                = "";
        my $MB_READ_PER_SEC        = "";
        my $MB_WRITE_PER_SEC       = "";
        my $PERCENT_DISK_BUSY      = "";
        my $PERCENT_FREE_CAPACITY  = "";
        my $PERCENT_USED_CAPACITY  = "";
        my $READS                  = "";
        my $RESPONSE_TIME_READ     = "";
        my $RESPONSE_TIME          = "";
        my $RESPONSE_TIME_WRITE    = "";
        my $USED_DISK_CAPACITY_GB  = "";
        my $WRITES                 = "";
        my $TOTAL_DISK_CAPACITY_GB = "";

        if ( defined $data{'DiskGroup'}{$id}{'timestamp'}             && isdigit( $data{'DiskGroup'}{$id}{'timestamp'} ) )             { $timestamp             = $data{'DiskGroup'}{$id}{'timestamp'} / 1000; }
        if ( defined $data{'DiskGroup'}{$id}{'IO_RATE'}               && isdigit( $data{'DiskGroup'}{$id}{'IO_RATE'} ) )               { $IO_RATE               = $data{'DiskGroup'}{$id}{'IO_RATE'}; }
        if ( defined $data{'DiskGroup'}{$id}{'PERCENT_DISK_BUSY'}     && isdigit( $data{'DiskGroup'}{$id}{'PERCENT_DISK_BUSY'} ) )     { $PERCENT_DISK_BUSY     = $data{'DiskGroup'}{$id}{'PERCENT_DISK_BUSY'}; }
        if ( defined $data{'DiskGroup'}{$id}{'PERCENT_FREE_CAPACITY'} && isdigit( $data{'DiskGroup'}{$id}{'PERCENT_FREE_CAPACITY'} ) ) { $PERCENT_FREE_CAPACITY = $data{'DiskGroup'}{$id}{'PERCENT_FREE_CAPACITY'}; }
        if ( defined $data{'DiskGroup'}{$id}{'PERCENT_USED_CAPACITY'} && isdigit( $data{'DiskGroup'}{$id}{'PERCENT_USED_CAPACITY'} ) ) { $PERCENT_USED_CAPACITY = $data{'DiskGroup'}{$id}{'PERCENT_USED_CAPACITY'}; }
        if ( defined $data{'DiskGroup'}{$id}{'READS'}                 && isdigit( $data{'DiskGroup'}{$id}{'READS'} ) )                 { $READS                 = $data{'DiskGroup'}{$id}{'READS'}; }
        if ( defined $data{'DiskGroup'}{$id}{'RESPONSE_TIME_READ'}    && isdigit( $data{'DiskGroup'}{$id}{'RESPONSE_TIME_READ'} ) )    { $RESPONSE_TIME_READ    = $data{'DiskGroup'}{$id}{'RESPONSE_TIME_READ'}; }
        if ( defined $data{'DiskGroup'}{$id}{'RESPONSE_TIME'}         && isdigit( $data{'DiskGroup'}{$id}{'RESPONSE_TIME'} ) )         { $RESPONSE_TIME         = $data{'DiskGroup'}{$id}{'RESPONSE_TIME'}; }
        if ( defined $data{'DiskGroup'}{$id}{'RESPONSE_TIME_WRITE'}   && isdigit( $data{'DiskGroup'}{$id}{'RESPONSE_TIME_WRITE'} ) )   { $RESPONSE_TIME_WRITE   = $data{'DiskGroup'}{$id}{'RESPONSE_TIME_WRITE'}; }
        if ( defined $data{'DiskGroup'}{$id}{'WRITES'}                && isdigit( $data{'DiskGroup'}{$id}{'WRITES'} ) )                { $WRITES                = $data{'DiskGroup'}{$id}{'WRITES'}; }

        if ( defined $data{'DiskGroup'}{$id}{'TOTAL_DISK_CAPACITY_GB'} && isdigit( $data{'DiskGroup'}{$id}{'TOTAL_DISK_CAPACITY_GB'} ) ) {
          $TOTAL_DISK_CAPACITY_GB = $data{'DiskGroup'}{$id}{'TOTAL_DISK_CAPACITY_GB'} / 1024;    # from GB to TB
        }
        if ( defined $data{'DiskGroup'}{$id}{'USED_DISK_CAPACITY_GB'} && isdigit( $data{'DiskGroup'}{$id}{'USED_DISK_CAPACITY_GB'} ) ) {
          $USED_DISK_CAPACITY_GB = $data{'DiskGroup'}{$id}{'USED_DISK_CAPACITY_GB'} / 1024;      # from GB to TB
        }
        if ( defined $data{'DiskGroup'}{$id}{'MB_RATE'} && isdigit( $data{'DiskGroup'}{$id}{'MB_RATE'} ) ) {
          $MB_RATE = $data{'DiskGroup'}{$id}{'MB_RATE'} * 1024;                                  # from MB to kB
        }
        if ( defined $data{'DiskGroup'}{$id}{'MB_READ_PER_SEC'} && isdigit( $data{'DiskGroup'}{$id}{'MB_READ_PER_SEC'} ) ) {
          $MB_READ_PER_SEC = $data{'DiskGroup'}{$id}{'MB_READ_PER_SEC'} * 1024;                  # from MB to kB
        }
        if ( defined $data{'DiskGroup'}{$id}{'MB_WRITE_PER_SEC'} && isdigit( $data{'DiskGroup'}{$id}{'MB_WRITE_PER_SEC'} ) ) {
          $MB_WRITE_PER_SEC = $data{'DiskGroup'}{$id}{'MB_WRITE_PER_SEC'} * 1024;                # from MB to kB
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$id,$stat_time,$timestamp,$IO_RATE,$MB_RATE,$MB_READ_PER_SEC,$MB_WRITE_PER_SEC,$PERCENT_DISK_BUSY,$PERCENT_FREE_CAPACITY,$PERCENT_USED_CAPACITY,$READS,$RESPONSE_TIME_READ,$RESPONSE_TIME,$RESPONSE_TIME_WRITE,$USED_DISK_CAPACITY_GB,$WRITES,$TOTAL_DISK_CAPACITY_GB\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print PERF "\n";
    } ## end if ( $section eq "DiskGroup"...)

    #CachePartition Level Statistics
    if ( $section eq "CachePartition" && $data{'CachePartition'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,IO_RATE,MB_RATE (kB),PERCENT_BUSY,PERCENT_HIT,SIZE (%),WP,WP_LIMIT\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'CachePartition'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp    = "";
        my $IO_RATE      = "";
        my $MB_RATE      = "";
        my $PERCENT_BUSY = "";
        my $PERCENT_HIT  = "";
        my $SIZE         = "";
        my $WP           = "";
        my $WP_LIMIT     = "";

        if ( defined $data{'CachePartition'}{$id}{'timestamp'}    && isdigit( $data{'CachePartition'}{$id}{'timestamp'} ) )    { $timestamp    = $data{'CachePartition'}{$id}{'timestamp'} / 1000; }
        if ( defined $data{'CachePartition'}{$id}{'IO_RATE'}      && isdigit( $data{'CachePartition'}{$id}{'IO_RATE'} ) )      { $IO_RATE      = $data{'CachePartition'}{$id}{'IO_RATE'}; }
        if ( defined $data{'CachePartition'}{$id}{'PERCENT_BUSY'} && isdigit( $data{'CachePartition'}{$id}{'PERCENT_BUSY'} ) ) { $PERCENT_BUSY = $data{'CachePartition'}{$id}{'PERCENT_BUSY'}; }
        if ( defined $data{'CachePartition'}{$id}{'PERCENT_HIT'}  && isdigit( $data{'CachePartition'}{$id}{'PERCENT_HIT'} ) )  { $PERCENT_HIT  = $data{'CachePartition'}{$id}{'PERCENT_HIT'}; }
        if ( defined $data{'CachePartition'}{$id}{'SIZE'}         && isdigit( $data{'CachePartition'}{$id}{'SIZE'} ) )         { $SIZE         = $data{'CachePartition'}{$id}{'SIZE'}; }
        if ( defined $data{'CachePartition'}{$id}{'WP'}           && isdigit( $data{'CachePartition'}{$id}{'WP'} ) )           { $WP           = $data{'CachePartition'}{$id}{'WP'}; }
        if ( defined $data{'CachePartition'}{$id}{'WP_LIMIT'}     && isdigit( $data{'CachePartition'}{$id}{'WP_LIMIT'} ) )     { $WP_LIMIT     = $data{'CachePartition'}{$id}{'WP_LIMIT'}; }

        if ( defined $data{'CachePartition'}{$id}{'MB_RATE'} && isdigit( $data{'CachePartition'}{$id}{'MB_RATE'} ) ) {
          $MB_RATE = $data{'CachePartition'}{$id}{'MB_RATE'} * 1024;    # from MB to kB
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$id,$stat_time,$timestamp,$IO_RATE,$MB_RATE,$PERCENT_BUSY,$PERCENT_HIT,$SIZE,$WP,$WP_LIMIT\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print PERF "\n";
    } ## end if ( $section eq "CachePartition"...)

    #ThinTier Level Statistics
    if ( $section eq "ThinTier" && $data{'ThinTier'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,ALLOCATED_POOL_CAPACITY_GB (TB),AVG_BE_DISK_TIME,BE_MB_READ_RATE (kB),BE_MB_TRANSFERED_PER_SEC (kB),BE_MB_WRITE_RATE (kB),BE_READS,BE_WRITES,BE_RESPONSE_TIME,BE_RESPONSE_TIME_READ,BE_RESPONSE_TIME_WRITE,ENABLED_POOL_CAPACITY_GB (TB),PERCENT_USED_CAPACITY,TOTAL_POOL_CAPACITY_GB (TB),USED_POOL_CAPACITY_GB (TB),io_rate_b\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'ThinTier'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp                  = "";
        my $ALLOCATED_POOL_CAPACITY_GB = "";
        my $AVG_BE_DISK_TIME           = "";
        my $BE_MB_READ_RATE            = "";
        my $BE_MB_TRANSFERED_PER_SEC   = "";
        my $BE_MB_WRITE_RATE           = "";
        my $io_rate_b                  = "";
        my $BE_READS                   = "";
        my $BE_WRITES                  = "";
        my $BE_RESPONSE_TIME           = "";
        my $BE_RESPONSE_TIME_READ      = "";
        my $BE_RESPONSE_TIME_WRITE     = "";
        my $ENABLED_POOL_CAPACITY_GB   = "";
        my $PERCENT_USED_CAPACITY      = "";
        my $TOTAL_POOL_CAPACITY_GB     = "";
        my $USED_POOL_CAPACITY_GB      = "";

        if ( defined $data{'ThinTier'}{$id}{'timestamp'}              && isdigit( $data{'ThinTier'}{$id}{'timestamp'} ) )              { $timestamp              = $data{'ThinTier'}{$id}{'timestamp'} / 1000; }
        if ( defined $data{'ThinTier'}{$id}{'AVG_BE_DISK_TIME'}       && isdigit( $data{'ThinTier'}{$id}{'AVG_BE_DISK_TIME'} ) )       { $AVG_BE_DISK_TIME       = $data{'ThinTier'}{$id}{'AVG_BE_DISK_TIME'}; }
        if ( defined $data{'ThinTier'}{$id}{'BE_READS'}               && isdigit( $data{'ThinTier'}{$id}{'BE_READS'} ) )               { $BE_READS               = $data{'ThinTier'}{$id}{'BE_READS'}; }
        if ( defined $data{'ThinTier'}{$id}{'BE_WRITES'}              && isdigit( $data{'ThinTier'}{$id}{'BE_WRITES'} ) )              { $BE_WRITES              = $data{'ThinTier'}{$id}{'BE_WRITES'}; }
        if ( defined $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME'}       && isdigit( $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME'} ) )       { $BE_RESPONSE_TIME       = $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME'}; }
        if ( defined $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME_READ'}  && isdigit( $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME_READ'} ) )  { $BE_RESPONSE_TIME_READ  = $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME_READ'}; }
        if ( defined $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME_WRITE'} && isdigit( $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME_WRITE'} ) ) { $BE_RESPONSE_TIME_WRITE = $data{'ThinTier'}{$id}{'BE_RESPONSE_TIME_WRITE'}; }
        if ( defined $data{'ThinTier'}{$id}{'PERCENT_USED_CAPACITY'}  && isdigit( $data{'ThinTier'}{$id}{'PERCENT_USED_CAPACITY'} ) )  { $PERCENT_USED_CAPACITY  = $data{'ThinTier'}{$id}{'PERCENT_USED_CAPACITY'}; }

        if ( defined $data{'ThinTier'}{$id}{'ALLOCATED_POOL_CAPACITY_GB'} && isdigit( $data{'ThinTier'}{$id}{'ALLOCATED_POOL_CAPACITY_GB'} ) ) {
          $ALLOCATED_POOL_CAPACITY_GB = $data{'ThinTier'}{$id}{'ALLOCATED_POOL_CAPACITY_GB'} / 1024;    # from GB to TB
        }
        if ( defined $data{'ThinTier'}{$id}{'ENABLED_POOL_CAPACITY_GB'} && isdigit( $data{'ThinTier'}{$id}{'ENABLED_POOL_CAPACITY_GB'} ) ) {
          $ENABLED_POOL_CAPACITY_GB = $data{'ThinTier'}{$id}{'ENABLED_POOL_CAPACITY_GB'} / 1024;        # from GB to TB
        }
        if ( defined $data{'ThinTier'}{$id}{'TOTAL_POOL_CAPACITY_GB'} && isdigit( $data{'ThinTier'}{$id}{'TOTAL_POOL_CAPACITY_GB'} ) ) {
          $TOTAL_POOL_CAPACITY_GB = $data{'ThinTier'}{$id}{'TOTAL_POOL_CAPACITY_GB'} / 1024;            # from GB to TB
        }
        if ( defined $data{'ThinTier'}{$id}{'USED_POOL_CAPACITY_GB'} && isdigit( $data{'ThinTier'}{$id}{'USED_POOL_CAPACITY_GB'} ) ) {
          $USED_POOL_CAPACITY_GB = $data{'ThinTier'}{$id}{'USED_POOL_CAPACITY_GB'} / 1024;              # from GB to TB
        }
        if ( defined $data{'ThinTier'}{$id}{'BE_MB_READ_RATE'} && isdigit( $data{'ThinTier'}{$id}{'BE_MB_READ_RATE'} ) ) {
          $BE_MB_READ_RATE = $data{'ThinTier'}{$id}{'BE_MB_READ_RATE'} * 1024;                          # from MB to kB
        }
        if ( defined $data{'ThinTier'}{$id}{'BE_MB_TRANSFERED_PER_SEC'} && isdigit( $data{'ThinTier'}{$id}{'BE_MB_TRANSFERED_PER_SEC'} ) ) {
          $BE_MB_TRANSFERED_PER_SEC = $data{'ThinTier'}{$id}{'BE_MB_TRANSFERED_PER_SEC'} * 1024;        # from MB to kB
        }
        if ( defined $data{'ThinTier'}{$id}{'BE_MB_WRITE_RATE'} && isdigit( $data{'ThinTier'}{$id}{'BE_MB_WRITE_RATE'} ) ) {
          $BE_MB_WRITE_RATE = $data{'ThinTier'}{$id}{'BE_MB_WRITE_RATE'} * 1024;                        # from MB to kB
        }
        if ( defined $BE_READS && isdigit($BE_READS) && defined $BE_WRITES && isdigit($BE_WRITES) ) {
          $io_rate_b = $BE_READS + $BE_WRITES;
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$id,$stat_time,$timestamp,$ALLOCATED_POOL_CAPACITY_GB,$AVG_BE_DISK_TIME,$BE_MB_READ_RATE,$BE_MB_TRANSFERED_PER_SEC,$BE_MB_WRITE_RATE,$BE_READS,$BE_WRITES,$BE_RESPONSE_TIME,$BE_RESPONSE_TIME_READ,$BE_RESPONSE_TIME_WRITE,$ENABLED_POOL_CAPACITY_GB,$PERCENT_USED_CAPACITY,$TOTAL_POOL_CAPACITY_GB,$USED_POOL_CAPACITY_GB,$io_rate_b\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print PERF "\n";
    } ## end if ( $section eq "ThinTier"...)

    #FEDirectorByPort Level Statistics
    if ( $section eq "FEDirectorByPort" && $data{'FEDirectorByPort'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,Reads,Writes,IOs,MBRead (kB),MBWritten (kB),MBs (kB),AvgIOSize,SpeedGBs,MaxSpeedGBs,PercentBusy,ResponseTime,ReadResponseTime,WriteResponseTime\n";

      my $idx = 0;
      foreach my $f_dir_id ( keys( %{ $data{'FEDirectorByPort'} } ) ) {
        chomp $f_dir_id;
        if ( defined $data{'FEDirectorByPort'}{$f_dir_id} && $data{'FEDirectorByPort'}{$f_dir_id} ne '' ) {
          foreach my $port_id ( keys( %{ $data{'FEDirectorByPort'}{$f_dir_id} } ) ) {
            chomp $port_id;
            $idx++;

            my $timestamp       = "";
            my $IO_RATE         = "";
            my $MB_RATE         = "";
            my $AVG_REQ_SIZE_KB = "";
            my $UTILIZATION     = "";

            if ( defined $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'timestamp'}       && isdigit( $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'timestamp'} ) )       { $timestamp       = $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'timestamp'} / 1000; }
            if ( defined $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'IO_RATE'}         && isdigit( $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'IO_RATE'} ) )         { $IO_RATE         = $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'IO_RATE'}; }
            if ( defined $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'AVG_REQ_SIZE_KB'} && isdigit( $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'AVG_REQ_SIZE_KB'} ) ) { $AVG_REQ_SIZE_KB = $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'AVG_REQ_SIZE_KB'}; }
            if ( defined $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'UTILIZATION'}     && isdigit( $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'UTILIZATION'} ) )     { $UTILIZATION     = $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'UTILIZATION'}; }
            if ( defined $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'MB_RATE'}         && isdigit( $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'MB_RATE'} ) ) {
              $MB_RATE = $data{'FEDirectorByPort'}{$f_dir_id}{$port_id}{'MB_RATE'} * 1024;    # from MB to kB
            }

            my $stat_time = "";
            if ( defined $timestamp && isdigit($timestamp) ) {
              my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
              my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
              my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

              $stat_time = $date . "T$time";
            } ## end if ( defined $timestamp...)

            print PERF "$f_dir_id-$port_id,$stat_time,$timestamp,,,$IO_RATE,,,$MB_RATE,$AVG_REQ_SIZE_KB,,,$UTILIZATION,,,\n";
          } ## end foreach my $port_id ( keys(...))
        } ## end if ( defined $data{'FEDirectorByPort'...})
      } ## end foreach my $f_dir_id ( keys...)
      print PERF "\n";
    } ## end if ( $section eq "FEDirectorByPort"...)

    #FEDirector Level Statistics
    if ( $section eq "FEDirector" && $data{'FEDirector'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,READ_HIT_PER_SEC,HIT_PER_SEC,RESPONSE_TIME_WRITE,RESPONSE_TIME_READ,WRITE_HIT_PER_SEC,PERCENT_READ_HIT,PERCENT_WRITE_HIT,PERCENT_HIT,PERCENT_BUSY,WRITES,READS,IO_RATE\n";

      my $idx = 0;
      foreach my $f_dir_id ( keys( %{ $data{'FEDirector'} } ) ) {
        chomp $f_dir_id;
        $idx++;

        my $timestamp           = "";
        my $READ_HIT_PER_SEC    = "";
        my $HIT_PER_SEC         = "";
        my $RESPONSE_TIME_WRITE = "";
        my $RESPONSE_TIME_READ  = "";
        my $WRITE_HIT_PER_SEC   = "";
        my $PERCENT_READ_HIT    = "";
        my $PERCENT_WRITE_HIT   = "";
        my $PERCENT_HIT         = "";
        my $PERCENT_BUSY        = "";
        my $WRITES              = "";
        my $READS               = "";
        my $IO_RATE             = "";

        if ( defined $data{'FEDirector'}{$f_dir_id}{'timestamp'}           && isdigit( $data{'FEDirector'}{$f_dir_id}{'timestamp'} ) )           { $timestamp           = $data{'FEDirector'}{$f_dir_id}{'timestamp'} / 1000; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'READ_HIT_PER_SEC'}    && isdigit( $data{'FEDirector'}{$f_dir_id}{'READ_HIT_PER_SEC'} ) )    { $READ_HIT_PER_SEC    = $data{'FEDirector'}{$f_dir_id}{'READ_HIT_PER_SEC'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'HIT_PER_SEC'}         && isdigit( $data{'FEDirector'}{$f_dir_id}{'HIT_PER_SEC'} ) )         { $HIT_PER_SEC         = $data{'FEDirector'}{$f_dir_id}{'HIT_PER_SEC'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'RESPONSE_TIME_WRITE'} && isdigit( $data{'FEDirector'}{$f_dir_id}{'RESPONSE_TIME_WRITE'} ) ) { $RESPONSE_TIME_WRITE = $data{'FEDirector'}{$f_dir_id}{'RESPONSE_TIME_WRITE'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'RESPONSE_TIME_READ'}  && isdigit( $data{'FEDirector'}{$f_dir_id}{'RESPONSE_TIME_READ'} ) )  { $RESPONSE_TIME_READ  = $data{'FEDirector'}{$f_dir_id}{'RESPONSE_TIME_READ'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'WRITE_HIT_PER_SEC'}   && isdigit( $data{'FEDirector'}{$f_dir_id}{'WRITE_HIT_PER_SEC'} ) )   { $WRITE_HIT_PER_SEC   = $data{'FEDirector'}{$f_dir_id}{'WRITE_HIT_PER_SEC'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'PERCENT_READ_HIT'}    && isdigit( $data{'FEDirector'}{$f_dir_id}{'PERCENT_READ_HIT'} ) )    { $PERCENT_READ_HIT    = $data{'FEDirector'}{$f_dir_id}{'PERCENT_READ_HIT'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'PERCENT_WRITE_HIT'}   && isdigit( $data{'FEDirector'}{$f_dir_id}{'PERCENT_WRITE_HIT'} ) )   { $PERCENT_WRITE_HIT   = $data{'FEDirector'}{$f_dir_id}{'PERCENT_WRITE_HIT'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'PERCENT_HIT'}         && isdigit( $data{'FEDirector'}{$f_dir_id}{'PERCENT_HIT'} ) )         { $PERCENT_HIT         = $data{'FEDirector'}{$f_dir_id}{'PERCENT_HIT'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'PERCENT_BUSY'}        && isdigit( $data{'FEDirector'}{$f_dir_id}{'PERCENT_BUSY'} ) )        { $PERCENT_BUSY        = $data{'FEDirector'}{$f_dir_id}{'PERCENT_BUSY'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'WRITES'}              && isdigit( $data{'FEDirector'}{$f_dir_id}{'WRITES'} ) )              { $WRITES              = $data{'FEDirector'}{$f_dir_id}{'WRITES'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'READS'}               && isdigit( $data{'FEDirector'}{$f_dir_id}{'READS'} ) )               { $READS               = $data{'FEDirector'}{$f_dir_id}{'READS'}; }
        if ( defined $data{'FEDirector'}{$f_dir_id}{'IO_RATE'}             && isdigit( $data{'FEDirector'}{$f_dir_id}{'IO_RATE'} ) )             { $IO_RATE             = $data{'FEDirector'}{$f_dir_id}{'IO_RATE'}; }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$f_dir_id,$stat_time,$timestamp,$READ_HIT_PER_SEC,$HIT_PER_SEC,$RESPONSE_TIME_WRITE,$RESPONSE_TIME_READ,$WRITE_HIT_PER_SEC,$PERCENT_READ_HIT,$PERCENT_WRITE_HIT,$PERCENT_HIT,$PERCENT_BUSY,$WRITES,$READS,$IO_RATE\n";
      } ## end foreach my $f_dir_id ( keys...)
      print PERF "\n";
    } ## end if ( $section eq "FEDirector"...)

    #BEDirector Level Statistics
    if ( $section eq "BEDirector" && $data{'BEDirector'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,IO_RATE,MB_RATE (kB),MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_BUSY,READS,WRITES\n";

      my $idx = 0;
      foreach my $b_dir_id ( keys( %{ $data{'BEDirector'} } ) ) {
        chomp $b_dir_id;
        $idx++;

        my $timestamp        = "";
        my $IO_RATE          = "";
        my $MB_RATE          = "";
        my $MB_READ_PER_SEC  = "";
        my $MB_WRITE_PER_SEC = "";
        my $PERCENT_BUSY     = "";
        my $READS            = "";
        my $WRITES           = "";

        if ( defined $data{'BEDirector'}{$b_dir_id}{'timestamp'}    && isdigit( $data{'BEDirector'}{$b_dir_id}{'timestamp'} ) )    { $timestamp    = $data{'BEDirector'}{$b_dir_id}{'timestamp'} / 1000; }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'IO_RATE'}      && isdigit( $data{'BEDirector'}{$b_dir_id}{'IO_RATE'} ) )      { $IO_RATE      = $data{'BEDirector'}{$b_dir_id}{'IO_RATE'}; }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'PERCENT_BUSY'} && isdigit( $data{'BEDirector'}{$b_dir_id}{'PERCENT_BUSY'} ) ) { $PERCENT_BUSY = $data{'BEDirector'}{$b_dir_id}{'PERCENT_BUSY'}; }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'READS'}        && isdigit( $data{'BEDirector'}{$b_dir_id}{'READS'} ) )        { $READS        = $data{'BEDirector'}{$b_dir_id}{'READS'}; }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'WRITES'}       && isdigit( $data{'BEDirector'}{$b_dir_id}{'WRITES'} ) )       { $WRITES       = $data{'BEDirector'}{$b_dir_id}{'WRITES'}; }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'MB_RATE'}      && isdigit( $data{'BEDirector'}{$b_dir_id}{'MB_RATE'} ) ) {
          $MB_RATE = $data{'BEDirector'}{$b_dir_id}{'MB_RATE'} * 1024;    # from MB to kB
        }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'MB_READ_PER_SEC'} && isdigit( $data{'BEDirector'}{$b_dir_id}{'MB_READ_PER_SEC'} ) ) {
          $MB_READ_PER_SEC = $data{'BEDirector'}{$b_dir_id}{'MB_READ_PER_SEC'} * 1024;    # from MB to kB
        }
        if ( defined $data{'BEDirector'}{$b_dir_id}{'MB_WRITE_PER_SEC'} && isdigit( $data{'BEDirector'}{$b_dir_id}{'MB_WRITE_PER_SEC'} ) ) {
          $MB_WRITE_PER_SEC = $data{'BEDirector'}{$b_dir_id}{'MB_WRITE_PER_SEC'} * 1024;    # from MB to kB
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$b_dir_id,$stat_time,$timestamp,$IO_RATE,$MB_RATE,$MB_READ_PER_SEC,$MB_WRITE_PER_SEC,$PERCENT_BUSY,$READS,$WRITES\n";
      } ## end foreach my $b_dir_id ( keys...)
      print PERF "\n";
    } ## end if ( $section eq "BEDirector"...)

    #RDFDirector Level Statistics
    if ( $section eq "RDFDirector" && $data{'RDFDirector'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "ID,time,timestamp,IO_RATE,MB_RATE (kB),MB_READ_PER_SEC (kB),MB_WRITE_PER_SEC (kB),PERCENT_BUSY,READS,WRITES\n";

      my $idx = 0;
      foreach my $b_dir_id ( keys( %{ $data{'RDFDirector'} } ) ) {
        chomp $b_dir_id;
        $idx++;

        my $timestamp        = "";
        my $IO_RATE          = "";
        my $MB_RATE          = "";
        my $MB_READ_PER_SEC  = "";
        my $MB_WRITE_PER_SEC = "";
        my $PERCENT_BUSY     = "";
        my $READS            = "";
        my $WRITES           = "";

        if ( defined $data{'RDFDirector'}{$b_dir_id}{'timestamp'}    && isdigit( $data{'RDFDirector'}{$b_dir_id}{'timestamp'} ) )    { $timestamp    = $data{'RDFDirector'}{$b_dir_id}{'timestamp'} / 1000; }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'IO_RATE'}      && isdigit( $data{'RDFDirector'}{$b_dir_id}{'IO_RATE'} ) )      { $IO_RATE      = $data{'RDFDirector'}{$b_dir_id}{'IO_RATE'}; }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'PERCENT_BUSY'} && isdigit( $data{'RDFDirector'}{$b_dir_id}{'PERCENT_BUSY'} ) ) { $PERCENT_BUSY = $data{'RDFDirector'}{$b_dir_id}{'PERCENT_BUSY'}; }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'READS'}        && isdigit( $data{'RDFDirector'}{$b_dir_id}{'READS'} ) )        { $READS        = $data{'RDFDirector'}{$b_dir_id}{'READS'}; }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'WRITES'}       && isdigit( $data{'RDFDirector'}{$b_dir_id}{'WRITES'} ) )       { $WRITES       = $data{'RDFDirector'}{$b_dir_id}{'WRITES'}; }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'MB_RATE'}      && isdigit( $data{'RDFDirector'}{$b_dir_id}{'MB_RATE'} ) ) {
          $MB_RATE = $data{'RDFDirector'}{$b_dir_id}{'MB_RATE'} * 1024;    # from MB to kB
        }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'MB_READ_PER_SEC'} && isdigit( $data{'RDFDirector'}{$b_dir_id}{'MB_READ_PER_SEC'} ) ) {
          $MB_READ_PER_SEC = $data{'RDFDirector'}{$b_dir_id}{'MB_READ_PER_SEC'} * 1024;    # from MB to kB
        }
        if ( defined $data{'RDFDirector'}{$b_dir_id}{'MB_WRITE_PER_SEC'} && isdigit( $data{'RDFDirector'}{$b_dir_id}{'MB_WRITE_PER_SEC'} ) ) {
          $MB_WRITE_PER_SEC = $data{'RDFDirector'}{$b_dir_id}{'MB_WRITE_PER_SEC'} * 1024;    # from MB to kB
        }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print PERF "$b_dir_id,$stat_time,$timestamp,$IO_RATE,$MB_RATE,$MB_READ_PER_SEC,$MB_WRITE_PER_SEC,$PERCENT_BUSY,$READS,$WRITES\n";
      } ## end foreach my $b_dir_id ( keys...)
      print PERF "\n";
    } ## end if ( $section eq "RDFDirector"...)

    #StorageGroupByTier Level Statistics
    if ( $section eq "StorageGroupByTier" && $data{'StorageGroupByTier'} ne '' ) {
      print PERF "$section Level Statistics\n";
      print PERF "---------------------\n";
      print PERF "SG_ID,Tier_id,time,timestamp,SG total capacity,Tier capacity,Tier capacity (%),SG total allocated tracks, Tier allocated tracks, Tier allocated tracks (%)\n";

      my $idx = 0;
      foreach my $sg_id ( keys( %{ $data{'StorageGroupByTier'} } ) ) {
        chomp $sg_id;
        $idx++;

        my $sg_alloc_total = 0;
        my $sg_cap_total   = 0;

        if ( defined $data{'StorageGroupByTier'}{$sg_id} && $data{'StorageGroupByTier'}{$sg_id} ne '' ) {
          foreach my $tier_id ( keys( %{ $data{'StorageGroupByTier'}{$sg_id} } ) ) {
            chomp $tier_id;

            my $tier_cap          = "";
            my $tier_alloc_tracks = "";

            if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} ) ) {
              $tier_cap = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'};
            }
            if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} ) ) {
              $tier_alloc_tracks = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'};
            }

            #print "$sg_id : $tier_id : $tier_cap : $tier_alloc_tracks\n";

            $sg_alloc_total += $tier_alloc_tracks;
            $sg_cap_total   += $tier_cap;
          } ## end foreach my $tier_id ( keys(...))

          #print "TOTAL : $sg_cap_total : $sg_alloc_total\n\n";

          if ( $sg_cap_total ne "0" && $sg_alloc_total ne "0" ) {
            foreach my $tier_id ( keys( %{ $data{'StorageGroupByTier'}{$sg_id} } ) ) {
              chomp $tier_id;

              my $timestamp         = "";
              my $tier_cap          = "";
              my $tier_alloc_tracks = "";

              my $tier_cap_prct          = "";
              my $tier_alloc_tracks_prct = "";

              if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} ) ) {
                $timestamp = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} / 1000;
              }
              if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} ) ) {
                $tier_cap      = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'};
                $tier_cap_prct = $tier_cap / ( $sg_cap_total / 100 );
                $tier_cap_prct = sprintf( '%.1f', $tier_cap_prct );
              }
              if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} ) ) {
                $tier_alloc_tracks      = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'};
                $tier_alloc_tracks_prct = $tier_alloc_tracks / ( $sg_alloc_total / 100 );
                $tier_alloc_tracks_prct = sprintf( '%.1f', $tier_alloc_tracks_prct );
              }

              my $stat_time = "";
              if ( defined $timestamp && isdigit($timestamp) ) {
                my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
                my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
                my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

                $stat_time = $date . "T$time";
              } ## end if ( defined $timestamp...)

              print PERF "$sg_id,$tier_id,$stat_time,$timestamp,$sg_cap_total,$tier_cap,$tier_cap_prct,$sg_alloc_total,$tier_alloc_tracks,$tier_alloc_tracks_prct\n";
            } ## end foreach my $tier_id ( keys(...))
          } ## end if ( $sg_cap_total ne ...)
        } ## end if ( defined $data{'StorageGroupByTier'...})
      } ## end foreach my $sg_id ( keys( %...))
      print PERF "\n";
    } ## end if ( $section eq "StorageGroupByTier"...)

    ##############################
    # only for testing all sections
    #print "$section Level Statistics\n";
    #print "---------------------\n";
    #my $idx    = 0;
    #my $header = "ID,";
    #foreach my $id (keys (%{$data{$section}})) {
    #  chomp $id;
    #  $idx++;
    #  my $data_line = "$id,";
    #  if ( $data{$section}{$id} ne '' ) {
    #    foreach my $metric (keys (%{$data{$section}{$id}})) {
    #      chomp $metric;
    #      #print "$id : $metric\n";
    #      if ( $idx == 1 ) { $header .= "$metric,"; }
    #      if ( $data{$section}{$id}{$metric} ne '' ) {
    #        $data_line .= "$data{$section}{$id}{$metric},";
    #      }
    #      else {
    #        $data_line .= ",";
    #      }
    #    }
    #  }
    #  if ( $idx == 1 ) { print "$header\n"; }
    #  print "$data_line\n";
    #}
    #print "\n";
    ##############################

  } ## end foreach my $section ( keys ...)

  close(PERF);

  return 1;
} ## end sub print_perf

sub print_conf {
  open( CONF, ">$conf_out" ) || error( "Couldn't open file $conf_out $!" . __FILE__ . ":" . __LINE__ ) && exit;

  print CONF "Configuration Data\n";
  print CONF "------------------\n";
  print CONF "symmetrix ID : $symmetrix_id\n";
  print CONF "\n";

  foreach my $section ( keys %data ) {
    chomp $section;

    #StorageGroup Level Configuration
    if ( $section eq "StorageGroup" && $data{'StorageGroup'} ne '' ) {
      print CONF "$section Level Configuration\n";
      print CONF "---------------------\n";
      print CONF "SG_ID,time,timestamp,DEVICE_CAPACITY,allocated tiers\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'StorageGroup'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp   = "";
        my $cap         = "";
        my $tiers_alloc = "";

        if ( defined $data{'StorageGroup'}{$id}{'timestamp'}       && isdigit( $data{'StorageGroup'}{$id}{'timestamp'} ) )       { $timestamp = $data{'StorageGroup'}{$id}{'timestamp'} / 1000; }
        if ( defined $data{'StorageGroup'}{$id}{'DEVICE_CAPACITY'} && isdigit( $data{'StorageGroup'}{$id}{'DEVICE_CAPACITY'} ) ) { $cap       = $data{'StorageGroup'}{$id}{'DEVICE_CAPACITY'}; }

        if ( defined $data{'TierByStorageGroup'}{$id} && $data{'TierByStorageGroup'}{$id} ne '' ) {
          foreach my $tier_id ( keys( %{ $data{'TierByStorageGroup'}{$id} } ) ) {
            chomp $tier_id;

            if ( defined $tiers_alloc && $tiers_alloc ne '' ) {
              $tiers_alloc .= " $tier_id";
            }
            else {
              $tiers_alloc = "$tier_id";
            }
          } ## end foreach my $tier_id ( keys(...))
        } ## end if ( defined $data{'TierByStorageGroup'...})

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print CONF "$id,$stat_time,$timestamp,$cap,$tiers_alloc\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print CONF "\n";
    } ## end if ( $section eq "StorageGroup"...)

    #ThinPool Level Configuration
    if ( $section eq "ThinPool" && $data{'ThinPool'} ne '' ) {
      print CONF "$section Level Configuration\n";
      print CONF "---------------------\n";
      print CONF "ID,time,timestamp\n";

      my $idx = 0;
      foreach my $id ( keys( %{ $data{'ThinPool'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp = "";

        if ( defined $data{'ThinPool'}{$id}{'timestamp'} && isdigit( $data{'ThinPool'}{$id}{'timestamp'} ) ) { $timestamp = $data{'ThinPool'}{$id}{'timestamp'} / 1000; }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print CONF "$id,$stat_time,$timestamp\n";
      } ## end foreach my $id ( keys( %{ $data...}))
      print CONF "\n";
    } ## end if ( $section eq "ThinPool"...)

    #ThinTier Level Configuration
    if ( $section eq "ThinTier" && $data{'ThinTier'} ne '' ) {
      print CONF "$section Level Configuration\n";
      print CONF "---------------------\n";
      print CONF "ID,time,timestamp\n";

      my $idx = 0;
      foreach my $id ( sort keys( %{ $data{'ThinTier'} } ) ) {
        chomp $id;
        $idx++;

        my $timestamp = "";

        if ( defined $data{'ThinTier'}{$id}{'timestamp'} && isdigit( $data{'ThinTier'}{$id}{'timestamp'} ) ) { $timestamp = $data{'ThinTier'}{$id}{'timestamp'} / 1000; }

        my $stat_time = "";
        if ( defined $timestamp && isdigit($timestamp) ) {
          my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
          my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
          my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

          $stat_time = $date . "T$time";
        } ## end if ( defined $timestamp...)

        print CONF "$id,$stat_time,$timestamp\n";
      } ## end foreach my $id ( sort keys(...))
      print CONF "\n";
    } ## end if ( $section eq "ThinTier"...)

    #StorageGroupByTier Level Statistics
    if ( $section eq "StorageGroupByTier" && $data{'StorageGroupByTier'} ne '' ) {
      print CONF "$section Level Configuration\n";
      print CONF "---------------------\n";
      print CONF "SG_ID,Tier_id,time,timestamp,SG total capacity,Tier capacity,Tier capacity (%),SG total allocated tracks, Tier allocated tracks, Tier allocated tracks (%)\n";

      my $idx = 0;
      foreach my $sg_id ( keys( %{ $data{'StorageGroupByTier'} } ) ) {
        chomp $sg_id;
        $idx++;

        my $sg_alloc_total = 0;
        my $sg_cap_total   = 0;

        if ( defined $data{'StorageGroupByTier'}{$sg_id} && $data{'StorageGroupByTier'}{$sg_id} ne '' ) {
          foreach my $tier_id ( keys( %{ $data{'StorageGroupByTier'}{$sg_id} } ) ) {
            chomp $tier_id;

            my $tier_cap          = "";
            my $tier_alloc_tracks = "";

            if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} ) ) {
              $tier_cap = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'};
            }
            if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} ) ) {
              $tier_alloc_tracks = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'};
            }

            #print "$sg_id : $tier_id : $tier_cap : $tier_alloc_tracks\n";

            $sg_alloc_total += $tier_alloc_tracks;
            $sg_cap_total   += $tier_cap;
          } ## end foreach my $tier_id ( keys(...))

          #print "TOTAL : $sg_cap_total : $sg_alloc_total\n\n";

          if ( $sg_cap_total ne "0" && $sg_alloc_total ne "0" ) {
            foreach my $tier_id ( keys( %{ $data{'StorageGroupByTier'}{$sg_id} } ) ) {
              chomp $tier_id;

              my $timestamp         = "";
              my $tier_cap          = "";
              my $tier_alloc_tracks = "";

              my $tier_cap_prct          = "";
              my $tier_alloc_tracks_prct = "";

              if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} ) ) {
                $timestamp = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} / 1000;
              }
              if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'} ) ) {
                $tier_cap      = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'CAPACITY'};
                $tier_cap_prct = $tier_cap / ( $sg_cap_total / 100 );
                $tier_cap_prct = sprintf( '%.1f', $tier_cap_prct );
              }
              if ( defined $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} && isdigit( $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'} ) ) {
                $tier_alloc_tracks      = $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'ALLOCATED_TRACKS'};
                $tier_alloc_tracks_prct = $tier_alloc_tracks / ( $sg_alloc_total / 100 );
                $tier_alloc_tracks_prct = sprintf( '%.1f', $tier_alloc_tracks_prct );
              }

              my $stat_time = "";
              if ( defined $timestamp && isdigit($timestamp) ) {
                my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($timestamp);
                my $date = sprintf( "%4d:%02d:%02d",  $year + 1900, $month + 1, $day );
                my $time = sprintf( "%02d:%02d:%02d", $hour,        $min,       $sec );

                $stat_time = $date . "T$time";
              } ## end if ( defined $timestamp...)

              print CONF "$sg_id,$tier_id,$stat_time,$timestamp,$sg_cap_total,$tier_cap,$tier_cap_prct,$sg_alloc_total,$tier_alloc_tracks,$tier_alloc_tracks_prct\n";
            } ## end foreach my $tier_id ( keys(...))
          } ## end if ( $sg_cap_total ne ...)
        } ## end if ( defined $data{'StorageGroupByTier'...})
      } ## end foreach my $sg_id ( keys( %...))
      print CONF "\n";
    } ## end if ( $section eq "StorageGroupByTier"...)

  }    # end of foreach

  close(CONF);

  return 1;
} ## end sub print_conf

sub perf_array {
  my $time_error_idx = 0;
  #
  # Array Keys
  #
  $parameters{'Array'}{'keys'}{'url'} = "$url/restapi/performance/Array/keys";
  my $array_keys_result = new_get_request( $parameters{'Array'}{'keys'}{'url'} );

  #print Dumper $array_keys_result;
  if ( $array_keys_result eq "0" ) { return 0; }
  #
  # Array Metrics
  #
  if ( defined $array_keys_result->{'arrayKeyResult'}{'arrayInfo'} && $array_keys_result->{'arrayKeyResult'}{'arrayInfo'} ne '' ) {
    foreach ( @{ $array_keys_result->{'arrayKeyResult'}{'arrayInfo'} } ) {
      if ( defined $_->{'symmetrixId'} && $_->{'symmetrixId'} eq "$symmetrix_id" && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        $parameters{'Array'}{'metrics'}{'param'}{'arrayParam'}{'startDate'}   = $start_timestamp_ms - 300000;
        $parameters{'Array'}{'metrics'}{'param'}{'arrayParam'}{'endDate'}     = $start_timestamp_ms;
        $parameters{'Array'}{'metrics'}{'param'}{'arrayParam'}{'symmetrixId'} = $symmetrix_id;
        $parameters{'Array'}{'metrics'}{'param'}{'arrayParam'}{'metrics'}     = [@array_metrics];
        $parameters{'Array'}{'metrics'}{'url'}                                = "$url/restapi/performance/Array/metrics";

        my $array_metrics_json_body = encode_json( $parameters{'Array'}{'metrics'}{'param'} );
        my $array_metrics_result = new_post_request( $parameters{'Array'}{'metrics'}{'url'}, $array_metrics_json_body );

        #print Dumper $array_metrics_result;
        if ( $array_metrics_result eq "0" ) { return 0; }

        # parse perf
        if ( defined $array_metrics_result->{'iterator'}{'resultList'}{'result'} && $array_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $array_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@array_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'Array'}{$symmetrix_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'Array'}{$symmetrix_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $array_metrics_result...})
        } ## end if ( defined $array_metrics_result...)
      } ## end if ( defined $_->{'symmetrixId'...})
    } ## end foreach ( @{ $array_keys_result...})
  } ## end if ( defined $array_keys_result...)
  return 1;
} ## end sub perf_array

sub perf_fedirectorbyport {
  my $time_error_idx = 0;
  #
  # FEDirector keys
  #
  $parameters{'FEDirector'}{'keys'}{'url'} = "$url/restapi/performance/FEDirector/keys";
  $parameters{'FEDirector'}{'keys'}{'param'}{'feDirectorKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $fedirector_keys_json_body = encode_json( $parameters{'FEDirector'}{'keys'}{'param'} );
  my $fedirector_keys_result = new_post_request( $parameters{'FEDirector'}{'keys'}{'url'}, $fedirector_keys_json_body );
  if ( $fedirector_keys_result eq "0" ) { return 0; }
  if ( defined $fedirector_keys_result->{'feDirectorKeyResult'}{'feDirectorInfo'} && $fedirector_keys_result->{'feDirectorKeyResult'}{'feDirectorInfo'} ne '' ) {
    foreach ( @{ $fedirector_keys_result->{'feDirectorKeyResult'}{'feDirectorInfo'} } ) {
      if ( defined $_->{'directorId'} && $_->{'directorId'} ne '' ) {
        my $director_id = $_->{'directorId'};

        #print "directorId : $_->{'directorId'}\n";
        #
        # FEDirectorByPort Keys
        #
        $parameters{'FEDirectorByPort'}{'keys'}{'url'}                                              = "$url/restapi/performance/FEDirectorByPort/keys";
        $parameters{'FEDirectorByPort'}{'keys'}{'param'}{'feDirectorByPortKeyParam'}{'symmetrixId'} = "$symmetrix_id";
        $parameters{'FEDirectorByPort'}{'keys'}{'param'}{'feDirectorByPortKeyParam'}{'directorId'}  = $_->{'directorId'};
        my $fedirectorbyport_keys_json_body = encode_json( $parameters{'FEDirectorByPort'}{'keys'}{'param'} );
        my $fedirectorbyport_keys_result = new_post_request( $parameters{'FEDirectorByPort'}{'keys'}{'url'}, $fedirectorbyport_keys_json_body );

        #print "$fedirectorbyport_keys_json_body\n";
        #print Dumper $fedirectorbyport_keys_result;
        if ( $fedirectorbyport_keys_result eq "0" ) { return 0; }
        #
        # FEDirectorByPort Metrics
        #
        $parameters{'FEDirectorByPort'}{'metrics'}{'url'}                                           = "$url/restapi/performance/FEDirectorByPort/metrics";
        $parameters{'FEDirectorByPort'}{'metrics'}{'param'}{'feDirectorByPortParam'}{'symmetrixId'} = "$symmetrix_id";
        $parameters{'FEDirectorByPort'}{'metrics'}{'param'}{'feDirectorByPortParam'}{'directorId'}  = "$director_id";
        $parameters{'FEDirectorByPort'}{'metrics'}{'param'}{'feDirectorByPortParam'}{'metrics'}     = [@fedirectorbyport_metrics];
        if ( defined $fedirectorbyport_keys_result->{'feDirectorByPortKeyResult'}{'feDirectorByPortInfo'} && $fedirectorbyport_keys_result->{'feDirectorByPortKeyResult'}{'feDirectorByPortInfo'} ne '' ) {
          foreach ( @{ $fedirectorbyport_keys_result->{'feDirectorByPortKeyResult'}{'feDirectorByPortInfo'} } ) {
            if ( defined $_->{'portId'} && $_->{'portId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
              if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
                my $start_timestamp_ms_old = $start_timestamp_ms;
                $start_timestamp_ms = $_->{'lastAvailableDate'};
                if ( $time_error_idx == 0 ) {
                  $time_error_idx++;
                  my $s_time = localtime( $start_timestamp_ms_old / 1000 );
                  my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
                  my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
                  error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
                } ## end if ( $time_error_idx ==...)
                else {    # report this error only once per vmaxper.pl run
                  $time_error_idx++;
                  next;
                }
              } ## end if ( $start_timestamp_ms...)
              my $port_id = $_->{'portId'};

              #print "portId : $port_id\n";
              $parameters{'FEDirectorByPort'}{'metrics'}{'param'}{'feDirectorByPortParam'}{'startDate'} = $start_timestamp_ms - 300000;
              $parameters{'FEDirectorByPort'}{'metrics'}{'param'}{'feDirectorByPortParam'}{'endDate'}   = $start_timestamp_ms;
              $parameters{'FEDirectorByPort'}{'metrics'}{'param'}{'feDirectorByPortParam'}{'portId'}    = $port_id;
              my $fedirectorbyport_metrics_json_body = encode_json( $parameters{'FEDirectorByPort'}{'metrics'}{'param'} );
              my $fedirectorbyport_metrics_result = new_post_request( $parameters{'FEDirectorByPort'}{'metrics'}{'url'}, $fedirectorbyport_metrics_json_body );

              #print Dumper $fedirectorbyport_metrics_result;
              if ( $fedirectorbyport_metrics_result eq "0" ) { next; }

              # parse perf
              if ( defined $fedirectorbyport_metrics_result->{'iterator'}{'resultList'}{'result'} && $fedirectorbyport_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
                foreach ( @{ $fedirectorbyport_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
                  foreach my $metric (@fedirectorbyport_metrics) {
                    if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                      $data{'FEDirectorByPort'}{$director_id}{$port_id}{$metric} = $_->{$metric};
                    }
                  }
                  if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
                    $data{'FEDirectorByPort'}{$director_id}{$port_id}{'timestamp'} = $_->{'timestamp'};
                  }
                } ## end foreach ( @{ $fedirectorbyport_metrics_result...})
              } ## end if ( defined $fedirectorbyport_metrics_result...)
            } ## end if ( defined $_->{'portId'...})
          } ## end foreach ( @{ $fedirectorbyport_keys_result...})
        } ## end if ( defined $fedirectorbyport_keys_result...)
      } ## end if ( defined $_->{'directorId'...})
    } ## end foreach ( @{ $fedirector_keys_result...})
  } ## end if ( defined $fedirector_keys_result...)
  return 1;
} ## end sub perf_fedirectorbyport

sub perf_cachepartition {
  my $time_error_idx = 0;
  #
  # CachePartition Keys
  #
  $parameters{'CachePartition'}{'keys'}{'url'} = "$url/restapi/performance/CachePartition/keys";
  $parameters{'CachePartition'}{'keys'}{'param'}{'cachePartitionKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $cachepartition_keys_json_body = encode_json( $parameters{'CachePartition'}{'keys'}{'param'} );
  my $cachepartition_keys_result = new_post_request( $parameters{'CachePartition'}{'keys'}{'url'}, $cachepartition_keys_json_body );

  #print Dumper $cachepartition_keys_result;
  if ( $cachepartition_keys_result eq "0" ) { return 0; }
  #
  # CachePartition Metrics
  #
  $parameters{'CachePartition'}{'metrics'}{'url'}                                         = "$url/restapi/performance/CachePartition/metrics";
  $parameters{'CachePartition'}{'metrics'}{'param'}{'cachePartitionParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'CachePartition'}{'metrics'}{'param'}{'cachePartitionParam'}{'metrics'}     = [@cachepartition_metrics];
  if ( defined $cachepartition_keys_result->{'cachePartitionKeyResult'}{'cachePartitionInfo'} && $cachepartition_keys_result->{'cachePartitionKeyResult'}{'cachePartitionInfo'} ne '' ) {
    foreach ( @{ $cachepartition_keys_result->{'cachePartitionKeyResult'}{'cachePartitionInfo'} } ) {
      if ( defined $_->{'cachePartitionId'} && $_->{'cachePartitionId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $cachepartition_id = $_->{'cachePartitionId'};
        $parameters{'CachePartition'}{'metrics'}{'param'}{'cachePartitionParam'}{'startDate'}        = $start_timestamp_ms - 300000;
        $parameters{'CachePartition'}{'metrics'}{'param'}{'cachePartitionParam'}{'endDate'}          = $start_timestamp_ms;
        $parameters{'CachePartition'}{'metrics'}{'param'}{'cachePartitionParam'}{'cachePartitionId'} = $_->{'cachePartitionId'};
        my $cachepartition_metrics_json_body = encode_json( $parameters{'CachePartition'}{'metrics'}{'param'} );
        my $cachepartition_metrics_result = new_post_request( $parameters{'CachePartition'}{'metrics'}{'url'}, $cachepartition_metrics_json_body );

        #print Dumper $cachepartition_metrics_result;
        if ( $cachepartition_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $cachepartition_metrics_result->{'iterator'}{'resultList'}{'result'} && $cachepartition_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $cachepartition_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@cachepartition_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'CachePartition'}{$cachepartition_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'CachePartition'}{$cachepartition_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $cachepartition_metrics_result...})
        } ## end if ( defined $cachepartition_metrics_result...)
      } ## end if ( defined $_->{'cachePartitionId'...})
    } ## end foreach ( @{ $cachepartition_keys_result...})
  } ## end if ( defined $cachepartition_keys_result...)
  return 1;
} ## end sub perf_cachepartition

sub perf_diskgroup {
  my $time_error_idx = 0;
  #
  # DiskGroup Keys
  #
  $parameters{'DiskGroup'}{'keys'}{'url'} = "$url/restapi/performance/DiskGroup/keys";
  $parameters{'DiskGroup'}{'keys'}{'param'}{'diskGroupKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $diskgroup_keys_json_body = encode_json( $parameters{'DiskGroup'}{'keys'}{'param'} );
  my $diskgroup_keys_result = new_post_request( $parameters{'DiskGroup'}{'keys'}{'url'}, $diskgroup_keys_json_body );

  #print Dumper $diskgroup_keys_result;
  if ( $diskgroup_keys_result eq "0" ) { return 0; }
  #
  # DiskGroup Metrics
  #
  $parameters{'DiskGroup'}{'metrics'}{'url'}                                    = "$url/restapi/performance/DiskGroup/metrics";
  $parameters{'DiskGroup'}{'metrics'}{'param'}{'diskGroupParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'DiskGroup'}{'metrics'}{'param'}{'diskGroupParam'}{'metrics'}     = [@diskgroup_metrics];
  if ( defined $diskgroup_keys_result->{'diskGroupKeyResult'}{'diskGroupInfo'} && $diskgroup_keys_result->{'diskGroupKeyResult'}{'diskGroupInfo'} ne '' ) {
    foreach ( @{ $diskgroup_keys_result->{'diskGroupKeyResult'}{'diskGroupInfo'} } ) {
      if ( defined $_->{'diskGroupId'} && $_->{'diskGroupId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $diskgroup_id = $_->{'diskGroupId'};
        $parameters{'DiskGroup'}{'metrics'}{'param'}{'diskGroupParam'}{'startDate'}   = $start_timestamp_ms - 300000;
        $parameters{'DiskGroup'}{'metrics'}{'param'}{'diskGroupParam'}{'endDate'}     = $start_timestamp_ms;
        $parameters{'DiskGroup'}{'metrics'}{'param'}{'diskGroupParam'}{'diskGroupId'} = $_->{'diskGroupId'};
        my $diskgroup_metrics_json_body = encode_json( $parameters{'DiskGroup'}{'metrics'}{'param'} );
        my $diskgroup_metrics_result = new_post_request( $parameters{'DiskGroup'}{'metrics'}{'url'}, $diskgroup_metrics_json_body );

        #print Dumper $diskgroup_metrics_result;
        if ( $diskgroup_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $diskgroup_metrics_result->{'iterator'}{'resultList'}{'result'} && $diskgroup_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $diskgroup_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@diskgroup_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'DiskGroup'}{$diskgroup_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'DiskGroup'}{$diskgroup_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $diskgroup_metrics_result...})
        } ## end if ( defined $diskgroup_metrics_result...)
      } ## end if ( defined $_->{'diskGroupId'...})
    } ## end foreach ( @{ $diskgroup_keys_result...})
  } ## end if ( defined $diskgroup_keys_result...)
  return 1;
} ## end sub perf_diskgroup

sub perf_thinpool {
  my $time_error_idx = 0;
  #
  # ThinPool Keys
  #
  $parameters{'ThinPool'}{'keys'}{'url'} = "$url/restapi/performance/ThinPool/keys";
  $parameters{'ThinPool'}{'keys'}{'param'}{'thinPoolKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $thinpool_keys_json_body = encode_json( $parameters{'ThinPool'}{'keys'}{'param'} );
  my $thinpool_keys_result = new_post_request( $parameters{'ThinPool'}{'keys'}{'url'}, $thinpool_keys_json_body );

  #print Dumper $thinpool_keys_result;
  if ( $thinpool_keys_result eq "0" ) { return 0; }
  #
  #
  #
  $parameters{'ThinPool'}{'metrics'}{'url'}                                   = "$url/restapi/performance/ThinPool/metrics";
  $parameters{'ThinPool'}{'metrics'}{'param'}{'thinPoolParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'ThinPool'}{'metrics'}{'param'}{'thinPoolParam'}{'metrics'}     = [@thinpool_metrics];
  if ( defined $thinpool_keys_result->{'poolKeyResult'}{'poolInfo'} && $thinpool_keys_result->{'poolKeyResult'}{'poolInfo'} ne '' ) {
    foreach ( @{ $thinpool_keys_result->{'poolKeyResult'}{'poolInfo'} } ) {
      if ( defined $_->{'poolId'} && $_->{'poolId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $thinpool_id = $_->{'poolId'};
        $parameters{'ThinPool'}{'metrics'}{'param'}{'thinPoolParam'}{'startDate'} = $start_timestamp_ms - 300000;
        $parameters{'ThinPool'}{'metrics'}{'param'}{'thinPoolParam'}{'endDate'}   = $start_timestamp_ms;
        $parameters{'ThinPool'}{'metrics'}{'param'}{'thinPoolParam'}{'poolId'}    = $_->{'poolId'};
        my $thinpool_metrics_json_body = encode_json( $parameters{'ThinPool'}{'metrics'}{'param'} );
        my $thinpool_metrics_result = new_post_request( $parameters{'ThinPool'}{'metrics'}{'url'}, $thinpool_metrics_json_body );

        #print Dumper $thinpool_metrics_result;
        if ( $thinpool_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $thinpool_metrics_result->{'iterator'}{'resultList'}{'result'} && $thinpool_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $thinpool_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@thinpool_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'ThinPool'}{$thinpool_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'ThinPool'}{$thinpool_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $thinpool_metrics_result...})
        } ## end if ( defined $thinpool_metrics_result...)
      } ## end if ( defined $_->{'poolId'...})
    } ## end foreach ( @{ $thinpool_keys_result...})
  } ## end if ( defined $thinpool_keys_result...)
  return 1;
} ## end sub perf_thinpool

sub perf_thintier {
  my $time_error_idx = 0;
  #
  # ThinTier Keys
  #
  $parameters{'ThinTier'}{'keys'}{'url'} = "$url/restapi/performance/ThinTier/keys";
  $parameters{'ThinTier'}{'keys'}{'param'}{'thinTierKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $thintier_keys_json_body = encode_json( $parameters{'ThinTier'}{'keys'}{'param'} );
  my $thintier_keys_result = new_post_request( $parameters{'ThinTier'}{'keys'}{'url'}, $thintier_keys_json_body );

  #print Dumper $thintier_keys_result;
  if ( $thintier_keys_result eq "0" ) { return 0; }
  #
  #
  #
  $parameters{'ThinTier'}{'metrics'}{'url'}                                   = "$url/restapi/performance/ThinTier/metrics";
  $parameters{'ThinTier'}{'metrics'}{'param'}{'thinTierParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'ThinTier'}{'metrics'}{'param'}{'thinTierParam'}{'metrics'}     = [@thintier_metrics];
  if ( defined $thintier_keys_result->{'tierKeyResult'}{'tierInfo'} && $thintier_keys_result->{'tierKeyResult'}{'tierInfo'} ne '' ) {
    foreach ( @{ $thintier_keys_result->{'tierKeyResult'}{'tierInfo'} } ) {
      if ( defined $_->{'tierId'} && $_->{'tierId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $thintier_id = $_->{'tierId'};
        $parameters{'ThinTier'}{'metrics'}{'param'}{'thinTierParam'}{'startDate'} = $start_timestamp_ms - 300000;
        $parameters{'ThinTier'}{'metrics'}{'param'}{'thinTierParam'}{'endDate'}   = $start_timestamp_ms;
        $parameters{'ThinTier'}{'metrics'}{'param'}{'thinTierParam'}{'tierId'}    = $_->{'tierId'};
        my $thintier_metrics_json_body = encode_json( $parameters{'ThinTier'}{'metrics'}{'param'} );
        my $thintier_metrics_result = new_post_request( $parameters{'ThinTier'}{'metrics'}{'url'}, $thintier_metrics_json_body );

        #print Dumper $thintier_metrics_result;
        if ( $thintier_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $thintier_metrics_result->{'iterator'}{'resultList'}{'result'} && $thintier_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $thintier_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@thintier_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'ThinTier'}{$thintier_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'ThinTier'}{$thintier_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $thintier_metrics_result...})
        } ## end if ( defined $thintier_metrics_result...)
      } ## end if ( defined $_->{'tierId'...})
    } ## end foreach ( @{ $thintier_keys_result...})
  } ## end if ( defined $thintier_keys_result...)
  return 1;
} ## end sub perf_thintier

sub perf_storagegroup {
  my $time_error_idx = 0;
  #
  # StorageGroup keys
  #
  $parameters{'StorageGroup'}{'keys'}{'url'} = "$url/restapi/performance/StorageGroup/keys";
  $parameters{'StorageGroup'}{'keys'}{'param'}{'storageGroupKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $storagegroup_keys_json_body = encode_json( $parameters{'StorageGroup'}{'keys'}{'param'} );
  my $storagegroup_keys_result = new_post_request( $parameters{'StorageGroup'}{'keys'}{'url'}, $storagegroup_keys_json_body );

  #print Dumper $storagegroup_keys_result;
  if ( $storagegroup_keys_result eq "0" ) { return 0; }
  #
  # StorageGroup Metrics
  #
  $parameters{'StorageGroup'}{'metrics'}{'url'}                                       = "$url/restapi/performance/StorageGroup/metrics";
  $parameters{'StorageGroup'}{'metrics'}{'param'}{'storageGroupParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'StorageGroup'}{'metrics'}{'param'}{'storageGroupParam'}{'metrics'}     = [@storagegroup_metrics];
  if ( defined $storagegroup_keys_result->{'storageGroupKeyResult'}{'storageGroupInfo'} && $storagegroup_keys_result->{'storageGroupKeyResult'}{'storageGroupInfo'} ne '' ) {
    foreach ( @{ $storagegroup_keys_result->{'storageGroupKeyResult'}{'storageGroupInfo'} } ) {
      if ( defined $_->{'storageGroupId'} && $_->{'storageGroupId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $storagegroup_id = $_->{'storageGroupId'};
        $parameters{'StorageGroup'}{'metrics'}{'param'}{'storageGroupParam'}{'startDate'}      = $start_timestamp_ms - 300000;
        $parameters{'StorageGroup'}{'metrics'}{'param'}{'storageGroupParam'}{'endDate'}        = $start_timestamp_ms;
        $parameters{'StorageGroup'}{'metrics'}{'param'}{'storageGroupParam'}{'storageGroupId'} = $_->{'storageGroupId'};
        my $storagegroup_metrics_json_body = encode_json( $parameters{'StorageGroup'}{'metrics'}{'param'} );
        my $storagegroup_metrics_result = new_post_request( $parameters{'StorageGroup'}{'metrics'}{'url'}, $storagegroup_metrics_json_body );

        #print Dumper $storagegroup_metrics_result;
        if ( $storagegroup_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $storagegroup_metrics_result->{'iterator'}{'resultList'}{'result'} && $storagegroup_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $storagegroup_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@storagegroup_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'StorageGroup'}{$storagegroup_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'StorageGroup'}{$storagegroup_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $storagegroup_metrics_result...})
        } ## end if ( defined $storagegroup_metrics_result...)
      } ## end if ( defined $_->{'storageGroupId'...})
    } ## end foreach ( @{ $storagegroup_keys_result...})
  } ## end if ( defined $storagegroup_keys_result...)
  return 1;
} ## end sub perf_storagegroup

sub perf_fedirector {
  my $time_error_idx = 0;
  #
  # FEDirector keys
  #
  $parameters{'FEDirector'}{'keys'}{'url'} = "$url/restapi/performance/FEDirector/keys";
  $parameters{'FEDirector'}{'keys'}{'param'}{'feDirectorKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $fedirector_keys_json_body = encode_json( $parameters{'FEDirector'}{'keys'}{'param'} );
  my $fedirector_keys_result = new_post_request( $parameters{'FEDirector'}{'keys'}{'url'}, $fedirector_keys_json_body );
  if ( $fedirector_keys_result eq "0" ) { return 0; }
  #
  # FEDirector metrics
  #
  $parameters{'FEDirector'}{'metrics'}{'url'}                                     = "$url/restapi/performance/FEDirector/metrics";
  $parameters{'FEDirector'}{'metrics'}{'param'}{'feDirectorParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'FEDirector'}{'metrics'}{'param'}{'feDirectorParam'}{'metrics'}     = [@fedirector_metrics];
  if ( defined $fedirector_keys_result->{'feDirectorKeyResult'}{'feDirectorInfo'} && $fedirector_keys_result->{'feDirectorKeyResult'}{'feDirectorInfo'} ne '' ) {
    foreach ( @{ $fedirector_keys_result->{'feDirectorKeyResult'}{'feDirectorInfo'} } ) {
      if ( defined $_->{'directorId'} && $_->{'directorId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $fedirector_id = $_->{'directorId'};

        #$parameters{'FEDirector'}{'metrics'}{'param'}{'feDirectorParam'}{'startDate'}   = $_->{'lastAvailableDate'} - 300000; #???
        $parameters{'FEDirector'}{'metrics'}{'param'}{'feDirectorParam'}{'startDate'}  = $start_timestamp_ms - 300000;
        $parameters{'FEDirector'}{'metrics'}{'param'}{'feDirectorParam'}{'endDate'}    = $start_timestamp_ms;
        $parameters{'FEDirector'}{'metrics'}{'param'}{'feDirectorParam'}{'directorId'} = $_->{'directorId'};
        my $fedirector_metrics_json_body = encode_json( $parameters{'FEDirector'}{'metrics'}{'param'} );
        my $fedirector_metrics_result = new_post_request( $parameters{'FEDirector'}{'metrics'}{'url'}, $fedirector_metrics_json_body );

        #print Dumper $fedirector_metrics_result;
        if ( $fedirector_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $fedirector_metrics_result->{'iterator'}{'resultList'}{'result'} && $fedirector_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $fedirector_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@fedirector_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'FEDirector'}{$fedirector_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'FEDirector'}{$fedirector_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $fedirector_metrics_result...})
        } ## end if ( defined $fedirector_metrics_result...)
      } ## end if ( defined $_->{'directorId'...})
    } ## end foreach ( @{ $fedirector_keys_result...})
  } ## end if ( defined $fedirector_keys_result...)
  return 1;
} ## end sub perf_fedirector

sub perf_bedirector {
  my $time_error_idx = 0;
  #
  # BEDirector keys
  #
  $parameters{'BEDirector'}{'keys'}{'url'} = "$url/restapi/performance/BEDirector/keys";
  $parameters{'BEDirector'}{'keys'}{'param'}{'beDirectorKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $bedirector_keys_json_body = encode_json( $parameters{'BEDirector'}{'keys'}{'param'} );
  my $bedirector_keys_result = new_post_request( $parameters{'BEDirector'}{'keys'}{'url'}, $bedirector_keys_json_body );
  if ( $bedirector_keys_result eq "0" ) { return 0; }
  #
  # BEDirector metrics
  #
  $parameters{'BEDirector'}{'metrics'}{'url'}                                     = "$url/restapi/performance/BEDirector/metrics";
  $parameters{'BEDirector'}{'metrics'}{'param'}{'beDirectorParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'BEDirector'}{'metrics'}{'param'}{'beDirectorParam'}{'metrics'}     = [@bedirector_metrics];
  if ( defined $bedirector_keys_result->{'beDirectorKeyResult'}{'beDirectorInfo'} && $bedirector_keys_result->{'beDirectorKeyResult'}{'beDirectorInfo'} ne '' ) {
    foreach ( @{ $bedirector_keys_result->{'beDirectorKeyResult'}{'beDirectorInfo'} } ) {
      if ( defined $_->{'directorId'} && $_->{'directorId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $bedirector_id = $_->{'directorId'};

        #$parameters{'BEDirector'}{'metrics'}{'param'}{'beDirectorParam'}{'startDate'}   = $_->{'lastAvailableDate'} - 300000; #???
        $parameters{'BEDirector'}{'metrics'}{'param'}{'beDirectorParam'}{'startDate'}  = $start_timestamp_ms - 300000;
        $parameters{'BEDirector'}{'metrics'}{'param'}{'beDirectorParam'}{'endDate'}    = $start_timestamp_ms;
        $parameters{'BEDirector'}{'metrics'}{'param'}{'beDirectorParam'}{'directorId'} = $_->{'directorId'};
        my $bedirector_metrics_json_body = encode_json( $parameters{'BEDirector'}{'metrics'}{'param'} );
        my $bedirector_metrics_result = new_post_request( $parameters{'BEDirector'}{'metrics'}{'url'}, $bedirector_metrics_json_body );

        #print Dumper $bedirector_metrics_result;
        if ( $bedirector_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $bedirector_metrics_result->{'iterator'}{'resultList'}{'result'} && $bedirector_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $bedirector_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@bedirector_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'BEDirector'}{$bedirector_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'BEDirector'}{$bedirector_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $bedirector_metrics_result...})
        } ## end if ( defined $bedirector_metrics_result...)
      } ## end if ( defined $_->{'directorId'...})
    } ## end foreach ( @{ $bedirector_keys_result...})
  } ## end if ( defined $bedirector_keys_result...)
  return 1;
} ## end sub perf_bedirector

sub perf_rdfdirector {
  my $time_error_idx = 0;
  #
  # RDFDirector keys
  #
  $parameters{'RDFDirector'}{'keys'}{'url'} = "$url/restapi/performance/RDFDirector/keys";
  $parameters{'RDFDirector'}{'keys'}{'param'}{'rdfDirectorKeyParam'}{'symmetrixId'} = "$symmetrix_id";
  my $rdfdirector_keys_json_body = encode_json( $parameters{'RDFDirector'}{'keys'}{'param'} );
  my $rdfdirector_keys_result = new_post_request( $parameters{'RDFDirector'}{'keys'}{'url'}, $rdfdirector_keys_json_body );
  if ( $rdfdirector_keys_result eq "0" ) { return 0; }
  #
  # BEDirector metrics
  #
  $parameters{'RDFDirector'}{'metrics'}{'url'}                                      = "$url/restapi/performance/RDFDirector/metrics";
  $parameters{'RDFDirector'}{'metrics'}{'param'}{'rdfDirectorParam'}{'symmetrixId'} = "$symmetrix_id";
  $parameters{'RDFDirector'}{'metrics'}{'param'}{'rdfDirectorParam'}{'metrics'}     = [@rdfdirector_metrics];
  if ( defined $rdfdirector_keys_result->{'rdfDirectorKeyResult'}{'rdfDirectorInfo'} && $rdfdirector_keys_result->{'rdfDirectorKeyResult'}{'rdfDirectorInfo'} ne '' ) {
    foreach ( @{ $rdfdirector_keys_result->{'rdfDirectorKeyResult'}{'rdfDirectorInfo'} } ) {
      if ( defined $_->{'directorId'} && $_->{'directorId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
        if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
          my $start_timestamp_ms_old = $start_timestamp_ms;
          $start_timestamp_ms = $_->{'lastAvailableDate'};
          if ( $time_error_idx == 0 ) {
            $time_error_idx++;
            my $s_time = localtime( $start_timestamp_ms_old / 1000 );
            my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
            my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
            error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
          } ## end if ( $time_error_idx ==...)
          else {    # report this error only once per vmaxper.pl run
            $time_error_idx++;
            next;
          }
        } ## end if ( $start_timestamp_ms...)
        my $rdfdirector_id = $_->{'directorId'};
        $parameters{'RDFDirector'}{'metrics'}{'param'}{'rdfDirectorParam'}{'startDate'}  = $start_timestamp_ms - 300000;
        $parameters{'RDFDirector'}{'metrics'}{'param'}{'rdfDirectorParam'}{'endDate'}    = $start_timestamp_ms;
        $parameters{'RDFDirector'}{'metrics'}{'param'}{'rdfDirectorParam'}{'directorId'} = $_->{'directorId'};
        my $rdfdirector_metrics_json_body = encode_json( $parameters{'RDFDirector'}{'metrics'}{'param'} );
        my $rdfdirector_metrics_result = new_post_request( $parameters{'RDFDirector'}{'metrics'}{'url'}, $rdfdirector_metrics_json_body );

        #print Dumper $rdfdirector_metrics_result;
        if ( $rdfdirector_metrics_result eq "0" ) { next; }

        # parse perf
        if ( defined $rdfdirector_metrics_result->{'iterator'}{'resultList'}{'result'} && $rdfdirector_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
          foreach ( @{ $rdfdirector_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
            foreach my $metric (@rdfdirector_metrics) {
              if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                $data{'RDFDirector'}{$rdfdirector_id}{$metric} = $_->{$metric};
              }
            }
            if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
              $data{'RDFDirector'}{$rdfdirector_id}{'timestamp'} = $_->{'timestamp'};
            }
          } ## end foreach ( @{ $rdfdirector_metrics_result...})
        } ## end if ( defined $rdfdirector_metrics_result...)
      } ## end if ( defined $_->{'directorId'...})
    } ## end foreach ( @{ $rdfdirector_keys_result...})
  } ## end if ( defined $rdfdirector_keys_result...)
  return 1;
} ## end sub perf_rdfdirector

sub perf_storagegroupbytier {
  my $time_error_idx = 0;
  if ( defined $data{'StorageGroup'} && $data{'StorageGroup'} ne '' ) {
    foreach my $sg_id ( keys( %{ $data{'StorageGroup'} } ) ) {

      #print "$sg_id\n";
      #
      # StorageGroupByTier keys
      #
      $parameters{'StorageGroupByTier'}{'keys'}{'url'}                                                   = "$url/restapi/performance/StorageGroupByTier/keys";
      $parameters{'StorageGroupByTier'}{'keys'}{'param'}{'storageGroupByTierKeyParam'}{'symmetrixId'}    = "$symmetrix_id";
      $parameters{'StorageGroupByTier'}{'keys'}{'param'}{'storageGroupByTierKeyParam'}{'storageGroupId'} = "$sg_id";
      my $storagegroupbytier_keys_json_body = encode_json( $parameters{'StorageGroupByTier'}{'keys'}{'param'} );
      my $storagegroupbytier_keys_result = new_post_request( $parameters{'StorageGroupByTier'}{'keys'}{'url'}, $storagegroupbytier_keys_json_body );
      if ( $storagegroupbytier_keys_result eq "0" ) { return 0; }

      #print Dumper $storagegroupbytier_keys_result;
      #
      # StorageGroupByTier metrics
      #
      $parameters{'StorageGroupByTier'}{'metrics'}{'url'}                                                = "$url/restapi/performance/StorageGroupByTier/metrics";
      $parameters{'StorageGroupByTier'}{'metrics'}{'param'}{'storageGroupByTierParam'}{'symmetrixId'}    = "$symmetrix_id";
      $parameters{'StorageGroupByTier'}{'metrics'}{'param'}{'storageGroupByTierParam'}{'storageGroupId'} = "$sg_id";
      $parameters{'StorageGroupByTier'}{'metrics'}{'param'}{'storageGroupByTierParam'}{'metrics'}        = [@storagegroupbytier_metrics];
      if ( defined $storagegroupbytier_keys_result->{'tierKeyResult'}{'tierInfo'} && $storagegroupbytier_keys_result->{'tierKeyResult'}{'tierInfo'} ne '' ) {
        foreach ( @{ $storagegroupbytier_keys_result->{'tierKeyResult'}{'tierInfo'} } ) {
          if ( defined $_->{'tierId'} && $_->{'tierId'} ne '' && defined $_->{'firstAvailableDate'} && $_->{'firstAvailableDate'} ne '' && defined $_->{'lastAvailableDate'} && $_->{'lastAvailableDate'} ne '' ) {
            if ( $start_timestamp_ms < $_->{'firstAvailableDate'} || $start_timestamp_ms > $_->{'lastAvailableDate'} ) {
              my $start_timestamp_ms_old = $start_timestamp_ms;
              $start_timestamp_ms = $_->{'lastAvailableDate'};
              if ( $time_error_idx == 0 ) {
                $time_error_idx++;
                my $s_time = localtime( $start_timestamp_ms_old / 1000 );
                my $f_time = localtime( $_->{'firstAvailableDate'} / 1000 );
                my $l_time = localtime( $_->{'lastAvailableDate'} / 1000 );
                error( "ERROR: no data available : START time is not in AvailableDate! start_t=$s_time : firstAvailableDate=$f_time : lastAvailableDate=$l_time $!" . __FILE__ . ":" . __LINE__ ) && next;
              } ## end if ( $time_error_idx ==...)
              else {    # report this error only once per vmaxper.pl run
                $time_error_idx++;
                next;
              }
            } ## end if ( $start_timestamp_ms...)
            my $tier_id = $_->{'tierId'};
            $parameters{'StorageGroupByTier'}{'metrics'}{'param'}{'storageGroupByTierParam'}{'startDate'} = $start_timestamp_ms;
            $parameters{'StorageGroupByTier'}{'metrics'}{'param'}{'storageGroupByTierParam'}{'endDate'}   = $start_timestamp_ms;
            $parameters{'StorageGroupByTier'}{'metrics'}{'param'}{'storageGroupByTierParam'}{'tierId'}    = $_->{'tierId'};
            my $storagegroupbytier_metrics_json_body = encode_json( $parameters{'StorageGroupByTier'}{'metrics'}{'param'} );
            my $storagegroupbytier_metrics_result = new_post_request( $parameters{'StorageGroupByTier'}{'metrics'}{'url'}, $storagegroupbytier_metrics_json_body );
            if ( $storagegroupbytier_metrics_result eq "0" ) { next; }

            # parse perf
            if ( defined $storagegroupbytier_metrics_result->{'iterator'}{'resultList'}{'result'} && $storagegroupbytier_metrics_result->{'iterator'}{'resultList'}{'result'} ne '' ) {
              foreach ( @{ $storagegroupbytier_metrics_result->{'iterator'}{'resultList'}{'result'} } ) {
                foreach my $metric (@storagegroupbytier_metrics) {
                  if ( defined $_->{$metric} && $_->{$metric} ne '' ) {
                    $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{$metric} = $_->{$metric};
                  }
                }
                if ( defined $_->{'timestamp'} && isdigit( $_->{'timestamp'} ) ) {
                  $data{'StorageGroupByTier'}{$sg_id}{$tier_id}{'timestamp'} = $_->{'timestamp'};
                }
              } ## end foreach ( @{ $storagegroupbytier_metrics_result...})
            } ## end if ( defined $storagegroupbytier_metrics_result...)
          } ## end if ( defined $_->{'tierId'...})
        } ## end foreach ( @{ $storagegroupbytier_keys_result...})
      } ## end if ( defined $storagegroupbytier_keys_result...)
    } ## end foreach my $sg_id ( keys( %...))
  } ## end if ( defined $data{'StorageGroup'...})
  return 1;
} ## end sub perf_storagegroupbytier

sub conf_tierbystoragegroup {
  my $time_error_idx = 0;
  if ( defined $data{'ThinTier'} && $data{'ThinTier'} ne '' ) {
    foreach my $tier_id ( keys( %{ $data{'ThinTier'} } ) ) {
      $parameters{'TierByStorageGroup'}{'keys'}{'url'}                                                = "$url/restapi/performance/TierByStorageGroup/keys";
      $parameters{'TierByStorageGroup'}{'keys'}{'param'}{'tierByStorageGroupKeyParam'}{'symmetrixId'} = "$symmetrix_id";
      $parameters{'TierByStorageGroup'}{'keys'}{'param'}{'tierByStorageGroupKeyParam'}{'tierId'}      = "$tier_id";
      my $tierbystoragegroup_keys_json_body = encode_json( $parameters{'TierByStorageGroup'}{'keys'}{'param'} );
      my $tierbystoragegroup_keys_result = new_post_request( $parameters{'TierByStorageGroup'}{'keys'}{'url'}, $tierbystoragegroup_keys_json_body );
      if ( $tierbystoragegroup_keys_result eq "0" ) { next; }

      #print Dumper $tierbystoragegroup_keys_result;
      if ( defined $tierbystoragegroup_keys_result->{'storageGroupKeyResult'}{'storageGroupInfo'} && $tierbystoragegroup_keys_result->{'storageGroupKeyResult'}{'storageGroupInfo'} ne '' ) {
        foreach ( @{ $tierbystoragegroup_keys_result->{'storageGroupKeyResult'}{'storageGroupInfo'} } ) {
          if ( defined $_->{'storageGroupId'} && $_->{'storageGroupId'} ne '' ) {
            my $sg_id = $_->{'storageGroupId'};
            $data{'TierByStorageGroup'}{$sg_id}{$tier_id} = $tier_id;
          }
        } ## end foreach ( @{ $tierbystoragegroup_keys_result...})
      } ## end if ( defined $tierbystoragegroup_keys_result...)
    } ## end foreach my $tier_id ( keys(...))
  } ## end if ( defined $data{'ThinTier'...})
  return 1;
} ## end sub conf_tierbystoragegroup

sub new_post_request {
  my $url       = shift;
  my $json_body = shift;

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( POST => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->content($json_body);
  $request->authorization_basic( $user, $pass );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;

    #my $res_con = $results->content;
    error("Request error: $st_line");

    #error("Request error: $res_con");
    error("POST url     : $url");
    error("json body    : $json_body");

    return 0;
  } ## end if ( !$results->is_success)

  my $result = decode_json( $results->content );

  return $result;
} ## end sub new_post_request

sub new_get_request {
  my $url = shift;

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( GET => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $user, $pass );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;

    #my $res_con = $results->content;
    error("Request error: $st_line");

    #error("Request error: $res_con");
    error("POST url     : $url");

    return 0;
  } ## end if ( !$results->is_success)

  my $result = decode_json( $results->content );

  return $result;
} ## end sub new_get_request

sub isdigit {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  return 0;
} ## end sub isdigit

sub file_time_diff {
  my $file = shift;

  my $act_time  = time();
  my $file_time = $act_time;
  my $time_diff = 0;

  if ( -f $file ) {
    $file_time = ( stat($file) )[9];
    $time_diff = $act_time - $file_time;
  }

  return ($time_diff);
} ## end sub file_time_diff

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  #print "$act_time: $text : $!\n" if $DEBUG > 2;;

  return 1;
} ## end sub error
