#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;
use Xorux_lib;

# cmd variables
my $sample_rate      = 300;
my $spa_ip           = "";
my $spb_ip           = "";
my $storage_name     = "";
my $VNX_BLOCK_CLIDIR = "";
my $naviseccli       = "";
my $sec_file_path    = "";

if ( defined $ENV{EMC_VNX_IP_SPA} )   { $spa_ip           = $ENV{EMC_VNX_IP_SPA}; }
if ( defined $ENV{EMC_VNX_IP_SPB} )   { $spb_ip           = $ENV{EMC_VNX_IP_SPB}; }
if ( defined $ENV{STORAGE_NAME} )     { $storage_name     = $ENV{STORAGE_NAME}; }
if ( defined $ENV{SAMPLE_RATE} )      { $sample_rate      = $ENV{SAMPLE_RATE}; }
if ( defined $ENV{VNX_BLOCK_CLIDIR} ) { $VNX_BLOCK_CLIDIR = $ENV{VNX_BLOCK_CLIDIR}; }

# test standard naviseccli paths
if ( -f "$VNX_BLOCK_CLIDIR/naviseccli" ) {    # path from stor2rrd.cfg
  $naviseccli = "$VNX_BLOCK_CLIDIR/naviseccli";
}
if ( ( !defined $naviseccli || $naviseccli eq '' ) && -f "/opt/Navisphere/bin/naviseccli" ) {
  $naviseccli = "/opt/Navisphere/bin/naviseccli";    # Linux
}
if ( ( !defined $naviseccli || $naviseccli eq '' ) && -f "/usr/bin/naviseccli" ) {
  $naviseccli = "/usr/bin/naviseccli";               # AIX
}
if ( !defined $naviseccli || $naviseccli eq '' ) {
  error( "Couldn't found path to Navisphere Secure CLI! $!" . __FILE__ . ":" . __LINE__ ) && exit;
}

my $basedir = $ENV{INPUTDIR};
my $webdir  = $ENV{WEBDIR};
my $wrkdir  = "$basedir/data";
my $tmpdir  = "$basedir/tmp";

if ( -f "$basedir/etc/SecuredCLIXMLEncrypted.key" && -f "$basedir/etc/SecuredCLISecurityFile.xml" ) {
  $sec_file_path = "-secfilepath $basedir/etc";
}

my %data;
my @get_all;
my @get_spb;
my @get_cache_spb;
my @storagepool;
my @lun_list;
my @health_status;

my $block_size = 2048;
my $timeout    = 300;
my $perf_out   = "";
my $conf_out   = "";

my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
my $time_to_output_file = sprintf( "%4d%02d%02d_%02d%02d", $year + 1900, $month + 1, $day, $hour, $min );
my $start_time = sprintf( "%4d:%02d:%02dT%02d:%02d:%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );

$perf_out = "$wrkdir/$storage_name/$storage_name\_vnxbperf_$time_to_output_file.out.tmp";
$conf_out = "$wrkdir/$storage_name/$storage_name\_vnxbconf_$time_to_output_file.out.tmp";

if ( !-f "$wrkdir/$storage_name/VNX" ) {
  open( VNX, ">$wrkdir/$storage_name/VNX" ) || error( "Couldn't open file $wrkdir/$storage_name/VNX $!" . __FILE__ . ":" . __LINE__ ) && exit;
  close(VNX);
}

# run getall cmd
eval {
  # Set alarm
  my $act_time = localtime();
  local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
  alarm($timeout);

  @get_all = `$naviseccli -h $spa_ip $sec_file_path getall`;

  # end of alarm
  alarm(0);
};

if ($@) {
  if ( $@ =~ /died in SIG ALRM/ ) {
    my $act_time = localtime();
    error("command timed out after : $timeout seconds");
    exit(0);
  }
} ## end if ($@)

# run storagepool cmd
eval {
  # Set alarm
  my $act_time = localtime();
  local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
  alarm($timeout);

  @storagepool = `$naviseccli -h $spa_ip $sec_file_path storagepool -list`;

  # end of alarm
  alarm(0);
};

if ($@) {
  if ( $@ =~ /died in SIG ALRM/ ) {
    my $act_time = localtime();
    error("command timed out after : $timeout seconds");
    exit(0);
  }
} ## end if ($@)

# run lun list cmd
eval {
  # Set alarm
  my $act_time = localtime();
  local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
  alarm($timeout);

  @lun_list = `$naviseccli -h $spa_ip $sec_file_path lun -list -all`;

  # end of alarm
  alarm(0);
};

if ($@) {
  if ( $@ =~ /died in SIG ALRM/ ) {
    my $act_time = localtime();
    error("command timed out after : $timeout seconds");
    exit(0);
  }
} ## end if ($@)

# run getall -sp cmd (SP performance)
if ( defined $spb_ip && $spb_ip ne '' ) {
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    @get_spb = `$naviseccli -h $spb_ip $sec_file_path getall -sp`;

    # end of alarm
    alarm(0);
  };

  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      my $act_time = localtime();
      error("command timed out after : $timeout seconds");
      exit(0);
    }
  } ## end if ($@)
} ## end if ( defined $spb_ip &&...)

# run getcache cmd for spb_ip
if ( defined $spb_ip && $spb_ip ne '' ) {
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    @get_cache_spb = `$naviseccli -h $spb_ip $sec_file_path getcache`;

    # end of alarm
    alarm(0);
  };

  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      my $act_time = localtime();
      error("command timed out after : $timeout seconds");
      exit(0);
    }
  } ## end if ($@)
} ## end if ( defined $spb_ip &&...)

# run health status cmd
eval {
  # Set alarm
  my $act_time = localtime();
  local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
  alarm($timeout);

  @health_status = `$naviseccli -h $spa_ip $sec_file_path faults -list`;

  # end of alarm
  alarm(0);
};

if ($@) {
  if ( $@ =~ /died in SIG ALRM/ ) {
    my $act_time = localtime();
    error("command timed out after : $timeout seconds");
    exit(0);
  }
} ## end if ($@)

# parse cmd output to hash
parse_get_all();
parse_lun_list();
parse_storagepool();
parse_get_spb();
parse_get_cache_spb();

# print perf file
print_perf_out();

# print conf file
my $config_html   = "$wrkdir/$storage_name/config.html";
my $cfg_time_diff = file_time_diff($config_html);

if ( !-f $config_html || $cfg_time_diff > 3599 ) {
  print_conf_out();
}

# Global health check
my $act_timestamp  = time();
my $main_state     = "OK";
my $state_suffix   = "ok";
my $component_name = $storage_name;
my $alert_status   = 0;

$component_name =~ s/\s+//g;

my $hs_state = grep {/The array is operating normally./} @health_status;

if ( $hs_state == 0 ) { $main_state = "NOT_OK"; $state_suffix = "nok"; }

if ( !-d "$tmpdir/health_status_summary" ) {
  mkdir( "$tmpdir/health_status_summary", 0755 ) || error( "Cannot mkdir $tmpdir/health_status_summary: $!" . __FILE__ . ":" . __LINE__ ) && exit;
}

if ( !-f "$tmpdir/health_status_summary/$component_name.ok" && !-f "$tmpdir/health_status_summary/$component_name.nok" ) {
  $alert_status = 1;
}

if ( -f "$tmpdir/health_status_summary/$component_name.ok" ) { unlink("$tmpdir/health_status_summary/$component_name.ok"); $alert_status = 1; }
if ( -f "$tmpdir/health_status_summary/$component_name.nok" ) { unlink("$tmpdir/health_status_summary/$component_name.nok"); }

if ( $state_suffix eq "nok" && $alert_status == 1 ) {
  ### alert health status
  if ( exists &Xorux_lib::health_status_alerting ) {
    Xorux_lib::health_status_alerting($component_name);
  }
} ## end if ( $state_suffix eq ...)

open( MAST, ">$tmpdir/health_status_summary/$component_name.$state_suffix" ) || error( "Couldn't open file $tmpdir/health_status_summary/$component_name.$state_suffix $!" . __FILE__ . ":" . __LINE__ ) && exit;
print MAST "STORAGE : $storage_name : $main_state : $act_timestamp\n";
close(MAST);

# health status
my $health_status_html = "$webdir/$storage_name/health_status.html";
my $hs_time_diff       = file_time_diff($health_status_html);

if ( !-f $health_status_html || $hs_time_diff > 599 ) {
  health_status($health_status_html);
}

#print Dumper \%data;
exit;

sub parse_get_cache_spb {
  foreach my $line (@get_cache_spb) {
    chomp $line;

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ /^Prct Dirty Cache Pages =/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /\s+=\s+/, $line );
      $data{'CACHE'}{$spb_ip}{$item} = $value;
    } ## end if ( $line =~ /^Prct Dirty Cache Pages =/)
    if ( $line =~ /^Prct Cache Pages Owned =/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /\s+=\s+/, $line );
      $data{'CACHE'}{$spb_ip}{$item} = $value;
    } ## end if ( $line =~ /^Prct Cache Pages Owned =/)
  } ## end foreach my $line (@get_cache_spb)
} ## end sub parse_get_cache_spb

sub parse_get_spb {

  # SP variables
  my $sp_name = "";

  foreach my $line (@get_spb) {
    chomp $line;

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ /^Storage Processor:/ ) {

      #print "$line\n";
      $sp_name = $line;
      $sp_name =~ s/^Storage Processor:\s+//;
    } ## end if ( $line =~ /^Storage Processor:/)
    if ( $line =~ /^Storage Processor IP Address:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Storage Processor IP Address:/)
    if ( $line =~ /^Statistics Logging:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Statistics Logging:/)
    if ( $line =~ /^Total Reads:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Total Reads:/)
    if ( $line =~ /^Total Writes:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Total Writes:/)
    if ( $line =~ /^Prct Busy:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Prct Busy:/)
    if ( $line =~ /^Prct Idle:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Prct Idle:/)
    if ( $line =~ /^Read_requests:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Read_requests:/)
    if ( $line =~ /^Write_requests:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Write_requests:/)
    if ( $line =~ /^Blocks_read:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Blocks_read:/)
    if ( $line =~ /^Blocks_written:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Blocks_written:/)
    if ( $line =~ /^Sum_queue_lengths_by_arrivals:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Sum_queue_lengths_by_arrivals:/)
    if ( $line =~ /^Arrivals_to_non_zero_queue:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Arrivals_to_non_zero_queue:/)
    if ( $line =~ /^Controller busy ticks:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Controller busy ticks:/)
    if ( $line =~ /^Controller idle ticks:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Controller idle ticks:/)
    if ( $line =~ /^Serial Number For The SP:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Serial Number For The SP:/)
    if ( $line =~ /^Write_cache_blocks_flushed:/ ) {

      #print "$line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{SP}{$sp_name}{$item} = $value;
    } ## end if ( $line =~ /^Write_cache_blocks_flushed:/)
  } ## end foreach my $line (@get_spb)
} ## end sub parse_get_spb

sub parse_storagepool {
  my $line_index = 0;
  my $last_line  = "";

  # LUN variables
  my $pool_id   = "";
  my $pool_name = "";

  foreach my $line (@storagepool) {
    chomp $line;
    $line_index++;

    #print "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ /^Pool Name:\s+/ ) {

      #print "$line\n";
      $pool_name = $line;
      $pool_name =~ s/^Pool Name:\s+//;

      $data{POOL}{$pool_name}{'Pool Name'} = $pool_name;
    } ## end if ( $line =~ /^Pool Name:\s+/)
    if ( $line =~ /^Pool ID:\s+/ ) {

      #print "$line\n";
      $pool_id = $line;
      $pool_id =~ s/^Pool ID:\s+//;

      $data{POOL}{$pool_name}{'Pool ID'} = $pool_id;
    } ## end if ( $line =~ /^Pool ID:\s+/)
    if ( $line =~ / Capacity / ) {

      #print "$pool_id : $pool_name : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{POOL}{$pool_name}{$item} = $value;
    } ## end if ( $line =~ / Capacity /)
    if ( $line =~ /^Percent Full:/ ) {

      #print "$pool_id : $pool_name : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{POOL}{$pool_name}{$item} = $value;
    } ## end if ( $line =~ /^Percent Full:/)
    if ( $line =~ /^LUNs:/ ) {

      #print "$pool_id : $pool_name : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{POOL}{$pool_name}{$item} = $value;
    } ## end if ( $line =~ /^LUNs:/)
    if ( $line =~ /^Raid Type:/ ) {

      #print "$pool_id : $pool_name : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{POOL}{$pool_name}{$item} = $value;
    } ## end if ( $line =~ /^Raid Type:/)

  } ## end foreach my $line (@storagepool)

} ## end sub parse_storagepool

sub parse_lun_list {
  my $line_index = 0;
  my $last_line  = "";

  # LUN variables
  my $lun_id = "";

  foreach my $line (@lun_list) {
    chomp $line;
    $line_index++;

    #print "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ /^LOGICAL UNIT NUMBER / ) {

      #print "$line\n";
      $lun_id = $line;
      $lun_id =~ s/^LOGICAL UNIT NUMBER //;

      #$data{LUN}{$lun_id}{'LUN_LIST'}{LUN_ID} = $lun_id;
    } ## end if ( $line =~ /^LOGICAL UNIT NUMBER /)

    #if ( $line =~ /^Name:/ ) {
    #  #print "$lun_id : $line\n";
    #  my ($item, $value) = split(/:\s+/, $line);
    #  $data{LUN}{$lun_id}{'LUN_LIST'}{$item} = $value;
    #}
    if ( $line =~ /^Read Requests:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Read Requests:/)
    if ( $line =~ /^Write Requests:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Write Requests:/)
    if ( $line =~ /^Blocks Read:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Blocks Read:/)
    if ( $line =~ /^Blocks Written:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Blocks Written:/)
    if ( $line =~ /^Pool Name:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Pool Name:/)
    if ( $line =~ /^Busy Ticks:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Busy Ticks:/)
    if ( $line =~ /^Idle Ticks:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Idle Ticks:/)
    if ( $line =~ /^Read Requests SP A:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value_line ) = split( /:\s+/, $line );
      my ( $value, undef ) = split( /\s+/, $value_line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Read Requests SP A:/)
    if ( $line =~ /^Read Requests SP B:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value_line ) = split( /:\s+/, $line );
      my ( $value, undef ) = split( /\s+/, $value_line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Read Requests SP B:/)
    if ( $line =~ /^Write Requests SP A:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value_line ) = split( /:\s+/, $line );
      my ( $value, undef ) = split( /\s+/, $value_line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Write Requests SP A:/)
    if ( $line =~ /^Write Requests SP B:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value_line ) = split( /:\s+/, $line );
      my ( $value, undef ) = split( /\s+/, $value_line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Write Requests SP B:/)
    if ( $line =~ /^Sum of Outstanding Requests:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Sum of Outstanding Requests:/)
    if ( $line =~ /^Non-Zero Request Count Arrivals:/ ) {

      #print "$lun_id : $line\n";
      my ( $item, $value ) = split( /:\s+/, $line );
      $data{LUN}{$lun_id}{$item} = $value;
    } ## end if ( $line =~ /^Non-Zero Request Count Arrivals:/)

    #if ( $line =~ /^Current Owner:/ ) {
    #  #print "$lun_id : $line\n";
    #  my ($item, $value) = split(/:\s+/, $line);
    #  $data{LUN}{$lun_id}{'LUN_LIST'}{$item} = $value;
    #}
    #if ( $line =~ /^UID:/ ) {
    #  #print "$lun_id : $line\n";
    #  my ($item, $value) = split(/:\s+/, $line);
    #  $data{LUN}{$lun_id}{'LUN_LIST'}{$item} = $value;
    #}

    $last_line = $line;
  } ## end foreach my $line (@lun_list)

} ## end sub parse_lun_list

sub parse_get_all {
  my $line_index  = 0;
  my $last_line   = "";
  my $act_section = "";    # actual section (e.g.: SP Information, All logical Units Information, All RAID Groups Information, ... )

  # LUN variables
  my $lun_id = "";

  # Host variables
  my $storage_group_name = "";

  # Raid Group variables
  my $rg_id = "";

  # Port variables
  my $sub_section_for_port = 0;    #Information about each SPPORT:
  my $port_SP              = "";
  my $port_id              = "";

  # Host variables
  my $sub_section_for_host = 0;    #Information about each SPPORT:
  my $host_name            = "";
  my $host_sp              = "";

  # Disk variables
  my $disk_id = "";

  # SP variables
  my $sp_name = "";

  foreach my $line (@get_all) {
    chomp $line;
    $line_index++;

    #print "$line\n";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ /^-+$/ ) {

      #print "$last_line\n";
      #print "$line\n";
      $act_section = $last_line;

      #print "\n";
      #print "$line_index : $act_section\n";
    } ## end if ( $line =~ /^-+$/ )

    # ALL sections:
    # Agent/Host Information
    # Array Information
    # HBA Information
    # SP Information
    # Cache Information
    # All Disks Information
    # All logical Units Information
    # All RAID Groups Information
    # All Storage Groups Information
    # Resume Information
    # Reserved LUN Pool Information
    # Snap View Information
    # SAN Copy Information
    # SAN Copy Destination Information
    # Mirror View Information
    # Clone Information
    # MetaLUN Information
    # Migration Information
    # FAST Cache Information
    # Backendbus Information

    # section: Agent/Host Information
    if ( $act_section eq "Agent/Host Information" ) {
      #Model:               VNX5300
      #Model Type:          Rackmount
      #Prom Rev:            7.20.00
      #Serial No:           CKM00112101169
      if ( $line =~ /^Model:/ ) {
        $line =~ s/^Model:\s+//;
        $data{'ARRAY'}{'model'} = $line;
      }
      if ( $line =~ /^Model Type:/ ) {
        $line =~ s/^Model Type:\s+//;
        $data{'ARRAY'}{'model_type'} = $line;
      }
      if ( $line =~ /^Serial No:/ ) {
        $line =~ s/^Serial No:\s+//;
        $data{'ARRAY'}{'serial'} = $line;
      }
    } ## end if ( $act_section eq "Agent/Host Information")

    # section: SP Information
    if ( $act_section eq "SP Information" ) {
      if ( $line =~ /^Storage Processor:/ ) {

        #print "$line\n";
        $sp_name = $line;
        $sp_name =~ s/^Storage Processor:\s+//;
      } ## end if ( $line =~ /^Storage Processor:/)
      if ( $line =~ /^Storage Processor IP Address:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Storage Processor IP Address:/)
      if ( $line =~ /^Statistics Logging:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Statistics Logging:/)
      if ( $line =~ /^Total Reads:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Total Reads:/)
      if ( $line =~ /^Total Writes:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Total Writes:/)
      if ( $line =~ /^Prct Busy:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Prct Busy:/)
      if ( $line =~ /^Prct Idle:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Prct Idle:/)
      if ( $line =~ /^Read_requests:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Read_requests:/)
      if ( $line =~ /^Write_requests:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Write_requests:/)
      if ( $line =~ /^Blocks_read:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Blocks_read:/)
      if ( $line =~ /^Blocks_written:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Blocks_written:/)
      if ( $line =~ /^Sum_queue_lengths_by_arrivals:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Sum_queue_lengths_by_arrivals:/)
      if ( $line =~ /^Arrivals_to_non_zero_queue:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Arrivals_to_non_zero_queue:/)
      if ( $line =~ /^Controller busy ticks:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Controller busy ticks:/)
      if ( $line =~ /^Controller idle ticks:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Controller idle ticks:/)
      if ( $line =~ /^Serial Number For The SP:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Serial Number For The SP:/)
      if ( $line =~ /^Write_cache_blocks_flushed:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{SP}{$sp_name}{$item} = $value;
      } ## end if ( $line =~ /^Write_cache_blocks_flushed:/)
    } ## end if ( $act_section eq "SP Information")

    # section: HBA Information
    if ( $act_section eq "HBA Information" ) {

      # HOSTs and Storage groups
      if ( $line =~ /^Information about each HBA:/ ) {    # start subsection for Hosts
        $sub_section_for_host++;
      }
      if ( $line =~ /^Server Name:/ && $sub_section_for_host > 0 ) {

        #print "$sub_section_for_host : $line\n";
        $host_name = $line;
        $host_name =~ s/^Server Name:\s+//;
      } ## end if ( $line =~ /^Server Name:/...)
      if ( $line =~ /^SP Name:/ && $sub_section_for_host > 0 ) {

        #print "$sub_section_for_host : $line\n";
        $host_sp = $line;
        $host_sp =~ s/^SP Name:\s+//;
      } ## end if ( $line =~ /^SP Name:/...)
      if ( $line =~ /^StorageGroup Name:/ && $sub_section_for_host > 0 ) {

        #print "$sub_section_for_host : $line\n";
        my $st_group = $line;
        $st_group =~ s/^StorageGroup Name:\s+//;

        $data{'Storage Group'}{$st_group}{'hosts'}{$host_name}{$host_sp} = $host_sp;
      } ## end if ( $line =~ /^StorageGroup Name:/...)

      #PORTs
      if ( $line =~ /^Information about each SPPORT:/ ) {    # start subsection for port informations
        $sub_section_for_port = 1;
        $sub_section_for_host = 0;
      }
      if ( $line =~ /^SP Name:/ && $sub_section_for_port == 1 ) {

        #print "\n$line\n";
        $port_SP = $line;
        $port_SP =~ s/^SP Name:\s+//;
      } ## end if ( $line =~ /^SP Name:/...)
      if ( $line =~ /^SP Port ID:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        $port_id = $line;
        $port_id =~ s/^SP Port ID:\s+//;
        $data{PORT}{$port_SP}{$port_id}{'SP Port ID'} = $port_id;
      } ## end if ( $line =~ /^SP Port ID:/...)
      if ( $line =~ /^SP UID:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my $value = $line;
        $value =~ s/^SP UID:\s+//;
        $data{PORT}{$port_SP}{$port_id}{'SP UID'} = $value;
      } ## end if ( $line =~ /^SP UID:/...)
      if ( $line =~ /^Link Status:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Link Status:/...)
      if ( $line =~ /^Port Status:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Port Status:/...)
      if ( $line =~ /^Speed Value :/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( / :\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Speed Value :/...)
      if ( $line =~ /^SFP State:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^SFP State:/...)
      if ( $line =~ /^Reads:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Reads:/...)
      if ( $line =~ /^Writes:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Writes:/...)
      if ( $line =~ /^Blocks Read:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Blocks Read:/...)
      if ( $line =~ /^Blocks Written:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Blocks Written:/...)
      if ( $line =~ /^I\/O Module Slot:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^I\/O Module Slot:/...)
      if ( $line =~ /^Physical Port ID:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Physical Port ID:/...)
      if ( $line =~ /^Usage:/ && $sub_section_for_port == 1 ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{PORT}{$port_SP}{$port_id}{$item} = $value;
      } ## end if ( $line =~ /^Usage:/...)

    } ## end if ( $act_section eq "HBA Information")

    # section: All Disks Information
    if ( $act_section eq "All Disks Information" ) {
      if ( $line =~ /^Bus / ) {

        #print "$line\n";
        my ( undef, $bus, undef, $enclosure, undef, $disk ) = split( /\s+/, $line );
        $disk_id = "$bus-$enclosure-$disk";

        $data{DISK}{$disk_id}{Location} = $line;
        $data{DISK}{$disk_id}{ID}       = $disk_id;

        #print "\n$disk_id\n";
      } ## end if ( $line =~ /^Bus / )
      if ( $line =~ /^State:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^State:/)
      if ( $line =~ /^Vendor Id:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Vendor Id:/)
      if ( $line =~ /^Lun:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Lun:/ )
      if ( $line =~ /^Capacity:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Capacity:/)
      if ( $line =~ /^Number of Reads:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Number of Reads:/)
      if ( $line =~ /^Number of Writes:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Number of Writes:/)
      if ( $line =~ /^Number of Luns:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Number of Luns:/)
      if ( $line =~ /^Raid Group ID:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Raid Group ID:/)
      if ( $line =~ /^Read Requests:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Read Requests:/)
      if ( $line =~ /^Write Requests:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Write Requests:/)
      if ( $line =~ /^Kbytes Read:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Kbytes Read:/)
      if ( $line =~ /^Kbytes Written:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Kbytes Written:/)
      if ( $line =~ /^Drive Type:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Drive Type:/)
      if ( $line =~ /^User Capacity:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^User Capacity:/)
      if ( $line =~ /^Idle Ticks:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Idle Ticks:/)
      if ( $line =~ /^Busy Ticks:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Busy Ticks:/)
      if ( $line =~ /^Current Speed:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Current Speed:/)
      if ( $line =~ /^Prct Idle:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Prct Idle:/)
      if ( $line =~ /^Prct Busy:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Prct Busy:/)
      if ( $line =~ /^Queue Length:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );

        $data{DISK}{$disk_id}{$item} = $value;
      } ## end if ( $line =~ /^Queue Length:/)
    } ## end if ( $act_section eq "All Disks Information")

    # section: All logical Units Information
    if ( $act_section eq "All logical Units Information" ) {
      if ( $line =~ /^LOGICAL UNIT NUMBER / ) {

        #print "$line\n";
        $lun_id = $line;
        $lun_id =~ s/^LOGICAL UNIT NUMBER //;

        $data{LUN}{$lun_id}{LUN_ID} = $lun_id;
      } ## end if ( $line =~ /^LOGICAL UNIT NUMBER /)
      if ( $line =~ /^Name/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /   \s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Name/ )
      if ( $line =~ /^Read Requests:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Read Requests:/)
      if ( $line =~ /^Write Requests:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Write Requests:/)
      if ( $line =~ /^Blocks read:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Blocks read:/)
      if ( $line =~ /^Blocks written:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Blocks written:/)
      if ( $line =~ /^Read cache hits:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Read cache hits:/)
      if ( $line =~ /^Write cache hits:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Write cache hits:/)
      if ( $line =~ /^Read Hit Ratio:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Read Hit Ratio:/)
      if ( $line =~ /^Write Hit Ratio:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Write Hit Ratio:/)
      if ( $line =~ /^RAIDGroup ID:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^RAIDGroup ID:/)
      if ( $line =~ /^Current owner:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Current owner:/)
      if ( $line =~ /^LUN Capacity\(Megabytes\):/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^LUN Capacity\(Megabytes\):/)
      if ( $line =~ /^LUN Capacity\(Blocks\):/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^LUN Capacity\(Blocks\):/)
      if ( $line =~ /^UID:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^UID:/ )
      if ( $line =~ /^LUN Busy Ticks SPA:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );

        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^LUN Busy Ticks SPA:/)
      if ( $line =~ /^LUN Busy Ticks SPB:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );

        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^LUN Busy Ticks SPB:/)
      if ( $line =~ /^LUN Idle Ticks SPA:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );

        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^LUN Idle Ticks SPA:/)
      if ( $line =~ /^LUN Idle Ticks SPB:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );

        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^LUN Idle Ticks SPB:/)
      if ( $line =~ /^Average Read Time:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Average Read Time:/)
      if ( $line =~ /^Average Write Time:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Average Write Time:/)
      if ( $line =~ /^Read Requests SPA:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Read Requests SPA:/)
      if ( $line =~ /^Read Requests SPB:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Read Requests SPB:/)
      if ( $line =~ /^Write Requests SPA:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Write Requests SPA:/)
      if ( $line =~ /^Write Requests SPB:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value_line ) = split( /:\s+/, $line );
        my ( $value, undef ) = split( /\s+/, $value_line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Write Requests SPB:/)
      if ( $line =~ /^Sum of Oustanding Requests:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Sum of Oustanding Requests:/)
      if ( $line =~ /^Non-zero Request Count Arrivals:/ ) {

        #print "$lun_id : $line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{LUN}{$lun_id}{$item} = $value;
      } ## end if ( $line =~ /^Non-zero Request Count Arrivals:/)
    } ## end if ( $act_section eq "All logical Units Information")

    # section: All RAID Groups Information
    if ( $act_section eq "All RAID Groups Information" ) {
      if ( $line =~ /^RaidGroup ID:/ ) {

        #print "$line\n";

        $rg_id = $line;
        $rg_id =~ s/^RaidGroup ID:\s+//;

        my ( $item, $value ) = split( /:\s+/, $line );
        $data{RG}{$rg_id}{$item} = $value;
      } ## end if ( $line =~ /^RaidGroup ID:/)
      if ( $line =~ /^RaidGroup Type:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{RG}{$rg_id}{$item} = $value;
      } ## end if ( $line =~ /^RaidGroup Type:/)
      if ( $line =~ /^List of luns:/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{RG}{$rg_id}{$item} = $value;
      } ## end if ( $line =~ /^List of luns:/)
      if ( $line =~ /^Raw Capacity/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{RG}{$rg_id}{$item} = $value;
      } ## end if ( $line =~ /^Raw Capacity/)
      if ( $line =~ /^Logical Capacity/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{RG}{$rg_id}{$item} = $value;
      } ## end if ( $line =~ /^Logical Capacity/)
      if ( $line =~ /^Free Capacity/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /:\s+/, $line );
        $data{RG}{$rg_id}{$item} = $value;
      } ## end if ( $line =~ /^Free Capacity/)
    } ## end if ( $act_section eq "All RAID Groups Information")

    # section: All Storage Groups Information
    if ( $act_section eq "All Storage Groups Information" ) {
      if ( $line =~ /^Storage Group Name:/ ) {

        #print "\n\n$line\n";

        $storage_group_name = $line;
        $storage_group_name =~ s/^Storage Group Name:\s+//;

        #my ($item, $value) = split(/:\s+/, $line);
        #$data{Storage_group}{$storage_group_name}{$item} = $value;
      } ## end if ( $line =~ /^Storage Group Name:/)

      #if ( $line =~ /^Storage Group UID:/ ) {
      #  print "$line\n";
      #  my ($item, $value) = split(/:\s+/, $line);
      #  $data{Storage_group}{$storage_group_name}{$item} = $value;
      #}
      if ( $line =~ /^[0-9]+\s+[0-9]+$/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /\s+/, $line );
        $data{'Storage Group'}{$storage_group_name}{luns}{$value} = $value;
      } ## end if ( $line =~ /^[0-9]+\s+[0-9]+$/)
    } ## end if ( $act_section eq "All Storage Groups Information")

    # section: Cache Information
    if ( $act_section eq "Cache Information" ) {
      if ( $line =~ /^Prct Dirty Cache Pages =/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /\s+=\s+/, $line );
        $data{'CACHE'}{$spa_ip}{$item} = $value;
      } ## end if ( $line =~ /^Prct Dirty Cache Pages =/)
      if ( $line =~ /^Prct Cache Pages Owned =/ ) {

        #print "$line\n";
        my ( $item, $value ) = split( /\s+=\s+/, $line );
        $data{'CACHE'}{$spa_ip}{$item} = $value;
      } ## end if ( $line =~ /^Prct Cache Pages Owned =/)
    } ## end if ( $act_section eq "Cache Information")

    $last_line = $line;
  } ## end foreach my $line (@get_all)

  return 1;
} ## end sub parse_get_all

sub print_perf_out {
  open( PERF, ">$perf_out" ) || error( "Couldn't open file $perf_out $!" . __FILE__ . ":" . __LINE__ ) && exit;

  # print LUN
  print PERF "Volume Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "Name,lun_id,Current_owner,RG_ID,Pool name,LUN_Capacity (MB),LUN_Capacity (Blocks),Read_Requests,Write_Requests,Blocks_read,Blocks_written,Read cache hits,Write cache hits,Read Hit Ratio,Write Hit Ratio,LUN_Capacity (MB),read (kB),write (kB),io rate,data rate,busy_ticks,idle_ticks,Average Read Time,Average Write Time,Read Requests SP A,Read Requests SP B,Write Requests SP A,Write Requests SP B,io_rate_spa,io_rate_spb,Non-zero Request Count Arrivals,Sum of Outstanding Requests\n";

  foreach my $lun_id ( sort { $a <=> $b } keys( %{ $data{LUN} } ) ) {
    my $Name = "";

    #my $UID                 = "";
    my $RG_ID               = "";
    my $pool_name           = "";
    my $Current_owner       = "";
    my $Read_Requests       = "";
    my $Write_Requests      = "";
    my $LUN_Capacity_MB     = "";
    my $LUN_Capacity_Blocks = "";
    my $Blocks_read         = "";
    my $Blocks_written      = "";
    my $Read_cache_hits     = "";
    my $Write_cache_hits    = "";
    my $Read_Hit_Ratio      = "";
    my $Write_Hit_Ratio     = "";

    my $Capacity_MB = "";
    my $read_kB     = "";
    my $write_kB    = "";
    my $io_rate     = "";
    my $data_rate   = "";
    my $busy_ticks  = "";
    my $idle_ticks  = "";
    my $resp_t_r    = "";
    my $resp_t_w    = "";

    my $read_io_spa  = "";
    my $read_io_spb  = "";
    my $write_io_spa = "";
    my $write_io_spb = "";
    my $io_rate_spa  = "";
    my $io_rate_spb  = "";

    my $arrivals = "";
    my $sum_que  = "";

    if ( defined $data{LUN}{$lun_id}{'Name'} && $data{LUN}{$lun_id}{'Name'} ne '' ) { $Name = $data{LUN}{$lun_id}{'Name'}; }
    if ( $Name =~ /Hot Spare LUN/ ) { next; }

    #if ( defined $data{LUN}{$lun_id}{'UID'} && $data{LUN}{$lun_id}{'UID'} ne '' ) { $UID = $data{LUN}{$lun_id}{'UID'}; }
    if ( defined $data{LUN}{$lun_id}{'Current owner'} && $data{LUN}{$lun_id}{'Current owner'} ne '' ) { $Current_owner = $data{LUN}{$lun_id}{'Current owner'}; }

    if ( defined $data{LUN}{$lun_id}{'RAIDGroup ID'} && $data{LUN}{$lun_id}{'RAIDGroup ID'} ne '' && isdigit( $data{LUN}{$lun_id}{'RAIDGroup ID'} ) ) {
      $RG_ID = $data{LUN}{$lun_id}{'RAIDGroup ID'};
    }
    if ( defined $data{LUN}{$lun_id}{'Pool Name'} && $data{LUN}{$lun_id}{'Pool Name'} ne '' ) {
      $pool_name = $data{LUN}{$lun_id}{'Pool Name'};
    }
    if ( defined $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'} && $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'} ) ) {
      $LUN_Capacity_MB = $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'};
    }
    if ( defined $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'} && $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'} ) ) {
      $LUN_Capacity_Blocks = $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'};
      $Capacity_MB         = $LUN_Capacity_Blocks / $block_size;            # from block to MB (1 block = 2048 MB)
      $Capacity_MB         = sprintf( '%.0f', $Capacity_MB );
    }
    if ( defined $data{LUN}{$lun_id}{'Read Requests'} && $data{LUN}{$lun_id}{'Read Requests'} ne '' && isdigit( $data{LUN}{$lun_id}{'Read Requests'} ) ) {
      $Read_Requests = $data{LUN}{$lun_id}{'Read Requests'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write Requests'} && $data{LUN}{$lun_id}{'Write Requests'} ne '' && isdigit( $data{LUN}{$lun_id}{'Write Requests'} ) ) {
      $Write_Requests = $data{LUN}{$lun_id}{'Write Requests'};
    }
    if ( defined $data{LUN}{$lun_id}{'Blocks read'} && $data{LUN}{$lun_id}{'Blocks read'} ne '' && isdigit( $data{LUN}{$lun_id}{'Blocks read'} ) ) {
      $Blocks_read = $data{LUN}{$lun_id}{'Blocks read'};
      $read_kB     = $Blocks_read / $block_size;                            # from block to MB (1 block = 2048 MB)
      $read_kB     = $read_kB * 1024;                                       # from MB to kB
      $read_kB     = sprintf( '%.0f', $read_kB );
    } ## end if ( defined $data{LUN...})
    if ( defined $data{LUN}{$lun_id}{'Blocks written'} && $data{LUN}{$lun_id}{'Blocks written'} ne '' && isdigit( $data{LUN}{$lun_id}{'Blocks written'} ) ) {
      $Blocks_written = $data{LUN}{$lun_id}{'Blocks written'};
      $write_kB       = $Blocks_written / $block_size;                      # from block to MB (1 block = 2048 MB)
      $write_kB       = $write_kB * 1024;                                   # from MB to kB
      $write_kB       = sprintf( '%.0f', $write_kB );
    } ## end if ( defined $data{LUN...})
    if ( defined $data{LUN}{$lun_id}{'Blocks Read'} && $data{LUN}{$lun_id}{'Blocks Read'} ne '' && isdigit( $data{LUN}{$lun_id}{'Blocks Read'} ) ) {
      $Blocks_read = $data{LUN}{$lun_id}{'Blocks Read'};
      $read_kB     = $Blocks_read / $block_size;                            # from block to MB (1 block = 2048 MB)
      $read_kB     = $read_kB * 1024;                                       # from MB to kB
      $read_kB     = sprintf( '%.0f', $read_kB );
    } ## end if ( defined $data{LUN...})
    if ( defined $data{LUN}{$lun_id}{'Blocks Written'} && $data{LUN}{$lun_id}{'Blocks Written'} ne '' && isdigit( $data{LUN}{$lun_id}{'Blocks Written'} ) ) {
      $Blocks_written = $data{LUN}{$lun_id}{'Blocks Written'};
      $write_kB       = $Blocks_written / $block_size;                      # from block to MB (1 block = 2048 MB)
      $write_kB       = $write_kB * 1024;                                   # from MB to kB
      $write_kB       = sprintf( '%.0f', $write_kB );
    } ## end if ( defined $data{LUN...})
    if ( defined $data{LUN}{$lun_id}{'Read cache hits'} && $data{LUN}{$lun_id}{'Read cache hits'} ne '' && isdigit( $data{LUN}{$lun_id}{'Read cache hits'} ) ) {
      $Read_cache_hits = $data{LUN}{$lun_id}{'Read cache hits'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write cache hits'} && $data{LUN}{$lun_id}{'Write cache hits'} ne '' && isdigit( $data{LUN}{$lun_id}{'Write cache hits'} ) ) {
      $Write_cache_hits = $data{LUN}{$lun_id}{'Write cache hits'};
    }
    if ( defined $data{LUN}{$lun_id}{'Read Hit Ratio'} && $data{LUN}{$lun_id}{'Read Hit Ratio'} ne '' && isdigit( $data{LUN}{$lun_id}{'Read Hit Ratio'} ) ) {
      $Read_Hit_Ratio = $data{LUN}{$lun_id}{'Read Hit Ratio'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write Hit Ratio'} && $data{LUN}{$lun_id}{'Write Hit Ratio'} ne '' && isdigit( $data{LUN}{$lun_id}{'Write Hit Ratio'} ) ) {
      $Write_Hit_Ratio = $data{LUN}{$lun_id}{'Write Hit Ratio'};
    }
    if ( defined $data{LUN}{$lun_id}{'Busy Ticks'} && $data{LUN}{$lun_id}{'Busy Ticks'} ne '' && isdigit( $data{LUN}{$lun_id}{'Busy Ticks'} ) ) {
      $busy_ticks = $data{LUN}{$lun_id}{'Busy Ticks'};

      #print "$lun_id : busy_ticks=$busy_ticks\n";
    }
    if ( defined $data{LUN}{$lun_id}{'Idle Ticks'} && $data{LUN}{$lun_id}{'Idle Ticks'} ne '' && isdigit( $data{LUN}{$lun_id}{'Idle Ticks'} ) ) {
      $idle_ticks = $data{LUN}{$lun_id}{'Idle Ticks'};

      #print "$lun_id : idle_ticks=$idle_ticks\n";
    }
    if ( defined $data{LUN}{$lun_id}{'LUN Busy Ticks SPA'} && $data{LUN}{$lun_id}{'LUN Busy Ticks SPA'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Busy Ticks SPA'} ) && defined $data{LUN}{$lun_id}{'LUN Busy Ticks SPB'} && $data{LUN}{$lun_id}{'LUN Busy Ticks SPB'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Busy Ticks SPB'} ) ) {
      $busy_ticks = $data{LUN}{$lun_id}{'LUN Busy Ticks SPA'} + $data{LUN}{$lun_id}{'LUN Busy Ticks SPB'};

      #print "$lun_id : busy_ticks=$busy_ticks : $data{LUN}{$lun_id}{'LUN Busy Ticks SPA'} + $data{LUN}{$lun_id}{'LUN Busy Ticks SPB'} = $busy_ticks\n";
    }
    if ( defined $data{LUN}{$lun_id}{'LUN Idle Ticks SPA'} && $data{LUN}{$lun_id}{'LUN Idle Ticks SPA'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Idle Ticks SPA'} ) && defined $data{LUN}{$lun_id}{'LUN Idle Ticks SPB'} && $data{LUN}{$lun_id}{'LUN Idle Ticks SPB'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Idle Ticks SPB'} ) ) {
      $idle_ticks = $data{LUN}{$lun_id}{'LUN Idle Ticks SPA'} + $data{LUN}{$lun_id}{'LUN Idle Ticks SPB'};

      #print "$lun_id : busy_ticks=$idle_ticks : $data{LUN}{$lun_id}{'LUN Idle Ticks SPA'} + $data{LUN}{$lun_id}{'LUN Idle Ticks SPB'} = $idle_ticks\n";
    }
    if ( defined $data{LUN}{$lun_id}{'Average Read Time'} && $data{LUN}{$lun_id}{'Average Read Time'} ne '' && isdigit( $data{LUN}{$lun_id}{'Average Read Time'} ) ) {
      $resp_t_r = $data{LUN}{$lun_id}{'Average Read Time'};
      if ( $resp_t_r == 0 ) { $resp_t_r = ""; }    # Pool luns have not response times counters. They have always 0.
    }
    if ( defined $data{LUN}{$lun_id}{'Average Write Time'} && $data{LUN}{$lun_id}{'Average Write Time'} ne '' && isdigit( $data{LUN}{$lun_id}{'Average Write Time'} ) ) {
      $resp_t_w = $data{LUN}{$lun_id}{'Average Write Time'};
      if ( $resp_t_w == 0 ) { $resp_t_w = ""; }    # Pool luns have not response times counters. They have always 0.
    }

    #sum io rate
    if ( isdigit($Read_Requests) && isdigit($Write_Requests) ) {
      $io_rate = $Read_Requests + $Write_Requests;
      $io_rate = sprintf( '%.0f', $io_rate );
    }

    #sum data rate
    if ( isdigit($read_kB) && isdigit($write_kB) ) {
      $data_rate = $read_kB + $write_kB;
      $data_rate = sprintf( '%.0f', $data_rate );
    }

    if ( defined $data{LUN}{$lun_id}{'Read Requests SP A'} && $data{LUN}{$lun_id}{'Read Requests SP A'} ne '' ) {
      $read_io_spa = $data{LUN}{$lun_id}{'Read Requests SP A'};
    }
    if ( defined $data{LUN}{$lun_id}{'Read Requests SPA'} && $data{LUN}{$lun_id}{'Read Requests SPA'} ne '' ) {
      $read_io_spa = $data{LUN}{$lun_id}{'Read Requests SPA'};
    }
    if ( defined $data{LUN}{$lun_id}{'Read Requests SP B'} && $data{LUN}{$lun_id}{'Read Requests SP B'} ne '' ) {
      $read_io_spb = $data{LUN}{$lun_id}{'Read Requests SP B'};
    }
    if ( defined $data{LUN}{$lun_id}{'Read Requests SPB'} && $data{LUN}{$lun_id}{'Read Requests SPB'} ne '' ) {
      $read_io_spb = $data{LUN}{$lun_id}{'Read Requests SPB'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write Requests SP A'} && $data{LUN}{$lun_id}{'Write Requests SP A'} ne '' ) {
      $write_io_spa = $data{LUN}{$lun_id}{'Write Requests SP A'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write Requests SPA'} && $data{LUN}{$lun_id}{'Write Requests SPA'} ne '' ) {
      $write_io_spa = $data{LUN}{$lun_id}{'Write Requests SPA'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write Requests SP B'} && $data{LUN}{$lun_id}{'Write Requests SP B'} ne '' ) {
      $write_io_spb = $data{LUN}{$lun_id}{'Write Requests SP B'};
    }
    if ( defined $data{LUN}{$lun_id}{'Write Requests SPB'} && $data{LUN}{$lun_id}{'Write Requests SPB'} ne '' ) {
      $write_io_spb = $data{LUN}{$lun_id}{'Write Requests SPB'};
    }

    #sum io rate spa
    if ( isdigit($read_io_spa) && isdigit($write_io_spa) ) {
      $io_rate_spa = $read_io_spa + $write_io_spa;
      $io_rate_spa = sprintf( '%.0f', $io_rate_spa );
    }

    #sum io rate spb
    if ( isdigit($read_io_spb) && isdigit($write_io_spb) ) {
      $io_rate_spb = $read_io_spb + $write_io_spb;
      $io_rate_spb = sprintf( '%.0f', $io_rate_spb );
    }
    if ( defined $data{LUN}{$lun_id}{'Non-zero Request Count Arrivals'} && $data{LUN}{$lun_id}{'Non-zero Request Count Arrivals'} ne '' ) {
      $arrivals = $data{LUN}{$lun_id}{'Non-zero Request Count Arrivals'};
    }
    if ( defined $data{LUN}{$lun_id}{'Non-Zero Request Count Arrivals'} && $data{LUN}{$lun_id}{'Non-Zero Request Count Arrivals'} ne '' ) {
      $arrivals = $data{LUN}{$lun_id}{'Non-Zero Request Count Arrivals'};
    }
    if ( defined $data{LUN}{$lun_id}{'Sum of Oustanding Requests'} && $data{LUN}{$lun_id}{'Sum of Oustanding Requests'} ne '' ) {
      $sum_que = $data{LUN}{$lun_id}{'Sum of Oustanding Requests'};
    }
    if ( defined $data{LUN}{$lun_id}{'Sum of Outstanding Requests'} && $data{LUN}{$lun_id}{'Sum of Outstanding Requests'} ne '' ) {
      $sum_que = $data{LUN}{$lun_id}{'Sum of Outstanding Requests'};
    }

    print PERF "$Name,$lun_id,$Current_owner,$RG_ID,$pool_name,$LUN_Capacity_MB,$LUN_Capacity_Blocks,$Read_Requests,$Write_Requests,$Blocks_read,$Blocks_written,$Read_cache_hits,$Write_cache_hits,$Read_Hit_Ratio,$Write_Hit_Ratio,$Capacity_MB,$read_kB,$write_kB,$io_rate,$data_rate,$busy_ticks,$idle_ticks,$resp_t_r,$resp_t_w,$read_io_spa,$read_io_spb,$write_io_spa,$write_io_spb,$io_rate_spa,$io_rate_spb,$arrivals,$sum_que\n";
  } ## end foreach my $lun_id ( sort {...})

  # print RG
  print PERF "\nRaid-Group Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "RG_ID,type,Capacity (MB),Capacity (Blocks),Free Capacity (MB),Free Capacity (Blocks),Luns,capacity (TB),capacity free (TB),capacity used (TB)\n";

  foreach my $RG_id ( sort { $a <=> $b } keys( %{ $data{RG} } ) ) {
    my $RG_type             = "";
    my $capacity_mb         = "";
    my $capacity_block      = "";
    my $free_capacity_mb    = "";
    my $free_capacity_block = "";
    my $list_of_luns        = "";
    my $cap_TB              = "";
    my $cap_free_TB         = "";
    my $cap_used_TB         = "";

    if ( defined $data{RG}{$RG_id}{'RaidGroup Type'} && $data{RG}{$RG_id}{'RaidGroup Type'} ne '' ) { $RG_type = $data{RG}{$RG_id}{'RaidGroup Type'}; }
    if ( $RG_type =~ /hot_spare/ ) { next; }

    if ( defined $data{RG}{$RG_id}{'Logical Capacity (Blocks)'} && isdigit( $data{RG}{$RG_id}{'Logical Capacity (Blocks)'} ) ) {
      $capacity_block = $data{RG}{$RG_id}{'Logical Capacity (Blocks)'};
      $capacity_mb    = $capacity_block / 2048;

      $cap_TB = $capacity_block / $block_size;    # from block to MB (1 block = 2048 MB)
      $cap_TB = $cap_TB / 1024;                   # from MB to GB
      $cap_TB = $cap_TB / 1024;                   # from GB to TB
                                                  #$cap_TB  = sprintf('%.0f', $cap_TB);
    } ## end if ( defined $data{RG}...)
    if ( defined $data{RG}{$RG_id}{'Free Capacity (Blocks,non-contiguous)'} && isdigit( $data{RG}{$RG_id}{'Free Capacity (Blocks,non-contiguous)'} ) ) {
      $free_capacity_block = $data{RG}{$RG_id}{'Free Capacity (Blocks,non-contiguous)'};
      $free_capacity_mb    = $free_capacity_block / 2048;

      $cap_free_TB = $free_capacity_block / $block_size;    # from block to MB (1 block = 2048 MB)
      $cap_free_TB = $cap_free_TB / 1024;                   # from MB to GB
      $cap_free_TB = $cap_free_TB / 1024;                   # from GB to TB
                                                            #$cap_free_TB  = sprintf('%.0f', $cap_free_TB);
    } ## end if ( defined $data{RG}...)
    if ( defined $data{RG}{$RG_id}{'List of luns'} && $data{RG}{$RG_id}{'List of luns'} ne '' ) {
      $list_of_luns = $data{RG}{$RG_id}{'List of luns'};
    }

    # sum used capacity from total and free
    if ( defined $cap_TB && isdigit($cap_TB) && defined $cap_free_TB && isdigit($cap_free_TB) ) {
      $cap_used_TB = $cap_TB - $cap_free_TB;
    }

    print PERF "$RG_id,$RG_type,$capacity_mb,$capacity_block,$free_capacity_mb,$free_capacity_block,$list_of_luns,$cap_TB,$cap_free_TB,$cap_used_TB\n";
  } ## end foreach my $RG_id ( sort { ...})

  # print POOL
  print PERF "\nPool Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "Pool name,Pool ID,Raid Type,Percent Full,Available Capacity (GBs),Available Capacity (Blocks),Consumed Capacity (GBs),Consumed Capacity (Blocks),User Capacity (GBs),User Capacity (Blocks),Luns,capacity (TB),capacity free (TB),capacity used (TB)\n";

  foreach my $pool_name ( keys( %{ $data{POOL} } ) ) {
    my $pool_id      = "";
    my $r_type       = "";
    my $percent_full = "";
    my $cap_avail_gb = "";
    my $cap_avail_bl = "";
    my $cap_used_gb  = "";
    my $cap_used_bl  = "";
    my $cap_user_gb  = "";
    my $cap_user_bl  = "";
    my $list_of_luns = "";
    my $cap_TB       = "";
    my $cap_free_TB  = "";
    my $cap_used_TB  = "";

    if ( defined $data{POOL}{$pool_name}{'Pool ID'} && $data{POOL}{$pool_name}{'Pool ID'} ne '' ) {
      $pool_id = $data{POOL}{$pool_name}{'Pool ID'};
    }
    if ( defined $data{POOL}{$pool_name}{'Raid Type'} && $data{POOL}{$pool_name}{'Raid Type'} ne '' ) {
      $r_type = $data{POOL}{$pool_name}{'Raid Type'};
    }
    if ( defined $data{POOL}{$pool_name}{'Percent Full'} && $data{POOL}{$pool_name}{'Percent Full'} ne '' ) {
      $percent_full = $data{POOL}{$pool_name}{'Percent Full'};
    }
    if ( defined $data{POOL}{$pool_name}{'Available Capacity (GBs)'} && $data{POOL}{$pool_name}{'Available Capacity (GBs)'} ne '' ) {
      $cap_avail_gb = $data{POOL}{$pool_name}{'Available Capacity (GBs)'};
    }
    if ( defined $data{POOL}{$pool_name}{'Available Capacity (Blocks)'} && $data{POOL}{$pool_name}{'Available Capacity (Blocks)'} ne '' ) {
      $cap_avail_bl = $data{POOL}{$pool_name}{'Available Capacity (Blocks)'};
      $cap_free_TB  = $cap_avail_bl / $block_size;                              # from block to MB (1 block = 2048 MB)
      $cap_free_TB  = $cap_free_TB / 1024;                                      # from MB to GB
      $cap_free_TB  = $cap_free_TB / 1024;                                      # from GB to TB
                                                                                #$cap_free_TB  = sprintf('%.3f', $cap_free_TB);
    } ## end if ( defined $data{POOL...})
    if ( defined $data{POOL}{$pool_name}{'Consumed Capacity (GBs)'} && $data{POOL}{$pool_name}{'Consumed Capacity (GBs)'} ne '' ) {
      $cap_used_gb = $data{POOL}{$pool_name}{'Consumed Capacity (GBs)'};
    }
    if ( defined $data{POOL}{$pool_name}{'Consumed Capacity (Blocks)'} && $data{POOL}{$pool_name}{'Consumed Capacity (Blocks)'} ne '' ) {
      $cap_used_bl = $data{POOL}{$pool_name}{'Consumed Capacity (Blocks)'};
      $cap_used_TB = $cap_used_bl / $block_size;                                # from block to MB (1 block = 2048 MB)
      $cap_used_TB = $cap_used_TB / 1024;                                       # from MB to GB
      $cap_used_TB = $cap_used_TB / 1024;                                       # from GB to TB
                                                                                #$cap_used_TB  = sprintf('%.3f', $cap_used_TB);
    } ## end if ( defined $data{POOL...})
    if ( defined $data{POOL}{$pool_name}{'User Capacity (GBs)'} && $data{POOL}{$pool_name}{'User Capacity (GBs)'} ne '' ) {
      $cap_user_gb = $data{POOL}{$pool_name}{'User Capacity (GBs)'};
    }
    if ( defined $data{POOL}{$pool_name}{'User Capacity (Blocks)'} && $data{POOL}{$pool_name}{'User Capacity (Blocks)'} ne '' ) {
      $cap_user_bl = $data{POOL}{$pool_name}{'User Capacity (Blocks)'};
      $cap_TB      = $cap_user_bl / $block_size;                                # from block to MB (1 block = 2048 MB)
      $cap_TB      = $cap_TB / 1024;                                            # from MB to GB
      $cap_TB      = $cap_TB / 1024;                                            # from GB to TB
                                                                                #$cap_TB  = sprintf('%.3f', $cap_TB);
    } ## end if ( defined $data{POOL...})
    if ( defined $data{POOL}{$pool_name}{'LUNs'} && $data{POOL}{$pool_name}{'LUNs'} ne '' ) {
      $list_of_luns = $data{POOL}{$pool_name}{'LUNs'};
      $list_of_luns =~ s/,//g;
    }

    print PERF "$pool_name,$pool_id,$r_type,$percent_full,$cap_avail_gb,$cap_avail_bl,$cap_used_gb,$cap_used_bl,$cap_user_gb,$cap_user_bl,$list_of_luns,$cap_TB,$cap_free_TB,$cap_used_TB\n";
  } ## end foreach my $pool_name ( keys...)

  # print PORT
  print PERF "\nPort Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "SP Name,SP Port ID,Physical Port ID,Link Status,Port Status,SFP State,Usage,Speed Value,SP UID,Reads (IOPS),Writes (IOPS),Blocks Read,Blocks Written,read (kB),write (kB),io_rate,data_rate\n";

  foreach my $SP_name ( keys( %{ $data{PORT} } ) ) {
    foreach my $SP_port_id ( sort { $a <=> $b } keys( %{ $data{PORT}{$SP_name} } ) ) {
      my $phys_port_id   = "";
      my $link_st        = "";
      my $port_st        = "";
      my $SFP_st         = "";
      my $usage          = "";
      my $speed          = "";
      my $SP_uid         = "";
      my $reads          = "";
      my $writes         = "";
      my $blocks_read    = "";
      my $blocks_written = "";
      my $read_kB        = "";
      my $write_kB       = "";
      my $io_rate        = "";
      my $data_rate      = "";

      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Physical Port ID'} && $data{PORT}{$SP_name}{$SP_port_id}{'Physical Port ID'} ne '' ) {
        $phys_port_id = $data{PORT}{$SP_name}{$SP_port_id}{'Physical Port ID'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Link Status'} && $data{PORT}{$SP_name}{$SP_port_id}{'Link Status'} ne '' ) {
        $link_st = $data{PORT}{$SP_name}{$SP_port_id}{'Link Status'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Port Status'} && $data{PORT}{$SP_name}{$SP_port_id}{'Port Status'} ne '' ) {
        $port_st = $data{PORT}{$SP_name}{$SP_port_id}{'Port Status'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'SFP State'} && $data{PORT}{$SP_name}{$SP_port_id}{'SFP State'} ne '' ) {
        $SFP_st = $data{PORT}{$SP_name}{$SP_port_id}{'SFP State'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Usage'} && $data{PORT}{$SP_name}{$SP_port_id}{'Usage'} ne '' ) {
        $usage = $data{PORT}{$SP_name}{$SP_port_id}{'Usage'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Speed Value'} && $data{PORT}{$SP_name}{$SP_port_id}{'Speed Value'} ne '' ) {
        $speed = $data{PORT}{$SP_name}{$SP_port_id}{'Speed Value'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'SP UID'} && $data{PORT}{$SP_name}{$SP_port_id}{'SP UID'} ne '' ) {
        $SP_uid = $data{PORT}{$SP_name}{$SP_port_id}{'SP UID'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Reads'} && $data{PORT}{$SP_name}{$SP_port_id}{'Reads'} ne '' ) {
        $reads = $data{PORT}{$SP_name}{$SP_port_id}{'Reads'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Writes'} && $data{PORT}{$SP_name}{$SP_port_id}{'Writes'} ne '' ) {
        $writes = $data{PORT}{$SP_name}{$SP_port_id}{'Writes'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Blocks Read'} && $data{PORT}{$SP_name}{$SP_port_id}{'Blocks Read'} ne '' ) {
        $blocks_read = $data{PORT}{$SP_name}{$SP_port_id}{'Blocks Read'};
        $read_kB     = $blocks_read / $block_size;                          # from block to MB (1 block = 2048 MB)
        $read_kB     = $read_kB * 1024;                                     # from MB to kB
        $read_kB     = sprintf( '%.0f', $read_kB );
      } ## end if ( defined $data{PORT...})
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Blocks Written'} && $data{PORT}{$SP_name}{$SP_port_id}{'Blocks Written'} ne '' ) {
        $blocks_written = $data{PORT}{$SP_name}{$SP_port_id}{'Blocks Written'};
        $write_kB       = $blocks_written / $block_size;                          # from block to MB (1 block = 2048 MB)
        $write_kB       = $write_kB * 1024;                                       # from MB to kB
        $write_kB       = sprintf( '%.0f', $write_kB );
      } ## end if ( defined $data{PORT...})

      #sum io rate
      if ( isdigit($reads) && isdigit($writes) ) {
        $io_rate = $reads + $writes;
        $io_rate = sprintf( '%.0f', $io_rate );
      }

      #sum data rate
      if ( isdigit($read_kB) && isdigit($write_kB) ) {
        $data_rate = $read_kB + $write_kB;
        $data_rate = sprintf( '%.0f', $data_rate );
      }

      print PERF "$SP_name,$SP_port_id,$phys_port_id,$link_st,$port_st,$SFP_st,$usage,$speed,$SP_uid,$reads,$writes,$blocks_read,$blocks_written,$read_kB,$write_kB,$io_rate,$data_rate\n";
    } ## end foreach my $SP_port_id ( sort...)
  } ## end foreach my $SP_name ( keys(...))

  # print DISK
  print PERF "\nDisk Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "Disk ID,State,Vendor ID,Drive Type,Raid Group ID,Current Speed,Capacity,User Capacity,Luns,Prct Idle,Prct Busy,Idle Ticks,Busy Ticks,Number of Reads,Number of Writes,Kbytes Read,Kbytes Written,Read Requests,Write Requests,Queue Length,io_rate,data_rate\n";

  foreach my $disk_id ( keys( %{ $data{DISK} } ) ) {
    my $state      = "";
    my $vendor_id  = "";
    my $drive_type = "";
    my $RG_id      = "";
    my $curr_speed = "";
    my $cap        = "";
    my $user_cap   = "";
    my $luns       = "";
    my $prct_idle  = "";
    my $prct_busy  = "";
    my $idle_ticks = "";
    my $busy_ticks = "";
    my $num_reads  = "";
    my $num_writes = "";
    my $read_kb    = "";
    my $written_kb = "";
    my $read_io    = "";
    my $write_io   = "";
    my $queue      = "";
    my $io_rate    = "";
    my $data_rate  = "";

    if ( defined $data{DISK}{$disk_id}{'State'} && $data{DISK}{$disk_id}{'State'} ne '' ) {
      $state = $data{DISK}{$disk_id}{'State'};
    }
    if ( defined $data{DISK}{$disk_id}{'Vendor Id'} && $data{DISK}{$disk_id}{'Vendor Id'} ne '' ) {
      $vendor_id = $data{DISK}{$disk_id}{'Vendor Id'};
    }
    if ( defined $data{DISK}{$disk_id}{'Drive Type'} && $data{DISK}{$disk_id}{'Drive Type'} ne '' ) {
      $drive_type = $data{DISK}{$disk_id}{'Drive Type'};
    }
    if ( defined $data{DISK}{$disk_id}{'Raid Group ID'} && $data{DISK}{$disk_id}{'Raid Group ID'} ne '' ) {
      $RG_id = $data{DISK}{$disk_id}{'Raid Group ID'};
    }
    if ( defined $data{DISK}{$disk_id}{'Current Speed'} && $data{DISK}{$disk_id}{'Current Speed'} ne '' ) {
      $curr_speed = $data{DISK}{$disk_id}{'Current Speed'};
    }
    if ( defined $data{DISK}{$disk_id}{'Capacity'} && $data{DISK}{$disk_id}{'Capacity'} ne '' ) {
      $cap = $data{DISK}{$disk_id}{'Capacity'};
    }
    if ( defined $data{DISK}{$disk_id}{'User Capacity'} && $data{DISK}{$disk_id}{'User Capacity'} ne '' ) {
      $user_cap = $data{DISK}{$disk_id}{'User Capacity'};
    }
    if ( defined $data{DISK}{$disk_id}{'Lun'} && $data{DISK}{$disk_id}{'Lun'} ne '' ) {
      $luns = $data{DISK}{$disk_id}{'Lun'};
    }
    if ( defined $data{DISK}{$disk_id}{'Prct Idle'} && $data{DISK}{$disk_id}{'Prct Idle'} ne '' ) {
      $prct_idle = $data{DISK}{$disk_id}{'Prct Idle'};
    }
    if ( defined $data{DISK}{$disk_id}{'Prct Busy'} && $data{DISK}{$disk_id}{'Prct Busy'} ne '' ) {
      $prct_busy = $data{DISK}{$disk_id}{'Prct Busy'};
    }
    if ( defined $data{DISK}{$disk_id}{'Idle Ticks'} && $data{DISK}{$disk_id}{'Idle Ticks'} ne '' ) {
      $idle_ticks = $data{DISK}{$disk_id}{'Idle Ticks'};
    }
    if ( defined $data{DISK}{$disk_id}{'Busy Ticks'} && $data{DISK}{$disk_id}{'Busy Ticks'} ne '' ) {
      $busy_ticks = $data{DISK}{$disk_id}{'Busy Ticks'};
    }
    if ( defined $data{DISK}{$disk_id}{'Number of Reads'} && $data{DISK}{$disk_id}{'Number of Reads'} ne '' ) {
      $num_reads = $data{DISK}{$disk_id}{'Number of Reads'};
    }
    if ( defined $data{DISK}{$disk_id}{'Number of Writes'} && $data{DISK}{$disk_id}{'Number of Writes'} ne '' ) {
      $num_writes = $data{DISK}{$disk_id}{'Number of Writes'};
    }
    if ( defined $data{DISK}{$disk_id}{'Kbytes Read'} && $data{DISK}{$disk_id}{'Kbytes Read'} ne '' ) {
      $read_kb = $data{DISK}{$disk_id}{'Kbytes Read'};
    }
    if ( defined $data{DISK}{$disk_id}{'Kbytes Written'} && $data{DISK}{$disk_id}{'Kbytes Written'} ne '' ) {
      $written_kb = $data{DISK}{$disk_id}{'Kbytes Written'};
    }
    if ( defined $data{DISK}{$disk_id}{'Read Requests'} && $data{DISK}{$disk_id}{'Read Requests'} ne '' ) {
      $read_io = $data{DISK}{$disk_id}{'Read Requests'};
    }
    if ( defined $data{DISK}{$disk_id}{'Write Requests'} && $data{DISK}{$disk_id}{'Write Requests'} ne '' ) {
      $write_io = $data{DISK}{$disk_id}{'Write Requests'};
    }
    if ( defined $data{DISK}{$disk_id}{'Queue Length'} && $data{DISK}{$disk_id}{'Queue Length'} ne '' ) {
      $queue = $data{DISK}{$disk_id}{'Queue Length'};
    }

    #sum io rate
    if ( isdigit($read_io) && isdigit($write_io) ) {
      $io_rate = $read_io + $write_io;
      $io_rate = sprintf( '%.0f', $io_rate );
    }

    #sum data rate
    if ( isdigit($read_kb) && isdigit($written_kb) ) {
      $data_rate = $read_kb + $written_kb;
      $data_rate = sprintf( '%.0f', $data_rate );
    }

    print PERF "$disk_id,$state,$vendor_id,$drive_type,$RG_id,$curr_speed,$cap,$user_cap,$luns,$prct_idle,$prct_busy,$idle_ticks,$busy_ticks,$num_reads,$num_writes,$read_kb,$written_kb,$read_io,$write_io,$queue,$io_rate,$data_rate\n";
  } ## end foreach my $disk_id ( keys(...))

  # print SP
  print PERF "\nSP Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "SP Name,Statistics Logging,Serial Number,SP IP,Idle (%),Busy (%),Controller idle ticks,Controller busy ticks,Total Reads,Total Writes,Read_requests,Write_requests,Blocks_read,Blocks_written,Arrivals_to_non_zero_queue,Sum_queue_lengths_by_arrivals,read (kB),write (kB),io_rate,data_rate,Write_cache_blocks_flushed,Prct Dirty Cache Pages,Prct Cache Pages Owned\n";

  foreach my $sp_name ( keys( %{ $data{SP} } ) ) {
    my $stat_log   = "";
    my $serial     = "";
    my $ip         = "";
    my $idle_prct  = "";
    my $busy_prct  = "";
    my $idle_ticks = "";
    my $busy_ticks = "";
    my $reads      = "";
    my $writes     = "";
    my $read_io    = "";
    my $write_io   = "";
    my $read_bl    = "";
    my $write_bl   = "";
    my $arrivals   = "";
    my $sum_que    = "";
    my $read_kB    = "";
    my $write_kB   = "";

    my $io_rate   = "";
    my $data_rate = "";

    my $write_cache_blocks_flushed = "";
    my $prct_dirty_cache_pages     = "";
    my $prct_cache_pages_owned     = "";

    if ( defined $data{SP}{$sp_name}{'Statistics Logging'} && $data{SP}{$sp_name}{'Statistics Logging'} ne '' ) {
      $stat_log = $data{SP}{$sp_name}{'Statistics Logging'};
    }
    if ( defined $data{SP}{$sp_name}{'Serial Number For The SP'} && $data{SP}{$sp_name}{'Serial Number For The SP'} ne '' ) {
      $serial = $data{SP}{$sp_name}{'Serial Number For The SP'};
    }
    if ( defined $data{SP}{$sp_name}{'Storage Processor IP Address'} && $data{SP}{$sp_name}{'Storage Processor IP Address'} ne '' ) {
      $ip = $data{SP}{$sp_name}{'Storage Processor IP Address'};
    }
    if ( defined $data{SP}{$sp_name}{'Prct Idle'} && $data{SP}{$sp_name}{'Prct Idle'} ne '' ) {
      $idle_prct = $data{SP}{$sp_name}{'Prct Idle'};
    }
    if ( defined $data{SP}{$sp_name}{'Prct Busy'} && $data{SP}{$sp_name}{'Prct Busy'} ne '' ) {
      $busy_prct = $data{SP}{$sp_name}{'Prct Busy'};
    }
    if ( defined $data{SP}{$sp_name}{'Controller idle ticks'} && $data{SP}{$sp_name}{'Controller idle ticks'} ne '' ) {
      $idle_ticks = $data{SP}{$sp_name}{'Controller idle ticks'};
    }
    if ( defined $data{SP}{$sp_name}{'Controller busy ticks'} && $data{SP}{$sp_name}{'Controller busy ticks'} ne '' ) {
      $busy_ticks = $data{SP}{$sp_name}{'Controller busy ticks'};
    }
    if ( defined $data{SP}{$sp_name}{'Total Reads'} && $data{SP}{$sp_name}{'Total Reads'} ne '' ) {
      $reads = $data{SP}{$sp_name}{'Total Reads'};
    }
    if ( defined $data{SP}{$sp_name}{'Total Writes'} && $data{SP}{$sp_name}{'Total Writes'} ne '' ) {
      $writes = $data{SP}{$sp_name}{'Total Writes'};
    }
    if ( defined $data{SP}{$sp_name}{'Read_requests'} && $data{SP}{$sp_name}{'Read_requests'} ne '' ) {
      $read_io = $data{SP}{$sp_name}{'Read_requests'};
    }
    if ( defined $data{SP}{$sp_name}{'Write_requests'} && $data{SP}{$sp_name}{'Write_requests'} ne '' ) {
      $write_io = $data{SP}{$sp_name}{'Write_requests'};
    }
    if ( defined $data{SP}{$sp_name}{'Blocks_read'} && $data{SP}{$sp_name}{'Blocks_read'} ne '' ) {
      $read_bl = $data{SP}{$sp_name}{'Blocks_read'};
      $read_kB = $read_bl / $block_size;               # from block to MB (1 block = 2048 MB)
      $read_kB = $read_kB * 1024;                      # from MB to kB
      $read_kB = sprintf( '%.0f', $read_kB );
    } ## end if ( defined $data{SP}...)
    if ( defined $data{SP}{$sp_name}{'Blocks_written'} && $data{SP}{$sp_name}{'Blocks_written'} ne '' ) {
      $write_bl = $data{SP}{$sp_name}{'Blocks_written'};
      $write_kB = $write_bl / $block_size;                 # from block to MB (1 block = 2048 MB)
      $write_kB = $write_kB * 1024;                        # from MB to kB
      $write_kB = sprintf( '%.0f', $write_kB );
    } ## end if ( defined $data{SP}...)
    if ( defined $data{SP}{$sp_name}{'Arrivals_to_non_zero_queue'} && $data{SP}{$sp_name}{'Arrivals_to_non_zero_queue'} ne '' ) {
      $arrivals = $data{SP}{$sp_name}{'Arrivals_to_non_zero_queue'};
    }
    if ( defined $data{SP}{$sp_name}{'Sum_queue_lengths_by_arrivals'} && $data{SP}{$sp_name}{'Sum_queue_lengths_by_arrivals'} ne '' ) {
      $sum_que = $data{SP}{$sp_name}{'Sum_queue_lengths_by_arrivals'};
    }

    # sum io_rate and data_rate
    if ( defined $read_io && isdigit($read_io) && defined $write_io && isdigit($write_io) ) {
      $io_rate = $read_io + $write_io;
      $io_rate = sprintf( '%.0f', $io_rate );
    }
    if ( defined $read_kB && isdigit($read_kB) && defined $write_kB && isdigit($write_kB) ) {
      $data_rate = $read_kB + $write_kB;
      $data_rate = sprintf( '%.0f', $data_rate );
    }
    if ( defined $data{SP}{$sp_name}{'Write_cache_blocks_flushed'} && $data{SP}{$sp_name}{'Write_cache_blocks_flushed'} ne '' ) {
      $write_cache_blocks_flushed = $data{SP}{$sp_name}{'Write_cache_blocks_flushed'};
    }

    #Prct Dirty Cache Pages,Prct Cache Pages Owned
    if ( defined $ip && $ip ne '' ) {
      if ( defined $data{CACHE}{$ip} && $data{CACHE}{$ip} ne '' ) {
        if ( defined $data{CACHE}{$ip}{'Prct Dirty Cache Pages'} && $data{CACHE}{$ip}{'Prct Dirty Cache Pages'} ne '' ) {
          $prct_dirty_cache_pages = $data{CACHE}{$ip}{'Prct Dirty Cache Pages'};
        }
        if ( defined $data{CACHE}{$ip}{'Prct Cache Pages Owned'} && $data{CACHE}{$ip}{'Prct Cache Pages Owned'} ne '' ) {
          $prct_cache_pages_owned = $data{CACHE}{$ip}{'Prct Cache Pages Owned'};
        }
      } ## end if ( defined $data{CACHE...})
    } ## end if ( defined $ip && $ip...)

    print PERF "$sp_name,$stat_log,$serial,$ip,$idle_prct,$busy_prct,$idle_ticks,$busy_ticks,$reads,$writes,$read_io,$write_io,$read_bl,$write_bl,$arrivals,$sum_que,$read_kB,$write_kB,$io_rate,$data_rate,$write_cache_blocks_flushed,$prct_dirty_cache_pages,$prct_cache_pages_owned\n";
  } ## end foreach my $sp_name ( keys(...))

  # print HOST
  print PERF "\nHost Level Statistics\n";
  print PERF "  Start: $start_time\n";
  print PERF "---------------------\n";
  print PERF "Host name,Volume IDs,Volume Names\n";

  foreach my $st_group ( keys( %{ $data{'Storage Group'} } ) ) {
    if ( defined $data{'Storage Group'}{$st_group}{'hosts'} ) {
      foreach my $host ( keys( %{ $data{'Storage Group'}{$st_group}{'hosts'} } ) ) {
        my $lun_ids   = "";
        my $lun_names = "";

        if ( defined $data{'Storage Group'}{$st_group}{'luns'} ) {
          foreach my $lun ( sort { $a <=> $b } keys( %{ $data{'Storage Group'}{$st_group}{'luns'} } ) ) {
            my $lun_name = "";
            if ( defined $data{LUN}{$lun}{'Name'} && $data{LUN}{$lun}{'Name'} ne '' ) {
              $lun_name = $data{LUN}{$lun}{'Name'};

              if ( $lun_ids eq '' ) { $lun_ids = $lun; }
              else                  { $lun_ids .= " $lun"; }

              if ( $lun_names eq '' ) { $lun_names = $lun_name; }
              else                    { $lun_names .= " $lun_name"; }
            } ## end if ( defined $data{LUN...})
          } ## end foreach my $lun ( sort { $a...})
          print PERF "$host,$lun_ids,$lun_names\n";
        } ## end if ( defined $data{'Storage Group'...})
      } ## end foreach my $host ( keys( %{...}))
    } ## end if ( defined $data{'Storage Group'...})
  } ## end foreach my $st_group ( keys...)

  close(PERF);
} ## end sub print_perf_out

sub print_conf_out {
  open( CONF, ">$conf_out" ) || error( "Couldn't open file $conf_out $!" . __FILE__ . ":" . __LINE__ ) && exit;

  print CONF "Configuration Data\n";
  print CONF "------------------\n";
  if ( defined $data{'ARRAY'}{'model'} && $data{'ARRAY'}{'model'} ne '' ) {
    print CONF "Model : $data{'ARRAY'}{'model'}";
  }
  if ( defined $data{'ARRAY'}{'model_type'} && $data{'ARRAY'}{'model_type'} ne '' ) {
    print CONF "Model Type : $data{'ARRAY'}{'model_type'}";
  }
  if ( defined $data{'ARRAY'}{'serial'} && $data{'ARRAY'}{'serial'} ne '' ) {
    print CONF "Serial No: $data{'ARRAY'}{'serial'}";
  }
  print CONF "\n";


  # print LUN
  print CONF "Volume Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "Name,lun_id,Current_owner,RG_ID,Pool name,UID,LUN_Capacity (MB),LUN_Capacity (Blocks)\n";

  foreach my $lun_id ( sort { $a <=> $b } keys( %{ $data{LUN} } ) ) {
    my $Name                = "";
    my $UID                 = "";
    my $RG_ID               = "";
    my $pool_name           = "";
    my $Current_owner       = "";
    my $Read_Requests       = "";
    my $Write_Requests      = "";
    my $LUN_Capacity_MB     = "";
    my $LUN_Capacity_Blocks = "";

    if ( defined $data{LUN}{$lun_id}{'Name'} && $data{LUN}{$lun_id}{'Name'} ne '' ) { $Name = $data{LUN}{$lun_id}{'Name'}; }
    if ( $Name =~ /Hot Spare LUN/ ) { next; }

    if ( defined $data{LUN}{$lun_id}{'UID'}           && $data{LUN}{$lun_id}{'UID'} ne '' )           { $UID           = $data{LUN}{$lun_id}{'UID'}; }
    if ( defined $data{LUN}{$lun_id}{'Current owner'} && $data{LUN}{$lun_id}{'Current owner'} ne '' ) { $Current_owner = $data{LUN}{$lun_id}{'Current owner'}; }

    if ( defined $data{LUN}{$lun_id}{'RAIDGroup ID'} && $data{LUN}{$lun_id}{'RAIDGroup ID'} ne '' && isdigit( $data{LUN}{$lun_id}{'RAIDGroup ID'} ) ) {
      $RG_ID = $data{LUN}{$lun_id}{'RAIDGroup ID'};
    }
    if ( defined $data{LUN}{$lun_id}{'Pool Name'} && $data{LUN}{$lun_id}{'Pool Name'} ne '' ) {
      $pool_name = $data{LUN}{$lun_id}{'Pool Name'};
    }
    if ( defined $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'} && $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'} ) ) {
      $LUN_Capacity_MB = $data{LUN}{$lun_id}{'LUN Capacity(Megabytes)'};
    }
    if ( defined $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'} && $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'} ne '' && isdigit( $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'} ) ) {
      $LUN_Capacity_Blocks = $data{LUN}{$lun_id}{'LUN Capacity(Blocks)'};
    }

    print CONF "$Name,$lun_id,$Current_owner,$RG_ID,$pool_name,$UID,$LUN_Capacity_MB,$LUN_Capacity_Blocks\n";
  } ## end foreach my $lun_id ( sort {...})

  # print RG
  print CONF "\nRaid-Group Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "RG_ID,type,Capacity (MB),Capacity (Blocks),Free Capacity (MB),Free Capacity (Blocks),Luns\n";

  foreach my $RG_id ( sort { $a <=> $b } keys( %{ $data{RG} } ) ) {
    my $RG_type             = "";
    my $capacity_mb         = "";
    my $capacity_block      = "";
    my $free_capacity_mb    = "";
    my $free_capacity_block = "";
    my $list_of_luns        = "";

    if ( defined $data{RG}{$RG_id}{'RaidGroup Type'} && $data{RG}{$RG_id}{'RaidGroup Type'} ne '' ) { $RG_type = $data{RG}{$RG_id}{'RaidGroup Type'}; }
    if ( $RG_type =~ /hot_spare/ ) { next; }

    if ( defined $data{RG}{$RG_id}{'Logical Capacity (Blocks)'} && isdigit( $data{RG}{$RG_id}{'Logical Capacity (Blocks)'} ) ) {
      $capacity_block = $data{RG}{$RG_id}{'Logical Capacity (Blocks)'};
      $capacity_mb    = $capacity_block / 2048;
    }
    if ( defined $data{RG}{$RG_id}{'Free Capacity (Blocks,non-contiguous)'} && isdigit( $data{RG}{$RG_id}{'Free Capacity (Blocks,non-contiguous)'} ) ) {
      $free_capacity_block = $data{RG}{$RG_id}{'Free Capacity (Blocks,non-contiguous)'};
      $free_capacity_mb    = $free_capacity_block / 2048;
    }
    if ( defined $data{RG}{$RG_id}{'List of luns'} && $data{RG}{$RG_id}{'List of luns'} ne '' ) {
      $list_of_luns = $data{RG}{$RG_id}{'List of luns'};
    }

    print CONF "$RG_id,$RG_type,$capacity_mb,$capacity_block,$free_capacity_mb,$free_capacity_block,$list_of_luns\n";
  } ## end foreach my $RG_id ( sort { ...})

  # print POOL
  print CONF "\nPool Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "Pool name,Pool ID,Raid Type,Percent Full,Available Capacity (GBs),Available Capacity (Blocks),Consumed Capacity (GBs),Consumed Capacity (Blocks),User Capacity (GBs),User Capacity (Blocks),Luns\n";

  foreach my $pool_name ( keys( %{ $data{POOL} } ) ) {
    my $pool_id      = "";
    my $r_type       = "";
    my $percent_full = "";
    my $cap_avail_gb = "";
    my $cap_avail_bl = "";
    my $cap_used_gb  = "";
    my $cap_used_bl  = "";
    my $cap_user_gb  = "";
    my $cap_user_bl  = "";
    my $list_of_luns = "";

    if ( defined $data{POOL}{$pool_name}{'Pool ID'} && $data{POOL}{$pool_name}{'Pool ID'} ne '' ) {
      $pool_id = $data{POOL}{$pool_name}{'Pool ID'};
    }
    if ( defined $data{POOL}{$pool_name}{'Raid Type'} && $data{POOL}{$pool_name}{'Raid Type'} ne '' ) {
      $r_type = $data{POOL}{$pool_name}{'Raid Type'};
    }
    if ( defined $data{POOL}{$pool_name}{'Percent Full'} && $data{POOL}{$pool_name}{'Percent Full'} ne '' ) {
      $percent_full = $data{POOL}{$pool_name}{'Percent Full'};
    }
    if ( defined $data{POOL}{$pool_name}{'Available Capacity (GBs)'} && $data{POOL}{$pool_name}{'Available Capacity (GBs)'} ne '' ) {
      $cap_avail_gb = $data{POOL}{$pool_name}{'Available Capacity (GBs)'};
    }
    if ( defined $data{POOL}{$pool_name}{'Available Capacity (Blocks)'} && $data{POOL}{$pool_name}{'Available Capacity (Blocks)'} ne '' ) {
      $cap_avail_bl = $data{POOL}{$pool_name}{'Available Capacity (Blocks)'};
    }
    if ( defined $data{POOL}{$pool_name}{'Consumed Capacity (GBs)'} && $data{POOL}{$pool_name}{'Consumed Capacity (GBs)'} ne '' ) {
      $cap_used_gb = $data{POOL}{$pool_name}{'Consumed Capacity (GBs)'};
    }
    if ( defined $data{POOL}{$pool_name}{'Consumed Capacity (Blocks)'} && $data{POOL}{$pool_name}{'Consumed Capacity (Blocks)'} ne '' ) {
      $cap_used_bl = $data{POOL}{$pool_name}{'Consumed Capacity (Blocks)'};
    }
    if ( defined $data{POOL}{$pool_name}{'User Capacity (GBs)'} && $data{POOL}{$pool_name}{'User Capacity (GBs)'} ne '' ) {
      $cap_user_gb = $data{POOL}{$pool_name}{'User Capacity (GBs)'};
    }
    if ( defined $data{POOL}{$pool_name}{'User Capacity (Blocks)'} && $data{POOL}{$pool_name}{'User Capacity (Blocks)'} ne '' ) {
      $cap_user_bl = $data{POOL}{$pool_name}{'User Capacity (Blocks)'};
    }
    if ( defined $data{POOL}{$pool_name}{'LUNs'} && $data{POOL}{$pool_name}{'LUNs'} ne '' ) {
      $list_of_luns = $data{POOL}{$pool_name}{'LUNs'};
      $list_of_luns =~ s/,//g;
    }

    print CONF "$pool_name,$pool_id,$r_type,$percent_full,$cap_avail_gb,$cap_avail_bl,$cap_used_gb,$cap_used_bl,$cap_user_gb,$cap_user_bl,$list_of_luns\n";
  } ## end foreach my $pool_name ( keys...)

  # print PORT
  print CONF "\nPort Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "SP Name,SP Port ID,Physical Port ID,Link Status,Port Status,SFP State,Usage,Speed Value,SP UID\n";

  foreach my $SP_name ( keys( %{ $data{PORT} } ) ) {
    foreach my $SP_port_id ( sort { $a <=> $b } keys( %{ $data{PORT}{$SP_name} } ) ) {
      my $phys_port_id = "";
      my $link_st      = "";
      my $port_st      = "";
      my $SFP_st       = "";
      my $usage        = "";
      my $speed        = "";
      my $SP_uid       = "";

      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Physical Port ID'} && $data{PORT}{$SP_name}{$SP_port_id}{'Physical Port ID'} ne '' ) {
        $phys_port_id = $data{PORT}{$SP_name}{$SP_port_id}{'Physical Port ID'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Link Status'} && $data{PORT}{$SP_name}{$SP_port_id}{'Link Status'} ne '' ) {
        $link_st = $data{PORT}{$SP_name}{$SP_port_id}{'Link Status'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Port Status'} && $data{PORT}{$SP_name}{$SP_port_id}{'Port Status'} ne '' ) {
        $port_st = $data{PORT}{$SP_name}{$SP_port_id}{'Port Status'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'SFP State'} && $data{PORT}{$SP_name}{$SP_port_id}{'SFP State'} ne '' ) {
        $SFP_st = $data{PORT}{$SP_name}{$SP_port_id}{'SFP State'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Usage'} && $data{PORT}{$SP_name}{$SP_port_id}{'Usage'} ne '' ) {
        $usage = $data{PORT}{$SP_name}{$SP_port_id}{'Usage'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'Speed Value'} && $data{PORT}{$SP_name}{$SP_port_id}{'Speed Value'} ne '' ) {
        $speed = $data{PORT}{$SP_name}{$SP_port_id}{'Speed Value'};
      }
      if ( defined $data{PORT}{$SP_name}{$SP_port_id}{'SP UID'} && $data{PORT}{$SP_name}{$SP_port_id}{'SP UID'} ne '' ) {
        $SP_uid = $data{PORT}{$SP_name}{$SP_port_id}{'SP UID'};
      }

      print CONF "$SP_name,$SP_port_id,$phys_port_id,$link_st,$port_st,$SFP_st,$usage,$speed,$SP_uid\n";
    } ## end foreach my $SP_port_id ( sort...)
  } ## end foreach my $SP_name ( keys(...))

  # print DISK
  print CONF "\nDisk Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "Disk ID,State,Vendor ID,Drive Type,Raid Group ID,Current Speed,Capacity,User Capacity,Luns\n";

  foreach my $disk_id ( keys( %{ $data{DISK} } ) ) {
    my $state      = "";
    my $vendor_id  = "";
    my $drive_type = "";
    my $RG_id      = "";
    my $curr_speed = "";
    my $cap        = "";
    my $user_cap   = "";
    my $luns       = "";

    if ( defined $data{DISK}{$disk_id}{'State'} && $data{DISK}{$disk_id}{'State'} ne '' ) {
      $state = $data{DISK}{$disk_id}{'State'};
    }
    if ( defined $data{DISK}{$disk_id}{'Vendor Id'} && $data{DISK}{$disk_id}{'Vendor Id'} ne '' ) {
      $vendor_id = $data{DISK}{$disk_id}{'Vendor Id'};
    }
    if ( defined $data{DISK}{$disk_id}{'Drive Type'} && $data{DISK}{$disk_id}{'Drive Type'} ne '' ) {
      $drive_type = $data{DISK}{$disk_id}{'Drive Type'};
    }
    if ( defined $data{DISK}{$disk_id}{'Raid Group ID'} && $data{DISK}{$disk_id}{'Raid Group ID'} ne '' ) {
      $RG_id = $data{DISK}{$disk_id}{'Raid Group ID'};
    }
    if ( defined $data{DISK}{$disk_id}{'Current Speed'} && $data{DISK}{$disk_id}{'Current Speed'} ne '' ) {
      $curr_speed = $data{DISK}{$disk_id}{'Current Speed'};
    }
    if ( defined $data{DISK}{$disk_id}{'Capacity'} && $data{DISK}{$disk_id}{'Capacity'} ne '' ) {
      $cap = $data{DISK}{$disk_id}{'Capacity'};
    }
    if ( defined $data{DISK}{$disk_id}{'User Capacity'} && $data{DISK}{$disk_id}{'User Capacity'} ne '' ) {
      $user_cap = $data{DISK}{$disk_id}{'User Capacity'};
    }
    if ( defined $data{DISK}{$disk_id}{'Lun'} && $data{DISK}{$disk_id}{'Lun'} ne '' ) {
      $luns = $data{DISK}{$disk_id}{'Lun'};
    }

    print CONF "$disk_id,$state,$vendor_id,$drive_type,$RG_id,$curr_speed,$cap,$user_cap,$luns\n";
  } ## end foreach my $disk_id ( keys(...))

  # print SP
  print CONF "\nSP Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "SP Name,Statistics Logging,Serial Number,SP IP\n";

  foreach my $sp_name ( keys( %{ $data{SP} } ) ) {
    my $stat_log = "";
    my $serial   = "";
    my $ip       = "";

    if ( defined $data{SP}{$sp_name}{'Statistics Logging'} && $data{SP}{$sp_name}{'Statistics Logging'} ne '' ) {
      $stat_log = $data{SP}{$sp_name}{'Statistics Logging'};
    }
    if ( defined $data{SP}{$sp_name}{'Serial Number For The SP'} && $data{SP}{$sp_name}{'Serial Number For The SP'} ne '' ) {
      $serial = $data{SP}{$sp_name}{'Serial Number For The SP'};
    }
    if ( defined $data{SP}{$sp_name}{'Storage Processor IP Address'} && $data{SP}{$sp_name}{'Storage Processor IP Address'} ne '' ) {
      $ip = $data{SP}{$sp_name}{'Storage Processor IP Address'};
    }

    print CONF "$sp_name,$stat_log,$serial,$ip\n";
  } ## end foreach my $sp_name ( keys(...))

  # print HOST
  print CONF "\nHost Level Configuration\n";
  print CONF "  Start: $start_time\n";
  print CONF "---------------------\n";
  print CONF "Host name,Volume IDs,Volume Names\n";

  foreach my $st_group ( keys( %{ $data{'Storage Group'} } ) ) {
    if ( defined $data{'Storage Group'}{$st_group}{'hosts'} ) {
      foreach my $host ( keys( %{ $data{'Storage Group'}{$st_group}{'hosts'} } ) ) {
        my $lun_ids   = "";
        my $lun_names = "";

        if ( defined $data{'Storage Group'}{$st_group}{'luns'} ) {
          foreach my $lun ( sort { $a <=> $b } keys( %{ $data{'Storage Group'}{$st_group}{'luns'} } ) ) {
            my $lun_name = "";
            if ( defined $data{LUN}{$lun}{'Name'} && $data{LUN}{$lun}{'Name'} ne '' ) {
              $lun_name = $data{LUN}{$lun}{'Name'};

              if ( $lun_ids eq '' ) { $lun_ids = $lun; }
              else                  { $lun_ids .= " $lun"; }

              if ( $lun_names eq '' ) { $lun_names = $lun_name; }
              else                    { $lun_names .= " $lun_name"; }
            } ## end if ( defined $data{LUN...})
          } ## end foreach my $lun ( sort { $a...})
          print CONF "$host,$lun_ids,$lun_names\n";
        } ## end if ( defined $data{'Storage Group'...})
      } ## end foreach my $host ( keys( %{...}))
    } ## end if ( defined $data{'Storage Group'...})
  } ## end foreach my $st_group ( keys...)

  close(CONF);
} ## end sub print_conf_out

sub health_status {
  my $health_status_html = shift;

  my $last_update = localtime();
  my $act_time    = time();
  my $last_days   = 10;

  if ( defined $ENV{VNX_events} && isdigit( $ENV{VNX_events} ) ) { $last_days = $ENV{VNX_events}; }
  my $last_sec = $last_days * 86400;

  my $start_time = $act_time - $last_sec;    # last 10 days
  my $end_time   = $act_time + 86400;        # end date must be tomorrow

  my ( undef, undef, undef, $start_day, $start_month, $start_year, undef, undef, undef ) = localtime($start_time);
  my $start_date = sprintf( "%02d/%02d/%4d", $start_month + 1, $start_day, $start_year + 1900 );

  my ( undef, undef, undef, $end_day, $end_month, $end_year, undef, undef, undef ) = localtime($end_time);
  my $end_date = sprintf( "%02d/%02d/%4d", $end_month + 1, $end_day, $end_year + 1900 );

  my @getlog_spa;
  my @getlog_spb;
  my %events;

  # run getlog cmds
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    if ( defined $spa_ip && $spa_ip ne '' ) {
      @getlog_spa = `$naviseccli -h $spa_ip $sec_file_path getlog -date $start_date $end_date`;
    }
    if ( defined $spb_ip && $spb_ip ne '' ) {
      @getlog_spb = `$naviseccli -h $spb_ip $sec_file_path getlog -date $start_date $end_date`;
    }

    # end of alarm
    alarm(0);
  };

  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      my $act_time = localtime();
      error("command timed out after : $timeout seconds");
      exit(0);
    }
  } ## end if ($@)

  # add correct SP name for this sp ip
  my $sp_name_for_spa_ip = "";
  if ( defined $data{SP}{'SP A'}{'Storage Processor IP Address'} && $data{SP}{'SP A'}{'Storage Processor IP Address'} eq $spa_ip ) {
    $sp_name_for_spa_ip = "SP A";
  }
  if ( defined $data{SP}{'SP B'}{'Storage Processor IP Address'} && $data{SP}{'SP B'}{'Storage Processor IP Address'} eq $spa_ip ) {
    $sp_name_for_spa_ip = "SP B";
  }

  my $sp_name_for_spb_ip = "";
  if ( defined $data{SP}{'SP A'}{'Storage Processor IP Address'} && $data{SP}{'SP A'}{'Storage Processor IP Address'} eq $spb_ip ) {
    $sp_name_for_spa_ip = "SP A";
  }
  if ( defined $data{SP}{'SP B'}{'Storage Processor IP Address'} && $data{SP}{'SP B'}{'Storage Processor IP Address'} eq $spb_ip ) {
    $sp_name_for_spb_ip = "SP B";
  }

  # parse event logs
  my $event_id = 0;

  foreach my $line (@getlog_spa) {
    chomp $line;

    my $date        = "";
    my $time        = "";
    my $cru         = "";
    my $event_code  = "";
    my $description = "";
    my $date2hash   = "";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line eq '' ) { next; }

    ( $date, $time, $cru, undef ) = split( /\s+/, $line );

    if ( $cru eq 'N/A' ) { next; }
    $event_id++;

    my ( $month, $day, $year ) = split( /\//, $date );
    my ( $hour,  $min, $sec )  = split( /:/,  $time );

    $date2hash = $year . $month . $day . $hour . $min . $sec;

    $description = $line;
    $description =~ s/^$date\s+$time//;
    $description =~ s/^\s+//g;
    $description =~ s/\s+$//g;
    $description =~ s/\s+/ /g;

    $cru = $description;
    $cru =~ s/\(.+$//;

    $event_code = $description;
    $event_code =~ s/\).+$//;
    $event_code =~ s/^$cru//;
    $event_code =~ s/^\s+//g;
    $event_code =~ s/\s+$//g;
    $event_code = $event_code . ")";

    if ( $event_code =~ m/7d/ || $event_code =~ m/60a/ ) { next; }

    $description =~ s/$cru//;
    $description =~ s/\Q$event_code\E//;
    $description =~ s/^\s+//g;
    $description =~ s/\s+$//g;

    if ( $description =~ m/Internal information only/ ) { next; }

    #print HS "<tr><td nowrap=\"\">$date2hash</td><td nowrap=\"\">$cru</td><td nowrap=\"\">$event_code</td><td nowrap=\"\">$description</td></tr>\n";
    my $print_line = "<tr><td nowrap=\"\">$date $time</td><td nowrap=\"\">$sp_name_for_spa_ip</td><td nowrap=\"\">$cru</td><td nowrap=\"\">$event_code</td><td nowrap=\"\">$description</td></tr>\n";

    $events{'events'}{$date2hash}{$event_id} = $print_line;
  } ## end foreach my $line (@getlog_spa)
  foreach my $line (@getlog_spb) {
    chomp $line;

    my $date        = "";
    my $time        = "";
    my $cru         = "";
    my $event_code  = "";
    my $description = "";
    my $date2hash   = "";

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line eq '' ) { next; }

    ( $date, $time, $cru, undef ) = split( /\s+/, $line );

    if ( $cru eq 'N/A' ) { next; }
    $event_id++;

    my ( $month, $day, $year ) = split( /\//, $date );
    my ( $hour,  $min, $sec )  = split( /:/,  $time );

    $date2hash = $year . $month . $day . $hour . $min . $sec;

    $description = $line;
    $description =~ s/^$date\s+$time//;
    $description =~ s/^\s+//g;
    $description =~ s/\s+$//g;
    $description =~ s/\s+/ /g;

    $cru = $description;
    $cru =~ s/\(.+$//;

    $event_code = $description;
    $event_code =~ s/\).+$//;
    $event_code =~ s/^$cru//;
    $event_code =~ s/^\s+//g;
    $event_code =~ s/\s+$//g;
    $event_code = $event_code . ")";

    if ( $event_code =~ m/7d/ || $event_code =~ m/60a/ ) { next; }

    $description =~ s/$cru//;
    $description =~ s/\Q$event_code\E//;
    $description =~ s/^\s+//g;
    $description =~ s/\s+$//g;

    if ( $description =~ m/Internal information only/ ) { next; }

    #print HS "<tr><td nowrap=\"\">$date2hash</td><td nowrap=\"\">$cru</td><td nowrap=\"\">$event_code</td><td nowrap=\"\">$description</td></tr>\n";
    my $print_line = "<tr><td nowrap=\"\">$date $time</td><td nowrap=\"\">$sp_name_for_spb_ip</td><td nowrap=\"\">$cru</td><td nowrap=\"\">$event_code</td><td nowrap=\"\">$description</td></tr>\n";

    $events{'events'}{$date2hash}{$event_id} = $print_line;
  } ## end foreach my $line (@getlog_spb)

  open( HS, ">$health_status_html" ) || error( "Couldn't open file $health_status_html $!" . __FILE__ . ":" . __LINE__ ) && exit;

  print HS "<br>$last_update\n";

  foreach (@health_status) {
    chomp $_;
    if ( $_ eq '' ) { next; }
    print HS "<br>$_\n";
  }

  print HS "<br>\n";
  print HS "<table class =\"tabcfgsum tablesorter tablesortercfgsum\"><thead><tr>\n";
  print HS "  <th class = \"sortable\" title=\"Date/Time\" nowrap=\"\">Date/Time&nbsp;&nbsp;&nbsp;&nbsp;</th>\n";
  print HS "  <th class = \"sortable\" title=\"Storage Processor\" nowrap=\"\">SP&nbsp;&nbsp;&nbsp;&nbsp;</th>\n";
  print HS "  <th class = \"sortable\" title=\"customer replaceable units\" nowrap=\"\">CRU&nbsp;&nbsp;&nbsp;&nbsp;</th>\n";
  print HS "  <th class = \"sortable\" title=\"Event code\" nowrap=\"\">EvtCd&nbsp;&nbsp;&nbsp;&nbsp;</th>\n";
  print HS "  <th class = \"sortable\" title=\"Description\" nowrap=\"\">Description&nbsp;&nbsp;&nbsp;&nbsp;</th>\n";
  print HS "</thead><tbody>\n";

  foreach my $date_line ( sort { $b <=> $a } keys( %{ $events{events} } ) ) {
    foreach my $id ( sort { $a <=> $b } keys( %{ $events{events}{$date_line} } ) ) {
      if ( defined $events{events}{$date_line}{$id} && $events{events}{$date_line}{$id} ne '' ) {
        print HS "$events{events}{$date_line}{$id}";
      }
    }
  } ## end foreach my $date_line ( sort...)

  print HS "</tbody></table>\n";

  if ( isdigit($last_days) && $last_days > 1 ) {
    print HS "<br>Last $last_days days.<br><br>\n";
  }
  else {
    print HS "<br>Last $last_days day.<br><br>\n";
  }

  close(HS);
  return 0;
} ## end sub health_status

sub file_time_diff {
  my $file = shift;

  my $act_time  = time();
  my $file_time = $act_time;
  my $time_diff = 0;

  if ( -f $file ) {
    $file_time = ( stat($file) )[9];
    $time_diff = $act_time - $file_time;
  }

  return ($time_diff);
} ## end sub file_time_diff

sub isdigit {
  my $digit = shift;

  if ( $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  #main::error ("there was expected a digit but a string is there, field: , value: $digit");
  return 0;
} ## end sub isdigit

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  #print "$act_time: $text : $!\n" if $DEBUG > 2;;

  return 1;
} ## end sub error

