#!/usr/bin/perl
use warnings;
use strict;
use Data::Dumper;
use Date::Parse;
no warnings 'portable';

my %performance_data;


my $STORAGE_NAME       = $ENV{STORAGE_NAME};
my $BASEDIR            = $ENV{INPUTDIR};
my $bindir             = $ENV{BINDIR};
my $webdir             = $ENV{WEBDIR};
my $health_status_html = "$webdir/$STORAGE_NAME/health_status.html";
my $inputdir           = $ENV{INPUTDIR};
my $wrkdir             = "$BASEDIR/data/$STORAGE_NAME";
my $configuration      = "$wrkdir/IOSTATS";
my $config_directory   = "";
my $data_directory     = "";
my $ip_adress          = $ENV{VSPG_IP};
my $login              = $ENV{VSPG_USER};
my $paswd              = $ENV{VSPG_PW};
my $cli_dir            = $ENV{VSP_CLIDIR};
my $id_storage         = $ENV{VSPG_CONF_ID};
my $autetization       = "-I$id_storage";
my $path               = $ENV{PATH};
my $java               = $ENV{HDS_EXPORT_TOOL_JAVAPATH};
if ( defined $java ) {
  $ENV{PATH} = "$java:$path";
}

## encode paswd ##
my $pom_paswd = unobscure_password($paswd);
$paswd = $pom_paswd;

my $data = "DATA";
my $time = "TIME";
my $from = "FROM";
my $to   = "TO";
my $len  = "LENGTH";
my %section_header;
my @values             = "";
my $act_directory      = localtime();
my $output             = get_name_perf_file();
my $sample             = 300;
my $timeout_cmd_high   = 360;
my $timeout_cmd_normal = 180;
my $timeout_cmd_low    = 60;
my $host_index         = 1;

sub create_command_file {
  my $file          = shift;
  my $file2         = shift;
  my $serial_number = shift;
  my $version       = shift;

  ### test command group PhyMPU for vspg 200, 400, 600, 800 and group PhyMainPK for HUS_VM ###
  ( my $version_short, undef ) = split( "/", $version );
  my $cmd = "";
  if ( -e "$cli_dir/$version_short/command.txt" ) {
    my @data     = get_array_data("$cli_dir/$version_short/command.txt");
    my $command1 = "group PhyMPU";
    my $command2 = "group PhyMainPK";
    my $command3 = "group PhyCMPK";
    my $command4 = "group PhyESW";
    my $test     = grep {/$command1/} @data;
    if ($test) {
      $cmd = $command1;
    }
    else {
      $test = grep {/$command2/} @data;
      if ($test) {
        $cmd = "group PhyMainPK Short";

      }
      else{
        $test = grep {/$command3/} @data;
        if ($test) {
          $cmd = "group PhyCMPK Short";
        }
        else{
          $test = grep {/$command4/} @data;
          if ($test) {
            $cmd = "group PhyESW Short";
          }
        }
      }
    } ## end else [ if ($test) ]
  } ## end if ( -e "$cli_dir/$version_short/command.txt")

  #if ($type_storage){
  open( DATA, "> $file" ) || error_die( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ );
  print DATA "ip $ip_adress        ; Specifies IP adress of SVP\n";
  print DATA "dkcsn $serial_number           ; Specifies Serial Number of SVP\n";
  print DATA "login \"$login\" \"$paswd\"   ; Logs user into DKC\n";
  print DATA "show                  ; output storing period\n";
  print DATA "; +------------------------------------------------------------\n";
  print DATA ";group PhyPG ; Parity Groups\n";
  print DATA ";group PhyLDEV ; Logical Volumes\n";
  print DATA "group PhyProc ; Micro-Processor usage\n";
  print DATA "$cmd; Access Paths and Write Pending\n";
  print DATA "; -------------------------------------------------------------------------------------------------\n";
  print DATA "group PG ; Parity Group Statistics\n";
  print DATA "group Port ; Port usage\n";
  print DATA "group LDEVEachOfCU virtual ; LDEV usage in CUs\n";
  print DATA "group PortWWN ; Stats for HBAs connected to ports\n";
  print DATA ";group LU ; LDEV usage Summarised by LU Path\n";
  print DATA "; +------------------------------------------------------------\n";
  print DATA "range -0005:\n";
  print DATA "outpath \"$data_directory\"  ; specifies the sub-directory in which files will be saved\n";
  print DATA "option nocompress\n";
  print DATA "apply\n";
  close(DATA);

  #}
  #else{
  open( DATA, "> $file2" ) || error_die( "Cannot read $file2: $!" . __FILE__ . ":" . __LINE__ );
  print DATA "svpip $ip_adress        ; Specifies IP adress of SVP\n";
  print DATA "login \"$login\" \"$paswd\"   ; Logs user into DKC\n";
  print DATA "show                  ; output storing period\n";
  print DATA "; +------------------------------------------------------------\n";
  print DATA ";group PhyPG ; Parity Groups\n";
  print DATA ";group PhyLDEV ; Logical Volumes\n";
  print DATA "group PhyProc Short ; Micro-Processor usage\n";
  print DATA "$cmd; Access Paths and Write Pending\n";
  print DATA "; -------------------------------------------------------------------------------------------------\n";
  print DATA "group PG ; Parity Group Statistics\n";
  print DATA "group Port ; Port usage\n";
  print DATA "group LDEVEachOfCU virtual ; LDEV usage in CUs\n";
  print DATA "group PortWWN ; Stats for HBAs connected to ports\n";
  print DATA ";group LU ; LDEV usage Summarised by LU Path\n";
  print DATA "; +------------------------------------------------------------\n";
  print DATA "shortrange -0005:\n";
  print DATA "outpath \"$data_directory\"  ; specifies the sub-directory in which files will be saved\n";
  print DATA "option nocompress\n";
  print DATA "apply\n";
  close(DATA);

  #}

} ## end sub create_command_file

sub get_type_storage {
  my $file       = create_configuration_info();
  my $metric     = "RS_GROUP,RGID,V_Serial#,V_ID,V_IF,Serial#";
  my @metrics    = split( ",", $metric );
  my @data       = get_array_data($file);
  my $key_header = "V_ID";
  my $section    = "CONFIG";
  my %header;

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ /$key_header/ ) {
      my $prev;
      foreach my $element (@metrics) {
        if ( $element eq "Serial#" ) {
          $line =~ s/V_Serial#/V_serial/g;    ### exeption index
        }
        my $start_element = index( $line, $element );
        if ( $start_element == -1 ) { next; }
        else {
          if ( defined $prev ) {
            $header{$section}{$prev}{END} = $start_element - 1;
          }
          $header{$section}{$start_element}{NAME}  = $element;
          $header{$section}{$start_element}{START} = $start_element;
          if ( !defined $prev ) {
            $prev = $start_element;
            next;
          }
          if ( defined $header{$section}{$prev}{END} ) {
            $header{$section}{$prev}{LENGTH} = $header{$section}{$prev}{END} - $header{$section}{$prev}{START};
          }
          $prev = $start_element;

        } ## end else [ if ( $start_element ==...)]
      } ## end foreach my $element (@metrics)
      foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
        if ( !defined $header{$section}{$id}{LENGTH} && defined $header{$section}{$id}{START} ) {
          $header{$section}{$id}{LENGTH} = "LAST";
          last;
        }
      } ## end foreach my $id ( sort { $a ...})
    } ## end if ( $line =~ /$key_header/)
    else {
      if ( $line =~ /User for/ ) { next; }
      my $storage_type = "";
      if ( defined $header{$section} ) {
        foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
          if ( defined $header{$section}{$id}{LENGTH} ) {
            my $name = $header{$section}{$id}{NAME};
            if ( $name ne "V_ID" ) { next; }
            my $start  = $header{$section}{$id}{START};
            my $length = $header{$section}{$id}{LENGTH};
            if ( $start == 0 ) { $length--; }
            my $value = substr( $line, $start, $length );
            $value = remove_whitespace($value);

            if ( $name eq "V_ID" ) {
              $storage_type = $value;
              if ( $storage_type eq "M8M" || $storage_type eq "M8S" || $storage_type eq "M8H" ) {
                return 1;
              }
              else {
                return 0;
              }
            } ## end if ( $name eq "V_ID" )
          } ## end if ( defined $header{$section...})
        } ## end foreach my $id ( sort { $a ...})
      } ## end if ( defined $header{$section...})
    } ## end else [ if ( $line =~ /$key_header/)]
  } ## end foreach my $line (@data)
} ## end sub get_type_storage

sub create_configuration_info {
  my $file = "$config_directory/config-STORAGE.txt";
  my $test = test_file_exist($file);
  if ($test) { return $file; }
  else {
    my $cmd = "raidcom get resource -key opt $autetization";
    cmd( $cmd, $file, $timeout_cmd_low );
    return $file;
  }

} ## end sub create_configuration_info

sub collect_metrics {
  my $file  = shift;
  my $file2 = shift;

  #`/usr/java8_64/jre/bin/java -classpath "$cli_dir/JSanExport.jar:$cli_dir/JSanRmiApiSx.jar:$cli_dir/JSanRmiServerUx.jar:$cli_dir/SanRmiApi.jar" -Xmx536870912 -Dmd.command=$file -Dmd.logpath=$wrkdir/log -Dmd.rmitimeout=20 sanproject.getmondat.RJMdMain`;

  #`/usr/java8_64/jre/bin/java -classpath "$cli_dir/JSanExportLoader.jar" -Del.tool.Xmx=536870912 -Dmd.command=$file -Del.logpath=$wrkdir/log -Dmd.rmitimeout=20 sanproject.getexptool.RJElMain`;

  ( my $version, my $serial ) = split( ",", get_version_serial() );
  create_command_file( $file, $file2, $serial, $version );
  ( my $version_short, undef ) = split( "/", $version );
  my $command  = "";
  my $command2 = "";

  ### test type storage file ###
  my $type_storage = "";                      ## 1 - vspg , 0 - hus_vm
  my $type_vspg    = "$wrkdir/type_vspg";
  my $type_hus_vm  = "$wrkdir/type_hus_vm";
  if ( -e $type_vspg ) {
    $type_storage = 1;
  }
  if ( -e $type_hus_vm ) {
    $type_storage = 0;
  }
  if ( !-e $type_hus_vm && !-e $type_vspg ) {
    $type_storage = get_type_storage();
  }

  ###

  ### check version_short exist ###
  my $dir = "$cli_dir/$version_short";
  if ( !-d "$dir" ) {
    error_die("directory $dir (version firmware) does not exist !");
  }
  my $collect_file = "$dir/runUnix.bat";
  my $test         = test_file_exist($collect_file);
  if ( !$test ) {
    error_die("file $collect_file does not exist !");
  }
  my @data = get_array_data($collect_file);

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^java/ ) {
      my $line_pom = $line;
      $line =~ s/command\.txt/$file/g;
      $line_pom =~ s/command\.txt/$file2/g;
      $line =~ s/\.\/lib/$dir\/lib/g;
      $line_pom =~ s/\.\/lib/$dir\/lib/g;
      $line =~ s/=log/=$wrkdir\/log/g;
      $line_pom =~ s/=log/=$wrkdir\/log/g;
      $command  = $line;
      $command2 = $line_pom;
    } ## end if ( $line =~ m/^java/)
  } ## end foreach my $line (@data)

  ### collect performance ###
  `rm -r $data_directory/*`;
  if ($type_storage) {
    `$command 2>>\$ERRLOG_VSPG`;
    if ( $? != 0 ) {
      error_die( "Java failed " . __FILE__ . ":" . __LINE__ );
    }
    else {
      `touch "$type_vspg"`;
    }

  } ## end if ($type_storage)
  else {
    `$command2 2>>\$ERRLOG_VSPG`;
    if ( $? != 0 ) {
      if ( -e $type_hus_vm ) {
        error_die( "Java failed " . __FILE__ . ":" . __LINE__ );
      }
      else {
        `$command 2>>\$ERRLOG_VSPG`;
        if ( $? != 0 ) {
          error_die( "Java failed " . __FILE__ . ":" . __LINE__ );
        }
        else {
          `touch "$type_vspg"`;
        }
      } ## end else [ if ( -e $type_hus_vm )]
    } ## end if ( $? != 0 )
    else { `touch "$type_hus_vm"`; }
  } ## end else [ if ($type_storage) ]

} ## end sub collect_metrics

sub unobscure_password {
  my $string    = shift;
  my $unobscure = DecodeBase64($string);
  $unobscure = unpack( chr( ord("a") + 19 + print "" ), $unobscure );
  return $unobscure;
} ## end sub unobscure_password

sub DecodeBase64 {
  my $d = shift;
  $d =~ tr!A-Za-z0-9+/!!cd;
  $d =~ s/=+$//;
  $d =~ tr!A-Za-z0-9+/! -_!;
  my $r = '';
  while ( $d =~ /(.{1,60})/gs ) {
    my $len = chr( 32 + length($1) * 3 / 4 );
    $r .= unpack( "u", $len . $1 );
  }
  $r;
} ## end sub DecodeBase64

sub create_firmware_storage_file {
  my $file = "$config_directory/config-STORAGE_FIRMWARE.txt";
  my $test = test_file_exist($file);
  if ($test) { return $file; }
  else {
    my $cmd = "raidqry -l $autetization";
    cmd( $cmd, $file, $timeout_cmd_low );
    return $file;
  }

} ## end sub create_firmware_storage_file

sub get_version_serial {

  my $file         = create_firmware_storage_file();
  my @data         = get_array_data($file);
  my $active       = 0;
  my $index_ver    = "";
  my $index_serial = "";
  my $version      = "";
  my $serial       = "";

  foreach my $line (@data) {
    chomp $line;
    if ( $line =~ /User for/ ) { next; }
    if ( $line eq "" ) {
      $active = 0;
      next;
    }
    $line = remove_whitespace($line);
    if ( $line =~ /Micro_ver/ && $line =~ /Serial#/ ) {
      $index_ver    = "";
      $index_serial = "";
      my @header = split( " ", $line );
      my $index = -1;
      foreach my $element (@header) {
        chomp $element;
        $index++;
        if ( $element eq "" ) { next; }
        $element = remove_whitespace($element);
        if ( $element eq "Micro_ver" ) {
          $index_ver = $index;
        }
        if ( $element eq "Serial#" ) {
          $index_serial = $index;
        }
      } ## end foreach my $element (@header)
      if ( $index_ver ne "" && $index_serial ne "" ) {
        $active = 1;
        next;
      }
    } ## end if ( $line =~ /Micro_ver/...)
    if ( $active == 1 ) {
      my @header = split( " ", $line );
      $version = remove_whitespace( $header[$index_ver] );
      $serial  = remove_whitespace( $header[$index_serial] );
    }
  } ## end foreach my $line (@data)

  # check version a serial not empty #
  if ( $version eq "" || $serial eq "" ) {
    error_die("Version or Serial is empty !");

  }
  return "$version,$serial";
} ## end sub get_version_serial

sub delete_file {
  my $file = shift;
  if ( -e $file ) {

    #unlink "$file" or  error_die ("Unable to unlink $file: $!");
  }

} ## end sub delete_file

sub create_parity_gr_file {
  my $file = "$config_directory/config-RG.txt";
  return $file;

  my $cmd = "raidcom get parity_grp $autetization";
  cmd( $cmd, $file );

  return $file;
} ## end sub create_parity_gr_file

sub create_parity_gr_file2 {
  my $file = "$config_directory/config-RG2.txt";
  return $file;

  my $file_in = "$config_directory/config-RG.txt";
  my @array   = get_array_data($file_in);

  foreach my $line (@array) {
    chomp $line;
    if ( $line =~ /User for/ ) { next; }
    if ( $line eq "" )         { next; }
    $line = remove_whitespace($line);
    if ( !( index( $line, "GROUP" ) > -1 ) ) {
      my @element = split( " ", $line );
      $element[1] = remove_whitespace( $element[1] );
      my $cmd      = "raidcom get parity_grp -parity_grp_id $element[1] $autetization";
      my $save_end = "save";
      cmd( $cmd, $file, $save_end );
    } ## end if ( !( index( $line, ...)))
  } ## end foreach my $line (@array)

  return $file;
} ## end sub create_parity_gr_file2

sub get_array_data {
  my $file = shift;
  my $test = test_file_exist($file);
  if ($test) {
    open( FH, "< $file" ) || error_die( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ );
    my @file_all = <FH>;
    close(FH);
    return @file_all;
  } ## end if ($test)
  else {
    error( "File not exist $file: $!" . __FILE__ . ":" . __LINE__ );
    my @file_empty = "";
    return @file_empty;
  }
} ## end sub get_array_data

sub sum_total_metric {
  my $metric1     = shift;
  my $metric2     = shift;
  my $metric3     = shift;
  my $sub_section = shift;
  my $section     = shift;
  my $sum         = 0;

  foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( defined $performance_data{$section}{$sub_section}{$id}{$metric1} && isdigit( $performance_data{$section}{$sub_section}{$id}{$metric1} ) ) {
      $sum = $performance_data{$section}{$sub_section}{$id}{$metric1};
    }
    if ( defined $performance_data{$section}{$sub_section}{$id}{$metric2} && isdigit( $performance_data{$section}{$sub_section}{$id}{$metric2} ) ) {
      $sum = $sum + $performance_data{$section}{$sub_section}{$id}{$metric2};
    }
    if ( $sum != 0 ) {
      $performance_data{$section}{$sub_section}{$id}{$metric3} = $sum;
      next;
    }
  } ## end foreach my $id ( keys %{ $performance_data...})
} ## end sub sum_total_metric

sub set_data_by_header_array {
  my ( $data_line_tmp, $header_index_tmp, $section, $sub_section ) = @_;
  my @data_line    = @{$data_line_tmp};
  my %header_index = %{$header_index_tmp};
  my $id           = "";
  if ( defined $header_index{id} ) {
    $id = $data_line[ $header_index{id} ];
  }
  else {
    $id = "ALL";
    $performance_data{$section}{$sub_section}{$id}{ID} = $id;
  }
  foreach my $element ( keys %header_index ) {
    my $value = $data_line[ $header_index{$element} ];
    $value =~ s/,/ /g;
    my $name_property = $element;
    $performance_data{$section}{$sub_section}{$id}{$name_property} = $value;
  } ## end foreach my $element ( keys ...)
} ## end sub set_data_by_header_array

sub get_header_index {
  my $line   = shift;
  my @header = split( ",", $line );
  my $index  = -1;
  my %header_index;

  foreach my $element (@header) {
    chomp $element;
    $index++;
    if ( $element eq "" ) { next; }
    $element = remove_whitespace($element);
    $header_index{$element} = $index;
  } ## end foreach my $element (@header)
  return %header_index;

} ## end sub get_header_index

sub set_data_port {

  ### FILES ###
  my $file_port_iops         = "$data_directory/Port_dat/Port_IOPS.csv";
  my $file_port_kbps         = "$data_directory/Port_dat/Port_KBPS.csv";
  my $file_port_response     = "$data_directory/Port_dat/Port_Response.csv";
  my $file_port_ini_iops     = "$data_directory/Port_dat/Port_Initiator_IOPS.csv";
  my $file_port_ini_kbps     = "$data_directory/Port_dat/Port_Initiator_KBPS.csv";
  my $file_port_ini_response = "$data_directory/Port_dat/Port_Initiator_Response.csv";

  my $section     = "Port Level Statistics";
  my $sub_section = "PORT";
  my $metric      = "IO Rate(IOPS)";

  #my $header = "ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Hit(%),Write Hit(%),Read Trans. Size(kB/s),Write Trans. Size(kB/s)";
  my $header = "ID,IO Rate(IOPS),Trans. Size(kB/s),Response time,IO Rate(IOPS)_Initiator,Trans. Size(kB/s)_Initiator,Response time_Initiator";
  $section_header{$section} = $header;

  ### PORT IOPS AND TIME ###

  set_data_first( $metric, $section, $sub_section, $file_port_iops );

  ### PORT KBPS ###
  $metric = "Trans. Size(kB/s)";
  set_data( $metric, $section, $sub_section, $file_port_kbps );

  ### PORT RESPONSE ###
  $metric = "Response time";
  set_data( $metric, $section, $sub_section, $file_port_response );

  ### PORT INITIATOR IOPS ###
  $metric = "IO Rate(IOPS)_Initiator";
  set_data( $metric, $section, $sub_section, $file_port_ini_iops );

  ### PORT INITIATOR KBPS ###
  $metric = "Trans. Size(kB/s)_Initiator";
  set_data( $metric, $section, $sub_section, $file_port_ini_kbps );

  ### PORT INITIATOR RESPONSE ###
  $metric = "Response time_Initiator";
  set_data( $metric, $section, $sub_section, $file_port_ini_response );

} ## end sub set_data_port

sub set_data_proc_usage {
  ### FILES ###
  my $file_proc_usage = "$data_directory/PhyProc_dat/PHY_MP.csv";
  my $test            = test_file_exist($file_proc_usage);
  if ( !$test ) {
    $file_proc_usage = "$data_directory/PhyProc_dat/PHY_Short_MP.csv";
  }
  my $section     = "CPU-CORE Level Statistics";
  my $sub_section = "PROC";
  my $metric      = "Usage(%)";

  my $header = "ID,$metric";
  $section_header{$section} = $header;

  ### PROC USAGE ###
  set_data_first( $metric, $section, $sub_section, $file_proc_usage );

  ### add usage rate of DRRs for vsp and hus_vm
  $file_proc_usage = "$data_directory/PhyProc_dat/PHY_Short_DRR.csv";
  $test            = test_file_exist($file_proc_usage);
  if ($test) {
    $section     = "CPU-CORE DRR Level Statistics";
    $sub_section = "PROC";
    $metric      = "Usage(%)";

    $header = "ID,$metric";
    $section_header{$section} = $header;

    ### usage rate of DDRs ###
    set_data_first( $metric, $section, $sub_section, $file_proc_usage );
  } ## end if ($test)

} ## end sub set_data_proc_usage

sub set_data_cpu_util {
  my $file_peding   = "$data_directory/PhyMPU_dat/PHY_Write_Pending_Rate.csv";
  my $file_usage    = "$data_directory/PhyMPU_dat/PHY_Cache_Usage_Rate.csv";
  my $file_allocate = "$data_directory/PhyMPU_dat/PHY_Cache_Allocate.csv";


  ### HUS-VM

  if ( !test_file_exist($file_peding) ) {
    $file_peding = "$data_directory/PhyMainPK_dat/PHY_Short_Write_Pending_Rate.csv";
  }
  if ( !test_file_exist($file_usage) ) {
    $file_usage = "$data_directory/PhyMainPK_dat/PHY_Short_Cache_Usage_Rate.csv";
  }
  ### HUS-VM

  ### G1000

  if ( !test_file_exist($file_peding) ) {
    $file_peding = "$data_directory/PhyCMPK_dat/PHY_Short_Write_Pending_Rate.csv";
  }
  if ( !test_file_exist($file_usage) ) {
    $file_usage = "$data_directory/PhyCMPK_dat/PHY_Short_Cache_Usage_Rate.csv";
  }
  ### G1000

  ### VSP

  if ( !test_file_exist($file_peding) ) {
    $file_peding = "$data_directory/PhyESW_dat/PHY_Short_Write_Pending_Rate.csv";
  }
  if ( !test_file_exist($file_usage) ) {
    $file_usage = "$data_directory/PhyESW_dat/PHY_Short_Cache_Usage_Rate.csv";
  }
  ###




  if ( !test_file_exist($file_peding) && !test_file_exist($file_usage) ) {
    return 0;
  }

  my $section     = "Node Cache Level Statistics";
  my $sub_section = "CPU";
  my $metric      = "Cache Write Peding(%)";
  my $header      = "ID,$metric";

  #$section_header{$section} = $header;

  ### Cache write peding ###
  set_data_first( $metric, $section, $sub_section, $file_peding );

  ### cache usage rate
  $metric = "Cache Usage Rate(%)";
  $header = $header . ",$metric";
  set_data( $metric, $section, $sub_section, $file_usage );

  ### Cache allocate ##
  $metric                   = "Cache Allocate(%)";
  $header                   = $header . ",$metric";
  $section_header{$section} = $header;
  if ( test_file_exist($file_allocate) ) {
    set_data( $metric, $section, $sub_section, $file_usage );
  }

} ## end sub set_data_cpu_util

#sub set_data_raid_group{
#  ### FILES ###
#  my $file_raid_group = "$config_directory/PhyPG_dat/PHY_PG.csv";
#  my $section = "RAID GROUP Level Statistics";
#  my $sub_section = "RAID GROUP";
#  my $metric = "Usage(%)";
#
#  my $header = "ID,$metric";
#  $section_header{$section} = $header;
#
#  ### PROC USAGE ###
#  set_data_first($metric, $section, $sub_section, $file_raid_group);
#
#}

sub remove_whitespace {
  my $value = shift;
  $value =~ s/^\s+|\s+$//g;
  return $value;

} ## end sub remove_whitespace

sub get_total_cap_rg {
  my ($data_tmp)  = @_;
  my @data        = @{$data_tmp};
  my $active      = 0;
  my $index_group = "";
  my $index_lba   = "";
  my %rg_cap;

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) {
      $active = 0;
      next;
    }
    $line = remove_whitespace($line);
    if ( $line =~ /GROUP/ && $line =~ /SIZE_LBA/ ) {
      $index_group = "";
      $index_lba   = "";
      my @header = split( " ", $line );
      my $index = -1;
      foreach my $element (@header) {
        chomp $element;
        $index++;
        if ( $element eq "" ) { next; }
        $element = remove_whitespace($element);
        if ( $element eq "GROUP" ) {
          $index_group = $index;
        }
        if ( $element eq "SIZE_LBA" ) {
          $index_lba = $index;
        }
      } ## end foreach my $element (@header)
      if ( $index_group ne "" && $index_lba ne "" ) {
        $active = 1;
        next;
      }
    } ## end if ( $line =~ /GROUP/ ...)
    if ( $active == 1 ) {
      my @header = split( " ", $line );
      chomp $header[$index_group];
      chomp $header[$index_lba];
      my $name      = remove_whitespace( $header[$index_group] );
      my $lba       = remove_whitespace( $header[$index_lba] );
      my $total_cap = "";
      {
        no warnings;
        $total_cap = round( ( hex($lba) * 512 ) / ( 1024 * 1024 * 1024 ) );
      }
      if ( defined $rg_cap{"$name"} ) {
        my $cap = $rg_cap{"$name"};
        $rg_cap{"$name"} = $cap + $total_cap;
      }
      else {
        $rg_cap{"$name"} = $total_cap;
      }
    } ## end if ( $active == 1 )
  } ## end foreach my $line (@data)

  return %rg_cap;
} ## end sub get_total_cap_rg

sub set_capacity_rg {
  my $section     = shift;
  my $sub_section = shift;
  my $metric      = shift;
  my $file        = shift;
  my $file2       = shift;
  my @metrics     = split( ",", $metric );

  my @data     = get_array_data($file);
  my @data_cap = get_array_data($file2);
  my %header;
  my $key_header = "GROUP";
  my @array_rg;
  my %total_capacity = get_total_cap_rg( \@data_cap );

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ /$key_header/ ) {
      my $prev;
      foreach my $element (@metrics) {
        my $start_element = index( $line, $element );
        if ( $start_element == -1 ) { next; }
        else {
          if ( defined $prev ) {
            $header{$section}{$prev}{END} = $start_element - 1;
          }
          $header{$section}{$start_element}{NAME}  = $element;
          $header{$section}{$start_element}{START} = $start_element;
          if ( !defined $prev ) {
            $prev = $start_element;
            next;
          }
          if ( defined $header{$section}{$prev}{END} ) {
            $header{$section}{$prev}{LENGTH} = $header{$section}{$prev}{END} - $header{$section}{$prev}{START};
          }
          $prev = $start_element;

        } ## end else [ if ( $start_element ==...)]
      } ## end foreach my $element (@metrics)
      foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
        if ( !defined $header{$section}{$id}{LENGTH} && defined $header{$section}{$id}{START} ) {
          $header{$section}{$id}{LENGTH} = "LAST";
          last;
        }
      } ## end foreach my $id ( sort { $a ...})
    } ## end if ( $line =~ /$key_header/)
    else {
      if ( $line =~ /User for/ ) { next; }
      my $group        = "";
      my $size_usage   = "";
      my $size_free    = "";
      my $percent_free = "";
      if ( defined $header{$section} ) {
        foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
          if ( defined $header{$section}{$id}{LENGTH} ) {
            my $name = $header{$section}{$id}{NAME};
            if ( $name ne "GROUP" && $name ne "U(%)" && $name ne "AV_CAP(GB)" ) { next; }
            my $start  = $header{$section}{$id}{START};
            my $length = $header{$section}{$id}{LENGTH};
            if ( $start == 0 ) { $length--; }
            my $value = substr( $line, $start, $length );
            $value = remove_whitespace($value);

            if ( $name eq "GROUP" ) {
              $group = $value;
            }
            if ( $name eq "AV_CAP(GB)" ) {
              $size_free = $value;
            }

            if ( $name eq "U(%)" ) {
              my $percent = $value;
              $percent_free = 100 - $percent;
            }
          } ## end if ( defined $header{$section...})
        } ## end foreach my $id ( sort { $a ...})
        if ( ( $size_free == 0 && defined $total_capacity{$group} && $percent_free == 0 ) || ( defined $total_capacity{$group} && $size_free == 0 && $percent_free != 0 ) ) {

          #my $total_cap = (100/$percent_free) * $size_free;
          #$total_cap = round($total_cap);
          my $total_cap = $total_capacity{$group};
          $size_free                                                          = 0;
          $size_usage                                                         = $total_cap;
          $performance_data{$section}{$sub_section}{$group}{"Total size(TB)"} = round( $total_cap / 1024 );
          $performance_data{$section}{$sub_section}{$group}{"Used size(TB)"}  = round( $size_usage / 1024 );
        } ## end if ( ( $size_free == 0...))
        else {
          if ( defined $total_capacity{$group} ) {
            my $total_cap = $total_capacity{$group};

            #$size_free = ($percent_free/100) * $total_cap;
            $size_usage                                                         = $total_cap - $size_free;
            $performance_data{$section}{$sub_section}{$group}{"Total size(TB)"} = round( $total_cap / 1024 );
            $performance_data{$section}{$sub_section}{$group}{"Used size(TB)"}  = round( $size_usage / 1024 );
          } ## end if ( defined $total_capacity...)
          else {
            push( @array_rg, ",$group,,,,,,,,,,,,,,,,\n" );
          }
        } ## end else [ if ( ( $size_free == 0...))]
      } ## end if ( defined $header{$section...})
    } ## end else [ if ( $line =~ /$key_header/)]
  } ## end foreach my $line (@data)
} ## end sub set_capacity_rg

sub set_data_raid_group {

  ### FILES ###
  #my $file_pool_iops = "$data_directory/PG_dat/PG_IOPS.csv";
  #my $file_pool_iops_read = "$data_directory/PG_dat/PG_Read_IOPS.csv";
  #my $file_pool_iops_write = "$data_directory/PG_dat/PG_Write_IOPS.csv";

  my $file_pool_iops_read_seq  = "$data_directory/PG_dat/PG_Seq_Read_IOPS.csv";
  my $file_pool_iops_read_rnd  = "$data_directory/PG_dat/PG_Rnd_Read_IOPS.csv";
  my $file_pool_iops_write_seq = "$data_directory/PG_dat/PG_Seq_Write_IOPS.csv";
  my $file_pool_iops_write_rnd = "$data_directory/PG_dat/PG_Rnd_Write_IOPS.csv";

  #my $file_pool_trans_rate ="$data_directory/PG_dat/PG_TransRate.csv";
  my $file_pool_trans_rate_read  = "$data_directory/PG_dat/PG_Read_TransRate.csv";
  my $file_pool_trans_rate_write = "$data_directory/PG_dat/PG_Write_TransRate.csv";

  my $file_pool_response       = "$data_directory/PG_dat/PG_Response.csv";
  my $file_pool_response_read  = "$data_directory/PG_dat/PG_Read_Response.csv";
  my $file_pool_response_write = "$data_directory/PG_dat/PG_Write_Response.csv";

  my $file_pool_back_trans = "$data_directory/PG_dat/PG_BackTrans.csv";
  my $file_pool_d2cs       = "$data_directory/PG_dat/PG_D2CS_Trans.csv";
  my $file_pool_d2cr       = "$data_directory/PG_dat/PG_D2CR_Trans.csv";

  my $section     = "Raid Group Level Statistics";
  my $sub_section = "RG";
  my $metric      = "seq_read_io";

  my $header = "ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Total size(TB),Used size(TB),Read response time,Write response time,Data rate back,IO rate back,seq_read_io,rnd_read_io,seq_write_io,rnd_write_io";
  $section_header{$section} = $header;

  ### POOL IOPS READ SEQUENTIAL AND TIME ###

  set_data_first( $metric, $section, $sub_section, $file_pool_iops_read_seq );

  ### POOL READ IOPS ###
  #$metric = "Read Rate(IOPS)";
  #set_data($metric, $section, $sub_section, $file_pool_iops_read);

  ### POOL WRITE IOPS ###
  #$metric = "Write Rate(IOPS)";
  #set_data($metric, $section, $sub_section, $file_pool_iops_write);

  ### POOL READ SEQUENTIAL IOPS ###
  #$metric = "IOPS_READ_SEQUENTIAL";
  #set_data($metric, $section, $sub_section, $file_pool_iops_read_seq);

  ### POOL READ RANDOM IOPS ###
  $metric = "rnd_read_io";
  set_data( $metric, $section, $sub_section, $file_pool_iops_read_rnd );

  ### POOL WRITE SEQUENTIAL IOPS ###
  $metric = "seq_write_io";
  set_data( $metric, $section, $sub_section, $file_pool_iops_write_seq );

  ### POOL WRITE RANDOM IOPS ###
  $metric = "rnd_write_io";
  set_data( $metric, $section, $sub_section, $file_pool_iops_write_rnd );

  ### IOPS READ ###
  set_total_metric( "seq_read_io", "rnd_read_io", "Read Rate(IOPS)", $section, $sub_section );

  ### IOPS WRITE ###
  set_total_metric( "seq_write_io", "rnd_write_io", "Write Rate(IOPS)", $section, $sub_section );

  ### TOTAL IOPS ###
  set_total_metric( "Read Rate(IOPS)", "Write Rate(IOPS)", "IO Rate(IOPS)", $section, $sub_section );

  ### TRANS RATE READ ###
  $metric = "Read Trans. Size(kB/s)";
  set_data( $metric, $section, $sub_section, $file_pool_trans_rate_read );

  ### TRANS_RATE ###
  #$metric = "TRANS_RATE";
  #set_data($metric, $section, $sub_section, $file_pool_trans_rate);

  ### TRANS RATE WRITE ###
  $metric = "Write Trans. Size(kB/s)";
  set_data( $metric, $section, $sub_section, $file_pool_trans_rate_write );

  ### Response TIME ###
  #$metric = "Response time";
  #set_data($metric, $section, $sub_section, $file_pool_response);

  ### Response READ ###
  $metric = "Read response time";
  set_data( $metric, $section, $sub_section, $file_pool_response_read );

  ### Response WRITE ###
  $metric = "Write response time";
  set_data( $metric, $section, $sub_section, $file_pool_response_write );

  ### C2D OR BACK_TRANS ###
  $metric = "Data rate back";
  set_data( $metric, $section, $sub_section, $file_pool_back_trans );

  ### D2CS ###
  $metric = "D2CS";
  set_data( $metric, $section, $sub_section, $file_pool_d2cs );

  ### D2CR ###
  $metric = "D2CR";
  set_data( $metric, $section, $sub_section, $file_pool_d2cr );

  ### D2CS + D2CR je io_rate_back
  $metric = "IO rate back";
  set_total_metric( "D2CR", "D2CS", $metric, $section, $sub_section );

  ### check zero value ###

  foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( $id =~ m/:/ ) {
      delete $performance_data{$section}{$sub_section}{$id};
      next;
    }

    foreach my $metric ( keys %{ $performance_data{$section}{$sub_section}{$id} } ) {
      if ( $metric eq "IO Rate(IOPS)" ) {
        if ( ( isdigit( $performance_data{$section}{$sub_section}{$id}{$metric} ) && $performance_data{$section}{$sub_section}{$id}{$metric} == 0 ) || ( !isdigit( $performance_data{$section}{$sub_section}{$id}{$metric} ) ) ) {    ## add isdigit
          delete $performance_data{$section}{$sub_section}{$id};
        }
        else {
          if ( defined $performance_data{"Volume Level Statistics"}{"VOLUME"}{$id} ) {
            delete $performance_data{$section}{$sub_section}{$id};
          }
        }
      } ## end if ( $metric eq "IO Rate(IOPS)")
    } ## end foreach my $metric ( keys %...)
  } ## end foreach my $id ( keys %{ $performance_data...})

  my $size = keys %{ $performance_data{$section}{$sub_section} };

  if ( $size == 0 ) {
    delete $performance_data{$section};
    return 1;
  }

  ### set capacity
  my $config_file  = create_parity_gr_file();
  my $config_file2 = create_parity_gr_file2();
  $metric = "T,GROUP,Num_LDEV,U(%),AV_CAP(GB),R_LVL,R_TYPE,SL,CL,DRIVE_TYPE";
  set_capacity_rg( $section, $sub_section, $metric, $config_file, $config_file2 );

} ## end sub set_data_raid_group

sub create_host1_file {
  my $file = "$config_directory/config-HOST1.txt";
  return $file;

  my $file_in = "$config_directory/config-PORT.txt";
  my @array   = get_array_data($file_in);

  foreach my $line (@array) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    if ( $line =~ /TAR/ ) {
      my @elements = split( " ", $line );
      $elements[0] = remove_whitespace( $elements[0] );
      my $cmd      = "raidcom get host_grp -port $elements[0] $autetization";
      my $save_end = "save";
      cmd( $cmd, $file, $save_end );
    } ## end if ( $line =~ /TAR/ )
  } ## end foreach my $line (@array)
  return $file;
} ## end sub create_host1_file

sub create_host3_file {
  my $file = "$config_directory/config-HOST3.txt";
  return $file;

  my $file_in = "$config_directory/config-HOST1.txt";
  my @array   = get_array_data($file_in);

  foreach my $line (@array) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    if ( !( index( $line, "GROUP_NAME" ) > -1 ) ) {
      my @elements = split( " ", $line );
      $elements[0] = remove_whitespace( $elements[0] );
      $elements[2] = remove_whitespace( $elements[2] );
      my $port = "$elements[0] $elements[2]";
      `echo $port >> "$file"`;
      my $cmd      = "raidcom get hba_wwn -port $port $autetization";
      my $save_end = "save";
      cmd( $cmd, $file, $save_end );
    } ## end if ( !( index( $line, ...)))
  } ## end foreach my $line (@array)

  return $file;
} ## end sub create_host3_file

sub set_data_pool {

  #print Dumper \%performance_data;
  my $section     = "Pool Level Statistics";
  my $sub_section = "POOL";

  my $sect_vol     = "Volume Level Statistics";
  my $sub_sect_vol = "VOLUME";
  my $metric       = "IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Data rate back,IO rate back,Read Hit(%),Write Hit(%),Read response time,Write response time";
  my @metrics      = split( ",", $metric );

  my $header = "ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Data rate back,IO rate back,Read response time,Write response time,Total size(TB),Used size(TB)";

  $section_header{$section}                 = $header;
  $performance_data{$section}{$time}{$from} = $performance_data{$sect_vol}{$time}{$from};
  $performance_data{$section}{$time}{$to}   = $performance_data{$sect_vol}{$time}{$to};
  $performance_data{$section}{$len}         = $performance_data{$sect_vol}{$len};

  my $iops                       = 0;
  my $sum_iops                   = 0;
  my $read_iops                  = 0;
  my $write_iops                 = 0;
  my $read_rate                  = 0;
  my $write_rate                 = 0;
  my $data_rate_back             = 0;
  my $io_rate_back               = 0;
  my $numerator_write_hit        = 0;
  my $numerator_read_hit         = 0;
  my $numerator_response_read    = 0;
  my $numerator_response_write   = 0;
  my $denominator_write_hit      = 0;
  my $denominator_read_hit       = 0;
  my $denominator_response_read  = 0;
  my $denominator_response_write = 0;

  #print Dumper \$performance_data{"Volume Level Statistics"};

  foreach my $pool ( keys %{ $performance_data{$section}{$sub_section} } ) {
    foreach my $volume ( keys %{ $performance_data{$section}{$sub_section}{$pool}{VOLUMES} } ) {
      if ( defined $performance_data{$section}{$sub_section}{$pool}{VOLUMES}{$volume}{ID} ) {
        my $volume_x = $volume;
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[0] } ) {
          if ( isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[0] } ) ) {
            $sum_iops = $sum_iops + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[0] };
            $iops     = $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[0] };
          }
          else {
            $iops = $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[0] };
          }
        } ## end if ( defined $performance_data...)
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[1] } && isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[1] } ) ) {
          $read_iops = $read_iops + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[1] };
        }
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[2] } && isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[2] } ) ) {
          $write_iops = $write_iops + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[2] };
        }
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[3] } && isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[3] } ) ) {
          $read_rate = $read_rate + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[3] };
        }
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[4] } && isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[4] } ) ) {
          $write_rate = $write_rate + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[4] };
        }
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[5] } && isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[5] } ) ) {
          $data_rate_back = $data_rate_back + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[5] };
        }
        if ( defined $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[6] } && isdigit( $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[6] } ) ) {
          $io_rate_back = $io_rate_back + $performance_data{$sect_vol}{$sub_sect_vol}{$volume_x}{ $metrics[6] };
        }
        my $last_num_read_hit   = $numerator_read_hit;
        my $last_num_write_hit  = $numerator_write_hit;
        my $last_num_read_resp  = $numerator_response_read;
        my $last_num_write_resp = $numerator_response_write;

        #$numerator_read_hit = get_hit_numerator_volume($sect_vol,$sub_sect_vol,$metrics[7],$metrics[0],$volume_x,$numerator_read_hit);
        #if ($numerator_read_hit != $last_num_read_hit){
        #  $denominator_read_hit = get_hit_denominator_volume($sect_vol,$sub_sect_vol,$metrics[0],$volume_x,$denominator_read_hit);
        #}

        #$numerator_write_hit = get_hit_numerator_volume($sect_vol,$sub_sect_vol,$metrics[8],$metrics[0],$volume_x,$numerator_write_hit);
        #if ($numerator_write_hit != $last_num_write_hit){
        #  $denominator_write_hit = get_hit_denominator_volume($sect_vol,$sub_sect_vol,$metrics[0],$volume_x,$denominator_write_hit);
        #}

        $numerator_response_read = get_response_numerator_volume( $sect_vol, $sub_sect_vol, $metrics[9], $iops, $volume_x, $numerator_response_read );
        if ( $numerator_response_read != $last_num_read_resp ) {
          $denominator_response_read = get_response_denominator_volume( $sect_vol, $sub_sect_vol, $iops, $volume_x, $denominator_response_read );
        }
        $numerator_response_write = get_response_numerator_volume( $sect_vol, $sub_sect_vol, $metrics[10], $iops, $volume_x, $numerator_response_write );
        if ( $last_num_write_resp != $numerator_response_write ) {
          $denominator_response_write = get_response_denominator_volume( $sect_vol, $sub_sect_vol, $iops, $volume_x, $denominator_response_write );
        }
      } ## end if ( defined $performance_data...)
    } ## end foreach my $volume ( keys %...)
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[0] } = $sum_iops;
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[1] } = $read_iops;
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[2] } = $write_iops;
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[3] } = $read_rate;
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[4] } = $write_rate;
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[5] } = $data_rate_back;
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[6] } = $io_rate_back;

    #if ($denominator_read_hit != 0){
    #  $performance_data{$section}{$sub_section}{$pool}{$metrics[7]} = round($numerator_read_hit / $denominator_read_hit);
    #}
    #else{
    #  $performance_data{$section}{$sub_section}{$pool}{$metrics[7]} = "NAN";
    #}
    #if ($denominator_write_hit != 0){
    #  $performance_data{$section}{$sub_section}{$pool}{$metrics[8]} = round($numerator_write_hit / $denominator_write_hit);
    #}
    #else{
    #  $performance_data{$section}{$sub_section}{$pool}{$metrics[8]} = "NAN";
    #}
    if ( $denominator_response_read != 0 ) {
      $performance_data{$section}{$sub_section}{$pool}{ $metrics[9] } = round( $numerator_response_read / $denominator_response_read );
    }
    else {
      $performance_data{$section}{$sub_section}{$pool}{ $metrics[9] } = "NAN";
    }
    if ( $denominator_response_write != 0 ) {
      $performance_data{$section}{$sub_section}{$pool}{ $metrics[10] } = round( $numerator_response_write / $denominator_response_write );
    }
    else {
      $performance_data{$section}{$sub_section}{$pool}{ $metrics[10] } = "NAN";
    }
    delete $performance_data{$section}{$sub_section}{$pool}{VOLUMES};

    $io_rate_back   = 0;
    $data_rate_back = 0;
    $write_rate     = 0;
    $read_rate      = 0;
    $write_iops     = 0;
    $read_iops      = 0;
    $sum_iops       = 0;

    $numerator_write_hit        = 0;
    $numerator_read_hit         = 0;
    $numerator_response_read    = 0;
    $numerator_response_write   = 0;
    $denominator_write_hit      = 0;
    $denominator_read_hit       = 0;
    $denominator_response_read  = 0;
    $denominator_response_write = 0;
  } ## end foreach my $pool ( keys %{ ...})

  my $file_pool = create_pool_file();
  $metric = "PID,POLS,U(%),SSCNT,Available(MB),Capacity(MB),Seq#,Num,LDEV#,H(%)";
  set_capacity_pool( $section, $sub_section, $metric, $file_pool );

  #create data for unknow pool - this is from reason because some volumes is not in ldev config but in performance is
  #my $metric = "IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Data rate back,IO rate back,Read Hit(%),Write Hit(%),Read response time,Write response time";

  $iops                       = 0;
  $sum_iops                   = 0;
  $read_iops                  = 0;
  $write_iops                 = 0;
  $read_rate                  = 0;
  $write_rate                 = 0;
  $data_rate_back             = 0;
  $io_rate_back               = 0;
  $numerator_write_hit        = 0;
  $numerator_read_hit         = 0;
  $numerator_response_read    = 0;
  $numerator_response_write   = 0;
  $denominator_write_hit      = 0;
  $denominator_read_hit       = 0;
  $denominator_response_read  = 0;
  $denominator_response_write = 0;

  $section     = "Volume Level Statistics";
  $sub_section = "VOLUME";

  foreach my $id_volume ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( !defined $performance_data{$section}{$sub_section}{$id_volume}{POOL} ) {
      $iops           = $iops + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[0] } );
      $read_iops      = $read_iops + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[1] } );
      $write_iops     = $write_iops + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[2] } );
      $read_rate      = $read_rate + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[3] } );
      $write_rate     = $write_rate + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[4] } );
      $data_rate_back = $data_rate_back + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[5] } );
      $io_rate_back   = $io_rate_back + get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[6] } );

      ### response time
      $numerator_response_read  = $numerator_response_read +  ( get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[0] } ) * get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[9] } ) );
      $numerator_response_write = $numerator_response_write + ( get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[0] } ) * get_value( $performance_data{$section}{$sub_section}{$id_volume}{ $metrics[10] } ) );

      $denominator_response_read  = $denominator_response_read + $iops;
      $denominator_response_write = $denominator_response_write + $iops;

    } ## end if ( !defined $performance_data...)
  } ## end foreach my $id_volume ( keys...)
  $section     = "Pool Level Statistics";
  $sub_section = "POOL";
  my $pool = "-1";
  $performance_data{$section}{$sub_section}{$pool}{ID}            = $pool;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[0] } = $iops;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[1] } = $read_iops;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[2] } = $write_iops;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[3] } = $read_rate;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[4] } = $write_rate;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[5] } = $data_rate_back;
  $performance_data{$section}{$sub_section}{$pool}{ $metrics[6] } = $io_rate_back;

  if ( $denominator_response_read != 0 ) {
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[9] } = round( $numerator_response_read / $denominator_response_read );
  }
  else {
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[9] } = "NAN";
  }
  if ( $denominator_response_write != 0 ) {
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[10] } = round( $numerator_response_write / $denominator_response_write );
  }
  else {
    $performance_data{$section}{$sub_section}{$pool}{ $metrics[10] } = "NAN";
  }
} ## end sub set_data_pool

sub get_value {
  my $value = shift;
  if ( !defined $value ) {
    return 0;
  }

  if ( isdigit($value) ) {
    return $value;
  }
  else {
    return 0;
  }

} ## end sub get_value

sub set_capacity_pool {
  my $section     = shift;
  my $sub_section = shift;
  my $metric      = shift;
  my $file        = shift;
  my @metrics     = split( ",", $metric );
  my @data        = get_array_data($file);
  my $key_header  = "PID";
  my %header;

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ /$key_header/ ) {
      my $prev;
      foreach my $element (@metrics) {
        my $start_element = index( $line, $element );
        if ( $start_element == -1 ) { next; }
        else {
          if ( defined $prev ) {
            $header{$section}{$prev}{END} = $start_element - 1;
          }
          $header{$section}{$start_element}{NAME}  = $element;
          $header{$section}{$start_element}{START} = $start_element;
          if ( !defined $prev ) {
            $prev = $start_element;
            next;
          }
          if ( defined $header{$section}{$prev}{END} ) {
            $header{$section}{$prev}{LENGTH} = $header{$section}{$prev}{END} - $header{$section}{$prev}{START};
          }
          $prev = $start_element;

        } ## end else [ if ( $start_element ==...)]
      } ## end foreach my $element (@metrics)
      foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
        if ( !defined $header{$section}{$id}{LENGTH} && defined $header{$section}{$id}{START} ) {
          $header{$section}{$id}{LENGTH} = "LAST";
          last;
        }
      } ## end foreach my $id ( sort { $a ...})
    } ## end if ( $line =~ /$key_header/)
    else {
      if ( $line =~ /User for/ ) { next; }
      my $id_pool    = "";
      my $size_usage = "";
      my $size_free  = "";
      my $size_total = "";
      if ( defined $header{$section} ) {
        foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
          if ( defined $header{$section}{$id}{LENGTH} ) {
            my $name = $header{$section}{$id}{NAME};
            if ( $name ne "PID" && $name ne "Available(MB)" && $name ne "Capacity(MB)" ) { next; }
            my $start  = $header{$section}{$id}{START};
            my $length = $header{$section}{$id}{LENGTH};
            if ( $start == 0 ) { $length--; }
            if ( $length eq "LAST" ) {
              $length = length($line);
            }
            my $value = substr( $line, $start, $length );
            $value = remove_whitespace($value);
            if ( $name eq "PID" ) {
              $id_pool = $value;
            }
            if ( $name eq "Available(MB)" ) {
              $size_free = $value;
            }

            if ( $name eq "Capacity(MB)" ) {
              $size_total = $value;
              if ( $size_total =~ /  / ) {
                $size_total =~ s/ +.*//g;
                $size_total = remove_whitespace($size_total);
              }
            } ## end if ( $name eq "Capacity(MB)")
          } ## end if ( defined $header{$section...})
        } ## end foreach my $id ( sort { $a ...})
        if ( $size_total ne "" && $size_free ne "" ) {
          $size_usage = $size_total - $size_free;
        }
        if ( $size_usage ne "" && $id_pool ne "" ) {
          $size_total = $size_total / ( 1024 * 1024 );
          $size_total = round($size_total);
          $size_free  = $size_free / ( 1024 * 1024 );
          $size_free  = round($size_free);
          $size_usage = $size_usage / ( 1024 * 1024 );
          $size_usage = round($size_usage);
          my $id_len = length($id_pool);
          my @chars = split( "", $id_pool );

          if ( $id_len > 1 ) {
            for ( my $i = 0; $i < $id_len - 1; $i++ ) {
              if ( $chars[$i] eq "0" ) {
                $id_pool =~ s/^.//g;
              }
              else {
                last;
              }
            } ## end for ( my $i = 0; $i < $id_len...)
          } ## end if ( $id_len > 1 )
          if ( defined $performance_data{$section}{$sub_section}{$id_pool}{ID} ) {
            $performance_data{$section}{$sub_section}{$id_pool}{"Total size(TB)"} = $size_total;
            $performance_data{$section}{$sub_section}{$id_pool}{"Used size(TB)"}  = $size_usage;
          }
          else {
            delete $performance_data{$section}{$sub_section}{$id_pool};
          }
        } ## end if ( $size_usage ne ""...)
      } ## end if ( defined $header{$section...})
    } ## end else [ if ( $line =~ /$key_header/)]
  } ## end foreach my $line (@data)
} ## end sub set_capacity_pool

sub create_pool_file {
  my $file = "$config_directory/config-POOL2.txt";
  return $file;
  my $cmd = "raidcom get pool $autetization";
  cmd( $cmd, $file );
  return $file;

} ## end sub create_pool_file

sub set_data_wwn {

  #check vspg_native_hosts
  my $VSPG_NATIVE_HOSTS = 1;
  if ( defined $ENV{VSPG_NATIVE_HOSTS} ) {
    $VSPG_NATIVE_HOSTS = $ENV{VSPG_NATIVE_HOSTS};
    if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} && $VSPG_NATIVE_HOSTS == 0 ) {
      $VSPG_NATIVE_HOSTS = 1;    # host are specified, switch default to host data back
      foreach my $storage_act ( split( / /, $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST} ) ) {
        if ( $STORAGE_NAME =~ m/^$storage_act$/ ) {
          $VSPG_NATIVE_HOSTS = 0;
        }
      }
    } ## end if ( defined $ENV{VSPG_NATIVE_HOSTS_EXCLUDED_LIST...})
  } ## end if ( defined $ENV{VSPG_NATIVE_HOSTS...})

  if ( $VSPG_NATIVE_HOSTS == 0 ) {
    return 0;
  }

  my $space_config_directory = check_space_name($data_directory);
  my @files                  = <$space_config_directory/PortWWN_dat/*>;
  my $size_array             = scalar @files;
  if ( $size_array == 0 ) { return 0; }
  my $port_file  = create_port_file();
  my $host_file  = create_host1_file();
  my $host_file3 = create_host3_file();
  my @data       = get_array_data($host_file3);
  my $metric_h   = "PORT,GID,GROUP_NAME,HWWN,Serial#,NICK_NAME";
  my @metrics    = split( ",", $metric_h );
  my %header;
  my $index = 0;
  my @host;

  my $metric;
  my $key_header  = "PORT";
  my $section     = "Host Level Statistics";
  my $sub_section = "HOST";
  my $header      = "ID,IO Rate(IOPS),Trans. Size(kB/s),Response time";
  $section_header{$section} = $header;

  foreach my $file (@files) {
    if ( $file =~ /IOPS/ ) {
      $metric = "IO Rate(IOPS)";
      set_data_first( $metric, $section, $sub_section, $file );
    }
    if ( $file =~ /KBPS/ ) {
      $metric = "Trans. Size(kB/s)";
      set_data_first( $metric, $section, $sub_section, $file );
    }
    if ( $file =~ /Response/ ) {
      $metric = "Response time";
      set_data_first( $metric, $section, $sub_section, $file );
    }
  } ## end foreach my $file (@files)

  foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( defined $performance_data{$section}{$sub_section}{$id}{ID} ) {
      my $ids = $performance_data{$section}{$sub_section}{$id}{ID};
      if ( grep { $ids eq $_ } @host ) {
        next;
      }
      push( @host, "$ids" );
    } ## end if ( defined $performance_data...)
  } ## end foreach my $id ( keys %{ $performance_data...})
  foreach my $host_id (@host) {
    my $iops        = 0;
    my $data_rate   = 0;
    my $numerator   = 0;
    my $denominator = 0;
    foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
      if ( $performance_data{$section}{$sub_section}{$id}{ID} eq $host_id ) {
        if ( defined $performance_data{$section}{$sub_section}{$id}{"IO Rate(IOPS)"} && $performance_data{$section}{$sub_section}{$id}{"IO Rate(IOPS)"} ne "NAN" ) {
          $iops = $iops + $performance_data{$section}{$sub_section}{$id}{"IO Rate(IOPS)"};
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{"Trans. Size(kB/s)"} && $performance_data{$section}{$sub_section}{$id}{"Trans. Size(kB/s)"} ne "NAN" ) {

          $data_rate = $data_rate + $performance_data{$section}{$sub_section}{$id}{"Trans. Size(kB/s)"};
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{"Response time"} && $performance_data{$section}{$sub_section}{$id}{"Response time"} ne "NAN" ) {
          ### find iops host from port
          my $resp = $performance_data{$section}{$sub_section}{$id}{"Response time"};
          my $port = $performance_data{$section}{$sub_section}{$id}{PORT};
          foreach my $id_host ( keys %{ $performance_data{$section}{$sub_section} } ) {
            if ( $performance_data{$section}{$sub_section}{$id_host}{ID} eq $performance_data{$section}{$sub_section}{$id}{ID} && defined $performance_data{$section}{$sub_section}{$id_host}{"IO Rate(IOPS)"} && $performance_data{$section}{$sub_section}{$id_host}{PORT} eq $port ) {
              if ( $performance_data{$section}{$sub_section}{$id_host}{"IO Rate(IOPS)"} ne "NAN" ) {
                $numerator = $numerator + ( $performance_data{$section}{$sub_section}{$id_host}{"IO Rate(IOPS)"} * $resp );
              }

            } ## end if ( $performance_data...)
          } ## end foreach my $id_host ( keys ...)

        } ## end if ( defined $performance_data...)

        #delete $performance_data{$section}{$sub_section}{$id};
      } ## end if ( $performance_data...)
    } ## end foreach my $id ( keys %{ $performance_data...})
    $performance_data{$section}{$sub_section}{$host_id}{"IO Rate(IOPS)"}     = $iops;
    $performance_data{$section}{$sub_section}{$host_id}{"Trans. Size(kB/s)"} = $data_rate;
    if ( $iops != 0 ) {
      $performance_data{$section}{$sub_section}{$host_id}{"Response time"} = round( $numerator / $iops );
    }
    else {
      $performance_data{$section}{$sub_section}{$host_id}{"Response time"} = "NAN";
    }
    $performance_data{$section}{$sub_section}{$host_id}{ID} = $host_id;
  } ## end foreach my $host_id (@host)

  foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( !grep { $id eq $_ } @host ) {
      delete $performance_data{$section}{$sub_section}{$id};
    }
  }

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ /$key_header/ ) {
      my $prev;
      foreach my $element (@metrics) {
        my $start_element = index( $line, $element );
        if ( $start_element == -1 ) { next; }
        else {
          if ( defined $prev ) {
            $header{$section}{$prev}{END} = $start_element - 1;
          }
          $header{$section}{$start_element}{NAME}  = $element;
          $header{$section}{$start_element}{START} = $start_element;
          if ( !defined $prev ) {
            $prev = $start_element;
            next;
          }
          if ( defined $header{$section}{$prev}{END} ) {
            $header{$section}{$prev}{LENGTH} = $header{$section}{$prev}{END} - $header{$section}{$prev}{START};
          }
          $prev = $start_element;

        } ## end else [ if ( $start_element ==...)]
      } ## end foreach my $element (@metrics)
      foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
        if ( !defined $header{$section}{$id}{LENGTH} && defined $header{$section}{$id}{START} ) {
          $header{$section}{$id}{LENGTH} = "LAST";
          last;
        }
      } ## end foreach my $id ( sort { $a ...})

    } ## end if ( $line =~ /$key_header/)
    else {
      if ( $line =~ /User for/ ) { next; }
      foreach my $ids ( keys %{ $performance_data{$section}{$sub_section} } ) {
        if ( !defined $performance_data{$section}{$sub_section}{$ids}{ID} ) {
          delete $performance_data{$section}{$sub_section}{$ids}{ID};
          next;
        }
        my $name       = $performance_data{$section}{$sub_section}{$ids}{ID};
        my $lc         = lc $name;
        my $group_name = "";

        #push(@wwn,$lc);
        $lc =~ s/\(.*\)//g;
        if ( index( $line, $lc ) > -1 ) {
          foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
            if ( defined $header{$section}{$id}{LENGTH} ) {
              my $name = $header{$section}{$id}{NAME};
              if ( $name ne "NICK_NAME" && $name ne "GROUP_NAME" ) { next; }
              my $start  = $header{$section}{$id}{START};
              my $length = $header{$section}{$id}{LENGTH};
              if ( $length eq "LAST" ) {
                $length = length $line;
              }
              if ( $start == 0 ) { $length--; }
              my $size = length $line;
              if ( $size < $start ) {next}    ### exception substr warnings;
              my $value = substr( $line, $start, $length );
              $value = remove_whitespace($value);
              if ( $name eq "GROUP_NAME" ) {
                if ( defined $value && $value =~ / / ) {
                  my @groups = split( " ", $value );
                  $value = $groups[0];
                }
                $group_name = "$value";
                next;
              } ## end if ( $name eq "GROUP_NAME")
              if ( $value eq "" || $value eq "-" ) {
                if ( $group_name ne "" ) {

                  #print "$group_name\n";
                  $performance_data{$section}{$sub_section}{$ids}{ID}         = $group_name;
                  $performance_data{$section}{$sub_section}{$ids}{CONVERSION} = "yes";
                } ## end if ( $group_name ne "")
                next;
              } ## end if ( $value eq "" || $value...)
              if ( $value =~ / / ) {
                my @host_name = split( " ", $value );
                $value = $host_name[-1];
              }
              $performance_data{$section}{$sub_section}{$ids}{ID}         = $value;
              $performance_data{$section}{$sub_section}{$ids}{CONVERSION} = "yes";

              #push(@wwn,$value);
            } ## end if ( defined $header{$section...})
          } ## end foreach my $id ( sort { $a ...})
        } ## end if ( index( $line, $lc...))
      } ## end foreach my $ids ( keys %{ $performance_data...})
    } ## end else [ if ( $line =~ /$key_header/)]
  } ## end foreach my $line (@data)
  foreach my $ids ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( defined $performance_data{$section}{$sub_section}{$ids}{CONVERSION} ) {
    }
    else {
      delete $performance_data{$section}{$sub_section}{$ids};
    }
  } ## end foreach my $ids ( keys %{ $performance_data...})

  ### total iops, data_Rate, resp for same host in ports
  my %host_total;
  my @host_same_name;

  #print Dumper \$performance_data{$section};

  foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( defined $performance_data{$section}{$sub_section}{$id}{ID} ) {
      my $name = $performance_data{$section}{$sub_section}{$id}{ID};
      if ( !defined $host_total{$name}{ID} ) {
        $host_total{$name}{ID} = $performance_data{$section}{$sub_section}{$id}{ID};
      }
      else {
        push( @host_same_name, "$name" );
      }
    } ## end if ( defined $performance_data...)
  } ## end foreach my $id ( keys %{ $performance_data...})
  foreach my $host (@host_same_name) {
    my $total_iops       = 0;
    my $total_data       = 0;
    my $total_resp       = 0;
    my $numerator_resp   = 0;
    my $denominator_resp = 0;
    foreach my $wwn ( keys %{ $performance_data{$section}{$sub_section} } ) {
      if ( !defined $performance_data{$section}{$sub_section}{$wwn}{ID} ) { next; }
      my $wwn_name = $performance_data{$section}{$sub_section}{$wwn}{ID};
      if ( $wwn_name ne $host ) { next; }
      my $iops = "NAN";
      my $resp = "NAN";
      my $data = "NAN";
      if ( defined $performance_data{$section}{$sub_section}{$wwn}{"IO Rate(IOPS)"} ) {
        $iops = $performance_data{$section}{$sub_section}{$wwn}{"IO Rate(IOPS)"};
      }
      if ( defined $performance_data{$section}{$sub_section}{$wwn}{"Trans. Size(kB/s)"} ) {
        $data = $performance_data{$section}{$sub_section}{$wwn}{"Trans. Size(kB/s)"};
      }
      if ( defined $performance_data{$section}{$sub_section}{$wwn}{"Response time"} ) {
        $resp = $performance_data{$section}{$sub_section}{$wwn}{"Response time"};
      }
      if ( isdigit($data) ) {
        $total_data = $total_data + $data;
      }
      if ( isdigit($iops) ) {
        $total_iops = $total_iops + $iops;
      }
      $numerator_resp = get_numerator_wwn( $numerator_resp, $iops, $resp );
      $denominator_resp = get_denominator_wwn( $denominator_resp, $iops );
      delete $performance_data{$section}{$sub_section}{$wwn};
    } ## end foreach my $wwn ( keys %{ $performance_data...})
    $performance_data{$section}{$sub_section}{$host}{"Trans. Size(kB/s)"} = $total_data;
    $performance_data{$section}{$sub_section}{$host}{"IO Rate(IOPS)"}     = $total_iops;
    if ( $denominator_resp == 0 ) {
      $performance_data{$section}{$sub_section}{$host}{"Response time"} = "NAN";
    }
    else {
      $performance_data{$section}{$sub_section}{$host}{"Response time"} = round( $numerator_resp / $denominator_resp );
    }
    $performance_data{$section}{$sub_section}{$host}{ID}         = $host;
    $performance_data{$section}{$sub_section}{$host}{CONVERSION} = "yes";

  } ## end foreach my $host (@host_same_name)

  #print Dumper \$performance_data{$section};

  #print Dumper \@host_same_name;

} ## end sub set_data_wwn

sub get_numerator_wwn {
  my $total = shift;
  my $iops  = shift;
  my $resp  = shift;

  if ( !isdigit($resp) || !isdigit($iops) ) {
    return $total;
  }
  else {
    $total = $total + ( $iops * $resp );
    return $total;
  }
} ## end sub get_numerator_wwn

sub get_denominator_wwn {
  my $total = shift;
  my $iops  = shift;

  if ( !isdigit($iops) ) {
    return $total;
  }
  else {
    $total = $total + $iops;
    return $total;
  }
} ## end sub get_denominator_wwn

sub create_ldev_file {
  my $file = "$config_directory/config-LDEV.txt";
  return $file;

  my $cmd = "raidcom get ldev -ldev_list defined -fx $autetization";
  cmd( $cmd, $file );
  return $file;
} ## end sub create_ldev_file

sub set_data_ldev {

  my $file_cap = create_ldev_file();

  ## rnd read iops
  my $space_config_directory = check_space_name($data_directory);
  my @files                  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Rnd_Read_IOPS/*>;
  my $section                = "Volume Level Statistics";
  my $sub_section            = "VOLUME";
  my $metric                 = "rnd_read_io";
  my $header                 = "ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Hit(%),Write Hit(%),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Read response time,Write response time,Capacity(MB),Used(MB),Data rate back,IO rate back,seq_read_io,rnd_read_io,seq_write_io,rnd_write_io,seq_read_hit,rnd_read_hit,seq_write_hit,rnd_write_hit";
  $section_header{$section} = $header;

  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "seq_read_io";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Seq_Read_IOPS/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "seq_write_io";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Seq_Write_IOPS/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "rnd_write_io";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Rnd_Write_IOPS/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "seq_read_hit";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Seq_Read_Hit/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "rnd_read_hit";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Rnd_Read_Hit/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "seq_write_hit";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Seq_Write_Hit/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "rnd_write_hit";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Rnd_Write_Hit/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "Read Trans. Size(kB/s)";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Read_TransRate/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "Write Trans. Size(kB/s)";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Write_TransRate/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "Read response time";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Read_Response/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  $metric = "Write response time";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_Write_Response/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  ### C2D ###
  $metric = "Data rate back";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_C2D_Trans/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  ### D2CS ###
  $metric = "D2CS";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_D2CS_Trans/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  ### D2CR ###
  $metric = "D2CR";
  @files  = <$space_config_directory/LDEVEachOfCU_dat/LDEV_D2CR_Trans/*>;
  set_data_sec( $section, $sub_section, $metric, \@files );

  foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
    if ( defined $performance_data{$section}{$sub_section}{$id}{ID} ) {
      my $ids = $performance_data{$section}{$sub_section}{$id}{ID};

      #$ids =~ s/X//g;
      $performance_data{$section}{$sub_section}{$id}{ID} = $ids;
    } ## end if ( defined $performance_data...)
  } ## end foreach my $id ( keys %{ $performance_data...})

  #print Dumper \%performance_data;

  ### set metric per id ldev ###
  #$metric = "seq_read_io,seq_write_io,rnd_read_io,rnd_write_io,Read Trans. Size(kB/s),Write Trans. Size(kB/s),D2CS,Data rate back,D2CR";
  #set_id_volume($section,$sub_section,$metric);

  ### IOPS READ ###
  set_total_metric( "seq_read_io", "rnd_read_io", "Read Rate(IOPS)", $section, $sub_section );

  ### IOPS WRITE ###
  set_total_metric( "seq_write_io", "rnd_write_io", "Write Rate(IOPS)", $section, $sub_section );

  ### total IOPS ###
  set_total_metric( "Read Rate(IOPS)", "Write Rate(IOPS)", "IO Rate(IOPS)", $section, $sub_section );

  ### D2CS + D2CR je io_rate_back
  $metric = "IO rate back";
  set_total_metric( "D2CR", "D2CS", $metric, $section, $sub_section );

  ### HIT READ ###
  $metric = "Read Hit(%)";
  set_total_metric( "seq_read_hit", "rnd_read_hit", $metric, $section, $sub_section, "seq_read_io", "rnd_read_io" );

  ### HIT WRITE ###
  $metric = "Write Hit(%)";
  set_total_metric( "seq_write_hit", "rnd_write_hit", $metric, $section, $sub_section, "seq_write_io", "rnd_write_io" );

  ### capacity ###
  $metric = "LDEV,F_POOLID,B_POOLID,VOL_Capacity(BLK),Used_Block(BLK),TIER#1(MB),TIER#2(MB),TIER#3(MB)";
  set_capacity_ldev( $section, $sub_section, $file_cap, $metric );

} ## end sub set_data_ldev

sub set_data_sec {

  my ( $section, $sub_section, $metric, $file_array ) = @_;
  my @data = @{$file_array};

  foreach my $file (@data) {
    set_data_first( $metric, $section, $sub_section, $file );
  }

} ## end sub set_data_sec

sub set_capacity_ldev {
  my $section     = shift;
  my $sub_section = shift;
  my $file        = shift;
  my $metric      = shift;

  my @metrics = split( ",", $metric );
  my @data = get_array_data($file);

  my $active    = 0;
  my $ldev_id   = "";
  my $total_cap = "";
  my $used_cap  = "";
  my $tier1     = "";
  my $tier2     = "";
  my $tier3     = "";
  my $f_pool    = "";
  my $b_pool    = "";

  #print Dumper \%performance_data;
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) {

      #print "$ldev_id a $f_pool\n";
      #$ldev_id = $ldev_id . "X";
      if ( defined $performance_data{$section}{$sub_section}{$ldev_id} && $f_pool eq "NONE" ) {
        $performance_data{$section}{$sub_section}{$ldev_id}{"Capacity(MB)"}   = $total_cap;
        $performance_data{$section}{$sub_section}{$ldev_id}{"Used(MB)"}       = $used_cap;
        $performance_data{$section}{$sub_section}{$ldev_id}{"Tier1(MB)_used"} = $tier1;
        $performance_data{$section}{$sub_section}{$ldev_id}{"Tier2(MB)_used"} = $tier2;
        $performance_data{$section}{$sub_section}{$ldev_id}{"Tier3(MB)_used"} = $tier3;

        #if ($tier1 eq ""){
        # $tier1 = 0;
        #}
        #if ($tier2 eq ""){
        # $tier2 = 0;
        #}
        #if ($tier3 eq ""){
        # $tier3 = 0;
        #}
        if ( $tier1 ne "0" ) {
          $performance_data{$section}{$sub_section}{$ldev_id}{"Tier1(MB)_cap"} = $total_cap;
        }
        else {
          $performance_data{$section}{$sub_section}{$ldev_id}{"Tier1(MB)_cap"} = 0;
        }

        if ( $tier2 ne "0" ) {
          $performance_data{$section}{$sub_section}{$ldev_id}{"Tier2(MB)_cap"} = $total_cap;
        }
        else {
          $performance_data{$section}{$sub_section}{$ldev_id}{"Tier2(MB)_cap"} = 0;
        }

        if ( $tier3 ne "0" ) {
          $performance_data{$section}{$sub_section}{$ldev_id}{"Tier3(MB)_cap"} = $total_cap;
        }
        else {
          $performance_data{$section}{$sub_section}{$ldev_id}{"Tier3(MB)_cap"} = 0;
        }

        $active    = 0;
        $ldev_id   = "";
        $total_cap = "";
        $used_cap  = "";
        $tier1     = "";
        $tier2     = "";
        $tier3     = "";
        $f_pool    = "";
        $b_pool    = "";
        next;
      } ## end if ( defined $performance_data...)
    } ## end if ( $line eq "" )
    $line = remove_whitespace($line);

    if ( index( $line, "Serial" ) > -1 ) {
      $active = 1;
      next;
    }
    foreach my $element (@metrics) {
      if ( index( $line, $element ) > -1 && $active == 1 ) {

        my @array_line = split( ":", $line );
        my $header_element = "";
        foreach my $array_element (@array_line) {
          chomp $array_element;
          $array_element =~ s/VIR_LDEV//g;    ### virtual ldev
          $array_element = remove_whitespace($array_element);
          if ( index( $array_element, $element ) > -1 ) {
            $header_element = $array_element;
            next;
          }
          if ( $header_element eq "LDEV" ) {
            $ldev_id = $array_element;

            #print "$ldev_id\n";
            my $len = length($ldev_id);

            #print "pocet $len\n";
            if ( $len < 6 ) {
              my $add_null = 6 - $len;
              for ( my $i = 0; $i < $add_null; $i++ ) {
                $ldev_id = "0$ldev_id";

                #print "$ldev_id\n";
              }
            } ## end if ( $len < 6 )
            my @chars = split( "", $ldev_id );
            $ldev_id = "$chars[0]$chars[1]:$chars[2]$chars[3]:$chars[4]$chars[5]";
            my $uc = uc($ldev_id);
            $ldev_id = $uc;
            next;
          } ## end if ( $header_element eq...)
          if ( $header_element eq "VOL_Capacity(BLK)" && $ldev_id ne "" ) {
            my $transfer_cap = ( $array_element * 512 ) / ( 1024 * 1024 );
            $array_element = round($transfer_cap);
            $total_cap     = $array_element;
            next;
          } ## end if ( $header_element eq...)
          if ( $header_element eq "F_POOLID" && $ldev_id ne "" ) {
            $f_pool = $array_element;
            next;
          }
          if ( $header_element eq "B_POOLID" && $ldev_id ne "" ) {
            $b_pool = $array_element;
            if ( isdigit($b_pool) ) {
              $performance_data{$section}{VOLUME}{$ldev_id}{POOL} = $b_pool;
            }
            if ( $b_pool ne "" && isdigit($b_pool) && !defined $performance_data{"Pool Level Statistics"}{POOL}{$b_pool}{ID} ) {
              $performance_data{"Pool Level Statistics"}{POOL}{$b_pool}{ID} = $b_pool;
              $performance_data{"Pool Level Statistics"}{POOL}{$b_pool}{VOLUMES}{$ldev_id}{ID} = $ldev_id;

            }
            else {
              if ( $b_pool ne "" && isdigit($b_pool) && !defined $performance_data{"Pool Level Statistics"}{POOL}{$b_pool}{VOLUMES}{$ldev_id}{ID} ) {
                $performance_data{"Pool Level Statistics"}{POOL}{$b_pool}{VOLUMES}{$ldev_id}{ID} = $ldev_id;
              }
            }
            next;
          } ## end if ( $header_element eq...)
          if ( $header_element eq "Used_Block(BLK)" && $ldev_id ne "" ) {
            my $transfer_cap = ( $array_element * 512 ) / ( 1024 * 1024 );
            $array_element = round($transfer_cap);
            $used_cap      = $array_element;
            next;
          } ## end if ( $header_element eq...)
          if ( $header_element eq "TIER#1(MB)" && $ldev_id ne "" ) {
            $tier1 = $array_element;
            next;
          }
          if ( $header_element eq "TIER#2(MB)" && $ldev_id ne "" ) {
            $tier2 = $array_element;
            next;
          }
          if ( $header_element eq "TIER#3(MB)" && $ldev_id ne "" ) {
            $tier3 = $array_element;
            next;
          }
        } ## end foreach my $array_element (...)
      } ## end if ( index( $line, $element...))
    } ## end foreach my $element (@metrics)
  } ## end foreach my $line (@data)
} ## end sub set_capacity_ldev

sub set_data_lun {

  ### FILES ###
  #my $file_lun_iops = "$config_directory/LU_dat/LU_IOPS.csv";
  my $file_lun_iops_read_seq  = "$config_directory/LU_dat/LU_Seq_Read_IOPS.csv";
  my $file_lun_iops_write_seq = "$config_directory/LU_dat/LU_Seq_Write_IOPS.csv";
  my $file_lun_iops_read_rnd  = "$config_directory/LU_dat/LU_Rnd_Read_IOPS.csv";
  my $file_lun_iops_write_rnd = "$config_directory/LU_dat/LU_Rnd_Write_IOPS.csv";

  #my $file_lun_trans_rate ="$config_directory/LU_dat/LU_TransRate.csv";
  my $file_lun_trans_rate_read  = "$config_directory/LU_dat/LU_Read_TransRate.csv";
  my $file_lun_trans_rate_write = "$config_directory/LU_dat/LU_Write_TransRate.csv";

  my $file_lun_hit_read_seq  = "$config_directory/LU_dat/LU_Seq_Read_Hit.csv";
  my $file_lun_hit_write_seq = "$config_directory/LU_dat/LU_Seq_Write_Hit.csv";
  my $file_lun_hit_read_rnd  = "$config_directory/LU_dat/LU_Rnd_Read_Hit.csv";
  my $file_lun_hit_write_rnd = "$config_directory/LU_dat/LU_Rnd_Write_Hit.csv";

  #my $file_lun_response = "$config_directory/LU_dat/LU_Response.csv";
  my $file_lun_response_read  = "$config_directory/LU_dat/LU_Read_Response.csv";
  my $file_lun_response_write = "$config_directory/LU_dat/LU_Write_Response.csv";

  my $file_lun_c2d  = "$config_directory/LU_dat/LU_C2D_Trans.csv";
  my $file_lun_d2cs = "$config_directory/LU_dat/LU_D2CS_Trans.csv";
  my $file_lun_d2cr = "$config_directory/LU_dat/LU_D2CR_Trans.csv";

  my $section     = "Lun Level Statistics";
  my $sub_section = "LUN";
  my $metric      = "seq_read_io";

  my $header = "ID,IO Rate(IOPS),Read Rate(IOPS),Write Rate(IOPS),Read Hit(%),Write Hit(%),Read Trans. Size(kB/s),Write Trans. Size(kB/s),Read response time,Write response time,Capacity(MB),Used(MB),Data rate back,IO rate back,seq_read_io,rnd_read_io,seq_write_io,rnd_write_io,seq_read_hit,rnd_read_hit,seq_write_hit,rnd_write_hit";
  $section_header{$section} = $header;

  ### LUN IOPS READ SEQUENTIAL AND TIME ###
  set_data_first( $metric, $section, $sub_section, $file_lun_iops_read_seq );

  ### SET ID LUN/VOLUME
  #set_id_volume($section,$sub_section,$metric);

  ### IOPS READ SEQUENTIAL ###
  #$metric = "IOPS_READ_SEQUENTIAL";
  #set_data($metric, $section, $sub_section, $file_lun_iops_read_seq);

  ### IOPS WRITE SEQUENTIAL ###
  $metric = "seq_write_io";
  set_data( $metric, $section, $sub_section, $file_lun_iops_write_seq );

  ### IOPS READ RANDOM ###
  $metric = "rnd_read_io";
  set_data( $metric, $section, $sub_section, $file_lun_iops_read_rnd );

  ### IOPS WRITE RANDOM ###
  $metric = "rnd_write_io";
  set_data( $metric, $section, $sub_section, $file_lun_iops_write_rnd );

  ### IOPS READ ###
  #set_total_metric("seq_read_io","rnd_read_io","Read Rate(IOPS)",$section,$sub_section);

  ### IOPS WRITE ###
  #set_total_metric("seq_write_io","rnd_write_io","Write Rate(IOPS)",$section,$sub_section);

  ### total IOPS ###
  #set_total_metric("Read Rate(IOPS)","Write Rate(IOPS)","IO Rate(IOPS)",$section,$sub_section);

  ### HIT READ SEQUENTIAL ###
  $metric = "seq_read_hit";
  set_data( $metric, $section, $sub_section, $file_lun_hit_read_seq );

  ### HIT READ RANDOM ###
  $metric = "rnd_read_hit";
  set_data( $metric, $section, $sub_section, $file_lun_hit_read_rnd );

  ### HIT WRITE RANDOM ###
  $metric = "rnd_write_hit";
  set_data( $metric, $section, $sub_section, $file_lun_hit_write_rnd );

  ### HIT WRITE SEQUENTIAL ###
  $metric = "seq_write_hit";
  set_data( $metric, $section, $sub_section, $file_lun_hit_write_seq );

  ### HIT READ ###
  #$metric = "Read Hit(%)";
  #set_total_metric("seq_read_hit","rnd_read_hit",$metric,$section,$sub_section, "seq_read_io","rnd_read_io");

  ### HIT WRITE ###
  #$metric = "Write Hit(%)";
  #set_total_metric("seq_write_hit","rnd_write_hit",$metric,$section,$sub_section, "seq_write_io", "rnd_write_io");

  ### TRANS RATE READ ###
  $metric = "Read Trans. Size(kB/s)";
  set_data( $metric, $section, $sub_section, $file_lun_trans_rate_read );

  ### TRANS_RATE ###
  #$metric = "TRANS_RATE";
  #set_data($metric, $section, $sub_section, $file_lun_trans_rate);

  ### TRANS RATE WRITE ###
  $metric = "Write Trans. Size(kB/s)";
  set_data( $metric, $section, $sub_section, $file_lun_trans_rate_write );

  ### Response TIME ###
  #$metric = "RESPONSE";
  #set_data($metric, $section, $sub_section, $file_lun_response);

  ### Response READ ###
  $metric = "Read response time";
  set_data( $metric, $section, $sub_section, $file_lun_response_read );

  ### Response WRITE ###
  $metric = "Write response time";
  set_data( $metric, $section, $sub_section, $file_lun_response_write );

  #print Dumper \%performance_data;

  ### C2D ###
  $metric = "Data rate back";
  set_data( $metric, $section, $sub_section, $file_lun_c2d );

  ### D2CS ###
  $metric = "D2CS";
  set_data( $metric, $section, $sub_section, $file_lun_d2cs );

  ### D2CR ###
  $metric = "D2CR";
  set_data( $metric, $section, $sub_section, $file_lun_d2cr );

  ### D2CS + D2CR je io_rate_back
  #$metric = "IO rate back";
  #set_total_metric("D2CR","D2CS",$metric,$section,$sub_section);

  ### set metric per id volume ###
  $metric = "seq_read_io,seq_write_io,rnd_read_io,rnd_write_io,Read Trans. Size(kB/s),Write Trans. Size(kB/s),D2CS,Data rate back,D2CR";
  set_id_volume( $section, $sub_section, $metric );

  ### IOPS READ ###
  set_total_metric( "seq_read_io", "rnd_read_io", "Read Rate(IOPS)", $section, $sub_section );

  ### IOPS WRITE ###
  set_total_metric( "seq_write_io", "rnd_write_io", "Write Rate(IOPS)", $section, $sub_section );

  ### total IOPS ###
  set_total_metric( "Read Rate(IOPS)", "Write Rate(IOPS)", "IO Rate(IOPS)", $section, $sub_section );

  ### D2CS + D2CR je io_rate_back
  $metric = "IO rate back";
  set_total_metric( "D2CR", "D2CS", $metric, $section, $sub_section );

  ### HIT READ ###
  $metric = "Read Hit(%)";
  set_total_metric( "seq_read_hit", "rnd_read_hit", $metric, $section, $sub_section, "seq_read_io", "rnd_read_io" );

  ### HIT WRITE ###
  $metric = "Write Hit(%)";
  set_total_metric( "seq_write_hit", "rnd_write_hit", $metric, $section, $sub_section, "seq_write_io", "rnd_write_io" );

  #print "sub volume\n";
  my $file_port = create_port_file();
  my $file_cap  = create_volume_file();
  $metric = "PORT#,/ALPA/C,TID#,LU#,Seq#,Num,LDEV#,Used(MB),LU_CAP(MB),U(%),T(%),PID";
  set_cap_volume( $file_cap, $metric, $section, $sub_section );

  #print Dumper \%performance_data;
} ## end sub set_data_lun

sub create_port_file {
  my $file = "$config_directory/config-PORT.txt";
  return $file;

  my $test = test_file_exist($file);
  if ($test) { return $file; }

  my $cmd = "raidcom get port $autetization";
  cmd( $cmd, $file );

  return $file;
} ## end sub create_port_file

sub create_volume_file {
  my $file = "$config_directory/config-VOLUME.txt";
  return $file;

  my $file_in = "$config_directory/config-PORT.txt";
  my @array   = get_array_data($file_in);
  foreach my $line (@array) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    if ( $line =~ /TAR/ ) {
      my @elements = split( " ", $line );
      $elements[0] = remove_whitespace( $elements[0] );
      my $cmd      = "raidvchkscan -v aou -p $elements[0] $autetization";
      my $save_end = "save";
      cmd( $cmd, $file, $save_end );
    } ## end if ( $line =~ /TAR/ )
  } ## end foreach my $line (@array)
  return $file;
} ## end sub create_volume_file

sub set_cap_volume {
  my ( $file, $metric, $section, $sub_section ) = @_;
  my @data = get_array_data($file);
  my %header;
  my $key_header = "LU#";
  my @metrics = split( ",", $metric );

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ /$key_header/ ) {
      my $prev;
      foreach my $element (@metrics) {
        my $start_element = index( $line, $element );
        if ( $start_element == -1 ) { next; }
        else {
          if ( defined $prev ) {
            $header{$section}{$prev}{END} = $start_element - 1;
          }
          $header{$section}{$start_element}{NAME}  = $element;
          $header{$section}{$start_element}{START} = $start_element;
          if ( !defined $prev ) {
            $prev = $start_element;
            next;
          }
          if ( defined $header{$section}{$prev}{END} ) {
            $header{$section}{$prev}{LENGTH} = $header{$section}{$prev}{END} - $header{$section}{$prev}{START};
          }
          $prev = $start_element;

        } ## end else [ if ( $start_element ==...)]
      } ## end foreach my $element (@metrics)
    } ## end if ( $line =~ /$key_header/)
    else {
      if ( $line =~ /User for/ ) { next; }
      my $lun       = "";
      my $cap_total = "";
      my $cap_used  = "";
      my $cap_free  = "";
      if ( defined $header{$section} ) {
        foreach my $id ( sort { $a <=> $b } keys %{ $header{$section} } ) {
          if ( defined $header{$section}{$id}{LENGTH} ) {
            my $name = $header{$section}{$id}{NAME};
            if ( $name ne "LU#" && $name ne "Used(MB)" && $name ne "LU_CAP(MB)" ) { next; }
            my $start  = $header{$section}{$id}{START};
            my $length = $header{$section}{$id}{LENGTH};
            if ( $start == 0 ) { $length--; }
            my $value = substr( $line, $start, $length );
            if ( !defined $value ) {

              #print "$line\n";
              #print "start $start length $length\n";
            }
            $value = remove_whitespace($value);

            if ( $name eq "LU#" ) {
              $lun = $value;
            }
            if ( $name eq "Used(MB)" ) {
              $cap_used = $value;
            }
            if ( $name eq "LU_CAP(MB)" ) {
              $cap_total = $value;
            }

            if ( $lun ne "" && $cap_total ne "" && $cap_used ne "" ) {
              $cap_free = $cap_total - $cap_used;
              if ( defined $performance_data{$section}{$sub_section}{$lun}{ID} ) {
                $performance_data{$section}{$sub_section}{$lun}{"Capacity(MB)"} = $cap_total;
                $performance_data{$section}{$sub_section}{$lun}{"Used(MB)"}     = $cap_used;
              }
              else { delete $performance_data{$section}{$sub_section}{$lun}; }
            } ## end if ( $lun ne "" && $cap_total...)
          } ## end if ( defined $header{$section...})
        } ## end foreach my $id ( sort { $a ...})
      } ## end if ( defined $header{$section...})
    } ## end else [ if ( $line =~ /$key_header/)]
  } ## end foreach my $line (@data)
} ## end sub set_cap_volume

sub set_data {
  my $metric      = shift;
  my $section     = shift;
  my $sub_section = shift;
  my $file        = shift;
  my $i           = 1;
  my @array       = get_array_data($file);
  my $active      = 0;
  my @header;

  foreach my $line (@array) {
    chomp $line;
    if ( $line =~ /No/ ) {
      @header = split( ",", $line );
      my @pom_array;
      if ( $sub_section eq "RG" ) {
        foreach my $name (@header) {

          #if ($name =~ /X/){
          #  $name =~ s/X//g;
          #  $name =~ s/\(.*//g;
          #  $name = $name . "\"";
          my $size = length $name;
          if ( $size > 9 ) {
            my $pom_name = substr( $name, 0, 9 );
            $name = $pom_name . "\"";
          }

          #}
          push( @pom_array, "$name" );
        } ## end foreach my $name (@header)
        @header = @pom_array;
      } ## end if ( $sub_section eq "RG")
      $i = 2;
      next;
    } ## end if ( $line =~ /No/ )
    if ( $i == 2 ) {
      my @array_line = split( ",", $line );
      foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
        my $position = $performance_data{$section}{$sub_section}{$id}{POSITION};
        if ( !defined $position ) { next; }

        #foreach my $header_element(@header){
        #  $header_element =~ s/"//g;
        #  $header_element = remove_whitespace($header_element);
        #  if ($header_element =~ /No/ || $header_element =~ /time/){
        #    next;
        #  }
        #  if ("$header_element" eq "$id"){
        #    $active = 1;
        #    last;
        #  }
        #}
        my $ids = "\"$id\"";
        if ( !grep { "$ids" eq $_ } @header ) {
          next;
        }

        #if ($active == 0){next;}
        if ( !defined $performance_data{$section}{$sub_section}{$id}{$metric} ) {
          $array_line[$position] =~ s/^\s+|\s+$//g;
          if ( $array_line[$position] =~ /-/ ) {
            $performance_data{$section}{$sub_section}{$id}{$metric} = "NAN";
            next;
          }
          else {
            if ( $metric eq "Response time" || $metric eq "Read response time" || $metric eq "Write response time" ) {
              $array_line[$position] = $array_line[$position] / 1000;     ### microseconds to miliseconds
              $array_line[$position] = round( $array_line[$position] );
            }
          } ## end else [ if ( $array_line[$position...])]
          $performance_data{$section}{$sub_section}{$id}{$metric} = $array_line[$position];
        } ## end if ( !defined $performance_data...)
        $active = 0;
      } ## end foreach my $id ( keys %{ $performance_data...})
      $i = 3;
    } ## end if ( $i == 2 )
  } ## end foreach my $line (@array)
} ## end sub set_data

sub get_hit_numerator_volume {
  my $section     = shift;
  my $sub_section = shift;
  my $metric      = shift;
  my $metric_io   = shift;
  my $id          = shift;
  my $numerator   = shift;

  if ( defined $performance_data{$section}{$sub_section}{$id}{$metric} && $performance_data{$section}{$sub_section}{$id}{$metric} ne "NAN" ) {
    $numerator = $numerator + ( $performance_data{$section}{$sub_section}{$id}{$metric} * $performance_data{$section}{$sub_section}{$id}{$metric_io} );
    return $numerator;
  }

  else {
    return $numerator;
  }

} ## end sub get_hit_numerator_volume

sub get_response_numerator_volume {
  my $section     = shift;
  my $sub_section = shift;
  my $metric      = shift;
  my $metric_io   = shift;
  my $id          = shift;
  my $numerator   = shift;

  if ( defined $performance_data{$section}{$sub_section}{$id}{$metric} && $performance_data{$section}{$sub_section}{$id}{$metric} ne "NAN" ) {
    $numerator = $numerator + ( $performance_data{$section}{$sub_section}{$id}{$metric} * $metric_io );
    return $numerator;
  }

  else {
    return $numerator;
  }

} ## end sub get_response_numerator_volume

sub get_hit_denominator_volume {
  my $section     = shift;
  my $sub_section = shift;
  my $metric_io   = shift;
  my $id          = shift;
  my $denominator = shift;

  if ( defined $performance_data{$section}{$sub_section}{$id}{$metric_io} && $performance_data{$section}{$sub_section}{$id}{$metric_io} ne "NAN" ) {
    $denominator = $denominator + $performance_data{$section}{$sub_section}{$id}{$metric_io};
    return $denominator;
  }

  else {
    return $denominator;
  }

} ## end sub get_hit_denominator_volume

sub get_response_denominator_volume {
  my $section     = shift;
  my $sub_section = shift;
  my $metric_io   = shift;
  my $id          = shift;
  my $denominator = shift;

  $denominator = $denominator + $metric_io;
  return $denominator;

} ## end sub get_response_denominator_volume

sub set_id_volume {

###
  # CL1-A.01(B1-LPAR1-Prod).0
  # CL1-A -         PORT
  # 01 -            HOST GROUP
  # B1-LPAR1-Prod - GROUP NAME
  # 0               ID LUN/VOLUME

  #print Dumper \%performance_data;
  my $sum_1   = 0;
  my $sum_2   = 0;
  my $sum_3   = 0;
  my $sum_4   = 0;
  my $sum_5   = 0;
  my $sum_6   = 0;
  my $sum_7   = 0;
  my $sum_8   = 0;
  my $sum_9   = 0;
  my $last_id = "";

  my $seq_read_hit             = "seq_read_hit";
  my $seq_read_hit_numerator   = 0;
  my $seq_read_hit_denominator = 0;

  my $rnd_read_hit             = "rnd_read_hit";
  my $rnd_read_hit_numerator   = 0;
  my $rnd_read_hit_denominator = 0;

  my $seq_write_hit             = "seq_write_hit";
  my $seq_write_hit_numerator   = 0;
  my $seq_write_hit_denominator = 0;

  my $rnd_write_hit             = "rnd_write_hit";
  my $rnd_write_hit_numerator   = 0;
  my $rnd_write_hit_denominator = 0;

  my $response_read              = "Read response time";
  my $response_read_numerator    = 0;
  my $response_read_denominator  = 0;
  my $response_write             = "Write response time";
  my $response_write_numerator   = 0;
  my $response_write_denominator = 0;

  my $section     = shift;
  my $sub_section = shift;
  my $metric      = shift;
  my @metrics     = split( ",", $metric );
  my @inventory_id;

  my $size = keys %{ $performance_data{$section}{$sub_section} };
  for ( my $i = 1; $i <= $size; $i++ ) {
    foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
      my $id_new = $id;

      #print "$id\n";
      if ( $sub_section eq "LUN" ) {
        $id_new =~ s/.*\)\.//g;
      }
      else {
        #$id_new =~ s/X//g;
        my ( undef, undef, $ldev ) = split( ":", $id_new );
        if ( defined $ldev ) {

          #print "$ldev\n";
          {
            no warnings;

            #$id_new = hex($ldev);
          }
          $id_new = $ldev;
        } ## end if ( defined $ldev )
      } ## end else [ if ( $sub_section eq "LUN")]
      my $check = 0;
      foreach my $line (@inventory_id) {
        if ( $line eq $id_new ) {
          $check = 1;
          last;
        }
      } ## end foreach my $line (@inventory_id)
      if ( $check == 1 ) { next; }

      #print $id . "\n";
      if ( $last_id eq "" ) {
        $last_id = $id_new;
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[0] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[0] } ne "NAN" ) {
          $sum_1 = $sum_1 + $performance_data{$section}{$sub_section}{$id}{ $metrics[0] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[1] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[1] } ne "NAN" ) {
          $sum_2 = $sum_2 + $performance_data{$section}{$sub_section}{$id}{ $metrics[1] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[2] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[2] } ne "NAN" ) {
          $sum_3 = $sum_3 + $performance_data{$section}{$sub_section}{$id}{ $metrics[2] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[3] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[3] } ne "NAN" ) {
          $sum_4 = $sum_4 + $performance_data{$section}{$sub_section}{$id}{ $metrics[3] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[4] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[4] } ne "NAN" ) {
          $sum_5 = $sum_5 + $performance_data{$section}{$sub_section}{$id}{ $metrics[4] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[5] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[5] } ne "NAN" ) {
          $sum_6 = $sum_6 + $performance_data{$section}{$sub_section}{$id}{ $metrics[5] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[6] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[6] } ne "NAN" ) {
          $sum_7 = $sum_7 + $performance_data{$section}{$sub_section}{$id}{ $metrics[6] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[7] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[7] } ne "NAN" ) {
          $sum_8 = $sum_8 + $performance_data{$section}{$sub_section}{$id}{ $metrics[7] };
        }
        $seq_read_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $seq_read_hit, $metrics[0], $id, $seq_read_hit_numerator );
        $seq_read_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[0], $id, $seq_read_hit_denominator );

        $rnd_read_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $rnd_read_hit, $metrics[2], $id, $rnd_read_hit_numerator );
        $rnd_read_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[2], $id, $rnd_read_hit_denominator );

        $seq_write_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $seq_write_hit, $metrics[1], $id, $seq_write_hit_numerator );
        $seq_write_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[1], $id, $seq_write_hit_denominator );

        $rnd_write_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $rnd_write_hit, $metrics[3], $id, $rnd_write_hit_numerator );
        $rnd_write_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[3], $id, $rnd_write_hit_denominator );

        ### response time ###
        my $iops = $performance_data{$section}{$sub_section}{$id}{ $metrics[0] } + $performance_data{$section}{$sub_section}{$id}{ $metrics[1] } + $performance_data{$section}{$sub_section}{$id}{ $metrics[2] } + $performance_data{$section}{$sub_section}{$id}{ $metrics[3] };
        $response_read_numerator  = get_response_numerator_volume( $section, $sub_section, $response_read,  $iops, $id, $response_read_numerator );
        $response_write_numerator = get_response_numerator_volume( $section, $sub_section, $response_write, $iops, $id, $response_write_numerator );

        $response_read_denominator  = get_response_denominator_volume( $section, $sub_section, $iops, $id, $response_read_denominator );
        $response_write_denominator = get_response_denominator_volume( $section, $sub_section, $iops, $id, $response_write_denominator );
        delete( $performance_data{$section}{$sub_section}{$id} );

        next;
      } ## end if ( $last_id eq "" )

      if ( $id_new eq $last_id ) {
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[0] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[0] } ne "NAN" ) {
          $sum_1 = $sum_1 + $performance_data{$section}{$sub_section}{$id}{ $metrics[0] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[1] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[1] } ne "NAN" ) {
          $sum_2 = $sum_2 + $performance_data{$section}{$sub_section}{$id}{ $metrics[1] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[2] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[2] } ne "NAN" ) {
          $sum_3 = $sum_3 + $performance_data{$section}{$sub_section}{$id}{ $metrics[2] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[3] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[3] } ne "NAN" ) {
          $sum_4 = $sum_4 + $performance_data{$section}{$sub_section}{$id}{ $metrics[3] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[4] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[4] } ne "NAN" ) {
          $sum_5 = $sum_5 + $performance_data{$section}{$sub_section}{$id}{ $metrics[4] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[5] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[5] } ne "NAN" ) {
          $sum_6 = $sum_6 + $performance_data{$section}{$sub_section}{$id}{ $metrics[5] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[6] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[6] } ne "NAN" ) {
          $sum_7 = $sum_7 + $performance_data{$section}{$sub_section}{$id}{ $metrics[6] };
        }
        if ( defined $performance_data{$section}{$sub_section}{$id}{ $metrics[7] } && $performance_data{$section}{$sub_section}{$id}{ $metrics[7] } ne "NAN" ) {
          $sum_8 = $sum_8 + $performance_data{$section}{$sub_section}{$id}{ $metrics[7] };
        }

        $seq_read_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $seq_read_hit, $metrics[0], $id, $seq_read_hit_numerator );
        $seq_read_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[0], $id, $seq_read_hit_denominator );
        $rnd_read_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $rnd_read_hit, $metrics[2], $id, $rnd_read_hit_numerator );
        $rnd_read_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[2], $id, $rnd_read_hit_denominator );

        $seq_write_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $seq_write_hit, $metrics[1], $id, $seq_write_hit_numerator );
        $seq_write_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[1], $id, $seq_write_hit_denominator );

        $rnd_write_hit_numerator = get_hit_numerator_volume( $section, $sub_section, $rnd_write_hit, $metrics[3], $id, $rnd_write_hit_numerator );
        $rnd_write_hit_denominator = get_hit_denominator_volume( $section, $sub_section, $metrics[3], $id, $rnd_write_hit_denominator );

        ### response time ###
        my $iops                         = $performance_data{$section}{$sub_section}{$id}{ $metrics[0] } + $performance_data{$section}{$sub_section}{$id}{ $metrics[1] } + $performance_data{$section}{$sub_section}{$id}{ $metrics[2] } + $performance_data{$section}{$sub_section}{$id}{ $metrics[3] };
        my $last_response_read_numerator = $response_read_numerator;
        $response_read_numerator = get_response_numerator_volume( $section, $sub_section, $response_read, $iops, $id, $response_read_numerator );
        if ( $response_read_numerator != $last_response_read_numerator ) {
          $response_read_denominator = get_response_denominator_volume( $section, $sub_section, $iops, $id, $response_read_denominator );
        }
        my $last_response_write_numerator = $response_write_numerator;
        $response_write_numerator = get_response_numerator_volume( $section, $sub_section, $response_write, $iops, $id, $response_write_numerator );
        if ( $response_write_numerator != $last_response_write_numerator ) {
          $response_write_denominator = get_response_denominator_volume( $section, $sub_section, $iops, $id, $response_write_denominator );
        }

        #$response_read_denominator = get_response_denominator_volume($section,$sub_section,$iops,$id,$response_read_denominator);
        #$response_write_denominator = get_response_denominator_volume($section,$sub_section,$iops,$id,$response_write_denominator);
        delete( $performance_data{$section}{$sub_section}{$id} );

      } ## end if ( $id_new eq $last_id)

      #print $performance_data{$section}{$sub_section}{$id} . "\n";

    } ## end foreach my $id ( keys %{ $performance_data...})
    if ( $last_id eq "" ) { next; }

    #print "$last_id $metric $sum\n";
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[0] } = $sum_1;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[1] } = $sum_2;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[2] } = $sum_3;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[3] } = $sum_4;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[4] } = $sum_5;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[5] } = $sum_6;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[6] } = $sum_7;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[7] } = $sum_8;
    $performance_data{$section}{$sub_section}{$last_id}{ $metrics[8] } = $sum_9;
    $performance_data{$section}{$sub_section}{$last_id}{ID}            = $last_id;

    if ( $seq_read_hit_denominator == 0 ) {
      $performance_data{$section}{$sub_section}{$last_id}{$seq_read_hit} = 0;
    }
    else {
      $performance_data{$section}{$sub_section}{$last_id}{$seq_read_hit} = round( $seq_read_hit_numerator / $seq_read_hit_denominator );
    }

    if ( $rnd_read_hit_denominator == 0 ) {
      $performance_data{$section}{$sub_section}{$last_id}{$rnd_read_hit} = 0;
    }
    else {
      $performance_data{$section}{$sub_section}{$last_id}{$rnd_read_hit} = round( $rnd_read_hit_numerator / $rnd_read_hit_denominator );
    }

    if ( $seq_write_hit_denominator == 0 ) {
      $performance_data{$section}{$sub_section}{$last_id}{$seq_write_hit} = 0;
    }
    else {
      $performance_data{$section}{$sub_section}{$last_id}{$seq_write_hit} = round( $seq_write_hit_numerator / $seq_write_hit_denominator );
    }

    if ( $rnd_write_hit_denominator == 0 ) {
      $performance_data{$section}{$sub_section}{$last_id}{$rnd_write_hit} = 0;
    }
    else {
      $performance_data{$section}{$sub_section}{$last_id}{$rnd_write_hit} = round( $rnd_write_hit_numerator / $rnd_write_hit_denominator );
    }
    if ( $response_read_denominator == 0 ) {
      $performance_data{$section}{$sub_section}{$last_id}{$response_read} = 0;
    }
    else {
      $performance_data{$section}{$sub_section}{$last_id}{$response_read} = round( $response_read_numerator / $response_read_denominator );
    }
    if ( $response_write_denominator == 0 ) {
      $performance_data{$section}{$sub_section}{$last_id}{$response_write} = 0;
    }
    else {
      $performance_data{$section}{$sub_section}{$last_id}{$response_write} = round( $response_write_numerator / $response_write_denominator );
    }

    push( @inventory_id, $last_id );
    $last_id                  = "";
    $sum_1                    = 0;
    $sum_2                    = 0;
    $sum_3                    = 0;
    $sum_4                    = 0;
    $sum_5                    = 0;
    $sum_6                    = 0;
    $sum_7                    = 0;
    $sum_8                    = 0;
    $sum_9                    = 0;
    $seq_read_hit_numerator   = 0;
    $seq_read_hit_denominator = 0;
    $rnd_read_hit_numerator   = 0;
    $rnd_read_hit_denominator = 0;

    $seq_write_hit_numerator   = 0;
    $seq_write_hit_denominator = 0;
    $rnd_write_hit_numerator   = 0;
    $rnd_write_hit_denominator = 0;

    $response_write_numerator   = 0;
    $response_write_denominator = 0;
    $response_read_numerator    = 0;
    $response_read_denominator  = 0;

  } ## end for ( my $i = 1; $i <= ...)

  #print Dumper \@inventory_id;

  #print Dumper \%performance_data;

} ## end sub set_id_volume

sub set_data_first {
  my $metric      = shift;
  my $section     = shift;
  my $sub_section = shift;
  my $file        = shift;
  my $i           = 1;
  my @array       = get_array_data($file);
  my $active      = 0;
  my @header_f;

  foreach my $line (@array) {

    #print $line . "\n";
    chomp $line;
    if ( $line =~ /From/ ) {
      my $start = get_start($line);

      #print $start;
      $performance_data{$section}{$time}{$from} = $start;

      #print Dumper \%performance_data;
    } ## end if ( $line =~ /From/ )
    if ( $line =~ /To/ ) {
      my $end = get_end($line);

      #print $start;
      $performance_data{$section}{$time}{$to} = $end;

      #print Dumper \%performance_data;
    } ## end if ( $line =~ /To/ )
    if ( defined $performance_data{$section}{$time}{$to} && $performance_data{$section}{$time}{$from} && $i == 1 ) {
      my $end_time   = str2time( $performance_data{$section}{$time}{$to} );
      my $start_time = str2time( $performance_data{$section}{$time}{$from} );
      my $length     = $end_time - $start_time;
      $performance_data{$section}{$len} = $length;
      $i++;
    } ## end if ( defined $performance_data...)
    if ( $line =~ /No/ ) {
      $i = 2;
      my @array_line = split( ",", $line );
      @header_f = split( ",", $line );
      my @pom_array;
      if ( $sub_section eq "VOLUME" || $sub_section eq "RG" || $sub_section eq "GAD-VOLUME" ) {
        foreach my $name (@header_f) {

          #if ($name =~ /X/){
          #  $name =~ s/X//g;
          #  $name =~ s/\(.*//g;
          #  $name = $name . "\"";
          #  $name =~ s/""/"/g;
          my $size = length $name;
          if ( $size > 9 ) {
            my $pom_name = substr( $name, 0, 9 );
            $name = $pom_name . "\"";
          }

          #}
          push( @pom_array, "$name" );
        } ## end foreach my $name (@header_f)
        @header_f = @pom_array;
      } ## end if ( $sub_section eq "VOLUME"...)
      my $position = -1;
      foreach my $header (@array_line) {
        $position++;
        if ( $header =~ /No/ || $header =~ /time/ ) {
          next;
        }
        else {
          $header =~ s/"//g;
          $header =~ s/^\s+|\s+$//g;
          if ( $sub_section eq "VOLUME" || $sub_section eq "RG" || $sub_section eq "GAD-VOLUME" ) {

            #if ($header =~ /X/){
            #  $header =~ s/X//g;
            #  $header =~ s/\(.*//g;
            my $size = length $header;
            if ( $size > 8 ) {
              my $pom_name = substr( $header, 0, 8 );
              $header = $pom_name;
            }

            #}
          } ## end if ( $sub_section eq "VOLUME"...)
          $performance_data{$section}{$sub_section}{$header}{ID}       = $header;
          $performance_data{$section}{$sub_section}{$header}{POSITION} = $position;
          next;
        } ## end else [ if ( $header =~ /No/ ||...)]
      } ## end foreach my $header (@array_line)
      next;
    } ## end if ( $line =~ /No/ )
    if ( $i == 2 ) {
      my @array_line = split( ",", $line );
      foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
        my $position = $performance_data{$section}{$sub_section}{$id}{POSITION};
        my $ids      = "\"$id\"";
        if ( !grep { "$ids" eq $_ } @header_f ) {
          next;
        }

        #if ($active == 0){next;}
        if ( !defined $performance_data{$section}{$sub_section}{$id}{$metric} ) {
          $array_line[$position] =~ s/^\s+|\s+$//g;
          if ( $array_line[$position] =~ /-/ ) {
            $performance_data{$section}{$sub_section}{$id}{$metric} = "NAN";
          }
          else {
            if ( $metric eq "Response time" || $metric eq "Read response time" || $metric eq "Write response time" ) {
              $array_line[$position] = $array_line[$position] / 1000;     ### microseconds to miliseconds
              $array_line[$position] = round( $array_line[$position] );
            }
            $performance_data{$section}{$sub_section}{$id}{$metric} = $array_line[$position];
          } ## end else [ if ( $array_line[$position...])]
          if ( $sub_section eq "HOST" ) {
            my @file_name = split( "/", $file );
            my $port = $file_name[-1];
            $port =~ s/PortWWN_//g;
            $port =~ s/_.*csv//g;
            $performance_data{$section}{$sub_section}{$id}{PORT} = $port;
          } ## end if ( $sub_section eq "HOST")
        } ## end if ( !defined $performance_data...)
        else {
          if ( $sub_section eq "HOST" ) {
            $array_line[$position] =~ s/^\s+|\s+$//g;
            my @file_name = split( "/", $file );
            my $port = $file_name[-1];
            $port =~ s/PortWWN_//g;
            $port =~ s/_.*csv//g;
            if ( $array_line[$position] !~ /-/ ) {
              if ( $metric eq "Response time" || $metric eq "Read response time" || $metric eq "Write response time" ) {
                $array_line[$position]                                          = $array_line[$position] / 1000;     ### microseconds to miliseconds
                $array_line[$position]                                          = round( $array_line[$position] );
                $performance_data{$section}{$sub_section}{$host_index}{$metric} = $array_line[$position];
                $performance_data{$section}{$sub_section}{$host_index}{ID}      = $id;
                $performance_data{$section}{$sub_section}{$host_index}{PORT}    = $port;
              } ## end if ( $metric eq "Response time"...)
              else {
                $performance_data{$section}{$sub_section}{$host_index}{$metric} = $array_line[$position];
                $performance_data{$section}{$sub_section}{$host_index}{ID}      = $id;
                $performance_data{$section}{$sub_section}{$host_index}{PORT}    = $port;
              }
            } ## end if ( $array_line[$position...])
            else {
              $performance_data{$section}{$sub_section}{$host_index}{$metric} = "NAN";
              $performance_data{$section}{$sub_section}{$host_index}{ID}      = $id;
              $performance_data{$section}{$sub_section}{$host_index}{PORT}    = $port;
            }
            $host_index++;
          } ## end if ( $sub_section eq "HOST")
        } ## end else [ if ( !defined $performance_data...)]

        $active = 0;
      } ## end foreach my $id ( keys %{ $performance_data...})
      $i = 3;
    } ## end if ( $i == 2 )
  } ## end foreach my $line (@array)
} ## end sub set_data_first

sub set_total_metric {
### random + sequential ###
  my $metric_random = shift;
  my $metric_seq    = shift;
  my $metric_total  = shift;
  my $section       = shift;
  my $sub_section   = shift;
  my $seq_io        = shift;
  my $rnd_io        = shift;
  my $sum           = "";
  if ( $metric_total eq "Data rate back" || $metric_total eq "IO rate back" || $metric_total eq "Read Rate(IOPS)" || $metric_total eq "Write Rate(IOPS)" || $metric_total eq "IO Rate(IOPS)" ) {

    foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
      my $random = $performance_data{$section}{$sub_section}{$id}{$metric_random};
      my $seq    = $performance_data{$section}{$sub_section}{$id}{$metric_seq};
      if ( !defined $random || !defined $seq ) { next; }
      if ( $random eq "NAN"  && $seq ne "NAN" )  { $sum = $seq; }
      if ( $random ne "NAN"  && $seq eq "NAN" )  { $sum = $random; }
      if ( $random ne "NAN"  && $seq ne "NAN" )  { $sum = $random + $seq; }
      if ( $random eq "NAN"  && $seq eq "NAN" )  { $sum = "NAN"; }
      if ( !isdigit($random) && !isdigit($seq) ) { $sum = "NAN"; }
      $performance_data{$section}{$sub_section}{$id}{$metric_total} = $sum;
    } ## end foreach my $id ( keys %{ $performance_data...})
  } ## end if ( $metric_total eq ...)

  if ( $metric_total eq "Read Hit(%)" || $metric_total eq "Write Hit(%)" ) {
    foreach my $id ( keys %{ $performance_data{$section}{$sub_section} } ) {
      my $random = $performance_data{$section}{$sub_section}{$id}{$metric_random};
      my $seq    = $performance_data{$section}{$sub_section}{$id}{$metric_seq};
      if ( !defined $random || !defined $seq ) { next; }
      if ( !isdigit($random) ) { $random = "NAN"; }
      if ( !isdigit($seq) )    { $seq    = "NAN"; }
      my $seq_iops = $performance_data{$section}{$sub_section}{$id}{$seq_io};
      my $rnd_iops = $performance_data{$section}{$sub_section}{$id}{$rnd_io};
      if ( !isdigit($seq_iops) && !isdigit($rnd_iops) ) {
        $performance_data{$section}{$sub_section}{$id}{$metric_total} = "NAN";
        next;
      }
      if ( !isdigit($seq_iops) && isdigit($rnd_iops) ) {
        $performance_data{$section}{$sub_section}{$id}{$metric_total} = $random;
        next;
      }
      if ( isdigit($seq_iops) && !isdigit($rnd_iops) ) {
        $performance_data{$section}{$sub_section}{$id}{$metric_total} = $seq;
        next;
      }
      if ( $rnd_iops + $seq_iops == 0 ) {
        my $total = 0;
        $performance_data{$section}{$sub_section}{$id}{$metric_total} = $total;
        next;
      }
      my $total = ( ( $seq * $seq_iops ) + ( $random * $rnd_iops ) ) / ( $rnd_iops + $seq_iops );
      $total = round($total);
      $performance_data{$section}{$sub_section}{$id}{$metric_total} = $total;
      next;
    } ## end foreach my $id ( keys %{ $performance_data...})

  } ## end if ( $metric_total eq ...)
} ## end sub set_total_metric

sub error_die {
  my $message = shift;
  print STDERR "$message\n";

  my $cmd  = "raidcom -logout $autetization";
  my $file = "/dev/null";
  cmd( $cmd, $file, $timeout_cmd_low );
  exit(1);
} ## end sub error_die

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

sub round {
  my $number = shift;
  my $num    = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > 3 ) {
      my $rounded = sprintf( "%.3f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub check_space_name {
  my $name       = shift;
  my $name_space = $name;
  if ( $name =~ m/ / ) {
    $name_space = "\"" . $name . "\"";    # it must be here to support space with server names
    return $name_space;
  }
  else {
    return $name;
  }
} ## end sub check_space_name

sub get_start {
  my $start = shift;
  $start =~ s/From//g;
  $start =~ s/://;
  $start =~ s/\//:/g;
  $start =~ s/^\s+|\s+$//g;
  $start =~ s/ /T/g;
  $start = $start . ":00";
  return $start;
} ## end sub get_start

sub get_end {
  my $end = shift;
  $end =~ s/To//g;
  $end =~ s/://;
  $end =~ s/\//:/g;
  $end =~ s/^\s+|\s+$//g;
  $end =~ s/ /T/g;
  $end = $end . ":00";
  return $end;
} ## end sub get_end

sub test_file_exist {
  my $file = shift;
  if ( !-e $file || -z $file ) {
    return 0;
  }
  else {
    return 1;
  }
} ## end sub test_file_exist

sub write_data {
  foreach my $title ( keys %section_header ) {
    open( DATA, ">>$output" ) or error_die("Cannot open file: $output : $!");

    #print "$title\n";
    get_data( "$title", "$section_header{$title}" );
    if ( !defined $performance_data{$title}{TIME}{FROM} || !defined $performance_data{$title}{TIME}{TO} || !defined $performance_data{$title}{LENGTH} ) {
      close DATA;
      next;
    }
    print DATA "$title\n";
    my $interval_start  = " Interval Start:  " . $performance_data{$title}{TIME}{FROM};
    my $interval_end    = " Interval End:    " . $performance_data{$title}{TIME}{TO};
    my $interval_length = " Interval Length: " . $performance_data{$title}{LENGTH};
    print DATA" $interval_start";
    print DATA"\n";
    print DATA" $interval_end";
    print DATA"\n";
    print DATA" $interval_length";
    print DATA"\n";
    print DATA "-----------------------------------\n";

    #print DATA"\n";
    #$section{$sect} =~ s/\n//g;
    print DATA $section_header{$title};
    print DATA @values;
    print DATA "\n\n";
    close DATA;
  } ## end foreach my $title ( keys %section_header)

} ## end sub write_data

sub get_data {
  my $name_section     = shift;
  my $section_contents = shift;
  @values = "";
  foreach my $id ( keys %{ $performance_data{$name_section} } ) {
    if ( $id eq "LENGTH" || $id eq "TIME" ) { next; }
    foreach my $section ( keys %{ $performance_data{$name_section}{$id} } ) {
      if ( !defined $performance_data{$name_section}{$id}{$section}{ID} ) {
        next;
      }
      if ( !@values eq "" ) {
        my $last = pop(@values);
        $last =~ s/,//g;
        push( @values, $last );
      }
      push( @values, "\n" );
      my @headers = split( ",", $section_contents );
      foreach my $element (@headers) {
        chomp $element;
        $element =~ s/^\s+|\s+$//g;
        if ( defined $performance_data{$name_section}{$id}{$section}{$element} && $performance_data{$name_section}{$id}{$section}{$element} ne "NAN" ) {
          push( @values, "$performance_data{$name_section}{$id}{$section}{$element}," );
        }
        else {
          push( @values, "," );
        }
      } ## end foreach my $element (@headers)
    } ## end foreach my $section ( keys ...)
  } ## end foreach my $id ( keys %{ $performance_data...})
  my $last = pop(@values);
  $last =~ s/,//g;
  push( @values, $last );
} ## end sub get_data

sub data_structure {
  my $directory = $configuration;
  if ( !-d $directory ) {
    mkdir $directory || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
  }
  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || error_die( "Cannot read $directory: $!" . __FILE__ . ":" . __LINE__ );
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
} ## end sub data_structure

sub cmd {
  my $cmd_raidcom = shift;
  my $file        = shift;
  my $timeout     = shift;
  my $save        = shift;
  my $pid         = fork();

  if ( !defined $pid ) {
    error_die( "no fork $!" . __FILE__ . ":" . __LINE__ );
  }
  elsif ( $pid == 0 ) {
    setpgrp( 0, 0 );
    if ( defined $save && $save eq "save" ) {
      `$cmd_raidcom >> "$file" 2>>\$ERRLOG_VSPG`;
    }
    else {
      `$cmd_raidcom > "$file" 2>>\$ERRLOG_VSPG`;
    }
    if ( $? != 0 ) {
      error( "command failed: $cmd_raidcom " . __FILE__ . ":" . __LINE__ );
    }
    exit(0);
  } ## end elsif ( $pid == 0 )
  eval {
    local $SIG{ALRM} = sub { die "die in SIGALRM"; };
    alarm($timeout);
    waitpid( $pid, 0 );
    alarm(0);
  };
  if ( $@ =~ /die in SIGALRM/ ) {
    my $act_time = localtime();
    error( "command: \"$cmd_raidcom > $file\" timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ );
    kill -15, $pid;
    waitpid( $pid, 0 );
  } ## end if ( $@ =~ /die in SIGALRM/)
  return;
} ## end sub cmd

sub get_name_perf_file {

  #my $dir = "/sys2rrd/HUS_CLI/configuration";
  my @time       = localtime();
  my $time_human = localtime();
  my $sec        = $time[0];
  my $minute     = $time[1];
  my $hour       = $time[2];
  my $day        = $time[3];
  my $month      = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;

  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $name_file = "$wrkdir/$STORAGE_NAME" . "_vspgperf_" . "$year$month$day" . "_" . "$hour$minute" . ".out";

  #print "$name_file\n";
  return $name_file;

} ## end sub get_name_perf_file

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( !defined($digit) ) {
    return 0;
  }
  if ( $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

### main function ####
#eval{
#Set alarm
#my $act_time = localtime();
#my $act_directory = $act_time;
#local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
#alarm($timeout);
my @entry_time = split( " ", $act_directory );
( undef, my $active_minute, undef ) = split( ":", $entry_time[3] );
$config_directory = "$configuration/configuration";

#$data_directory = "$configuration/55";
$data_directory = "$configuration/$active_minute";

if ( !-d $data_directory ) {
  mkdir $data_directory || error_die( "Cannot read $data_directory: $!" . __FILE__ . ":" . __LINE__ );
}

#$config_directory= "$configuration/00";
my $command_file  = "$configuration/$STORAGE_NAME-type-vspg$active_minute.txt";
my $command_file2 = "$configuration/$STORAGE_NAME-type-hus_vm$active_minute.txt";
data_structure();

my $cmd  = "raidcom -login $login $paswd $autetization";
my $file = "/dev/null";
cmd( $cmd, $file, $timeout_cmd_low );

collect_metrics( $command_file, $command_file2 );
set_data_port();

#set_data_lun();
set_data_ldev();

#print Dumper \%performance_data;
set_data_proc_usage();
set_data_cpu_util();
set_data_raid_group();
set_data_wwn();
set_data_pool();

$cmd = "raidcom -logout $autetization";
cmd( $cmd, $file, $timeout_cmd_low );

#print Dumper \%performance_data;
#print Dumper \%section_header;
write_data();

#if ($active_minute eq "00"){
#  system "perl $bindir/vspgconf.pl";

#}

#alarm(0);
#};

#if ($@){
#  if ($@ =~ /died in SIG ALRM/){
#    my $act_time = localtime();
#    error_die(" script vspgperf.pl timed out after : $timeout seconds");
#  }
#}

### end main function ###
