#!/bin/ksh

# Parameters:
# Param1 - SAN name

#set -x 

PATH=$PATH:/usr/bin:/bin:/usr/local/bin:/opt/freeware/bin
export PATH

LANG=C
export LANG=C

if [ `locale 2>/dev/null| grep  LC_NUMERIC| egrep "^C$|US" | wc -l` -eq 0 ]; then
  # locale LC_NUMERIC might cause a problem in decimal separator, setting to "C"
  LC_NUMERIC=C
  export LC_NUMERIC
fi


# it is necessary as files need to be readable also for WEB server user
umask 022

# Load STORAGE2RRD configuration
dir=`dirname $0`
CFG="$dir/etc/stor2rrd.cfg"
. $CFG
DEBUG=0

PERL5LIB=$BINDIR:$PERL5LIB
export PERL5LIB

# Load "magic" setup
if [ -f "$dir/etc/.magic" ]; then
  . $dir/etc/.magic
fi

UPGRADE=0
if [ ! -f $dir/tmp/$version ]; then
  UPGRADE=1
fi

if [ "$INPUTDIR" = "." ]; then
   INPUTDIR=`pwd`
   export INPUTDIR
fi
if [ x"$STORAGE_USER" = "x" ]; then
  echo "STORAGE_USER does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$SAN_CFG" = "x" ]; then
  echo "SAN_CFG does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$PERL" = "x" ]; then
  echo "PERL does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$SAMPLE_RATE" = "x" ]; then
  echo "SAMPLE_RATE does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi

cd $INPUTDIR

# Check if it runs under the right user
if [ `uname -s` = "SunOS" ]; then
  # Solaris does not have -X
  install_user=`ls -l etc/stor2rrd.cfg|awk '{print $3}'`
else
  install_user=`ls -lX etc/stor2rrd.cfg|awk '{print $3}'`  # must be X to do not cut user name to 8 chars
fi

running_user=`id |awk -F\( '{print $2}'|awk -F\) '{print $1}'`
if [ ! "$install_user" = "$running_user" ]; then
  echo "You probably trying to run it under wrong user"
  echo "STOR2RRD files are owned by : $install_user"
  echo "You are : $running_user"
  echo "STOR2RRD should run only under user which owns installed package"
  echo "Do you want to really continue? [n]:"
  read answer
  if [ "$answer"x = "x" -o "$answer" = "n" -o "$anwer" = "N" ]; then
    exit
  fi
fi

if [ ! -f "$SAN_CFG" ]; then
  echo "$SAN_CFG does not exist, exiting"
  exit 0
fi

#SNMP port in etc/san-list.cfg
SNMP_PORT=`cat $SAN_CFG|sed 's/#.*$//'|egrep "^SNMP_PORT"|egrep -v "^#"|sed -e 's/^SNMP_PORT=//'`
export SNMP_PORT

SNMP_VERSION=`cat $SAN_CFG|sed 's/#.*$//'|egrep "^SNMP_VERSION"|egrep -v "^#"|sed -e 's/^SNMP_VERSION=//'`
if [ "$SNMP_VERSION"x = "x" ]; then
  SNMP_VERSION=1
fi
SNMP_VERSION_GLOBAL=$SNMP_VERSION

date=`date`
SAMPLE_RATE_DEF=$SAMPLE_RATE
for line in `cat $SAN_CFG|sed 's/#.*$//'|egrep ".*:.*:BNA:"|egrep -v "^#"`
do
  # Only SAN load ....
  # IP:SAN::
  SAN_IP=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
  export SAN_IP

  SAN_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
  export SAN_TYPE

  SAMPLE_RATE=$SAMPLE_RATE_DEF
  echo "$SAN_IP: $date : starting"

  if [ ! "$SAMPLE_RATE_min"x = "x" -a -f "$INPUTDIR/tmp/$SAN_IP-run" ]; then
    (( SAMPLE_RATE = SAMPLE_RATE_min * 60 ))
    #SAMPLE_RATE=`echo "$SAMPLE_RATE_min*60"|bc` # bc does not have to exist everywhere
    # check how offten it should run as per timestamp and skip it if it too early
    time_last_tmp=`$PERL -e '$inp=shift;$st_name=shift;$v = (stat("$inp/tmp/$st_name-run"))[9]; print "$v\n";' $INPUTDIR $SAN_IP`
    time_act=`date "+%s"`
    d=`date`
    #(( time_last = SAMPLE_RATE + time_last - 50 )) # put 50 sec to make sure
    grace_time=50
    time_last=`expr $SAMPLE_RATE + $time_last_tmp - $grace_time`
    echo "$SAN_IP: $d : $time_last -gt $time_act  -- $SAMPLE_RATE -- $INPUTDIR/tmp/$SAN_IP-run"
    if [ $time_last -gt $time_act ]; then
      echo "Skipping now   : $SAN_IP: not process this time due to sample_rate : $SAMPLE_RATE"
      continue
    else
      # go further for the data load
      touch "$INPUTDIR/tmp/$SAN_IP-run"
    fi
  else 
    touch "$INPUTDIR/tmp/$SAN_IP-run"
  fi
  echo "Continue now   : $SAN_IP "
  export SAMPLE_RATE

  DATEEXT=`date "+%Y%m%d"`
  DATETIMEEXT=`date "+%Y%m%d-%H%M"`
  ERRLOG_SAN="$ERRLOG-${SAN_IP}"
  OUTLOG="$INPUTDIR/logs/output.log-${SAN_IP}"
  export ERRLOG_SAN OUTLOG

  if [ "$SAN_TYPE" = "BNA" ]; then
    BNA_IP=`echo $line | awk 'BEGIN{FS=":"}{print $1}'`
    export BNA_IP

    BNA_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $2}'`
    export BNA_PORT

    BNA_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $3}'`
    export BNA_TYPE

    BNA_USER=`echo $line | awk 'BEGIN{FS=":"}{print $4}'`
    export BNA_USER

    BNA_PASS=`echo $line | awk 'BEGIN{FS=":"}{print $5}'`
    export BNA_PASS

    BNA_IP_SEC=`echo $line | awk 'BEGIN{FS=":"}{print $6}'`
    export BNA_IP_SEC

    $BINDIR/bna_stor_load.sh $BNA_IP 2>>$ERRLOG_SAN 1>>$OUTLOG &
    if [ ! $? -eq 0 ]; then
      echo "`date` : An error occured in bnaperf.pl, check $ERRLOG and output of $0" >> $ERRLOG
      echo "`date` : An error occured in bnaperf.pl, check $ERRLOG and output of $0"
    fi
  fi

done

# wait for all jobs to let see the ouput
for job in `jobs -p`
do
    echo "Waiting for $job"
    wait $job
done

# It must be here for rrdtool purposes!!!! Only for AIX
if [ "$PERL" = "/opt/freeware/bin/perl" ]; then
  export LIBPATH=/opt/freeware/lib:$LIBPATH
fi

# update output files to rrd
$PERL -w $BINDIR/san_rrdupdate.pl 2>>$ERRLOG
