#!/bin/ksh

# Parameters:
# Param1 - Storage name

#set -x 

PATH=$PATH:/usr/bin:/bin:/usr/local/bin:/opt/freeware/bin
export PATH

LANG=C
export LANG=C

if [ `locale 2>/dev/null| grep  LC_NUMERIC| egrep "^C$|US" | wc -l` -eq 0 ]; then
  # locale LC_NUMERIC might cause a problem in decimal separator, setting to "C"
  LC_NUMERIC=C
  export LC_NUMERIC
fi


# it is necessary as files need to be readable also for WEB server user
umask 022

# Load STORAGE2RRD configuration
dir=`dirname $0`
CFG="$dir/etc/stor2rrd.cfg"
. $CFG
DEBUG=0

PERL5LIB=$BINDIR:$PERL5LIB
export PERL5LIB

# Load "magic" setup
if [ -f "$dir/etc/.magic" ]; then
  . $dir/etc/.magic
fi

UPGRADE=0
if [ ! -f $dir/tmp/$version ]; then
  UPGRADE=1
fi
export UPGRADE

if [ "$INPUTDIR" = "." ]; then
   INPUTDIR=`pwd`
   export INPUTDIR
fi

if [ x"$STORAGE_USER" = "x" ]; then
  echo "STORAGE_USER does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$STORAGE_CFG" = "x" ]; then
  echo "STORAGE_CFG does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$PERL" = "x" ]; then
  echo "PERL does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi
if [ x"$SAMPLE_RATE" = "x" ]; then
  echo "SAMPLE_RATE does not seem to be set up, correct it in stor2rrd.cfg"
  exit 0
fi

cd $INPUTDIR

cd $INPUTDIR
# Check if it runs under the right user
if [ `uname -s` = "SunOS" ]; then
  # Solaris does not have -X
  install_user=`ls -l etc/stor2rrd.cfg|awk '{print $3}'`
else
  install_user=`ls -lX etc/stor2rrd.cfg|awk '{print $3}'`  # must be X to do not cut user name to 8 chars
fi

running_user=`id |awk -F\( '{print $2}'|awk -F\) '{print $1}'`
if [ ! "$install_user" = "$running_user" ]; then
  echo "You probably trying to run it under wrong user"
  echo "STOR2RRD files are owned by : $install_user"
  echo "You are : $running_user"
  echo "STOR2RRD should run only under user which owns installed package"
  echo "Do you want to really continue? [n]:"
  read answer
  if [ "$answer"x = "x" -o "$answer" = "n" -o "$anwer" = "N" ]; then
    exit
  fi
fi
SAMPLE_RATE_DEF=$SAMPLE_RATE
for line in `cat $STORAGE_CFG|sed 's/#.*$//'|egrep ":COMPELLENT:"|egrep -v "^#"|sed 's/ /===space===/g'`
do
  # Storage Alias:COMPELLENT:IP address:PORT:USER:PASSWORD:VOLUME_AGG_DATA_LIM:VOLUME_AGG_IO_LIM:SAMPLE_RATE_MINS
  STORAGE_NAME=`echo $line | awk 'BEGIN{FS=":"}{print $1}' | sed 's/===space===/ /g'`
  STORAGE_TYPE=`echo $line | awk 'BEGIN{FS=":"}{print $2}' | sed 's/===space===/ /g'`


  # COMPELLENT
  COMPELLENT_IP_SPA=`echo $line | awk 'BEGIN{FS=":"}{print $3}' | sed 's/===space===/ /g'`
  COMPELLENT_PORT=`echo $line | awk 'BEGIN{FS=":"}{print $4}' | sed 's/===space===/ /g'`
  COMPELLENT_USER=`echo $line | awk 'BEGIN{FS=":"}{print $5}' | sed 's/===space===/ /g'`
  COMPELLENT_PW=`echo $line | awk 'BEGIN{FS=":"}{print $6}' | sed 's/===space===/ /g'`

  if [ x"$COMPELLENT_PORT" = "x" ]; then
    COMPELLENT_PORT=3033 ### Defalt
  fi

  export STORAGE_NAME STORAGE_TYPE COMPELLENT_IP_SPA COMPELLENT_USER COMPELLENT_PW COMPELLENT_PORT

  SAMPLE_RATE=$SAMPLE_RATE_DEF
  echo "$STORAGE_NAME: $date : starting"

  if [ ! "$SAMPLE_RATE_min"x = "x" -a -f "$INPUTDIR/tmp/$STORAGE_NAME-run" ]; then
    (( SAMPLE_RATE = SAMPLE_RATE_min * 60 ))
    #SAMPLE_RATE=`echo "$SAMPLE_RATE_min*60"|bc` # bc does not have to exist everywhere
    # check how offten it should run as per timestamp and skip it if it too early
    time_last_tmp=`$PERL -e '$inp=shift;$st_name=shift;$v = (stat("$inp/tmp/$st_name-run"))[9]; print "$v\n";' $INPUTDIR $STORAGE_NAME`
    time_act=`date "+%s"`
    d=`date`
    #(( time_last = SAMPLE_RATE + time_last - 50 )) # put 50 sec to make sure
    grace_time=50
    time_last=`expr $SAMPLE_RATE + $time_last_tmp - $grace_time`
    echo "$STORAGE_NAME: $d : $time_last -gt $time_act  -- $SAMPLE_RATE -- $INPUTDIR/tmp/$STORAGE_NAME-run"
    if [ $time_last -gt $time_act ]; then
      echo "Skipping now   : $STORAGE_NAME: not process this time due to sample_rate : $SAMPLE_RATE"
      continue
    else
      # go further for the data load
      touch "$INPUTDIR/tmp/$STORAGE_NAME-run"
    fi
  else-
    touch "$INPUTDIR/tmp/$STORAGE_NAME-run"
  fi
  echo "Continue now   : $STORAGE_NAME: sample_rate : $SAMPLE_RATE"
  export SAMPLE_RATE

  DATEEXT=`date "+%Y%m%d"`
  DATETIMEEXT=`date "+%Y%m%d-%H%M"`
  ERRLOG_COMPELLENT="$ERRLOG-${STORAGE_NAME}"
  OUTLOG="$INPUTDIR/logs/output.log-${STORAGE_NAME}"
  export ERRLOG_COMPELLENT OUTLOG
  if [ ! -d "$INPUTDIR/data/$STORAGE_NAME" ]; then
    mkdir "$INPUTDIR/data/$STORAGE_NAME"
  fi

  $BINDIR/compellent_stor_load.sh "$STORAGE_NAME" 2>>"$ERRLOG_COMPELLENT" 1>>"$OUTLOG" &
  if [ ! $? -eq 0 ]; then
    echo "`date` : An error occured in compellentperf.pl, check $ERRLOG and output of $0" >> $ERRLOG
    echo "`date` : An error occured in compellentperf.pl, check $ERRLOG and output of $0"
  fi

done

# wait for all jobs to let see the ouput
for job in `jobs -p`
do
    echo "Waiting for $job"
    wait $job
done

